// This file is generated by xtask. Do not edit it manually.

//! Constants and MDS matrix for the BN254 curve with the following properties:
//!
//! * x^5 S-boxes
//! * 3 prime fields (one zero prime field and two inputs from the caller)
//! * 8 full rounds and 57 partial rounds
//!
//! Those parameters are used for our Poseidon hash implementation.
//!
//! They were generated using the official script from the Poseidon paper:
//! [generate_parameters_grain.sage](https://extgit.iaik.tugraz.at/krypto/hadeshash/-/blob/master/code/generate_parameters_grain.sage)
//! with the following parameters:
//!
//! ```bash
//! sage generate_parameters_grain.sage 1 0 254 3 8 57 0x30644e72e131a029b85045b68181585d2833e84879b9709143e1f593f0000001
//! ```
pub const FULL_ROUNDS: usize = 8;
pub const PARTIAL_ROUNDS: [usize; 15] =
    [56, 57, 56, 60, 60, 63, 64, 63, 60, 66, 60, 65, 70, 60, 64];
pub const ALPHA: u64 = 5;

use crate::{PoseidonError, PoseidonParameters};
/// Returns Poseidon parameters for the BN254 curve with the following
/// properties:
///
/// * x^5 S-boxes
/// * 3 inputs (one input with zeros and two inputs from the syscall)
/// * 8 full rounds and 57 partial rounds
///
/// The argument of this macro is a type which implements
/// [`ark_ff::PrimeField`](ark_ff::PrimeField).
use ark_ff::PrimeField;
// to avoid warnings when width_limit_13 feature is used
#[allow(unused_variables)]
pub fn get_poseidon_parameters<F: PrimeField + std::convert::From<ark_ff::BigInteger256>>(
    t: u8,
) -> Result<PoseidonParameters<F>, PoseidonError> {
    if t == 0_u8 {
        Err(PoseidonError::InvalidWidthCircom {
            width: t as usize,
            max_limit: 13usize,
        })
    } else if 2 == t {
        let ark = vec![
            F::from(ark_ff::BigInteger256::new([
                7817616578516342695,
                12041114087599168795,
                18311542326445259320,
                703809069991762900,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4331763550625516417,
                14201023056939694132,
                3069182888826189263,
                865630468011650756,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3566474286914161200,
                11533337848482785469,
                1454188893941615860,
                2173164774482348753,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5546713666651497759,
                1033391138293734294,
                579106019186219236,
                2859554329514026824,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8767088134270176771,
                4695306501711252744,
                4209243863731017562,
                2192991121640820561,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17805298208823395027,
                307985100983112664,
                9602098390607458494,
                3055472776560074307,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3909282107706262992,
                13368341216138434697,
                1588073989677247728,
                821623203604019809,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15755426777904750113,
                803549746874466029,
                9443029210981129783,
                85256046229640304,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3247251991413109177,
                7841264143924049595,
                18269122915921683453,
                167401055085182909,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17247355753810440945,
                6068911149477229014,
                7812597725061720419,
                1699111382917971815,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9086616471667026833,
                11249876738920507827,
                10531948423244928237,
                3180709101917510650,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6045638319889455877,
                14048485700492070853,
                12407180079391136579,
                2072174480898319934,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6746110078988393302,
                33862480341740752,
                12893055833257516694,
                2688747040681077131,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12762121410920968153,
                12583693102466367891,
                8782570957304574643,
                2901298802732085988,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3713045009585348675,
                1753181911864277204,
                2064049868111244977,
                2891709276830441329,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1377859005849043657,
                17213302918879987073,
                2250098761913978943,
                3179338808143010527,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7236023902221217737,
                15698137225854421603,
                9669998922288655229,
                552725606465661892,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4024542742229642300,
                10676338206571928785,
                2835917563533532670,
                157668069081079593,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8178076668328059049,
                12133895062126733470,
                11597165300144392768,
                2002257975968913669,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3056574329694873596,
                15152835891335545375,
                2336296560329179629,
                3322569079616917512,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12034835072381781688,
                11425246710696692215,
                1304950287065922640,
                1307437582895567736,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7625324869890990606,
                15546400408522906137,
                10556636253062694543,
                208545047093026385,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7874519155181355406,
                5699848720424976135,
                9177764430905078766,
                2242454803620536339,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16918768489580524930,
                1451360297318946841,
                8369495911896295217,
                1617545695573194041,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2102853383857409991,
                14818696684473492633,
                6265715398128913129,
                2899825424943956492,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7012031407790312396,
                7509561033518701014,
                15049632849442349075,
                2029501729809436367,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10865896729759530941,
                1457647043961857778,
                17836922434109140734,
                2409290043080073586,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1270566041830377352,
                14891794189842536868,
                7258252898854293803,
                3174236381978470231,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14648639749528776515,
                1660232324255600783,
                5210356524639407775,
                1649235115699342888,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5246011419136659397,
                8583939463376414991,
                3988539085160041585,
                1683479457214066523,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15620172062379863602,
                7684285279883317076,
                18351497879714015238,
                897786720739953686,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10540561140639464845,
                8506771935836737836,
                14152240211101678991,
                1890431831991524277,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10793138928641828495,
                600618435932543590,
                13877963886452429632,
                923200061044958949,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16205809318213838742,
                7397066335257516860,
                3160198027865047320,
                1510601990922667766,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9866801240052691026,
                14702449656399697352,
                10881482235068204458,
                676305374017174802,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12677365286172731501,
                1380435406082811253,
                13772227141419739068,
                1911790097914468531,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5489908542194921755,
                2355146368963641151,
                14798397298817373515,
                1210238513617216306,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8460619148043677965,
                18025577457283642374,
                13675017155226021032,
                395862298079042366,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12655186675713445102,
                13358011002929926133,
                15297284634119018636,
                2917372813350125739,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8034273717284931869,
                2185982791782152782,
                12951413382393270415,
                2440726185204156347,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4539500151132553907,
                8691376595824906200,
                4478210354694403277,
                2010233294366166640,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5248272632199042497,
                15189336710541847786,
                7498899605340474637,
                2778056244288907515,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13620651143643946710,
                14256350727656881413,
                14270146554555854060,
                1015468296665738832,
            ])),
            F::from(ark_ff::BigInteger256::new([
                292554386543128384,
                10515732338267378288,
                8858109166056940592,
                348791369923272870,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3104459525386259054,
                3706509501996465542,
                9346457373566313197,
                998628960351368213,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9955704627847062859,
                3816150674512517976,
                8539215117740000962,
                2851037169766766418,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9563222423978047977,
                4134090717682400521,
                4931886900955358576,
                2181286346732320574,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14179168894901588400,
                18055191287518198047,
                7173507097360274271,
                2426987407020299813,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6365929671746204939,
                9605027792591653205,
                5594195411713214760,
                250501994217948813,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10132915144326598729,
                14559655054176323244,
                7670837463673856084,
                12761215476027430,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1135687413980796326,
                15748351523129783305,
                2119141318581095358,
                1310926709634850203,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6973840191868739265,
                1133547488339520846,
                7358060163725254722,
                856422543455136065,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2537781629768649754,
                3952926295132561306,
                6300924289786463548,
                1930189990533446405,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5893579505698628222,
                6778702467274214034,
                6866130142038735678,
                1555505760660470978,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15478953192464200210,
                17403454457297442222,
                7721529128250960608,
                1360294703589482407,
            ])),
            F::from(ark_ff::BigInteger256::new([
                353188889562277050,
                14892186255215643417,
                11432173571329174096,
                1137487999655651371,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11276615796778280802,
                12776567518989086543,
                15488694486527196992,
                2277707700932731970,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10166206986740036819,
                364106907529202693,
                17928232400318857972,
                1062132786353612405,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8828067068861466321,
                4253212145573359161,
                7613892610833863104,
                3222308960075869855,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10432355210345241023,
                14877110326726120038,
                6198435499051035424,
                1917810163665871564,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9789163323282868015,
                15042360471542282800,
                6820767863046574897,
                3031296629605872474,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10751711984671309920,
                16661266258360878322,
                1565276276401255440,
                2592303526438568973,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2742332881621149863,
                11803538530403661451,
                2637639035157985731,
                398335568721703586,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3126142991524034215,
                151352264296347077,
                14564917722384201204,
                1369303116767000612,
            ])),
            F::from(ark_ff::BigInteger256::new([
                517437299484701558,
                10194692261461724337,
                16589035092694756392,
                1853713095538544151,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2639673604112600959,
                1428143039648754754,
                13095801650290172662,
                97933583764930581,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16902791728491724753,
                18311104903645370571,
                1988066165513084494,
                2977850697400937375,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5541440123987594748,
                9591427865345548116,
                9669088081248008287,
                794484372090197528,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5717472324945572905,
                6174074361583416910,
                1577632871765737854,
                3481129492503380273,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13325680800061527036,
                12309078185183640140,
                16763311912973755637,
                1566110581857700361,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12249548568115988678,
                11209261032218673420,
                4940466595080791277,
                771487592615259358,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2674544028287596442,
                17248485300825939625,
                10949481555690624629,
                1023121073329425969,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11747515358778335254,
                7464561602524039318,
                12859524779717724390,
                727145153511787684,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4779681098544026607,
                10801352859288203177,
                5187327735492849400,
                309704060256086324,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15238782239290182539,
                18305014652360812689,
                13719472254031951616,
                3374147316341246576,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15578969976403941546,
                12537435997175236533,
                14478162230856265230,
                2277595356037893698,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13204649732309636058,
                11785500951278913133,
                10339958013586985354,
                1955459727616805928,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14569030102282623316,
                10385392823757262256,
                8359554707346512716,
                2608113925887867088,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12074698654214521341,
                11618779204221940505,
                3516582691944782661,
                589852033284418243,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11157311367097653628,
                14371626200785010329,
                11766745241084052131,
                3141834832237377712,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12733235032607510141,
                9323601586081538062,
                14895748692241263038,
                1016926342461848108,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9859264004464829177,
                11058394381060735221,
                14506646442508249481,
                3323282756208557411,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2582612150703885187,
                8306177362261553980,
                897928877326560533,
                918639075433083910,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10956170673865248644,
                8564036573374780377,
                15445891645117163942,
                644115059586624836,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2711973265794170614,
                16656643648783866019,
                5962640706602075203,
                2481938773001882888,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17590887190243061952,
                8924991201578943168,
                13862352318032358220,
                297054414896745259,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1872614578063478665,
                151963239291045450,
                9755516631272594764,
                2539981115033014794,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17278980520978006631,
                15212604883303952510,
                4838520370261379088,
                370619450042907230,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14719036969121305105,
                4766984980478767513,
                14864126913127613826,
                3185439275271214870,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17523523959145140574,
                3301704068096314519,
                2341590068010164151,
                819420160480278553,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12608605747964306847,
                13939825926796408526,
                14246908603479019478,
                2459097707443965012,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2847753020822552945,
                14842493287604615071,
                5630649191723782325,
                2654162296948548606,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8327608375789042030,
                3984233256092441221,
                4266911563483004824,
                791139856424493144,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17393623960378222830,
                8556845277389089803,
                11063746623933562198,
                3295116572797994866,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6556981464207204531,
                14679551020872837925,
                6266901575482294147,
                2752996138607489391,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2545246976953366491,
                15571854123925772603,
                7520613325025670458,
                2715526405698945879,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2268689995884986655,
                8137678705750911999,
                13876952357584857273,
                2717023506413470608,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10140370558899107377,
                16040958931821867185,
                4070783410166728933,
                2505483470326499820,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16097710549234583465,
                4782418641697995316,
                5748722374410837534,
                159588883507037298,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11291816946150689618,
                7290161082082722149,
                1044034546628418201,
                2969291899118471681,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17154880108210828863,
                7344728179590740873,
                7769584270863497050,
                1075648365980683583,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1803399554144300973,
                18288289715294841213,
                246138044464968337,
                561843230295672562,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2330968321100917994,
                10414440925487288588,
                17625909583140434097,
                2502093891459445253,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12526288117101684134,
                8362912079772406840,
                9992675896193181808,
                91505079800739868,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16646710225556983433,
                16370209032864520791,
                7060165441931734303,
                2544652218228964676,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9190675348599428386,
                3041910325904342697,
                13456873213918960905,
                3334999382905368224,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4990425590775788876,
                12631006258972777509,
                4007773506108580907,
                1102194232115527012,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16582422522765043534,
                5861473983963984992,
                18047721925441662696,
                2255525574963620325,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6047098222080324489,
                18077495893277810065,
                2765439401678121352,
                663459475751375352,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5357278748647445464,
                4894994898466515186,
                5073755966281596982,
                1935053366752426707,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1883309416397200082,
                4002604494382115756,
                15080032785222886426,
                1617550320980187549,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3518572601742433300,
                14256394187580425454,
                10323645900473976460,
                2160975567672505059,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13582610698040532064,
                16633901617425400798,
                12036492491914469759,
                894114751296418035,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9586227146206078216,
                6997012586024043659,
                10570874657405218792,
                1935907046324692848,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8733995415077984145,
                17742211468904935423,
                6633993672740213564,
                60609538245954180,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15210507094558518286,
                2809927191747993009,
                122303601839491740,
                1444545258862845613,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13876121165305895300,
                13982005152139757037,
                10500856908045162502,
                3390148465756674001,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15619953150446111841,
                621725405936757634,
                3460714094183172112,
                886900683506979925,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2865370493750824041,
                10674917767472006754,
                3932750800682271584,
                1658717153232615468,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4158634352451534074,
                17327208825304179714,
                12078702696456144043,
                3399401658749671310,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15476451516339742249,
                15960447891634313671,
                5788235009515645148,
                1936609771979262650,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5638973398222657237,
                3843419977431318862,
                5895548160828353119,
                680512758060564959,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6871413275513517638,
                15736227474207349895,
                10508251890877065022,
                1443618183687786010,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4713332816877647293,
                7821048273009846355,
                2633126125869664571,
                1614527264469018704,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4777309128810888759,
                6593676141362790707,
                11856581453418389393,
                1422992746367173494,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1854369959592209452,
                165389863918942103,
                3465148226042280996,
                531999173462682860,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1618443999406896336,
                8490012754175994739,
                3938620013027383472,
                1546080401611773023,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13866024322801563888,
                6616940439054472445,
                18254911664339537644,
                2782744476025532954,
            ])),
        ];
        let mds = vec![
            vec![
                F::from(ark_ff::BigInteger256::new([
                    13613439442855361237,
                    12612317059679343271,
                    17010153325034879546,
                    463711907270265477,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    380039182625911272,
                    2353060624420563571,
                    8611693173801816027,
                    3142750857801084569,
                ])),
            ],
            vec![
                F::from(ark_ff::BigInteger256::new([
                    17686748653477138425,
                    13168698300709470193,
                    3167853822857717295,
                    920278981868324822,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    2431048004232414152,
                    12531362618959840161,
                    11752828045514564013,
                    1329940993925894997,
                ])),
            ],
        ];
        return Ok(crate::PoseidonParameters::new(
            ark,
            mds,
            FULL_ROUNDS,
            PARTIAL_ROUNDS[0],
            t.into(),
            ALPHA,
        ));
    } else if 3 == t {
        let ark = vec![
            F::from(ark_ff::BigInteger256::new([
                10169643024046198382,
                1496489459468413331,
                15013179087956216865,
                1074572035719075096,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6239455757362194532,
                9834463921409911007,
                6450940760387492813,
                67910589270332556,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16751688058965816309,
                3514822508764902252,
                2245502192544547001,
                639497164796119454,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2348173180299641296,
                6251932254013741678,
                14109259725315707848,
                3397893802321178182,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10052922493383044338,
                13214685027855777508,
                2600475563429683764,
                3110546625639709581,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16594432877155917050,
                6436419528597111786,
                14746450667121398997,
                223438775977242494,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10301900068242346605,
                14463238905455560233,
                9959550014935311931,
                2918681975259572866,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1515787476805471096,
                2320345919784178364,
                17402400686162957325,
                2821337195423189251,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13329421459288540722,
                7997218926952286166,
                16666341364125177037,
                2544186741652662210,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14815153299398317096,
                12326536988431973148,
                9189081236192950112,
                1564197350901735823,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15492898315278275062,
                10236271341713535282,
                6834674029815110883,
                993319518520783711,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10916819061073399196,
                16883629385166233580,
                6921874708522579303,
                2003422334954896330,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1714630715101799769,
                14370321670889575288,
                16856964725142963021,
                1176014593347144415,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16756379877222262774,
                2968515920474853548,
                11179479702445996788,
                2264545440983580541,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14570479280712353541,
                18444734288884600265,
                15176568821682145435,
                2131985990035738644,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10368363968134106237,
                12047691299872059909,
                12705208366813915547,
                351098625659878575,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17792640394833160232,
                9696427064407893159,
                1520876545687531284,
                464672949723461184,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11511719757092779449,
                908211774484017294,
                12831319216039648607,
                691674951170147412,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9510773949154959270,
                5252381917909929361,
                8955462025988621972,
                372877525797294092,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1587374528693423580,
                9430068611203050871,
                12735463350683146671,
                831949795656129422,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13358750343521025489,
                5314631703920803920,
                6493880948315916231,
                1268631432301053871,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14849984304993963273,
                3191591746076102373,
                2677903913737681795,
                222096387432894843,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13720721153165534565,
                6013694977031309706,
                4391455072429085237,
                3226292761893557358,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10579744368738693369,
                4762779664445951531,
                5823120412175020918,
                22573339300296900,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9870159944135898805,
                7235529342485293563,
                13480538772416018769,
                2536150988941404724,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16720789321578578560,
                2522999998976288164,
                14519837292043943523,
                416852298297550563,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17158443766907913819,
                5961907595872497041,
                13404357466265350738,
                755230009370792854,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6770390416031239872,
                17390336258944747484,
                12772936794330968795,
                3058989918289202421,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8549908775410123872,
                17012614519708971561,
                10615274498289114563,
                1930267710583523745,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10382466218761732032,
                2783011596995200052,
                4587954668146979961,
                1351136628495840386,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9387305021536313998,
                5949266803577039558,
                9176955995885039695,
                825414034829436333,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3651076418516780104,
                8972364766045991658,
                15788598065422186641,
                251119977008056894,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5043382280892331119,
                15357393874202951105,
                16800243441893211803,
                407154788704645140,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8892180399712360046,
                4645155678429152180,
                7968129386745339652,
                3001532499684763796,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5799476994968113711,
                7965868846460524636,
                5018001173082862978,
                2324871211209925355,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5344206134167390343,
                13497554678722349933,
                10730987543428800497,
                3317931201752125258,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16632037294831661512,
                1693965690577373394,
                14571655515630349125,
                1459114695872543527,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14430416535844145905,
                17897610966497868991,
                6311387765811975117,
                893062843581543608,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17384577287839799891,
                16667598527773455984,
                5115149256409481050,
                183011890581373942,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2654438065656423180,
                7668446424519157688,
                6893111809130332402,
                3336873827275337690,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13358667107604709390,
                9882503929482424440,
                7194379942859558459,
                36531667966592230,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13441731135665126200,
                16305593838641322507,
                1764900527919325278,
                3374868242345334601,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11029036156622950918,
                3767439400139705484,
                5458142085128053488,
                3481451364176733008,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12056858118866257581,
                17339159417937361729,
                2457694693050504889,
                2035381829023865427,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1659725671961508204,
                14153602109795078942,
                330192436874582254,
                1203842034152894467,
            ])),
            F::from(ark_ff::BigInteger256::new([
                259699493296123375,
                15961029778693674869,
                4457811829436721853,
                3033599411683850228,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6678010397103840401,
                13421605108260313849,
                18365044507182290269,
                2050525847674035638,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11254465069202884256,
                11407387557876501754,
                16913602912723147595,
                527834611401260952,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9948922113988233040,
                2908314857315156352,
                11624323827010875732,
                2402506864120038473,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8375966539063315102,
                13612191774966483206,
                625257772314144528,
                3393392318161610646,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16601869878711972622,
                10913809729668532042,
                12078243733072605372,
                3262715235694311176,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15608316279448174251,
                12662471127068531388,
                10439508139178259089,
                1598586842980050115,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8608532833637053201,
                15539768828324126602,
                2760369212817214746,
                3120064577965505096,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4779699930828877946,
                7904642866356798671,
                2142298301270611063,
                3210992535745164252,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16566647052584656297,
                11392807339846396333,
                17509790013340347161,
                2120971635856277866,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5536220222296839465,
                18007269287740125490,
                16988379011452997949,
                2326967073555529329,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5762632206312391938,
                11442382750973961555,
                5697416040074198063,
                886287582240954391,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11124293347412048724,
                2187704089670443107,
                11696998627716706352,
                1678389218646739097,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4222149753036133103,
                16909372562319609761,
                17823522529666649570,
                1915744512019107209,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9386254665973801738,
                369343627046108186,
                15055393180748924159,
                463031500367058716,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2811225156588656644,
                4043324678723349205,
                2419972489884392763,
                3047898763605249269,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16526798864378746441,
                6805642193013234135,
                5181939895697731017,
                1417276069191301076,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5489393743654840986,
                13956742994665100569,
                1860634826424985449,
                1234361436619539990,
            ])),
            F::from(ark_ff::BigInteger256::new([
                695054542422134769,
                12019305300333709961,
                9843681412023287159,
                67383789733625196,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9537157493527123923,
                4615195029880018393,
                10026734106030658022,
                1019300322962990368,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5063327708825434761,
                4132830927089380748,
                2617264721062385806,
                3023383046488159874,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1028346471269770686,
                6490335944564795927,
                16130261465708739976,
                3050889264512630458,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17168111284362386161,
                17486960142991188712,
                17076376746195272713,
                2054593364731744093,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15262887479649442245,
                9505100495929542591,
                13855096146465136747,
                1520808608629131791,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16676344953669021952,
                12246747301111320017,
                13957281728738336067,
                343989352091453469,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11421158882397661543,
                5440847660503758404,
                1213793981452479827,
                1419614475451705339,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10601982260714973412,
                3234541434695713580,
                17688098355818888058,
                118717801572347293,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9449177667644177154,
                5375657923623462618,
                6543835113666090251,
                3076897632389067651,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11492857592340264518,
                12443281895567735798,
                2310318942922528275,
                2111500766629273819,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1519060835352555293,
                13341127823664892785,
                10328870498885827883,
                293460558823836962,
            ])),
            F::from(ark_ff::BigInteger256::new([
                299896046629691334,
                11021848952288240530,
                2040032867427221576,
                1534260234819767393,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4982201902518914468,
                5815106221790245040,
                4534518171699180816,
                807161977694049528,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9362303805132752942,
                11020562407162685405,
                6099272804889998405,
                672695088330099010,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3829537558432663078,
                12224726650097720431,
                2790510079905979068,
                465113627828659524,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12434856935128332283,
                4099816137562300822,
                5291509800583676536,
                1087889381526684220,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3058931662356704787,
                5455375754292263881,
                9999573367242040862,
                357735772164585264,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13940484895148816216,
                9804876323883312219,
                13953255888728739331,
                3122849638617761013,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9511239974733687279,
                10237687677678873181,
                15913306046644034507,
                1312634215549218900,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10675073936499958761,
                12216566237822522633,
                9866914788429900494,
                2398083397542529034,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16885311708936791373,
                7241347522894119874,
                1903990226363296297,
                1142822869681276272,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10686875188351513074,
                13614366883246235843,
                16098305773841909460,
                1312582147797590020,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15963606144910950453,
                11980267062975283715,
                12501611241057470952,
                2442346185398548571,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9980420217207238983,
                13074906607698438413,
                16119634457238226018,
                1595462938248686755,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11617670042665110210,
                12376478257583768892,
                17757582124498240376,
                2232629200712726708,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4280439214426330778,
                15305717883324493701,
                9837552695788434311,
                570372229075787216,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12249253053461168316,
                7282067329116856604,
                13237337846485506915,
                407792342528658083,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3286879747248941119,
                4573432664601249672,
                18158049416766721452,
                2373493147218089556,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11230019463442324329,
                7912136838618106113,
                10841000784467734883,
                333599382087531880,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16473825850870796314,
                13624353028681918311,
                16451401310565375468,
                2875703994879278542,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1687796451943245009,
                7928980879331832355,
                13305261426485117430,
                3453928292668169945,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7598338644619817656,
                10729144591383079437,
                4630335118873107699,
                2222469752301051004,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16080230830081207066,
                12423042439722618314,
                1549230324095215118,
                1545911809876140835,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2904543854463076229,
                8579485197604225815,
                14453736105001743365,
                1921955480208825986,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17038711017906273399,
                9452543770650697585,
                18004737915611345751,
                2703598197550172048,
            ])),
            F::from(ark_ff::BigInteger256::new([
                998897782403394151,
                6032549410726290231,
                14877717775171186163,
                1559795874367549122,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3392697467836765991,
                15922000626033260368,
                1079313812342659430,
                2599093181141465496,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6377813262293748891,
                5181859273934949669,
                1433048174057411302,
                13070502006410567,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4145621046567048658,
                12650345898048364247,
                10942059026714787025,
                3456438562529005033,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16806278217174170814,
                3356626440802756849,
                6836507271113723160,
                52217025170580825,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9040885813031482642,
                2063783757152207784,
                9696596405493190145,
                733897032897314666,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9163181484798974349,
                1106091070615971483,
                4199500626945875606,
                10275305867431991,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5830843384116006313,
                4139255191202704419,
                2940723024787082394,
                1665582135731026707,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11317622713099790344,
                6974195757466602969,
                8072839204361927383,
                1660226640530079177,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5201316823598627553,
                14715480498374840762,
                3383068957560025966,
                3002673069591357036,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13094889931869145459,
                16648982407050338536,
                18289605278935059783,
                2205569170731292504,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11285488793507827465,
                18350717580005913679,
                6410861464604314458,
                2148895943228264365,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6709750789426163512,
                8992917079336798153,
                4616613824157795742,
                2507867874581904081,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15400784301878071037,
                18244452579437791868,
                4224191343226798030,
                1076433859378617427,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4264996793658278532,
                1376105531089050016,
                17476204074859227269,
                1252174447925563362,
            ])),
            F::from(ark_ff::BigInteger256::new([
                626157971226405295,
                18360428839510729767,
                11418101606126458807,
                1253929125255671226,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17894393997393060966,
                1693282321587506006,
                10610815137722143707,
                3307095170886567317,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8412444619122534829,
                17661690963694250141,
                11127731205362461725,
                3460712092451636609,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7076365036357439622,
                2996018067437664761,
                18288959755437174594,
                3095133733569333888,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9345554969304508355,
                9734718780845491687,
                16548111906976335323,
                1269566828719773023,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14985592217270212608,
                13047571069448810567,
                1768764949568828727,
                1744502220636328542,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7800483618336468662,
                15083323447214890130,
                5582574125022657056,
                2891339774727348131,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5365510678292251879,
                9677253646602045298,
                10729843634325729280,
                594190901955360016,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11387234443688137019,
                1279424678775871569,
                7014609431566646248,
                824027491960199506,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1457211823867907931,
                6383027606162415120,
                7228109047101594260,
                1065216652437508569,
            ])),
            F::from(ark_ff::BigInteger256::new([
                61141279752947089,
                17065237724822033421,
                1927762415130120514,
                3094885811492450599,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3912754945728835960,
                3771356032485721483,
                15031711080582401655,
                815716758723016167,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14565351617278385472,
                11584783007181865516,
                15626181124208896302,
                805425446120956330,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16142621932647703252,
                6320039675520759995,
                15672753973526549481,
                2267858513861341332,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15749878102686188083,
                614109551756748456,
                12585727881591095317,
                104123295342755102,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8512653459191138419,
                8722136429807568464,
                3646779388833721243,
                877373150580798793,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6717122165374165389,
                8988252081842455491,
                5645968194207689527,
                1252839797334746901,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1595382035876223936,
                2894788246433044882,
                7254516605129339151,
                3392435059111642139,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13998303702462746555,
                13567266803394983698,
                11055620437644727489,
                2193238868414204989,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8541111410171021429,
                15510862156074330629,
                7499768617590159362,
                129069364530034896,
            ])),
            F::from(ark_ff::BigInteger256::new([
                836365385714984206,
                7494700099001437856,
                9725935530670171100,
                2267339694435901381,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5135733616265261081,
                2426332595100882671,
                4656936268612107177,
                2675901305471578230,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10151513934083020635,
                3753973305408449599,
                5932656183844634499,
                2589779655048364701,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7411025101745302873,
                6707027749470441567,
                14604097463168980328,
                3152993128539731999,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9228435110421995615,
                7077145627258986441,
                8287837705857547985,
                1122529595160356003,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14632877491481866408,
                1722445538081828426,
                7473487843190403406,
                2549223116576907002,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7911624047840495944,
                13006776561127361280,
                9244905133407413598,
                1551237582814894454,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5734626924445778447,
                3904426628295653716,
                758384697192160672,
                1817860466175176350,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16442809390655319567,
                4264455610716866612,
                3430587810186965360,
                3042544677360207512,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12976680908701618498,
                9884505158002686887,
                4506805106606625170,
                1754597687761070431,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4651435674279269163,
                3103468488976219594,
                86770003631272336,
                351588734636199512,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2260337876880351011,
                12758337300762156683,
                4666632919716891739,
                1142547878393382557,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12238467918978150848,
                8498440410987792772,
                12272022433944218978,
                399903969432840125,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15083162859014497690,
                2572883854687877808,
                5295861190485996843,
                1405947505286439302,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16277807055507155125,
                9178820837597334391,
                583720080208146622,
                235956917338752750,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12275617654227012184,
                8055449209487420250,
                18443150053558188092,
                2591750978202878614,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1829412907376403186,
                1784551171442285202,
                9110382820117307087,
                745804616459797009,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17636728153934123131,
                16340444422133123832,
                13081815878669958460,
                498866876760309842,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10821885995694136577,
                12723584804667044276,
                5146301989088820936,
                1263385986896629429,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10800072172023390641,
                5971967654560104426,
                6545357310958328301,
                1429505446243742742,
            ])),
            F::from(ark_ff::BigInteger256::new([
                18269496140486461307,
                16002478335803382869,
                8120489888303010331,
                1628640051152447506,
            ])),
            F::from(ark_ff::BigInteger256::new([
                18122091663874610253,
                4233014431675774575,
                17240608988078162137,
                355592338756513175,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15417051770040314348,
                1867975149401080782,
                1066645743524851259,
                2697118902655154489,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11529920096058011314,
                8488218827988262189,
                16569498673561862223,
                3463317764937792863,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2293207290640910226,
                307269496307905699,
                2019605757530151924,
                1196143593917332415,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17054418173490849888,
                11471621437112287983,
                2736637320199971288,
                1819134904140512630,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17466733393954671453,
                15116232513785238565,
                15003334136884310855,
                1735727526517594389,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3018231278213947665,
                11756906826253427310,
                865102719128852662,
                2970189933199996587,
            ])),
            F::from(ark_ff::BigInteger256::new([
                53857651063310097,
                8083318015622867638,
                11670144255177893450,
                1181189051546590777,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7264345978047633403,
                3427453013614820152,
                17687308340198489875,
                858883541620126021,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16139757679130157878,
                3528952606930721195,
                15856398903938866312,
                1645649108355563827,
            ])),
            F::from(ark_ff::BigInteger256::new([
                18259613611884802074,
                744602176078339578,
                4356130164393161908,
                750620222363875571,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8117441027290110048,
                12767141919626113394,
                10277542331312621822,
                1902664590289501020,
            ])),
            F::from(ark_ff::BigInteger256::new([
                18142567679047472086,
                2784389961213710191,
                4140168213814042183,
                684823272106514114,
            ])),
            F::from(ark_ff::BigInteger256::new([
                282509298302633771,
                14901915657276303392,
                13769774206967582656,
                1592939612075046988,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15029079146827225611,
                5277947701434344677,
                10740941382983966314,
                781955803547370784,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13046325154127803473,
                15271841070263964187,
                3603800691510092686,
                1905881597770513026,
            ])),
            F::from(ark_ff::BigInteger256::new([
                18147836268617712982,
                10413298699095885023,
                9327065618741831658,
                3206880635179173199,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7961104417159548784,
                13344422238921471806,
                11924406431390384663,
                2052665222524073279,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13005032304728461175,
                17582243758993224695,
                10248473147828664992,
                3231875006450919708,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9604737890175299306,
                11952591534011753808,
                14943182360968004505,
                2593879447300027479,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16136936607574934125,
                12635460431001064455,
                14910545630443985360,
                690275998716546918,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9974859801427953200,
                8970509559472668454,
                13147802568752216335,
                1769242106777687234,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16195182707833107583,
                15827858176195376533,
                9364816585327216436,
                3481532623310383999,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11349797944779537784,
                7848337389964465624,
                1585489724280364098,
                1562486739353219241,
            ])),
            F::from(ark_ff::BigInteger256::new([
                299574738534894284,
                15774196221256035113,
                11560070881270290821,
                2136624426723258025,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16521154246496397771,
                5642919338817456453,
                17724106494152086094,
                2852725321712119593,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8984735811309350240,
                2167745649215877194,
                226808028189901552,
                2318647723425753931,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11717711903977946719,
                11734121989528668209,
                1543209836968400498,
                679312990471436121,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7167914974923007396,
                4867725154692709913,
                9475724683532666747,
                382518031130098914,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7263451747656492529,
                15612406511579619656,
                15794977949985463993,
                1951799731841132862,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14253531380344450780,
                17149421595627640462,
                18275357549111665566,
                2442398084493626838,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8882024121061072648,
                14660249638811034002,
                2479936304909085374,
                2088357835029043913,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3009881635143434496,
                3213792448625028097,
                13467690131828994998,
                1090178415541778203,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13614238871538377504,
                14133257625619989830,
                12680744397006639806,
                1617164053432527250,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15283478423327660144,
                13704112697969002700,
                3365542576422386393,
                1127004209850885532,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9649179334161025978,
                15235357479981583552,
                6326192347223033870,
                1818343763663989123,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14849751341126543608,
                11488527144984798584,
                9120114908867576777,
                1164989258755683746,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4658640436750183089,
                3560836902621619118,
                16331544763316127153,
                1144107236774418521,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16764062968747494690,
                3289301024319987453,
                599939870177378126,
                2409257677298851547,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17910303629776146785,
                4333503720265116478,
                5349829146288687643,
                2136215616631132703,
            ])),
        ];
        let mds = vec![
            vec![
                F::from(ark_ff::BigInteger256::new([
                    18364386637727142283,
                    10718140578549479307,
                    12877775606795118513,
                    1196690043610129609,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    15476133327266186208,
                    10598338402256349781,
                    7701464610378726588,
                    1652049073940316358,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    8726449643484246893,
                    17800103023430280315,
                    17732896993093103117,
                    3139215236411295113,
                ])),
            ],
            vec![
                F::from(ark_ff::BigInteger256::new([
                    15431936778085062513,
                    3227108208197516766,
                    13385661772669246410,
                    2984182855068918912,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    3004517874245172771,
                    13302916677726082624,
                    5519286447998647745,
                    3324810986103434297,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    14459594221771332602,
                    1480925685775052901,
                    10894586239177917757,
                    1157550379892963766,
                ])),
            ],
            vec![
                F::from(ark_ff::BigInteger256::new([
                    3630540040312041895,
                    16631083012873467731,
                    963663003087537756,
                    1454699978888462131,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    12784839371101448465,
                    15012715472039172022,
                    9414510034565922201,
                    1687935244450844709,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    8297773286174348768,
                    11980778987040554871,
                    1709819669659146559,
                    1847597393482099700,
                ])),
            ],
        ];
        return Ok(crate::PoseidonParameters::new(
            ark,
            mds,
            FULL_ROUNDS,
            PARTIAL_ROUNDS[1],
            t.into(),
            ALPHA,
        ));
    } else if 4 == t {
        let ark = vec![
            F::from(ark_ff::BigInteger256::new([
                10018390284920759269,
                196898842818127395,
                5249540449481148995,
                1853312570062057576,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12486221224710452438,
                2372038863109147677,
                8230667498854222355,
                2764611904404804029,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4466505105966356650,
                4686185096558265002,
                16210260819355521378,
                1844031548168280073,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15002325471271702008,
                5581154705073500415,
                1229208533183169201,
                1549225070791782920,
            ])),
            F::from(ark_ff::BigInteger256::new([
                18309653156114024706,
                798761732958817262,
                6904962453156279281,
                3335412762186210716,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2824096028161810206,
                14640933461146357672,
                957840840567621315,
                1024001058677493842,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14339023814126516630,
                12239068001133297662,
                428134084092645147,
                2673682960814460689,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6214865908119297870,
                17923963059035301363,
                10985380589240272449,
                1430464474809378870,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5109255232332580664,
                11913027714091798733,
                4449570166290740355,
                864862123557185234,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2323272968957708806,
                354488099726909104,
                115174089281514891,
                80808271106704719,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9646436663147525449,
                3404572679246369876,
                2350204275212843361,
                1069216089054537871,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5059356740217174171,
                4245857056683447103,
                2426504795124362174,
                350059533408463330,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14876286709841668328,
                6932857857384975351,
                7976037835777844091,
                738350885205242785,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16522097747524989503,
                4157368317794149558,
                10343110624935622906,
                2709590753056582169,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8805379462752425633,
                8594508728147436821,
                15629690186821248127,
                2936193411053712582,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17046614324338172999,
                14086280776151114414,
                2804088968006330580,
                728643340397380469,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12986735346000814543,
                6140074342411686364,
                6041575944194691717,
                896092723329689904,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9573905030842087441,
                12243211539080976096,
                15287161151491266826,
                1310836290481124728,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8865134002163281525,
                6813849753829831047,
                9066778847678578696,
                2801725307463304665,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4931814869361681093,
                13712769805002511750,
                1776191062268299644,
                2068661504023016414,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8161631444256445904,
                3049786034047984668,
                1021328518293651309,
                2147500022207188878,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12766468767470212468,
                926098071429114297,
                17691598410912255471,
                76565467953470566,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15547843034426617484,
                13465733818561903358,
                11157089789589945854,
                3107062195097242290,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16908372174309343397,
                17264932925429761530,
                11508063480483774160,
                2682419245684831641,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4870692136216401181,
                17645600130793395310,
                2758876031472241166,
                874943362207641089,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4540479402638267003,
                13477556963426049071,
                6055112305493291757,
                1810598527648098537,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7894770769272900997,
                9595210915998428021,
                7642295683223718917,
                2210716392790471408,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10910178561156475899,
                15811627963917441510,
                16460518660187536520,
                1698297851221778809,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7831732902708890908,
                1464390598836302271,
                8568564606321342514,
                3007171090439369509,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12758232712903990792,
                5937193763836963893,
                4629415695575460109,
                2476198378403296665,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16185652584871361881,
                3161867062328690813,
                8447947510117581907,
                452436262606194895,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10531967515434376071,
                5577695765815843856,
                9164856352050088505,
                1205339682110411496,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3898841196333713180,
                14650521577519770525,
                5736581618852866049,
                1010789789328495026,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12103741763020280571,
                14760208106156268938,
                15246749619665902195,
                1987439155030896717,
            ])),
            F::from(ark_ff::BigInteger256::new([
                326429241861474059,
                11335157279655967493,
                16233357323017397007,
                2124770605461456708,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13507610432344102875,
                9765425316929074945,
                10455054851855122687,
                3371280263716451574,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9433430149246843174,
                16916651192445074064,
                12002862125451454299,
                3293088726774108791,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15895963712096768440,
                10975964170403460506,
                7594578539046143282,
                441635248990433378,
            ])),
            F::from(ark_ff::BigInteger256::new([
                55564641555031451,
                2316046008873247993,
                6273091099984972305,
                531938487375579818,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17845282940759944461,
                6735239388814238924,
                3181517889518583601,
                2376846283559998361,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14097127963645492314,
                1165420652731038559,
                12527303660854712762,
                2717289076364278965,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15600044695084040011,
                255324662529267034,
                11859356122961343981,
                2571979992654075442,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1589817027469470176,
                1086723465680833706,
                6948011514366564799,
                2482410610948543635,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6071201116374785253,
                16554668458221199618,
                16319484688832471879,
                2792452762383364279,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13535048470209809113,
                1831807297936988201,
                16757520396573457190,
                508291910620511162,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6946737468087619802,
                14033399912488027565,
                12701200401813783486,
                1348363389498465135,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6788008051328210729,
                13866524545426155292,
                4317879914214157329,
                2633928310905799638,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1183626302001490602,
                10035686235057284266,
                1656321729167440177,
                1887128381037099784,
            ])),
            F::from(ark_ff::BigInteger256::new([
                964566190254741199,
                17650087760652370459,
                14904592615785317921,
                2929864473487096026,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13584300701347139198,
                512534187550045064,
                13489711551083721364,
                41824696873363624,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17586611824788147557,
                6430987250922925699,
                9294838151373947091,
                348446557360066429,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15025298913764434311,
                14393211163878018166,
                7154440178410267241,
                3057088631006286899,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13451769229280519155,
                17839347496757587523,
                10553299811918798519,
                2523373819901075642,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16267315463205810352,
                13830706729545301172,
                15413288900478726729,
                287556136711008934,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4573780169675443044,
                8758089751960064775,
                2470295096511057988,
                51551212240288730,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7093949836145798554,
                12771428392262798771,
                17021632567931004395,
                1558106578814965657,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8205915653008540447,
                10376314495036230740,
                5774593793305666491,
                2231830927015656581,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10783762484003267341,
                10229708558604896492,
                1831638669050696278,
                2190429714552610800,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7310961803978392383,
                12793746113455595394,
                17036245927795997300,
                3106081169494120044,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17421859032088162675,
                7339791467855418851,
                4622175020331968961,
                590786792834928630,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14242884250645212438,
                12806057845811725595,
                7743423753614082490,
                213381026777379804,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1110713325513004805,
                8318407684973846516,
                15952888485475298710,
                1018983205230111328,
            ])),
            F::from(ark_ff::BigInteger256::new([
                533883137631233338,
                333001117808183237,
                16968583542443855481,
                329716098711096173,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4449676039486426793,
                7760073051300251162,
                5615103291054015906,
                2516053143677338215,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16503526645482286870,
                6358830762575712333,
                12313512559299087688,
                2716767262544184013,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5426798011730033104,
                13085704829880126552,
                6356732802364281819,
                2175930396888807151,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8262282602783970021,
                2576069526442506486,
                14199683559983367515,
                3432491072538425468,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14778817021916755205,
                6110468871588391807,
                2850248286812407967,
                3411084787375678665,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4906200604739023933,
                12096549814065429793,
                5988343102643160344,
                309820751832846301,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8709336210313678885,
                10520000332606345601,
                4756441214598660785,
                2483744946546306397,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9617950371599090517,
                6702332727289490762,
                7078214601245292934,
                215269160536524476,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14694170287735041964,
                13462371741453101277,
                7691247574208617782,
                1078917709155142535,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17559938410729200952,
                12326273425107991305,
                8641129484519639030,
                1699848340767391255,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3946956839294125797,
                10123891284815211853,
                3676846437799665248,
                753827773683953838,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10815195850656127580,
                17940782720817522247,
                11666428030894512886,
                2305765957929457259,
            ])),
            F::from(ark_ff::BigInteger256::new([
                437280840171101279,
                6885928680245806601,
                6031863836827793624,
                2698250255620259624,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16961604592822056794,
                12516844188945734293,
                2404426354458718742,
                901141949721836097,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3152898413090790038,
                16108523113696338432,
                11492645026300260534,
                1417477149741880787,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10578217394647568846,
                6637113826221079930,
                1364449097464563400,
                2379869735503406314,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6332539588517624153,
                17422837239624809585,
                12296960536238467913,
                2434905421004621494,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10311634121439582299,
                2959376558854333994,
                6697398963915560134,
                417944321386245900,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16872849857899172004,
                1640712307042701286,
                16457516735210998920,
                1084862449077757478,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10329879351081882815,
                5178010365334480003,
                7014208314719145622,
                385149140585498380,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13199866221884806229,
                10541991787372042848,
                14909749656931548440,
                708152185224876794,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1717216310632203061,
                17455832130858697862,
                5278085098799702411,
                227655898188482835,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17164141620747686731,
                16689913387728553544,
                2568326884589391367,
                3166155980659486882,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1233442753680249567,
                15490006495937952898,
                7249042245074469654,
                2138985910652398451,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4115849303762846724,
                2230284817967990783,
                5095423606777193313,
                1685862792723606183,
            ])),
            F::from(ark_ff::BigInteger256::new([
                18087059749829081307,
                14597940091345208276,
                11798127311484826936,
                3382732163134819975,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1691643954595343914,
                11247744324691988234,
                1707622876639858711,
                2480528034929290191,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3851508649196064239,
                17876067345026233181,
                2887488286234095119,
                1494319128927001460,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16523935542317028039,
                13324220361730126584,
                8371122380455762760,
                3309018818741234613,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15551003374354129095,
                4733249908186564525,
                18168529914254682980,
                3260751056816567734,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11254913643308260393,
                10502246588649407704,
                3799086616373140507,
                874361864942043036,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8900047886550157555,
                11072664836547990954,
                14067690250186104876,
                222120495052738670,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8862674475629615516,
                15981292778298457016,
                12707315746863502731,
                820078558277844600,
            ])),
            F::from(ark_ff::BigInteger256::new([
                319020525646737272,
                13307653856642240278,
                17864555577809705080,
                2323294512214543786,
            ])),
            F::from(ark_ff::BigInteger256::new([
                528913733934289222,
                13438428507040452908,
                3992004248394831445,
                1892847789713859281,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13328028178140586,
                13321926867853313751,
                3819245912595584684,
                249585818967781437,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4419507824394510725,
                15491296526060492671,
                7323441311493520434,
                33789035094519140,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16850313695997997525,
                13212925895639178375,
                16969717250207688921,
                1186502353743521213,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13071735289386612456,
                937867514930142591,
                338297992309721356,
                1214967615784395659,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8072276821399088150,
                12835106910674049377,
                12882375598172350360,
                23726925003953432,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1422103134736368268,
                5972060781611222310,
                3327741120806881763,
                2462344296021899375,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1520550024709727173,
                13222423213650094918,
                6471594054199938885,
                2900547158719341234,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5804579144463292523,
                312232116955447702,
                44072266925618011,
                443414315285602586,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6172438645164754758,
                2060767095070702602,
                9776673209354320039,
                433502090548086188,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3106863340529713165,
                3204921228853971160,
                18415230105665777666,
                334730862702895825,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5466895657389284138,
                18304437713220353520,
                1285400929215246941,
                1433021517987524568,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3420924115454337265,
                10554697431561455965,
                18140848270910970266,
                79981240397196109,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11667892943350068910,
                15378827967189494736,
                2716130655971101605,
                796329625544841029,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15585967042268697444,
                13137141366443867411,
                8082024980430782985,
                1489529056035368393,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10992446072450441478,
                138224143000960237,
                18217951800741742472,
                1876717591120043048,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2811521800456418823,
                7206285738874881233,
                8892713678098300901,
                2847756496817017136,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11773876037318181465,
                4541661022584540555,
                6306836169905751871,
                1651511859680343759,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4738709976041415754,
                11141434812704456588,
                5702683616473326790,
                2695305801712532292,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13295380262981145560,
                12355872011545154982,
                12081814135484567415,
                3002244520363334352,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16286908647809927556,
                7660343138745399436,
                15329123807288104192,
                1567412250881355744,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16962696400723776719,
                16674360835917832095,
                613266670392544631,
                1638603428702031454,
            ])),
            F::from(ark_ff::BigInteger256::new([
                18439556095094824997,
                13841863600186822152,
                12881105544243265652,
                205846478311636494,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1421596393284776217,
                9670089260651961164,
                8892365077566249889,
                424642527071812767,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1174656589459585602,
                13111961777171832740,
                11010160508517699816,
                2151038282169938108,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14785351810514061712,
                10509749256713706405,
                17745791112576401212,
                109422259592528263,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12473402395094382828,
                3981546577109849463,
                6872073528861716252,
                1484115331546415110,
            ])),
            F::from(ark_ff::BigInteger256::new([
                18317077064691995805,
                14913449958680164452,
                17623596895715902442,
                2734827927745965252,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14636684619416311376,
                16730968059669888955,
                4570969327469932201,
                707935733096272276,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3006455902127606224,
                11822286354056382123,
                4297786762918050768,
                3420439350374660530,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5978702745168944063,
                1998810337363776705,
                8348590052117104260,
                1104467115195932484,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16039251789478834082,
                1038491901047320342,
                7332881201062291194,
                2720734319770534994,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9889453805010587490,
                4733166420935746511,
                16386579815948162750,
                3067000315321614250,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6135943761382356265,
                7383331705498486994,
                182268831961983977,
                2081041303672646899,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3442174528586580054,
                127768021078442829,
                11860827353689030792,
                2640806201107799091,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9161848213852793716,
                9502732263284396375,
                11511904210561473861,
                2730639040471913229,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7464406005706715485,
                6445986204771625364,
                10612325414022949253,
                2585060567289166458,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11049893713287239413,
                9250777710965695546,
                5311843568161138628,
                447032443011861900,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14735198370254960933,
                5149919212516786612,
                13213290762788959515,
                2833909042855306566,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13087145610003905781,
                12389987606219446205,
                9668150611494432222,
                1567708440254857640,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8769832092537020222,
                3922419636542440982,
                13026535536833317566,
                2765175409727047930,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10226545596341077950,
                15237837378859331332,
                17450524747430601511,
                1426786253838892745,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11510299753779474109,
                15648423234006941291,
                17878397267458107527,
                1686087287210284857,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16720173690211773789,
                9424462116992110308,
                17228629356371214127,
                2428175180575224264,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10637134511767983964,
                11242800519361680153,
                4908564602791985560,
                1734858341154991546,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4165526492516941448,
                15226518715815154615,
                15884846109142235074,
                3152658976624961768,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1035863612704006206,
                12976895843024868525,
                6212786364173882366,
                3304285567232300618,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10005631916186190839,
                16762977863219942256,
                483042612146611721,
                2845911091306459233,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13510666336410935269,
                12916258535049845584,
                11316527754552285782,
                958673525965206566,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12578708801015947020,
                11476392957080179530,
                4793646082078730863,
                829861623373275572,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8368954597798115573,
                2801668492221606245,
                7928185876878687562,
                1296573905590198696,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1661347606740312278,
                16094588869337733251,
                5559087869789274145,
                2468015437555659234,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2002406138875656228,
                12208073193810858486,
                16172511668802172005,
                805201680755746951,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11648526116807373857,
                2092569519493367703,
                15279680017188561003,
                2457667794297898489,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15916552878365887188,
                8400265427301508270,
                4413844844380978440,
                3465621380650955221,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17881105919101264368,
                5995231121957490471,
                12216544104261232595,
                2591209649951794123,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10263136083366592059,
                9863410988395927086,
                2015399722589794803,
                2816553220431107581,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14497305553569961148,
                4401150916821698173,
                345120779567927454,
                682106606418554786,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16725317612241898289,
                6868385450026281733,
                170452525768695106,
                1460548320601423832,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11533585941794454499,
                13708952860559896594,
                13742534273619525352,
                2801321869310591600,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16098540659734549714,
                68837467740084950,
                3814064711874348696,
                2496615949693424126,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11489937043019385047,
                9134207088259910754,
                7709431569153213043,
                1678528579534425948,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15378815369136544647,
                10169560859035861315,
                1254527813982941824,
                2139732686486537203,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4067667284746944465,
                8027097857353886566,
                10390007508266638866,
                1526930346003824264,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9886256804720927831,
                3957214837900214600,
                11457652020877856409,
                825273720773945845,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8922435769675346974,
                6375612147484993400,
                4995242859637458061,
                3009066607375901792,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8637719193338729766,
                8844650175977910963,
                9584744566971233287,
                3112221808291847163,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14942673260225526994,
                8461147456600437287,
                8897844479136715682,
                1366515947882425003,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2304324338946818323,
                6173591794042328196,
                7035759562505791802,
                2675386131426564671,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14307665510588266847,
                12437752028785505171,
                15972277002907396455,
                2218544992262648300,
            ])),
            F::from(ark_ff::BigInteger256::new([
                624560312293185317,
                9493408516823400572,
                4841190197548351044,
                763839392792653358,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3565367490910848146,
                571405999987496865,
                606729989618056164,
                562292334834148255,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6721810132034896803,
                5313546692873814963,
                5968801888617564891,
                633255313806810761,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4035728573944667294,
                11165912396443621843,
                5285270876170160621,
                2826000855337691059,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12245702703220489578,
                10418007625547552787,
                15170217045852219575,
                1173652548850031088,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6963992341973233115,
                16035181459457139950,
                13901022825544142539,
                1780327675184989282,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11887048032338337049,
                12030373338541060530,
                4903063948191501314,
                3455025062474631634,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9951606279144162047,
                16770912808200625105,
                16531720941691714739,
                3370367015516629298,
            ])),
            F::from(ark_ff::BigInteger256::new([
                631015221670859870,
                4776714374936325695,
                16373639051681677933,
                2763625555247573365,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2328780616525686684,
                8614934655976826559,
                11340700184122887784,
                2595738753578164498,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8565169891571008535,
                18281546118878910059,
                9800255022221437893,
                670077401787843630,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3678480084579115192,
                5803856042855409286,
                2502197757278418509,
                3184280177206307932,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7683981478385642741,
                5226122082337106334,
                2724709414990864941,
                119158915364822888,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17616941447736995787,
                7869366046051367420,
                5553690445668138620,
                734809016286385130,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17064732684030364788,
                14056325806675590261,
                18353158459724902616,
                1534283387411244685,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1415455234720958322,
                11034697553301592284,
                1952078964540452590,
                2858252211571919883,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11329187203818480737,
                6249834409763675988,
                1912673301082184151,
                3239219660216551472,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12719358865497538161,
                5850737403655511787,
                2107492751703142913,
                3467332969984936267,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1704849561708626876,
                3664888786442708493,
                10037844209542279203,
                2993745196209066589,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14920887292419025557,
                15114443281475397642,
                5549557518859781918,
                1960805757712091128,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13555161582631241527,
                12258506710131302076,
                8449642417020347349,
                993221926246727859,
            ])),
            F::from(ark_ff::BigInteger256::new([
                625896906369790585,
                14637106271608487994,
                422374132680136568,
                740954543177489948,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6078649022830123246,
                1145162337946185076,
                6959950705288931679,
                758637870525025503,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9419993483735924061,
                9810790349883058675,
                17316674788286674479,
                1971214646685895029,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13036445330247342955,
                1673803854189541767,
                5182484438622780522,
                72089432943764509,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14351823125190380082,
                8845232206820368363,
                17799627044042300018,
                3447918686569258420,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11234057375005780420,
                8511050498564179161,
                14039915112570352542,
                2576062947883961043,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11468401263830344285,
                12593004431246133479,
                3214459584619915969,
                1861181681126874117,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10493278677973794393,
                10084293371263846448,
                18292724474499654402,
                3286238782581877786,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15772435725552599655,
                2853356086045427740,
                14158836739093231054,
                2618784274335834894,
            ])),
            F::from(ark_ff::BigInteger256::new([
                310180290368703285,
                9304129411132879043,
                6350173486904328226,
                773556113925659269,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2102103242839957168,
                1252471534676407056,
                931063388181255411,
                1860774294339572180,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11804967427389311432,
                4530872927467054660,
                624587641892097337,
                1123558154106282006,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9950251026031115978,
                18020401287390128360,
                2742199922334115751,
                1647129509260656568,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1740502732123791483,
                17258713949348080957,
                1652968360151151150,
                2598488100394277305,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5114060257993093551,
                16913972691740618175,
                18333174632466487152,
                728369784163925772,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14170300970818903885,
                4095373524327219965,
                12008940400080733386,
                3334881845696561137,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6895368543510426294,
                7438451633326870003,
                9659276349790829025,
                1616330404555883336,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17082427599758594415,
                913330018831249409,
                12620028449979343476,
                1780097780460743247,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2440576134660978434,
                5747551907442774914,
                5265966416627793581,
                620566218983837534,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15701527848304892113,
                9942034399578495265,
                12645105509567191775,
                2606121003572721451,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8408086382523997243,
                16918229844406252132,
                13783252080351045782,
                1679766864673039106,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8650332277023272043,
                2923393798247047487,
                7057991143640974973,
                1671977243803338349,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11744605028750980786,
                17336811244614932819,
                17298095512771285028,
                2201768377544572226,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13781288815197088097,
                5879666315369048233,
                16272866522462474503,
                371979686145988871,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17752945385386925967,
                16773510165179538811,
                13798437426692318428,
                2405614127507218480,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2431649435704700614,
                12247865874662182020,
                4389642990441486530,
                1353143906027749231,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11292533156863168654,
                12048194650434558407,
                18210521867034541393,
                2965132050029356719,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2141152992295375192,
                13168662051870371831,
                2112915122322738133,
                57686144052558296,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12752061185225153355,
                13146632332229278754,
                15953228898560326314,
                1755592556327667991,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1440681259053764127,
                7455959725985909775,
                8371029599828507160,
                1496915276947903455,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15005678507503273118,
                2961647472612508036,
                16563083189985700124,
                1095896715735947004,
            ])),
            F::from(ark_ff::BigInteger256::new([
                867624000822817768,
                9445230896058761402,
                6769232364762513587,
                1986045650897292807,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16793980290185781284,
                18175554892156760243,
                11815320694620334612,
                1046709514597569548,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12719876333406082504,
                3981116340966140694,
                1032558712212934098,
                3345152568137697143,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3235270470525863428,
                18419689082252916631,
                8941045840022285257,
                420972154397804323,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14594241719175790600,
                2138978708215668469,
                7799413393191345420,
                3000260487308225255,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5436741407933477652,
                15884587357169328006,
                8341692049766298956,
                2987252677898154530,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14765549605422319580,
                11717443854524401855,
                12872138901986128172,
                2220550621253837385,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15567970721324924571,
                12549817771378911506,
                7300320073330606062,
                2051740309460947235,
            ])),
            F::from(ark_ff::BigInteger256::new([
                18137306828541254703,
                10129132284853846377,
                8743216946413260529,
                2191753406760997653,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10362117754252389365,
                15003588778870996372,
                16584292146351747678,
                1882695578478064878,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3878886834973636972,
                7218556758240584979,
                18193023122632710550,
                1806000448421554854,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3670231520872114555,
                14370740168506259634,
                4216031874442345151,
                31060627329810165,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2233657878081548397,
                18146550295609420163,
                13759594780910591904,
                796431701878345995,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3191680642632703901,
                5387551707124450577,
                1604230073230224596,
                2723259064212077509,
            ])),
            F::from(ark_ff::BigInteger256::new([
                602126742684059434,
                17867877877025854664,
                11166877134585354354,
                1351619238070263309,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2154650112197077678,
                16548804321914436024,
                13311483468327444462,
                1634570722071784361,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7553399159779977180,
                8096214204325625281,
                15458600426259187345,
                87502732314826893,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13356031970608918665,
                11387265826046624589,
                7704137580768293191,
                1601489866217783574,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9089156687021644662,
                9325873694834017551,
                1439153316335971334,
                276460926712749379,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2636911312384286292,
                17832598938988685692,
                13029277307551076322,
                1274214484122586692,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7217973157041260580,
                7667112746676943433,
                8774672420635971299,
                1439385670256956052,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5795198734668976475,
                6764106300381545484,
                14431869472555671395,
                1028917458975922077,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4178292235426418100,
                13084952883411706155,
                16018141439777827032,
                1961775089259242535,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4921620798036603006,
                17754425319824335429,
                8205961119827003185,
                3165386697124192725,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8320908517239250412,
                12721289315251225250,
                9997153959194077311,
                3238264716804133807,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8447785023153615751,
                9014606803370671536,
                9836719943963572434,
                3250428568094537642,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7137501526584210324,
                17852502689704248878,
                17592136596972909881,
                456570858924221585,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7219187072465138536,
                14726657982829893100,
                5770150334882458603,
                3284038860053914964,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4344223950627153291,
                61613562035523724,
                17828504851151794389,
                1921299805207021840,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8705073189572905949,
                2812884615077932192,
                14359836866311691371,
                179845228946293425,
            ])),
            F::from(ark_ff::BigInteger256::new([
                208284969092561147,
                11486327909402984504,
                1574938258323442756,
                3133751403457203131,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4574597420363930383,
                5085599881153043953,
                8973168100283762679,
                899655753974327928,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11972780978748231800,
                8799826700948803556,
                7108850007176869408,
                50760541399695228,
            ])),
            F::from(ark_ff::BigInteger256::new([
                231734206753624021,
                12718138108554634781,
                16633507802992129151,
                2180300950737339727,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12077945078567862,
                2441425468131118729,
                8281677366490978372,
                1319691946544315340,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15527635912650175963,
                10946903111623733220,
                14912283569785481821,
                1428827490177187947,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5081757460474398421,
                5079375049711630438,
                7526115622033988197,
                1602937536327996483,
            ])),
        ];
        let mds = vec![
            vec![
                F::from(ark_ff::BigInteger256::new([
                    9769558989247416493,
                    16565252363839940605,
                    9958898962517555741,
                    2552717700379466948,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    4390385258812911129,
                    10844419161943762334,
                    14171052955486594283,
                    2843607864154932411,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    9557855149468195938,
                    5848418228398144266,
                    830176943711826768,
                    161485854821576908,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    8879538689549763265,
                    10011656847463815109,
                    15352039111291781015,
                    2104761180406443026,
                ])),
            ],
            vec![
                F::from(ark_ff::BigInteger256::new([
                    17314577599372204556,
                    18346477388402037823,
                    13053048163652971304,
                    3059529030018236541,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    8014382316945569550,
                    627369506834793328,
                    8710668102824499944,
                    592997363715428848,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    7541839986222558063,
                    2654243558414584693,
                    9062540963752379309,
                    1899066707007035561,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    8723316744271491247,
                    10687047034213259752,
                    9689249627916665550,
                    969505166981163003,
                ])),
            ],
            vec![
                F::from(ark_ff::BigInteger256::new([
                    17538903595536411970,
                    5294776234988437058,
                    6425038848567247373,
                    2337482136946705135,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    12944117837419347627,
                    499023354391579318,
                    17870410428075111897,
                    3321625001393700701,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    18025723937318745608,
                    17502176093717372728,
                    9165783803354318065,
                    1128273817540809693,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    4543046333068739563,
                    6269959438944906408,
                    2888652664128384694,
                    284825165950480742,
                ])),
            ],
            vec![
                F::from(ark_ff::BigInteger256::new([
                    13441904791694243447,
                    10371896868661851640,
                    2632446402411070390,
                    3410454547762694515,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    6442985728581388142,
                    13263184683363784449,
                    13832787558104280907,
                    2558339223781899289,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    2341543658554445270,
                    2019458683841350626,
                    17199181642962620210,
                    94519923773959515,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    13503511481480623402,
                    10062352967750810818,
                    15579616735936788351,
                    54429964343538493,
                ])),
            ],
        ];
        return Ok(crate::PoseidonParameters::new(
            ark,
            mds,
            FULL_ROUNDS,
            PARTIAL_ROUNDS[2],
            t.into(),
            ALPHA,
        ));
    } else if 5 == t {
        let ark = vec![
            F::from(ark_ff::BigInteger256::new([
                5709486634755270801,
                15603208330859234182,
                9174926028928814807,
                1059829148828130778,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13435151555604064459,
                11643967437604800275,
                17307293407029460924,
                384168495989622370,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17656499060093797076,
                6093508768815579189,
                13607423987096732535,
                3423784509791480680,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1807386625533712003,
                7057527142772858611,
                2614247787438821812,
                3217556051884718497,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5676534217206202025,
                11043273723905977437,
                10497975883995225236,
                2068833595310908510,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13396529737527378210,
                2434120040341524674,
                17916482132268545558,
                3365589271740770027,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15418021208676454926,
                1137117723951502262,
                14202525191686707690,
                2470832312318920103,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2556192569385911031,
                3959577542639286648,
                15613941884691717168,
                1113534474368712942,
            ])),
            F::from(ark_ff::BigInteger256::new([
                443316488524326115,
                1851551941083068826,
                11726935069089959320,
                1125070165016089941,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7643403544667028384,
                10894966515326481483,
                8021655946622347554,
                2205240841378904921,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14036960630577299004,
                4395041454368048582,
                9835113421217291865,
                610243336402961280,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6480989746377831932,
                14904178184753081690,
                14417323833696385218,
                2820547478722997370,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16415884019766327263,
                5025231080565268240,
                5439424799568627435,
                139488021564701356,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10946573832950074672,
                10270580329202598058,
                5602635313366604563,
                314019806692597345,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12534177755930434662,
                11809597266636284807,
                11517663152414195598,
                775156918826687121,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15073100384172167476,
                1141223120433682809,
                14409728945282668701,
                182983068691223728,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14999993004554296764,
                5819103990952586505,
                10822823350348882798,
                2788880469346871884,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5400734615295706667,
                417108559831873193,
                13451178293129126238,
                3437897604311791404,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15926352737404770384,
                12505010425057301379,
                4971751096952435273,
                2455257435145554674,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6688065275224777443,
                11756488339097930753,
                6405490512371498582,
                1946710444063352898,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12186288347468009215,
                1463590592391211078,
                10289879847026024147,
                720988719790639979,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17839712777150320195,
                11885908612911790119,
                17531738362508399945,
                3133193024850150347,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5484117929960390141,
                6967141352294906880,
                5545997647314465932,
                3013640116432554142,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1863895427737075742,
                11828441638860349917,
                6297189109671795316,
                2037460044600332947,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8330218780817429097,
                3902769326585237376,
                17541874437619808480,
                492970902720138098,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1085521930605466380,
                15878641196749475222,
                14213363123603915022,
                2954323733505425718,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13325032106070099433,
                13720200562067814061,
                17896723074015824663,
                3343913851502109333,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16693702715190546215,
                14579987761459750204,
                1051575468067101229,
                2796235066704695395,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12658691410024947121,
                4813007794191410977,
                5609127907338747884,
                3345620110661121987,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4056358364306004172,
                9840596591682916934,
                6886522344944850171,
                3073512637182560756,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3368915050432525310,
                9975808783723075646,
                3205446996530476123,
                2917602595669473966,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1288767484872571908,
                11948912309388552763,
                17628221439889849725,
                2390234503652240046,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4037972413180359940,
                17978327329879284409,
                1922950502481311840,
                1248895466487252671,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15741345657436226055,
                13785890714328196483,
                5059654587595731311,
                2161555772451975618,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7199633339120426588,
                4616650645556307240,
                13871600651802349122,
                1028023043121434230,
            ])),
            F::from(ark_ff::BigInteger256::new([
                18407007357199773688,
                9320045401694561059,
                12370093193055634161,
                226246388256237252,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13190810660770529949,
                2718149974683554093,
                1818251906882749583,
                1329806251377889369,
            ])),
            F::from(ark_ff::BigInteger256::new([
                18231871688567038528,
                15305140525692147518,
                18274372773902298229,
                1257550829557295000,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6907193654582068186,
                12170258066876559038,
                8561876523466805565,
                1429079314231569736,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17628653266216073889,
                14659907551036239847,
                18414598187998091075,
                38082767495671121,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11757304474605908111,
                1074751533573117910,
                7335839785634907386,
                2512764582543145048,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10085225350110507349,
                10972216360679106241,
                9730225146006207550,
                862831660412066704,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7616771925528817458,
                7209028748629977484,
                11746159820621225486,
                698495116519298492,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15453075522789613123,
                18139310771595982445,
                11871420266552606811,
                670848552474226230,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8669391628108296223,
                11809072369959895499,
                5881602578926877205,
                2997632299147874832,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7590634728973810979,
                9525143785790755126,
                2065267353424291968,
                3416148269733115447,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7068200716974095563,
                6223388776565523192,
                17012022004637386560,
                515256071297219609,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12018234027717710495,
                7088734015651079817,
                1385419247248460267,
                1757100601503993264,
            ])),
            F::from(ark_ff::BigInteger256::new([
                533928697388275500,
                13149997190323249781,
                16470046317097510365,
                1775534490766120918,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8888284565823907613,
                8241074171689122304,
                8632307772207956108,
                730139602187139288,
            ])),
            F::from(ark_ff::BigInteger256::new([
                808010436049238946,
                9451016593447599040,
                2283147949544711458,
                1563459131374585471,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10147328920399769953,
                826258488857654648,
                10090518116967673174,
                1606501261591911664,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8801430400155757168,
                7045350816033990237,
                9716945417499119347,
                617197692902967919,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14712944074172918050,
                11972727648492947340,
                11828443723872724202,
                3440368405956187173,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1170231525117704874,
                8894791295596935609,
                6053686325770909790,
                329535971000777301,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7680499207910572622,
                2117120264201660255,
                5629540337850598102,
                425906726507201552,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9095030953212800028,
                11050591183035472955,
                10652248742236441296,
                1368632629978772280,
            ])),
            F::from(ark_ff::BigInteger256::new([
                490049980730472526,
                2244527621709318505,
                10776963427979260947,
                162504167597104455,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9805393244268549456,
                15535309122138455006,
                3957859653679567034,
                1766055410268090186,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13476626328037709989,
                16106180895415260427,
                16857630767377661405,
                903895511449916699,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4905425530068338150,
                6507603054411034502,
                3908572617951655548,
                1671503761541126257,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2073574377898262574,
                16523838148121299959,
                1310366031267041073,
                363359316019713305,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6301077409336986688,
                5457063769613363592,
                255775579631400566,
                1084544637655443779,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11365900716619365590,
                8769527431760125012,
                17802679792080338202,
                2021982742846540237,
            ])),
            F::from(ark_ff::BigInteger256::new([
                18326107955115426278,
                5313723800598452473,
                7866595442681485214,
                3157297394972060537,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8999684224146367179,
                11560383071262436630,
                5150077941561111423,
                2876423446573371027,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17388888965788871402,
                2904748300632551856,
                10346665908224294051,
                3335067085690011226,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6945883126069584645,
                14522172750187029979,
                6714521702291577219,
                1819423304397913404,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4020327993758180645,
                7968664068098072142,
                14862258134802252013,
                3166884234040405115,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11525531993204439848,
                15413539024102608114,
                12217381135254859101,
                1364993305495252294,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12506948078507994415,
                3743570834341252080,
                11539961906053632415,
                3058529624628730355,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8637756393200003444,
                733413615448698409,
                7909587125510003807,
                2431396432242894053,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1405075490168587338,
                8194387178284537774,
                2830973839694344794,
                1021611356878984971,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11474898985784393262,
                14058790042889037276,
                7021295737499818124,
                2767453154966479140,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9443503600510579717,
                1535939863509986162,
                3505915654255700346,
                3228077367003398151,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3370106638778134109,
                12568912046903135268,
                5693114279715143711,
                614893721629633699,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7696595949199900458,
                11588601762137737395,
                12484221445992807895,
                186753376831814838,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1204125732958202778,
                13821512999321439894,
                18262015992342488648,
                1300639660169017180,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11583023990583000892,
                10554320165488737651,
                14419993394135304588,
                719118153791737491,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17739907610767338863,
                1003136618105596810,
                741740501086433530,
                171297302681240789,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13332059161446922644,
                2685214475529579260,
                13510722594825814817,
                2972457816835906423,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16654332663712744441,
                17437894212935778941,
                15341648630208093006,
                2830872302208069433,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14363524761672196642,
                11739726288968853258,
                4148144355494233547,
                688070129514684274,
            ])),
            F::from(ark_ff::BigInteger256::new([
                113100990117648694,
                8469808637894479268,
                3540830240629423452,
                1787636037085521022,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7183991719641800288,
                4376386839113169293,
                15341308121272512797,
                1375700400724398208,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12179941823238500349,
                2795695192257127920,
                13873608510470894166,
                147676055085376574,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1648361686689300514,
                10651355104533709853,
                18050526472939277750,
                2820114206656614345,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10401013854774175772,
                7248473615532823875,
                15442805964935438712,
                100303024497986305,
            ])),
            F::from(ark_ff::BigInteger256::new([
                18287510897546856003,
                1840127135088837996,
                7945366276415480415,
                3184678123790778185,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6555141981764355528,
                12691919063799037579,
                4975301142146589409,
                1135051004262689000,
            ])),
            F::from(ark_ff::BigInteger256::new([
                18341552195368198638,
                13733236756286064510,
                15622365155513083315,
                2329136663412716041,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10221479030229363136,
                856618301502588506,
                12652535643945313049,
                2946516207180905397,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5320687108831381146,
                6185322572898180277,
                11491000713646067126,
                2695840496051952505,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3808277647320436021,
                11716033137413860033,
                14956449940154683465,
                1588250600712739415,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7233954208810529748,
                10051167908532039840,
                3808275709781341119,
                2434505909465367484,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15062699372733100876,
                15486366370265760694,
                15181796272438846277,
                1930029994119981174,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13320467532645982252,
                6737992616714261828,
                8941442567095734818,
                771213592162700174,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14245487589991330679,
                11902719074503341281,
                2874335296205595164,
                2046694175812231342,
            ])),
            F::from(ark_ff::BigInteger256::new([
                903693634869390661,
                16391495424045209437,
                4437326617789518562,
                3232398815996129277,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4488147173559909166,
                2608555397427888003,
                8596235329438933811,
                2573108608164239280,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3403913031559014821,
                942162471765852437,
                10139626071512437409,
                2206248357169511071,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6970680459015168731,
                14531811614219641028,
                12376990325905483590,
                1009337817565442097,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1407331297170706024,
                4843656659678829131,
                8894933843402347972,
                290628195454438396,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11574670931530847848,
                3836810854519963344,
                897522105680914998,
                1804570027951569232,
            ])),
            F::from(ark_ff::BigInteger256::new([
                18170003560559622956,
                13970112605287735653,
                15010307325935066120,
                1605470779349943176,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9524648511222862877,
                11480965008187493122,
                8761894689994913318,
                2607584610634175583,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3612314621152024495,
                11350541119748750027,
                3913917951484028918,
                1692115445034325985,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4157135835775235320,
                706312872307278388,
                5645409649616610086,
                334428445665729257,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14842385381414034575,
                16007026201832287436,
                1178404862798321680,
                2724925126993459708,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3192295646244899367,
                3606452384826060101,
                9624294380582280924,
                308335572121809661,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16823213836116455342,
                2029143030596177077,
                10072804297993326058,
                1193665791505825147,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13774428559020267454,
                3994461161768993038,
                4938885682379412909,
                2682242001837321552,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10939600771648611888,
                17114315788958381094,
                18196283229494728752,
                2707538640781833457,
            ])),
            F::from(ark_ff::BigInteger256::new([
                285766519571309302,
                9965680037255807106,
                14423531322980661711,
                2564460582677822438,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12966089842574379140,
                11807471102694565813,
                6118831181689104304,
                2784178156394296713,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12078906763471841431,
                14527355876024606571,
                10191387802246861944,
                2867122240254964385,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10172790104122789974,
                12400088934858119197,
                16457890121600675022,
                2799595181320361852,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8257098839150873481,
                7302433696222926197,
                8087955122552490444,
                390321479958236083,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3274850261246896784,
                13989666574848249877,
                14370160240413383331,
                2754150729801749678,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16896315381358951663,
                14068759961286864000,
                3671720959751062781,
                3478842401657181703,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8442781705012910508,
                5254689592339415056,
                5858925122295352311,
                58954162139295235,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14233878450320842659,
                13095526981216015385,
                4880258613542417146,
                67025388743409756,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7059719515083757938,
                1198414609893623692,
                5365675712779130365,
                791220511478963530,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12148861395190959370,
                7323473682813839966,
                9713778313026144494,
                802847944142620766,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7673534841041542614,
                17782164550804299175,
                14552110062054305941,
                2335975000578901431,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5964983632670572700,
                14627625227876630184,
                13156956748162174166,
                1147191930313198716,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9328055929912841683,
                4253352092437934305,
                9882715272879407697,
                2688015300024376075,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3995149027766863714,
                10810545371484326635,
                5067981747140254177,
                782976760767467030,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2424495528909854553,
                7828620474956046690,
                14382529426264553052,
                2788391716648861335,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1828656086397303715,
                11846010299176704859,
                4700505995351656421,
                651216236940542528,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2987038348368362363,
                16937350183057961460,
                14227602846790176419,
                638345862898979314,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6409027523929511839,
                4827467688296491879,
                7466879624955263203,
                714009272291692951,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8919315857020435804,
                9150490530548730187,
                7318440275644219756,
                2155822447507750469,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3890376857960646777,
                2632802482862050035,
                12188693035483834862,
                1848462418812637336,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16975100403542597476,
                7364856775542612977,
                9770989273425938693,
                2438544605350394762,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17794960278834395393,
                10927615196497605447,
                4476782966043432403,
                716504515174063217,
            ])),
            F::from(ark_ff::BigInteger256::new([
                631089284022079939,
                5258550190258919894,
                3039943363056277901,
                409622590419322189,
            ])),
            F::from(ark_ff::BigInteger256::new([
                18019742707438105875,
                6859948797481481580,
                7370816837197911859,
                1126359324505867077,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6009311504365685680,
                17593329328336549624,
                13664116708447278403,
                208425853984490400,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3918721654536867602,
                2320498700366006544,
                13328700483947996343,
                1510803646773488606,
            ])),
            F::from(ark_ff::BigInteger256::new([
                18424386149805520844,
                7217790796851026322,
                2692292334573092238,
                1165755894729726019,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14674564491561366466,
                2365734759741081400,
                5496633202064214563,
                305635596537853661,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5547786795373108986,
                8002470448966807663,
                12015835365913195111,
                2414835508500984212,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15876075074264968019,
                16955711904879288010,
                13003201099741621411,
                1092150559820932681,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10650562521192347862,
                16096531191964161500,
                4931613628074197582,
                954240267382229995,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12447883330461826116,
                15568803910144965449,
                6268880946420571701,
                3231322019897013865,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9718070987370937592,
                10469129982285258106,
                5551029655348661816,
                3020045394956712287,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17920884006027313470,
                11161262496134720343,
                2769428621149634831,
                2511432701511790609,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4829879170758956362,
                2453819263386509110,
                3468696503637552291,
                1145835211886963990,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14402651019371599487,
                15209447864596619244,
                1314003187648545338,
                22773371006195977,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9825935037949840417,
                7381291897287792581,
                13705804225604056883,
                2391591342793953826,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16272394848994314746,
                12278628011936920192,
                15624544290159881252,
                3312940862092057059,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11868951356208785250,
                6346455430740642539,
                17469100720285283649,
                2015315635322326028,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6721473816800267860,
                14950684637259595747,
                7055723592049930550,
                891765096194151815,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13132407457834696481,
                5860317854772441006,
                3621032117985750397,
                3337416662628082181,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12995438244769510113,
                17758987863531232066,
                6273749752756893570,
                1712019909642794370,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12874425050650591912,
                2392391425869992694,
                7495094730181281883,
                796383900875814695,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7032139753104915732,
                12226278783928786245,
                6436726381336856135,
                1105127791863707973,
            ])),
            F::from(ark_ff::BigInteger256::new([
                239983887705380210,
                2627867924265101609,
                16231587218416801105,
                1062703082072258564,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7357493123626368994,
                16253373127767919191,
                793361554790392643,
                1945295314229020648,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6274567990749719559,
                15272583616294916083,
                15718800468384700514,
                2379107712229126241,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13347211085114152983,
                1860517649335717956,
                9040357696066002366,
                613106111092773061,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1703722798126366266,
                10146217014289985622,
                15711049494382043079,
                2035781123800346785,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10438621924301879555,
                10339270121508848644,
                2508917386631645029,
                523851184678325279,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11554793405455260863,
                14908413423927086863,
                9822997141758841268,
                202748415721807293,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12114035578311703315,
                16977240533387309338,
                9430888078137768792,
                3366335244457011472,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16574104354912409525,
                5560475216086729476,
                2999245442738833845,
                3386046658006436592,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14202985707561738340,
                7036660083952681236,
                3377065140806511951,
                655728715918003812,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8509583055897105762,
                14656223418619783674,
                5607854391113184693,
                2777697308730276252,
            ])),
            F::from(ark_ff::BigInteger256::new([
                83409972693178588,
                10931448580963985031,
                2787985057451628422,
                2523043090752145176,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3965207605377629151,
                13570282371265620178,
                14118857344238770466,
                2865755440181571298,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3181669039585700622,
                11054565928946536692,
                17306163681059473668,
                1297503429719680536,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7833083828861813821,
                14626761102958126448,
                12502578164971628990,
                3189417786992285342,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5744768478049786180,
                5712477148861705788,
                16862537335286580873,
                469317136988606528,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8609012027791823235,
                12700817345624676731,
                12662216778238906735,
                239912365325200182,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17128196222445424235,
                8362037870518827120,
                15230927375655180072,
                1888066338791978521,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17526009840909487224,
                7964887651198928870,
                7471682897303089049,
                2305395957413907669,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1188754600960045971,
                1445054447668184195,
                17055230506963927284,
                1141220517878434047,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8927042020248130475,
                7819963812187330378,
                1833972566751359358,
                2754714916437452536,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7978091822771224646,
                4312334471452266461,
                5788486310253445438,
                1631470628296003682,
            ])),
            F::from(ark_ff::BigInteger256::new([
                18214236100628811162,
                6038255975158901001,
                12498948034097574114,
                3298060600585666828,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14648704397912405733,
                4772779286087826050,
                2004606313472449955,
                2656999043651885918,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13687084048955302415,
                8514095339415836001,
                4383628104390689512,
                3156061263155677039,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13009747483354109919,
                15215797625120686054,
                10224664515299492905,
                534836028911127693,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12218542376675395297,
                16433319893041976445,
                2019054853780299343,
                699731440351423555,
            ])),
            F::from(ark_ff::BigInteger256::new([
                469411707172125703,
                7271415981873885674,
                12288307584237414059,
                2332609180314279470,
            ])),
            F::from(ark_ff::BigInteger256::new([
                926931044104630313,
                654853666374115698,
                10459109597131065037,
                2890286664896673487,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5402706510652776949,
                43422908028986638,
                6867258550918782195,
                1874973438295736564,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9328124957053290060,
                7561486090771349225,
                575190725244637012,
                1156800393303211568,
            ])),
            F::from(ark_ff::BigInteger256::new([
                800711763403098568,
                12864820122923860680,
                13513018588550182751,
                1657929886579730775,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16195632571371287762,
                11509148164698085133,
                8143979856621818147,
                1190880486359104651,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13285005716169965668,
                3117048831037231983,
                163830414078914476,
                1529059912510060083,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5881231704952180744,
                5166095322549571509,
                5473188310634228852,
                3367383689734614338,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15678994885154927240,
                5544626209461953518,
                18357728644529278042,
                1966769599675967020,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13130243332324085847,
                13495728433422908000,
                9232167592519715579,
                2622508108522154117,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7929354413406706159,
                15753316487826704575,
                674514880695311255,
                1247271618748221901,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17161204709243777352,
                3341201543508623651,
                7726062425692666284,
                3154505451567754031,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1361384503448196117,
                12070571389831899300,
                8601376529521211318,
                128844595604546091,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11478215353676852891,
                6183360331803957197,
                8929987327891852995,
                366112076185856648,
            ])),
            F::from(ark_ff::BigInteger256::new([
                280102981543402366,
                2347028563324174767,
                1878892246125318305,
                2773222228711656648,
            ])),
            F::from(ark_ff::BigInteger256::new([
                18092398926685398486,
                17393504928101820764,
                14379013647835099309,
                2960850892807723221,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3953414371045905689,
                6660552975301659317,
                12624686877119585331,
                116929218233867827,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6054474205023133929,
                44711546241489070,
                10030799078001482791,
                1802753000306398282,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4292821355928616730,
                12956541524238595655,
                17865854030728979490,
                2806156480638338681,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13675964982855241589,
                4666984869414769438,
                1818103439487051072,
                1929344896460137532,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11902874548928800317,
                14770672448389609837,
                1832312237996525414,
                690978606290832647,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6063449929115748377,
                8417634495416762043,
                18253319861700640358,
                219717477558047188,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17159399771644920975,
                12962806624716596802,
                4130705766396366729,
                496431055902871085,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11757078959090340206,
                7335119905500778835,
                7739577677175619829,
                2119848888677224481,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4455058425522762226,
                14074181888688630787,
                6462737291599350472,
                2946722748897358122,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8807701991395719700,
                12713989709125138284,
                10146407990324147878,
                627698407660074320,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9900106143898483538,
                13185842599889490794,
                4921254382530593179,
                1597883734980539889,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11064220335754197622,
                13099187637999133649,
                409646222161104732,
                2950385777779579836,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6931498028256841232,
                83266748746985782,
                11787520182635880362,
                2007204825524584797,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5778855809983662539,
                1137590398495115915,
                14639255051103796750,
                2555093039624263824,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16802142277011487568,
                3687824109001897046,
                12500960564627456570,
                2281185566478704563,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4575542004743501460,
                5107921094990697359,
                17901948646788090020,
                2184154756685014769,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7039396633942032359,
                13419548556790813650,
                7923791060795369352,
                2383062610300004861,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17627378779120518565,
                17591166683133616961,
                5732565754540229400,
                3335206118123032234,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8470467745026149305,
                1940182863326563499,
                4319886157978011681,
                928198184793251711,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9558967774780069067,
                10215857554512039452,
                4378506136792455284,
                5002595038465673,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1488665627850152780,
                14582903516020978805,
                9988427545109060535,
                1718017637807795242,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14605175186665772538,
                12801679152113609262,
                15885416346114665222,
                1848678874574705555,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12280256903764785062,
                3598452616449920423,
                3062600776759280371,
                219551171499380154,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10624201023757226908,
                10618692974840400588,
                12548626186719541294,
                1638382584950771955,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4117798026031917853,
                16582575931142142829,
                6819835800666057111,
                1311703381887486626,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15891269240374992884,
                13443239741558852,
                13198528451933441156,
                1046754006555443280,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15013033422967566946,
                1176180032296643348,
                15228887004232545037,
                641458659369647462,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16216416410401405720,
                294366421203411150,
                5132081976262007308,
                3360005852781475528,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7371742751929268847,
                4725815944490859199,
                3766944442648453904,
                3299575789787986114,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14288275424666151531,
                17507378665460297636,
                7147950760486577331,
                504296321175420548,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14281095096307545904,
                16331756546805787178,
                5185835312275082744,
                2623928114884905113,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6671717386641172113,
                3423978651797322455,
                1761363586765033189,
                2496741325155617680,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4678122793411995959,
                13336934586771147425,
                8176736772905116877,
                3408187286024608198,
            ])),
            F::from(ark_ff::BigInteger256::new([
                926108520172050046,
                4102984363646637237,
                1419055328559733278,
                1224899567962889341,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1875020499501188014,
                8664968792210139055,
                11877002815986291744,
                1052980148648989630,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11810755844422118209,
                11629824305528605048,
                13993619618664457391,
                553807122640674426,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16807702006602056645,
                1889596758087546424,
                17932771665144338029,
                3287119345769760609,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8527131019902346439,
                15231231883520694340,
                3736609991259285233,
                2878251396302793555,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9101037612689293309,
                1733562185909622221,
                6102942971475941242,
                25044935775671859,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8109123196916508117,
                12700513927025006429,
                7464008331348751422,
                294466490178697225,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9910328290573796653,
                15673091058142672485,
                9179231419817839852,
                30052529763335900,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17025478780995507096,
                17009141474151213476,
                12815207562869617542,
                86041182261310036,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14916781907843347894,
                8617491479631120803,
                4699756985432842883,
                2038564182705360762,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15129725906738103798,
                18365759193123172023,
                10693042721751213177,
                284473062826207103,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5580934352231849058,
                9949422275242563926,
                4114615192073251172,
                3289721931800177417,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4036961375486364163,
                14543354548447304135,
                15859429380945806030,
                1524332561342362829,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10184897397777549171,
                3811046567907986031,
                14515963783786338516,
                1368574813167502570,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2427563288811938742,
                15589339873089097070,
                5496028829135981695,
                1484952576280647050,
            ])),
            F::from(ark_ff::BigInteger256::new([
                782581084919554020,
                6188021743049485466,
                1716314670587919512,
                581782065792924805,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3055295659757323803,
                6149687040474626794,
                15627349044471245439,
                210980398022255577,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11614854049965100472,
                6189604886238030699,
                5614687093837071006,
                1062013039803595601,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4967036864369237849,
                6175534131639722123,
                416225131083137666,
                1569316271941168194,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14023783437394539306,
                2071976140556520389,
                12749004067480607545,
                1627230791698565021,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7895199330101514223,
                629873512935339881,
                2560562891149348886,
                2686291348451397528,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14846477286382657649,
                12103677888874160534,
                4360940949427869580,
                1461238084872979743,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6651380376462260121,
                15251518547733286239,
                14536197351081746911,
                313678643220735678,
            ])),
            F::from(ark_ff::BigInteger256::new([
                88818322596669059,
                1475646447664896912,
                13273142235796579312,
                2738923099393915295,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13263133181777614401,
                8344627666578193989,
                17854879810365171208,
                1603189878507795195,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12650294746586082856,
                1432261652621133448,
                2132053242684233196,
                3202427219973585717,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15360512030853128102,
                15985006323135594466,
                6537683384293731321,
                2023912708655445175,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12160774974824262769,
                7364989905338403326,
                6799014758274713554,
                479240729805102078,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3549237303441478762,
                8420565468489815495,
                14254608929544954003,
                1242246187298922773,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3216035503911756033,
                16516091812200812922,
                4107168336500645954,
                3078369786887460326,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4143399440429535667,
                3787581752132941053,
                10819670249736607046,
                270528958859181566,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7700878353165870748,
                17867604143573468268,
                9800199870281981345,
                2812898632546730040,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12294196642566225421,
                7411650708048843532,
                15349627483881451836,
                3306533629975834356,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2690711955809751041,
                2938379807394406205,
                18256805720914799149,
                991083231436767442,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9022058399388238639,
                13608821356334005884,
                14512947250862160597,
                1759205098447077732,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10118800521569069101,
                13120837696218294484,
                17880796844788881475,
                157295733400089035,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1208845691662640828,
                10591518486504784486,
                4126683826145267314,
                2402061273137157790,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4514175333862125189,
                17008975762970833883,
                14621273070144435387,
                1000285626847367034,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7156747194887841123,
                1328408009156420475,
                14974006916442217016,
                422129178673200811,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14519134777911082334,
                65172331912703436,
                15337251019888841516,
                419983324069761982,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10525723912405416750,
                17345191957055522695,
                11233389717977818991,
                130033417631398955,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1948407342328889468,
                14184022161102919879,
                12646255086095158835,
                2289701183813746518,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6236939965459771330,
                12241212284588186051,
                6723128386751013931,
                1171145861623894068,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12107950093807627533,
                3108021775422920984,
                18387231574638938701,
                1636948764305594080,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4376967622653069134,
                11763526356269277121,
                4849033279353112774,
                2368877719597800299,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16120632139181852173,
                445245133872589451,
                14002981313048047864,
                149371037126421166,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1425408658431716873,
                12695014242201371714,
                9699083012147602300,
                3398308367620850754,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6356789037520473082,
                8340278624526149708,
                2732721540707278898,
                685895601485874645,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16937731612150657064,
                16470004808479920540,
                948460544166273549,
                939011978783580272,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14885134368029427012,
                2426534594091165261,
                7460585543903677144,
                1037461350089688367,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6167501161436189195,
                16361081704872408742,
                5190670340798074974,
                3323806564254138658,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5359514086886521557,
                12551034672382814710,
                8548026703044016833,
                2993191357213585363,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4212206843832341593,
                10484668710603262526,
                10530208684846006478,
                1240210518416376119,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3803497717205177400,
                15069792170220926572,
                11905794531976505973,
                2201417216749689469,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2391097777429514035,
                11583876448048344209,
                8910392397684641315,
                1675169384077586146,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11032647648435690969,
                6748045335325874640,
                12677223729910623294,
                127065795601637147,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5363601621991417186,
                17471663781493639572,
                12293333590476555775,
                1392646584051019188,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2373898905902796084,
                44248602955561707,
                1756657973412269796,
                3340190041184959661,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4762317713679402332,
                1431780653842643475,
                15001188798291269157,
                1809704187742601939,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17183829475327805598,
                9412607285544869533,
                9413196116529731997,
                1285769799236396610,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1994259360536775025,
                3766640194245914517,
                10445988104194594711,
                297677234941471213,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2861787368689948381,
                178863829444205618,
                325270493538525698,
                31656243235202131,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13969045486663022689,
                7451234827061499444,
                14440141729824382363,
                255078570206288295,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17408770430530332389,
                1097136723036802026,
                14818046407305698509,
                695224478106610304,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11319271188276235660,
                4440207966828541768,
                13360422271946804056,
                123154667680778184,
            ])),
            F::from(ark_ff::BigInteger256::new([
                18363742150646650022,
                6079396326119036951,
                7933925134479660057,
                829909496406889999,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9640728957051940835,
                16939467272560297209,
                955783664750297152,
                2568854559281306470,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17811509850728338004,
                5491017124820675788,
                4493653676865691295,
                3315622009556794932,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4022867939356212040,
                5443937916418767340,
                9185663938977480694,
                527710097306213453,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11088573932220863911,
                177069936983375916,
                14071957416844695461,
                472746101392447962,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12931016570370654901,
                9693766220869121464,
                7454992397969437689,
                3145493929170163017,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13285793107312707624,
                718616179054072457,
                11760255282741334547,
                3190372593878739958,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10867922948973982871,
                16403565789991507314,
                72227104662153345,
                416004673671780391,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4235857543940225137,
                923382658745620978,
                9523486069434540742,
                992542752251476965,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15156225351992686236,
                250184766665577054,
                1861280847398113191,
                1453881493605351294,
            ])),
            F::from(ark_ff::BigInteger256::new([
                70395964386404287,
                17541387128294515548,
                18266157641642938272,
                2999431416009894036,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15177604654879029878,
                772904652714104247,
                4090575808463432803,
                370983943932829777,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17399603568686669256,
                9085536577522771534,
                17327150401327735414,
                2197894926872057217,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15827162580613887356,
                5476979545962947887,
                10615274688464843987,
                253885204329352649,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17679147757923789313,
                14964290588039900954,
                16259729195935655327,
                249458143455347011,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13250928065650914484,
                6905545382526877776,
                17377821213224771310,
                1387559047428335897,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8873304464238898407,
                8336117018266486117,
                6237605655745727191,
                477012629360675571,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16868524488281765526,
                15164289555784247810,
                10787122273164977734,
                1665218614759864035,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6157846160241118936,
                14681113809190819577,
                13180723030729768580,
                3206986575570531612,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16400854778944660996,
                12703309859695405356,
                16407527883178199850,
                3015234443533956569,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12062501223658910347,
                15627876247206309082,
                9527009653284112208,
                602754543984529725,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11397023899480736660,
                15848691892499165228,
                1199166578831077420,
                1261574573335518634,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1177762458650108695,
                16128257840517341021,
                195358346922395069,
                2312938868822530044,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14982738250366463642,
                5078176838250319246,
                16319520210525855805,
                760026558334857642,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15352133508426897527,
                7281025369336364575,
                336598789189337270,
                1731133775445386442,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4511254541159486315,
                17896364464002809542,
                310632999827888291,
                674538711646592601,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10178284298849170576,
                9490800502366439297,
                2366764744498989589,
                1712245956675385219,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16489097120957245723,
                4330115892299785741,
                7670236975993787549,
                412198438385430923,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15478437314088958587,
                12328028551718445948,
                8067932095009120967,
                2764024201695511687,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15476768366451783977,
                608011486596805811,
                15689385031316221801,
                2627053544744446920,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16538265882335858430,
                8707187308931673829,
                11590134941561014305,
                1477543645720160846,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14362607902830559421,
                11059870933236644669,
                13646003024394621825,
                1215682938406659193,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1965138206652372831,
                18017644324325858563,
                9182958445625928963,
                1510868026331228511,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8486414050989499785,
                11379692271515198695,
                11498370395078145489,
                1940858616016228789,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10741670830975209306,
                606906424023356930,
                13431363977478012475,
                295325938905970318,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4008575113225383096,
                329884205540213144,
                6285846968494949740,
                2775975067878839761,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7288857083489334191,
                17703072143096546604,
                16950838741026506697,
                2321306227105270117,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12686540033356000923,
                3512332126132631636,
                8533023432103258570,
                896544630386252957,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3987710779638126233,
                8862520448144452771,
                2859725960167679616,
                576827625135236734,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13055946278812575396,
                10936109118879230195,
                12117579633153370667,
                1841135627386813122,
            ])),
            F::from(ark_ff::BigInteger256::new([
                18236614538741146994,
                2812975831053997165,
                12842930442200455075,
                3020540840880296833,
            ])),
        ];
        let mds = vec![
            vec![
                F::from(ark_ff::BigInteger256::new([
                    5748560962366832033,
                    11466979916847568886,
                    579569008118457398,
                    2674715827031838848,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    5212077121234302821,
                    18313582893206132451,
                    5173461357374052295,
                    2736869942695514032,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    7971902536992323635,
                    7920515792123700309,
                    3845856292408103344,
                    2971638520741306626,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    7009273702295781828,
                    7375099962815381215,
                    10808940011733010243,
                    1174597919655905374,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    4527728475262011767,
                    8731133959348414911,
                    9073132789881727545,
                    768485934559375121,
                ])),
            ],
            vec![
                F::from(ark_ff::BigInteger256::new([
                    7635397639335626809,
                    371832689374390889,
                    13852535625055263490,
                    3058148595291901147,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    1250749652313886074,
                    13447628286476990841,
                    7139799234418280987,
                    2887181629368966772,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    13288584625014402103,
                    10145923708961862751,
                    816214975370426931,
                    2937119018259380575,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    14189874474942401043,
                    11196632074693646556,
                    7872992026635568059,
                    2416322220620954964,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    3822823683466938689,
                    16722027105816434506,
                    9230509609244479275,
                    1503557939615592923,
                ])),
            ],
            vec![
                F::from(ark_ff::BigInteger256::new([
                    11887959819863015453,
                    6176999147643262618,
                    16043003182785902573,
                    3416445520537697479,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    7884174877245911122,
                    7474340593201843825,
                    12507369967952837554,
                    7915235535248243,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    12103886558629708587,
                    4162465562863683183,
                    14857142721807016900,
                    1565437278888115300,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    7279257234337681308,
                    14675055849775582610,
                    6410412150051602591,
                    1353299449905040985,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    5989438814592684391,
                    12639297900496483432,
                    5260628926173581870,
                    3385628920950297416,
                ])),
            ],
            vec![
                F::from(ark_ff::BigInteger256::new([
                    17415374429808097328,
                    4341463741422707988,
                    1838002315875461537,
                    882429594306981653,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    17124788337805206382,
                    10029232897166297715,
                    18047536113072813296,
                    1003270167506407941,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    8991257380578735866,
                    10968035383490218235,
                    2315510005694667104,
                    2037925172591850975,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    13309834195327537482,
                    581716623130446431,
                    5941134273328944182,
                    2748829959026088514,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    11509497001386452255,
                    17113389287907895483,
                    9725467663124186377,
                    571983162659663312,
                ])),
            ],
            vec![
                F::from(ark_ff::BigInteger256::new([
                    16354694695851275669,
                    8961694600247256475,
                    17977416854754739178,
                    3035676445061250450,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    4628295235559466313,
                    8076770176523098564,
                    12320474853942739182,
                    1173741998614951199,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    7041705240262253710,
                    17236817744044256085,
                    7494037994470992715,
                    210732847386178741,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    15342584230641263483,
                    13676142722266711070,
                    7496514862548845523,
                    3258946873459557446,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    14523405368506605156,
                    12942301318929430233,
                    14532296991539854921,
                    1443205430461869185,
                ])),
            ],
        ];
        return Ok(crate::PoseidonParameters::new(
            ark,
            mds,
            FULL_ROUNDS,
            PARTIAL_ROUNDS[3],
            t.into(),
            ALPHA,
        ));
    } else if 6 == t {
        let ark = vec![
            F::from(ark_ff::BigInteger256::new([
                3352568211928303185,
                15665882624935136478,
                16694419313858959419,
                1461525030627184536,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4290377113084622787,
                14823123334282545908,
                4882144023266965566,
                772282197906185374,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6651065023389854431,
                9569559133943690673,
                11136285386258182686,
                3133585193174496556,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3154520903713793628,
                983367962825823740,
                7309415363683066303,
                3154403055443472411,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2232300700915662140,
                880666068138387751,
                4404717036819992278,
                1313456400327784527,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8782732014235913299,
                9849399154458933218,
                3502242086813472997,
                249107332433037337,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2028201798691749201,
                13376923110475111088,
                12760551180132080310,
                2026926477915957355,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11366433171340357523,
                18081129955307092708,
                12234252201088091035,
                3177326046630588037,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12503967048847188017,
                10316025059164421618,
                300029224932979057,
                3380231706103695130,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1644114885531670292,
                13444431062321679251,
                344842210592276369,
                2938121085072681664,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3757490842055894496,
                7506520851064248091,
                16014382215831532111,
                1774450826139695840,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6116359373731605459,
                16592005864461783389,
                12841139639362312980,
                2027140090011680371,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10044247620011427761,
                11341935675000483245,
                9906993445121564347,
                780253912827552149,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10930108068509234055,
                6680620330183117963,
                13163381361089709460,
                764325405825322899,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1733118117236324975,
                6984983575789749222,
                5145647562514619473,
                1374334832188517741,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14745207914374596593,
                3393672858463774732,
                12812257796871139784,
                971195651005173898,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10102122569110912677,
                5313648529849581246,
                12702518775516008762,
                374634312930361409,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9572920725147007122,
                13541902470974023228,
                2971937252018549168,
                485205198002192897,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7529074541182438195,
                6371454443662928719,
                17734663423730679667,
                1104941349886733319,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8945451578965163672,
                892907077634034519,
                10294516986602416562,
                3161879495290085167,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1002621463155897905,
                1349083620610811851,
                11157095569393742085,
                2038376608060972923,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4055403560852411099,
                12068100814974921808,
                16032590254798248474,
                1660731444472951084,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14051798280251058161,
                14321380421592545748,
                5714559542204076849,
                1856183496904957091,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13752187924651424264,
                12856431544684473092,
                1905901998938088666,
                3394548990324843020,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6104129921423645936,
                15883512120589736209,
                11325402095834360272,
                411003149051753683,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9806743709199239477,
                6904909680089878639,
                11479465503777727532,
                1306048320411802325,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10467750596216697916,
                5270882302508831085,
                10693245324846499805,
                2736308842988395395,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8752932084981703748,
                1488621990777723156,
                8361897040582103523,
                106958577811954407,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2087615626128427471,
                9859169786179142673,
                8694594134791458581,
                3052674617284798237,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3061178402515457498,
                8902779782829494939,
                581846153225536606,
                1724928871697989180,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10297185298116702227,
                13448160024581985648,
                7346687998193977917,
                1862909570758154651,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15161064956159499781,
                17147278807860273996,
                4786361008681020019,
                1119393641176946715,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9652514892926976023,
                2886525324077238698,
                5259728146268426979,
                2297711387500375161,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10366172514849002094,
                14720215079129072483,
                4088668498126934500,
                1224347297074902655,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12028972770174073839,
                13825396758345012924,
                628750428649087806,
                2244508689751430981,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2781550813082050003,
                15713610596194291240,
                12331941714314326932,
                1342046284554094248,
            ])),
            F::from(ark_ff::BigInteger256::new([
                18079143402278242531,
                4300884792196378089,
                13935013878772311226,
                1562701576712550432,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3789040737205202962,
                5883877645020728503,
                968993489601373832,
                2246298657910849079,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7929113769126243644,
                1869028654401205938,
                3935290222999108046,
                1238737868226491129,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3686006330144529866,
                16303811423631726153,
                13203256667029618110,
                1392242404869696429,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15973629807109295316,
                5996137675102049895,
                17403064937128285062,
                1502584690719226469,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3556004092308196230,
                9395509756189417462,
                12131485213582372942,
                41088079530043774,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6648723137691307467,
                173314348563881674,
                6600646291592887146,
                3473165640367457617,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8320706948676354587,
                5809582167559236059,
                17320065824759639064,
                1507940500657512898,
            ])),
            F::from(ark_ff::BigInteger256::new([
                355384996147858795,
                15936835164306701803,
                1201708745795702106,
                1252547201829664216,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13513779276233277985,
                14596534425048485551,
                17450252772398671953,
                1704793925206468359,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1349165484470062946,
                7231123029148949347,
                17983524993900076936,
                1904004321030942562,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11993696130933694771,
                5405857957292312221,
                5498593988560979164,
                2273142071146070118,
            ])),
            F::from(ark_ff::BigInteger256::new([
                18214297492084440623,
                14967937388725786878,
                10924027567366519180,
                2304919774400873082,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12846150906918969112,
                15527520067915570653,
                7874715962153795578,
                242759174150101427,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4814758808287853958,
                6153951909470471677,
                9487441728974089339,
                2160231266020988119,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4080852493712059194,
                5025229105647605232,
                5766196743889245193,
                2891653477830766233,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1679600274605962273,
                3602841538899076963,
                12755914257313415215,
                2140516940231779696,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11870207176592557258,
                9036819351783276352,
                11616607754862802977,
                1495046132281843895,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1197940817780573285,
                12076069682394992935,
                16533858565658154115,
                3237334703533551495,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16128636456396563252,
                11334687629146435163,
                11876067116522726164,
                2152347165981668525,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4580461142064934512,
                16269801305619316839,
                2523397751647624328,
                3330047874049866566,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14622734236107640158,
                1606452993045447495,
                6478598593051057546,
                894035795831580380,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3219698525532648844,
                2750095312455227547,
                2255348779134820739,
                1404610979922648390,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2109225363658541741,
                9432044677689841019,
                9578395525517110123,
                2668720224876744492,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3117992020847014957,
                15325764224537550743,
                15941599853273531628,
                1660265388147166214,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1203240719386360705,
                11679486736832926877,
                15405676457148659547,
                966246138974346872,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5679686292726034808,
                8043952748884515278,
                7231212136824623408,
                914077163094823519,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2692652692319294506,
                1616673892318236139,
                10375791267792040043,
                2353079955614584039,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15226693522398020709,
                3712630116241247204,
                8690691995096466500,
                442464517567574194,
            ])),
            F::from(ark_ff::BigInteger256::new([
                44161246942771487,
                4802353451667154085,
                1105588123134338066,
                1599394172048137849,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14653119439192913023,
                13617255889676616959,
                16426426788373385399,
                3306901318284201236,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17338642748595670118,
                2712667785851673762,
                12500031506359378199,
                2111351297685958812,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14804989204692517188,
                10413255462079888408,
                11431856088133616521,
                324513467889788877,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4953876315890172493,
                9041090688730720215,
                10458600096924422033,
                3477635445688286378,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1167140253849405250,
                3107223511422885138,
                5832839994687733195,
                817319164431727948,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13042838842928606095,
                2637857957365298038,
                16255168954206097114,
                3239364692205007424,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11204176770423472516,
                8438108681618893663,
                1359672043738339075,
                2171329122390346749,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3338525469630709157,
                16495675253345464981,
                15106637972623141596,
                2945482273790466350,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15951454800754562272,
                9476075290192062548,
                7475235055001907676,
                3406082855218845404,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4621192588980189890,
                14875785030392697199,
                16338552638474738810,
                363233270636990943,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9018812417467323561,
                10138837933029533844,
                15528752854511043421,
                3411692627306060391,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8054659214729439860,
                15809696586754317219,
                1010128136589703211,
                2731673532733075076,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4910379242371682175,
                5708492197885690517,
                282043360468288626,
                3453261791554954016,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15479429144034238191,
                14633590347775326493,
                12172410828172989364,
                2921261077484812680,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13337220702316670084,
                11107469655056699973,
                13726761001728334695,
                2367986063698858585,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12703181595380178387,
                15212307857045289159,
                14666627596438047599,
                1726052527834849061,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8523818571256064933,
                4511742248742071278,
                148069449724773984,
                184343594739804420,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11496735735530495296,
                6737596327536660878,
                15274566782164826857,
                800933665883790122,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3252655429027942151,
                2141712247794290546,
                11247440845483943462,
                306301522309074631,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13298684924551605596,
                17631727091721796729,
                7106682511905969412,
                489313993957211161,
            ])),
            F::from(ark_ff::BigInteger256::new([
                18175590386085112291,
                13326001261377665627,
                4454527748726068167,
                3360002704615523337,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15831169705872602109,
                8630156840390215530,
                919502366159765034,
                3150047300102581884,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1884142188678363795,
                11728090514268498901,
                16689448521395693817,
                1022557240867052851,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1377415642914059553,
                17802788321626435796,
                13337929629268372400,
                1006495634739508816,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12003289928301601493,
                3420567520954290122,
                5384843184506971978,
                2867764504194024825,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4883948548551550399,
                3687659762955642380,
                16455288772119996482,
                2474098431296566109,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6974647867898384519,
                11004502176266704698,
                11486461806396563452,
                1386429198808348886,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12451775786980202378,
                1705057237565259508,
                2775913477627652040,
                1014797457725234494,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10244718758176794286,
                8395524159540650026,
                16185588597031651170,
                1090135977731778719,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6107067971956710329,
                7075939774241384745,
                17504170437937877426,
                3337785818563173247,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5386245056569541876,
                4213223824500841866,
                7987743873695821531,
                194242117856265626,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6574484857648409716,
                12498396564595859774,
                4076929280451507069,
                1864651285896219757,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8426093237312632581,
                18431902885226917023,
                7161571924367054812,
                953215027284244357,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5605009686447627826,
                11968798967067332176,
                12420782285800235373,
                3213121630302992922,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1493713131474090991,
                3178770909379491157,
                8110450009134535874,
                533324638804167923,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11249557458757215788,
                15333204801054444103,
                3594281170220901132,
                2583598245008125889,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11156459156058836351,
                9731948054461344484,
                16421066158546310006,
                1717555274936709136,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4523866371549560867,
                164069393402144400,
                1412819918059492137,
                1683724793591529766,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1278767399724062293,
                16554808393162805926,
                12517662759288671377,
                887156968205439521,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13232237434627881,
                4084019054513242053,
                3780133203116206970,
                2844362609000918330,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10874169839975893864,
                6918879970845234472,
                17415846723076241741,
                1047144747663091459,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5003794712184461634,
                17178757666596915851,
                541080848268053224,
                2241420942369296334,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9542706678081991889,
                4635741509362080650,
                13560742430471692195,
                3364564496785159901,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14103592242894563187,
                4990505311244246722,
                3842143004673590227,
                2039494852387759972,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6287062450625962932,
                16125735000228912584,
                10258353934940623778,
                2554850175725046955,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9750118789452690164,
                10731921454495928571,
                12220298665718863515,
                1863703155188745731,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14765037493359594703,
                14326018049748184879,
                18037066042684812979,
                2413198067944633327,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3062351619183601231,
                18205656777091710060,
                10147634376924688286,
                582648277912912131,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6868152063887647087,
                1149384759910268826,
                12016017581661653227,
                3150623854899745878,
            ])),
            F::from(ark_ff::BigInteger256::new([
                720228326462751465,
                9359523243508567127,
                17184814337586809765,
                1271445833591919891,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6674075455722843636,
                12570891697526476397,
                1958783845480417627,
                1977854088426412112,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16877749137551795005,
                10660539790299930860,
                12544648844139843799,
                1481732685996935319,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13953381105086018548,
                13206163278780526020,
                11527259305442034675,
                702905443670169252,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16151216795161465020,
                8383634417003692823,
                11705372857336147336,
                411386939033760385,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5006479370742976567,
                6164959480341065764,
                6803585700877689791,
                3164010276253126621,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7636346118869342637,
                9348905595909358147,
                7731976286559690090,
                440634746135620060,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10743015098219401961,
                8325827516856092091,
                17586889549496869189,
                860621766308919997,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16875099482938686406,
                4222126476833695701,
                10600783988008970142,
                3347262010460517100,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12474632298323876079,
                5178702612342186268,
                1384780321392851539,
                1118090004455529192,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10135132586300346767,
                9013059070306287701,
                2824247915369382734,
                21726603938259579,
            ])),
            F::from(ark_ff::BigInteger256::new([
                362540374456874945,
                10694253540328230216,
                15720261602519884556,
                2541509777918528112,
            ])),
            F::from(ark_ff::BigInteger256::new([
                630881150181171572,
                1345815579758717742,
                13679962542011745987,
                941884672081816466,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5708880485904893371,
                17179831425839794807,
                8113496939214027781,
                3437197006694377373,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13256797854869312294,
                9078628048746488296,
                2840742032144166971,
                245547367312712039,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3802998910793138631,
                13004894728179805072,
                3176781225468904806,
                2741268172366465399,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6900346501549401959,
                10479361759838106196,
                5941570876393596244,
                2910086304894589801,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1814006809626388693,
                3393659729879961343,
                2817175985115484214,
                191844093971673346,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16172484038460656239,
                3658915027903975972,
                12402700889307366413,
                640940159588933848,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2426448900957997483,
                3608445602197665807,
                66837549163578487,
                1444048282655319956,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17192384724123220263,
                1297798566876888517,
                15661745567956745991,
                2984466877709508087,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6297517769605952329,
                9608214103185047120,
                7009996540674556145,
                108981566570450049,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4062806626278404704,
                5758190997744856336,
                10834180542608091021,
                1910886794895809853,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12744519316913643843,
                11321897945028582559,
                15181350396943205242,
                291810559009437991,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12801358287953641007,
                17089754994078895847,
                14622328630034021604,
                75944403289676983,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8271591524712249987,
                4255872517116318732,
                4910594113502932402,
                571326213542504993,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15252799690453461991,
                10623329115216531922,
                2398245548247202710,
                3380596592400430943,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2826124580373159705,
                17617267403553483272,
                1154555171336129783,
                2179954322941404326,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4477022830537521270,
                17770435356045197808,
                5905405339913937160,
                2567746719084958186,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15885674182685642921,
                4155691960627029642,
                16589971451219369791,
                3334443491137420864,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17867985888778614258,
                321421640174125770,
                16706329248033913285,
                1290188777209966530,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11040978435359701223,
                5161465272946390712,
                8323249696497135845,
                1845721810092625548,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14504349515479793921,
                9627144313021087709,
                10967485708934226812,
                1105454964437466829,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17530080024746520663,
                1751063537676957572,
                16584042508611840774,
                2038845964250428440,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4445619495904528677,
                7093092155812611640,
                2489951266509526004,
                3299833374506345375,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4924508333512128737,
                816024615645494437,
                7772129714001136805,
                1387297500395803416,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14162286765538089563,
                8023675882848451444,
                9831615858776034816,
                2551542706717820875,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5439724181416531126,
                10532360638520333879,
                6265124343489785645,
                2282087912783366735,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2451490647706200079,
                5497770748902209117,
                16427043999063518677,
                900574249579527276,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6307938042530168579,
                7586193311595993515,
                6954517210255363900,
                1573513868258105660,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10561880016570957214,
                419796918556719202,
                2698245012379592941,
                858010145221445605,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3532386477394234522,
                402646526151671991,
                7581212483571410971,
                1383296250299192523,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17296566925892753026,
                9884197614752115180,
                854916026519871470,
                2685152567088777036,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16138226963465217858,
                11516088735627626902,
                10205890169617482682,
                395908959486712465,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6339454124059304605,
                3833918597379975651,
                4271788326453949789,
                2162095298165413839,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7203200329273074809,
                6447745186042852536,
                15473718509669566822,
                1895683727464574652,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8143725252319603344,
                11444791367679950631,
                9672276753154162947,
                2680802864452071850,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17206777408172698374,
                14149966025231529437,
                18439657350074728478,
                2483349558398120090,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3675780059548854192,
                12535679999315287925,
                18245227691822438483,
                2924381870211639193,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9069970251046887432,
                7391236644343906854,
                10076618995877707550,
                3447828163075740872,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7163582581324512666,
                13005283309468083908,
                5172423262987341141,
                2087600713338881201,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8410687093887373576,
                813794934017341894,
                11416756131462506019,
                248766413562321835,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12346935858713659968,
                5241144284292189214,
                17659697231076116004,
                433053456417104863,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14464430466771461692,
                17875239781674472061,
                17527718156052580123,
                984958310594709312,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8013107385104712899,
                9713530067896542516,
                10253212112150212774,
                917181589289585450,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13648366155761881373,
                2141862088585735080,
                17601668667496886258,
                2180698635976624580,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10225862510620183181,
                14561217378710114006,
                13584808519959836357,
                1995899074139298483,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12574382247028849098,
                10465694157204356012,
                1075143504062894498,
                780179946238471924,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13893105836024419080,
                5981024354124912176,
                6644032659066510841,
                121765179506232507,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12556516758344408127,
                11002506543315512690,
                6452484876296461865,
                1872754238142348007,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5534825722943471128,
                14286349578817732219,
                8147192065758848039,
                956814636236235761,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7872483069618027983,
                16975204820217268817,
                711765775318520370,
                2214643472133266360,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13545606924681396664,
                4483676442562525930,
                15479981864614638370,
                2811624637555644260,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10427681687341118947,
                6056518888303360419,
                12542017217390403165,
                2406577825050199473,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16595907208561724298,
                16527452981506665312,
                7784102134458589426,
                3329453033588251225,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8767202631199497951,
                15933700659878268749,
                7786577554089008506,
                224516101276227291,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14541307267961194614,
                3933455058274536113,
                640705624878054420,
                621160662629524937,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9008697810120398919,
                9167114407605059687,
                5185149532174018252,
                3410281953039930172,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12670644386104049555,
                12730031650999108074,
                16103220474118144200,
                1453153390739491536,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3682117409600172735,
                11372277760778988238,
                8469212703167824023,
                1706482566313004813,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2842944684908418855,
                7203969499910909720,
                14151735329379459075,
                1051322343044368600,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13419205316993481145,
                3673887952220946202,
                3411963763076021684,
                341338567319459172,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9568037600427888846,
                18362989607371415460,
                1575086392212222460,
                2606471460455872247,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11608900026619258768,
                541296042304887536,
                1163049582201772297,
                2727795181045744647,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16047283660298349184,
                8980538071464950082,
                14828034817576088856,
                3458286897385331177,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16829714569727117219,
                15047012767580644483,
                13539779489905196024,
                1322844682809691928,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12428656877444122300,
                18269821236435812407,
                5543929335496632928,
                582862394420329915,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2102869614527198676,
                14691373534760949063,
                944465581016588507,
                1229870798713842742,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2899616685623120785,
                18111696033996350489,
                2736656384547470840,
                1402756445545392777,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9097863133460771735,
                9499081595919257317,
                4381410051099946938,
                1004234377667897071,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5829482401150066367,
                5104805218370617075,
                6722189734533371184,
                3427140117925490234,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3520893404272216103,
                16933788435108761794,
                7044213982874038651,
                1885483765638570970,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2152500592861961711,
                18231117654762622460,
                4695928947556561657,
                2080530058230849027,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16390392261243930950,
                887179592705513113,
                16387516812235197196,
                2809455923966821226,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10547786266763567294,
                16220437663862898555,
                17511018730136565319,
                537574972234417238,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3088855529539995904,
                13362783655343186535,
                10151072404593290918,
                103504947144346211,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8031872075226486743,
                1321847530504422575,
                18204949679420225843,
                3186630706532965954,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2930222547163593905,
                15800814829144865442,
                4490240490748237167,
                1763768641114499578,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16503074104377417934,
                8703635296485716723,
                4755699179835289042,
                176107006325728038,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8181167888822366891,
                18116102772017851404,
                11533255710936543506,
                1537774336712427850,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5820580237168123681,
                593722665430584362,
                7172071942325600137,
                3066431047683308954,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8729303261165138637,
                16009117662800247529,
                13049534972371725225,
                2435168810614866931,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13922851633315936236,
                6505201597882600706,
                14802721388344955787,
                3181470952294589562,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17924669275236364059,
                2774809784147051916,
                16999164898206524300,
                162155351572453422,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17483204165855549682,
                9569497661114763244,
                15669392514663222538,
                2753635162343758875,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8927781520764324544,
                9364919209304189083,
                6935937179186722230,
                1866829246897822025,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7371981240675377424,
                16317069984994229030,
                14564420154211893640,
                1062718697190912583,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10340219925958652396,
                5520378639830002793,
                10040100512894957135,
                2722401215456206862,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15294962061110073167,
                5587934863821245182,
                12595058539796955585,
                307592356505539557,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5880217280718870858,
                16232264429350501123,
                2779795059538546297,
                1408985154708330079,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3148364391367450333,
                5628060710982763445,
                16152916319068814498,
                2653762874305216158,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12868221525153975948,
                14932399741792766196,
                13593005462585734340,
                2442742573025212256,
            ])),
            F::from(ark_ff::BigInteger256::new([
                42660036768132380,
                11230478766689277203,
                17308751831782349930,
                2633998398103751131,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16777504723041080162,
                1062189694385518239,
                3025633690285225103,
                3135130693601828030,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12199334255300403959,
                8229743006503897410,
                9351652320277164479,
                2610395788258833281,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16374291871890137068,
                14360691847571826626,
                11728708282475883032,
                2308473192735417883,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3444166510820556562,
                1872046231902597840,
                935198548096383927,
                1017473070783194614,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1553430373441128996,
                11631039886761394615,
                8547159629984303233,
                2789192890523576876,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7439996060521640372,
                10349976890147198730,
                12627537440806702967,
                2374439292004985336,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10559717105555880727,
                11499748606610424454,
                12080386948939303852,
                2386447292788253984,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4814691267880858804,
                14774149922120216418,
                14219679432961765421,
                844218145517347113,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2988105510234596811,
                6740274599090894419,
                16485237316293381074,
                2579197412682296945,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3833190311618382793,
                7661192211614069533,
                12993830156319956831,
                262152859749519821,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3276675708060219031,
                2413387887755031718,
                10662998050591859792,
                2846336849082767722,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3584163138937646951,
                13717480436876268418,
                8757333857694335465,
                915207338762807295,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15838860237110849427,
                15754133402434712088,
                15366911686737001308,
                2233828093177335325,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10299524511921501129,
                7900352789018442642,
                15199199019499427179,
                191358709879507566,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5901103373557731834,
                6544199950029965635,
                15715289125713132640,
                936044219374777544,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17121211584862649566,
                1610527701689577669,
                7514779824681811799,
                1732497959874079253,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12393439477283311633,
                3208554467242733082,
                730890065295266136,
                353931239743175994,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7644054344203891629,
                3835417606977791396,
                7309110042877718620,
                1554620897355495419,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9947743043519854769,
                11908726999418419928,
                12453138978857208163,
                3326311123111269129,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10749977960539663528,
                2008141834207822873,
                13485518473182762023,
                2744235675725235307,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7941248695167217219,
                13122102987012914339,
                1772155964818721878,
                2419817875670657807,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13165097568167405391,
                6332436014991149274,
                10471341364730063480,
                2544744025017439881,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4540870670945842475,
                6054319298913082880,
                12612829429161578873,
                1893885735901186765,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11223832689073767169,
                3850929711919335893,
                12120545961450036755,
                1970004842310155460,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2538541025416931621,
                13024329157569918597,
                184998268882015218,
                1906321686990359410,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15933509102026534331,
                13805776739806099405,
                7707738137390805227,
                1778456761338521961,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16040901793810739785,
                4949954798760307018,
                5047794620845081105,
                2433205107689709030,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2378657629037992187,
                6645296534613338919,
                2825532191070619543,
                320843803682879461,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15390021771951500802,
                6540606202522761474,
                2128274457373068900,
                797975826727254664,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12091548131561185539,
                3197283053367821178,
                6339088307267590285,
                625567237836689254,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9840075026020617768,
                12122680250281612682,
                16749941627085989940,
                260557216651152256,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17998499341674550029,
                17499000165958088550,
                17383625552084897289,
                2731347237191433333,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3288738140976225914,
                1276121073487476006,
                1751059418018502731,
                2202211166241945444,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8806024146398214564,
                18379604173215432151,
                4710151561450483802,
                2602460810843750538,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5443515462214055239,
                11230568018350209050,
                6427426816579171188,
                2304771345171282293,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3007724245397149678,
                13905427582757744108,
                14450447307807717675,
                1685205403776050448,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3466094638381967081,
                12383530558983668388,
                15464716005786045791,
                2990460585157424355,
            ])),
            F::from(ark_ff::BigInteger256::new([
                18044027719277087956,
                12781660770729252424,
                7911413516285930943,
                3473677345559225324,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7497871652305448432,
                4118988047377505796,
                17571685338270392482,
                1920759901179390431,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4169309686758604811,
                7236913052509896763,
                3976503559657988455,
                2279267779693944418,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11589937762709550803,
                4347231379797269395,
                10785376097934399129,
                1090981373052698465,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15651984551688886057,
                12955359736116760505,
                6536142118820868433,
                100582201803180659,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8029458489374275339,
                12443687373635033994,
                14244585481339717500,
                2583143660875419919,
            ])),
            F::from(ark_ff::BigInteger256::new([
                420543097780452211,
                7524024107610550347,
                4160134443102838258,
                3336060031356756722,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10604286477477947731,
                18183914450758582967,
                6779753945092009979,
                1315486222239097773,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10657422783262583149,
                479509608193243922,
                6590641919813164332,
                2182709444534254621,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9264706396083894101,
                16075497638007615816,
                9740196283652307244,
                1098480356701297622,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10928259308260257895,
                17195628041204296936,
                13987594449045493377,
                84503516453747036,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16305141647574963233,
                16529790647470704157,
                2025415873368621553,
                1435154821789432347,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10059992863941341758,
                1884656288408678836,
                7784017431927332555,
                2367879975243134789,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1683288447506055183,
                6334672549962953617,
                3924990938984722864,
                3200685138776657452,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12874824001731478955,
                8118666507010966657,
                4443191726023912070,
                1768949704696379311,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1011182065626600690,
                12360677172891280355,
                16854435346808557686,
                3429793621762237398,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5307121149417767046,
                18113677011958622670,
                16651205442149715238,
                3112972769349375750,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4368305853270625033,
                680191743700995091,
                14653802645314843503,
                2795798314353705767,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2680124928328783327,
                3101922147010471978,
                13213518640269653279,
                1951938861455392908,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2257491977820003044,
                10743532972377666605,
                4968771177885242607,
                1283731803589881640,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15440351207835916062,
                3741438015772184650,
                14992919930632464529,
                2535581035283311594,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13181218355820645834,
                12477716878795146926,
                16950584573708224201,
                2271196816219235384,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2482355527115866582,
                4987174789433607469,
                13111205488736799074,
                2846133794630470699,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1528343455685564329,
                9018431933212817174,
                12304182926446519323,
                2232831659334764171,
            ])),
            F::from(ark_ff::BigInteger256::new([
                18084636077035224900,
                12222317231477874948,
                12416165459923915351,
                967926987293077243,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5874967952389386259,
                17438992129151725570,
                9013417247904352993,
                1012884939160525211,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7972448242999914150,
                8637433264241695964,
                14491493266021913035,
                2412785011856164572,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16775647094297815226,
                15175782375934696561,
                13046789024753402951,
                394078748568196401,
            ])),
            F::from(ark_ff::BigInteger256::new([
                18125198418957022397,
                4926454035065158123,
                14487392022922096163,
                1761814034500465588,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2361277489372835962,
                9636112747052699735,
                2062415456177285594,
                2358424282381200907,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14495753252528721295,
                7779212473893957434,
                12417126665781343213,
                1616411376569968518,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12685594020707551054,
                12551397191109289997,
                14935030528065480310,
                3477450344298185738,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15524082116586658667,
                411772368626979146,
                7905942498825811699,
                2480265618177551967,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6352072169977252044,
                14250492366672746040,
                7695123144206733403,
                2788560747770380959,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17424411921203443025,
                17191974255300051186,
                11953897859006716674,
                477658589336627515,
            ])),
            F::from(ark_ff::BigInteger256::new([
                18098250196246396707,
                2658684462996595129,
                12459991785667252873,
                3222820593104802625,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12434412938153956641,
                10460150578838286650,
                575907299951527014,
                2371297268745971421,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13469626385931539708,
                134421547046548063,
                12046910672733066836,
                1532460386536078907,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16930135711960924421,
                4579795047659605721,
                14145146591447819662,
                1837476844234430226,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17033980226565463799,
                10469605793339351185,
                15258342844168201462,
                2725190352888559365,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13312703857341648124,
                465073409717160694,
                241582839250296436,
                3453303303811035105,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4714225255676439649,
                14229465641576567600,
                17574315742383591442,
                2006729894718428238,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9141955533336621072,
                8872753725240829917,
                6259907092006316403,
                49536097577205820,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12889432063825658251,
                14389630792695576498,
                17329729982519849256,
                2551022810463365249,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9660060219236880093,
                14298884485018153007,
                16298475901596361609,
                2275343873359451335,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8308627135845678647,
                6130170983142983988,
                12691486186613985109,
                3306819288232066401,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5002295690368943108,
                7491609575509755995,
                15239402350472947242,
                2721678256835754012,
            ])),
            F::from(ark_ff::BigInteger256::new([
                18047307576137705004,
                7239309614225351252,
                9823347492609870591,
                2338714794975059096,
            ])),
            F::from(ark_ff::BigInteger256::new([
                18174851757402176203,
                13208760866171837308,
                10709071874368120036,
                2826067423988908344,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15812606590941178454,
                13245302131941541663,
                10851274200587028060,
                1428602951159245697,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4879539697583617712,
                483119505428169443,
                10095609936468210024,
                622272504027906356,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9536240475426218284,
                4752876538190541064,
                6343245262587009552,
                400981978827648296,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7491371307797062883,
                10407561351447561989,
                15621510571984853274,
                1592343616324000466,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9604824086730013211,
                11108563450726642619,
                528053582029764838,
                2047108748226426457,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4227479198296331666,
                6458542998874217951,
                17808246999220697829,
                2879436491985814023,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6217921201758781801,
                12999747344087052994,
                6716132211465930335,
                1066226422690228887,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15441647206313674087,
                17504521614468144986,
                5002650751726052463,
                2845234698559261531,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10795539287177431218,
                18359380741669986927,
                13232812077037370199,
                1950487638551876433,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16236164958102037326,
                13656824120493161587,
                15956843828772385992,
                479380221465153531,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11750924838948120253,
                15362719829167185271,
                9186474237165267865,
                2526038762316077447,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14747133946403611938,
                9827559670899429864,
                7467306612295445868,
                1202753263049415751,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2647942861568132497,
                6020533490220227211,
                12777392900371051383,
                1018114204044688340,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2140454143338891128,
                17678326096865904391,
                17776122493932725839,
                2767238329794564962,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15275931682900533294,
                974685361225313270,
                2999744789723489915,
                360305175789110752,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16723920170632621600,
                617142348654746620,
                11962196274217001604,
                2760035909396946753,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16655588054977829959,
                14496846132062516165,
                3530648578093608402,
                578553794141609569,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2400936301955390308,
                15550611293703225180,
                15384061384000353131,
                1379069794367245085,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15152740200322620875,
                13120613836799289560,
                16128593519618623618,
                648255660632451007,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15610567760929191200,
                14325273485460563528,
                3134114037258845657,
                3155628117464835040,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13772866520044180538,
                8572509349589091248,
                15562371791915596473,
                1363017072958881482,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11955486306886081151,
                10187011388340230398,
                8793695097792092119,
                2829277170376280070,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6845552372306517558,
                9518889509152438256,
                16365792166545923772,
                212809422489998930,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13128091564024446965,
                942286719017573140,
                7498219314370982045,
                3440473211306241615,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17777522181683513063,
                7600043184719917639,
                8827771078848961788,
                2010442534096636629,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16377556084598748543,
                16859134542995255232,
                10846127043559690191,
                2775259870295380111,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17140630766688786769,
                13718423692505420768,
                2139711113975224427,
                1646855779652735715,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17511175120018460379,
                10035490960501809465,
                3356922798771420292,
                2856163941071177986,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10801786351914493521,
                15566563986914738299,
                1018578355287593090,
                490002996444142955,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7418784003191095334,
                13171817237350424865,
                3165807403441891607,
                739115089974940533,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10920647218811171173,
                15093864899590281285,
                10725692833122509428,
                2452775730703845292,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8257530980593087673,
                11302106766983850753,
                13502802548063070913,
                1018189159094705980,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9880547591127855219,
                13389352027926067245,
                7648937772324441695,
                1085114374103650644,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12999523427568336029,
                888265565040996081,
                4187336019238910932,
                1050002599634131786,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16455854364753772267,
                4535307056955365942,
                3231021850331363839,
                3084966506913250177,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9118896291649455698,
                4586377761973054201,
                10224927198217681967,
                749191981085945198,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4076839355326650017,
                2746574645166212754,
                2122559817861312335,
                172219010404102598,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17013571122823869349,
                15101243829428707628,
                4682995945860390796,
                2954881728119608496,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5275915069037864595,
                8176855318421163750,
                16109414193744413621,
                1768172649728395602,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1983168329773456074,
                1514025244711421782,
                9544388485554605448,
                2549148264154187340,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13889654088480401326,
                3918508809162023568,
                7124781874534629055,
                3456408762910163608,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2481857575944810331,
                11703002140402029539,
                235699010200794949,
                1866500848088173233,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11026123933873168831,
                6356173538767972370,
                6746679885948228504,
                283396412663598700,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9279252673153186352,
                11424793366641921212,
                12431345674373449354,
                640276575689519297,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17588512491656350956,
                3097218526956064594,
                9816054668776390356,
                553323348705359730,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1662226093234559808,
                5539739477145900172,
                146941311885070712,
                1666036611424117425,
            ])),
            F::from(ark_ff::BigInteger256::new([
                390166836760163834,
                3256942488508866021,
                454542241207258163,
                2377865193510457349,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11043815437858915911,
                17549889178832989515,
                9708248508125475976,
                643125958822551277,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9415985988332621003,
                6064244236680858561,
                10993313762085986695,
                2035448390553313246,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13595604376451197939,
                16411283473195920782,
                3707967319664990887,
                3010112663563617807,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4623944403649248086,
                6817071480173130575,
                2489043305314840426,
                1004254729914910504,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16888744975838745000,
                18422336212509257428,
                434052107654518831,
                753484473061905206,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16063862737778353792,
                1719291135440749859,
                15174786906517643852,
                262189479101404160,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5729439285806257000,
                7608386105822492341,
                3890866199307714558,
                1296499653556813657,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4313897288086189711,
                13771972846778079470,
                17452687184117824276,
                108479484436439925,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15402566596561667577,
                7435460266988516623,
                13199812373890883766,
                2746547466659650207,
            ])),
            F::from(ark_ff::BigInteger256::new([
                326553537994270660,
                7598190551740893191,
                10879539386286392804,
                3042959114724140775,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8360400754997778283,
                7625086512303768528,
                11331032661048015859,
                2579054847052458398,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14278127335493329159,
                7410924842475946472,
                10228033135482986272,
                2871930971066868873,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4087041380819987583,
                13887619055078578865,
                7751080820131122918,
                2832601849461897293,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9089354521286176161,
                14940762866348130773,
                653524597226805924,
                3186905733644210794,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9415470920109665901,
                15743088074467654466,
                4503634048633474370,
                2207345398934811469,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3827158337171572451,
                649557978580556854,
                13959610323765300577,
                2688014237888198297,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8159824360315682141,
                17616900787942902316,
                1030095532090104071,
                404893084162813630,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8996982494277160955,
                17235344800725020113,
                7015977553836221004,
                818279558910586206,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7866257055566072978,
                9644176769344469974,
                14594896623136545740,
                3061323352564771927,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12090003268930124625,
                15396661589853455719,
                14944026200174424347,
                3005931444789254777,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14708235615412699457,
                8004868947672378869,
                11166015605332231836,
                2906868676818887632,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15732566843510122261,
                14031017737351904560,
                10103521860688810830,
                3283154371730458988,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5582906091429528466,
                2682316822514875893,
                8672162195837783196,
                1803079572881009338,
            ])),
            F::from(ark_ff::BigInteger256::new([
                671725462778236063,
                16104589315232403140,
                7264758494776562735,
                2587338887110113280,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16525537109237234561,
                55164694137340135,
                12808732236882477058,
                2576236670276634598,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8981883734258851370,
                17086635467390262022,
                1422964642735204456,
                1280339923495176484,
            ])),
            F::from(ark_ff::BigInteger256::new([
                103720196520338361,
                2595756607504632315,
                3917544833207791606,
                3229175403716167302,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9061094132723422290,
                6582190168604362547,
                11677743270894412112,
                377352318410170281,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11120396555021396639,
                9672408613265707916,
                12387320998353775774,
                735702423959288506,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17716532453386191870,
                1134488958164647478,
                15393388301964796592,
                2022486039366263308,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5538379934814509712,
                6066117965119241768,
                4314120311488113484,
                2605706237686805245,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13150193688591680020,
                2071019975873062850,
                10629792599598595713,
                1020836285235314908,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5968077134024075893,
                7389877018246127061,
                5891831423279399016,
                2894467667447025879,
            ])),
            F::from(ark_ff::BigInteger256::new([
                546109984919010265,
                3340083116100872030,
                16891695678111734095,
                2923742508576625369,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14860119864177959887,
                5746163904129430202,
                11020048865569510733,
                692066112103627281,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12875277990906058699,
                6656749837643597735,
                15744448625966750242,
                3094555517136940431,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12298799110293813967,
                11000432566785120694,
                696809752203470366,
                923362109167847976,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10020898830619256170,
                3857287713464204612,
                14269135331702181670,
                2772315398896172080,
            ])),
            F::from(ark_ff::BigInteger256::new([
                390526240196444573,
                7392963495118987701,
                17889896033179617445,
                2475822485261420001,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11406314814228268638,
                14237347634363623664,
                6112157419856014236,
                150676291479154001,
            ])),
            F::from(ark_ff::BigInteger256::new([
                18104878740929016515,
                15914543075875129366,
                5099017389141979325,
                1109162162339381774,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6202598084453032039,
                15114457216626790047,
                10512196141628471689,
                1813407803886916457,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3684150792304805767,
                892350100651252619,
                2833794147122853798,
                3017356205202889407,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16798356484040538715,
                5961145552210384813,
                8303226359282657130,
                2225476228531726439,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16961182957521757383,
                6520709997001129967,
                15257233709472738361,
                771372549006966666,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15061126701076975425,
                4804047749732606127,
                2318422219482827861,
                1424072755121836422,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15947220427001482772,
                3172143662795211204,
                15175431433224284665,
                3069149459243122623,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7687761194435733250,
                17080071084603047889,
                10132936108017069338,
                53704221412998527,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3643598108254413174,
                14248720143551102197,
                1983861302299716905,
                40045150354225257,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8684057619839068696,
                4054363914170500033,
                15650558166137332741,
                3107583297766256739,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10556670791222060028,
                3715414045688300737,
                1192650548023555551,
                3287120474240538366,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15833057630945725079,
                12382142061567437505,
                16099856578901085589,
                3000279882986274093,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10522833171693866051,
                1499366481887106343,
                3409195635700361388,
                466344386946588417,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7654681207755823907,
                1684409257054897192,
                18311820669351975819,
                1130389925368321940,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6503710028808175827,
                6044961067592720703,
                14891270950595596805,
                2274136809467936441,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10834092554139527687,
                13932038735355855492,
                8016597289437163090,
                1587648440422499065,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14696932442279013790,
                7478072326152365144,
                15495475185321913558,
                1646200154323359393,
            ])),
        ];
        let mds = vec![
            vec![
                F::from(ark_ff::BigInteger256::new([
                    902893760510840088,
                    7877908785409135978,
                    1653648380007359628,
                    1316853156480675161,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    10705339016267238146,
                    16417801999626805600,
                    6172924150884730136,
                    1266161319499378834,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    14863116828697032773,
                    17993876359370822655,
                    10736388359837589534,
                    2630005357215238681,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    10983122654047050815,
                    8919747086682085277,
                    2519623219950210599,
                    287521302538651167,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    10045217766573353478,
                    6521756444173488806,
                    11125989239529913685,
                    1782652648709583191,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    544046452267346908,
                    6282744128070038135,
                    3872825360575175022,
                    3276583845881571839,
                ])),
            ],
            vec![
                F::from(ark_ff::BigInteger256::new([
                    1293509516558947338,
                    3785656624009608201,
                    10799526533318644300,
                    743457173069057260,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    8599381401745072237,
                    673462607405386005,
                    8900703113694802670,
                    1814412788887808805,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    9878616509694323728,
                    12802218273224682460,
                    3875072113823102259,
                    2994014975637404601,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    10022759097576423368,
                    10285483923608477033,
                    8206714962876821525,
                    670109354004950994,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    1674166773633681297,
                    3715905891675757096,
                    15441065623484032875,
                    277112360926788647,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    1964356468812466817,
                    15556192890475052790,
                    16448196626820369847,
                    2949486582042111336,
                ])),
            ],
            vec![
                F::from(ark_ff::BigInteger256::new([
                    14284527812175265290,
                    119035395984546851,
                    11848602251051100456,
                    2849569764828660910,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    1193512870511801559,
                    1464590756366507263,
                    481759499197545962,
                    450041936495522293,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    13730512151906109279,
                    12908201642802092332,
                    11826560353538768793,
                    1994797540579063078,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    13150200420380108244,
                    7543922910448955214,
                    2406304860736563427,
                    1379322265060899298,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    1199455312041335827,
                    453256626201095586,
                    10134482682154615670,
                    24595769209047834,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    3069750179252540557,
                    2295753891303233085,
                    5526729693631587258,
                    1209747027556915403,
                ])),
            ],
            vec![
                F::from(ark_ff::BigInteger256::new([
                    2683303607878153890,
                    13667807793035644957,
                    8800103967528555915,
                    880909609603842160,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    8179888752487057910,
                    14074795531534780176,
                    229943340450368140,
                    594209017218217245,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    6890328587995230819,
                    1838647816334183323,
                    2356720305425845361,
                    1902600236065342631,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    5256074031805589562,
                    3212817152092841983,
                    18043317754471275154,
                    79485492745912199,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    9193030122251117780,
                    9962455722816543442,
                    17522361951008400424,
                    633454368728560968,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    15902283965759374473,
                    1579644672937911437,
                    10760303134874582697,
                    115266684215792481,
                ])),
            ],
            vec![
                F::from(ark_ff::BigInteger256::new([
                    15699579780208673842,
                    3596746540312936955,
                    8035087091741441706,
                    2119855529695340994,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    11708692820111945024,
                    7259354541796714797,
                    2523871131729141382,
                    1269717714524295379,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    3198878283229790366,
                    7954081379808565023,
                    16127447995210508395,
                    3287076553760478175,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    3549292669025511000,
                    6306219936935893550,
                    9125208221771218707,
                    2734078812642280975,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    9136564034461848817,
                    9714699551935797833,
                    12370894718933941421,
                    1462678196084964445,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    15546666891165427950,
                    13328207268311019432,
                    18351513939441619020,
                    1123385746308670451,
                ])),
            ],
            vec![
                F::from(ark_ff::BigInteger256::new([
                    12002945045106419853,
                    9487730137142499505,
                    15927750887305090504,
                    2647642693886501382,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    16928349613206845881,
                    14022416368914245123,
                    2808802991504671142,
                    2384111618495231860,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    13868641943525554684,
                    8619110066493552746,
                    17338647041441367630,
                    1560072412991697398,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    7527608575225271704,
                    7455058445979923219,
                    11729664840166242288,
                    329651154467326994,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    16042037572292661674,
                    15091818303870577285,
                    18418642109037794326,
                    2524620383062271576,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    14093780331851508068,
                    14721048150550481038,
                    2922844820204067221,
                    1950652394741241246,
                ])),
            ],
        ];
        return Ok(crate::PoseidonParameters::new(
            ark,
            mds,
            FULL_ROUNDS,
            PARTIAL_ROUNDS[4],
            t.into(),
            ALPHA,
        ));
    } else if 7 == t {
        let ark = vec![
            F::from(ark_ff::BigInteger256::new([
                3456592035413222631,
                15082085994371811600,
                8966032743832851431,
                2420526744088136688,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12439783567747264377,
                2259044841039639909,
                13658115582807049909,
                1378929418172254292,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16460450546882843481,
                12161277329091888670,
                2459303021980646786,
                1865639201081436717,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14699140980819402138,
                7345762834583288121,
                15443487270009351532,
                2460110648044919991,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2051379564450038198,
                8886102940148176372,
                17075858758819307058,
                216460953837391356,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4005187704398041326,
                13248566568668233101,
                5343014872025512801,
                2494463708226552079,
            ])),
            F::from(ark_ff::BigInteger256::new([
                18285330010954704501,
                4564046337154461669,
                1724512051382547068,
                1022841555703171496,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9118709127063015210,
                7750328075936303168,
                14544263675641275964,
                2462469647139169030,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4608558321395606214,
                15070613602376689925,
                14938378544857698285,
                915381587356657705,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15191418532194074794,
                11087552299833978291,
                10023569932873029459,
                617682117695457933,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11593167997674617969,
                2609657447776404615,
                10926115147578677985,
                1793451464717178042,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15336916604195530256,
                315763859645237650,
                11365734727920984711,
                2693374675952496294,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16571386152318497554,
                15743686126352701718,
                17228510974545353979,
                2431630132710258470,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16730850713440121018,
                16496026013123266502,
                15329615525389640670,
                3462735025777065408,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12047923600863841315,
                9419630875070664164,
                15936376330058313545,
                3189856951892634217,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12420582804946492413,
                14500895271571115473,
                6803438039901464497,
                904478048089232649,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6483585942808262837,
                14745178823540212887,
                5234039339263430032,
                490754629979674067,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9408744425188296220,
                1080293665793814248,
                16290734888850451623,
                3422996308071386370,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15737792652045557314,
                13276979202061357359,
                8863767995386007692,
                855657500123221486,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17400811292719983025,
                12905792858266166958,
                6572110376919698357,
                489846644505052505,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5873354431852346517,
                4035716845079312036,
                3651388105402755913,
                3055678854213768345,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15579011801834012970,
                9279934856403788300,
                16097388118024977387,
                2698344859362002712,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11203100297204383655,
                7312099174243152263,
                2240947712934811158,
                268948707346826673,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1985721390352154923,
                4121638849840671010,
                14018576147718056543,
                2599282177925418811,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13676594450719289157,
                15783332454389188046,
                3492666401928288165,
                990621311417062116,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9666764948710475435,
                12794817379799731043,
                18395648798336990013,
                2831645687703724103,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15059537502905815869,
                2382915114689632428,
                17094965100043328794,
                2580351932284709872,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15908101896037570200,
                4609674941816176870,
                17646457467013871805,
                2137706955199689576,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1000901000122340693,
                11230059725064348461,
                8180322249864610275,
                1157486169484980653,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6504992972926801415,
                15062393518645299350,
                14333149115746082424,
                2740141996146918,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17069881642259492846,
                12335819634962300331,
                1679312470958182111,
                1374379810030389691,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1580780497572135071,
                1614587624467890767,
                12589029107589048179,
                2317465148513389605,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8792523160678448785,
                1053125421297719796,
                15580515794142189711,
                2729399895843385419,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13397127518125887443,
                4088759658288909608,
                11137701400792631130,
                37331411174896492,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1365948931171088346,
                11662492726116101656,
                2778569392593884279,
                2927408900848298667,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3646349113253789402,
                499453432689753165,
                16191390048914352081,
                821788288547437074,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3644547030678677613,
                10788702232878475881,
                14305323154356411524,
                97841594902069903,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9887369047767113595,
                11612421376334163098,
                3898412245718371049,
                2017140017536769884,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3184535085450338211,
                8566360004660189445,
                10057296882584466465,
                2077695439743634498,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2261844603172342455,
                13809432871474696750,
                8312454712701315804,
                2390597479972143616,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12374004000430353852,
                4001705424891207154,
                15637528500701434633,
                793893448177477763,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2799064191396993706,
                4357128584384351991,
                11345613571979041873,
                76830735010482487,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6507926470335949204,
                13945034242746366187,
                874973623443841557,
                631962535933343311,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3716666907109026979,
                4955429275517281343,
                17613379192518539095,
                3333274241570917386,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6285762131887742950,
                14169212578552445122,
                4995793772497357547,
                3319795227553711917,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4784980473329954102,
                211828615679181742,
                14885188173069173858,
                1319455708024499564,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8231136740136021479,
                16181255434814436257,
                11544882269997147315,
                861966505078934047,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14636668243874518115,
                11913512418695899137,
                14676450197792631481,
                2433761199083702081,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1538490840437076801,
                1818922837832365788,
                9446418244071040138,
                2400297478020593022,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14040222486179939675,
                7095525419775268249,
                7813576171490273730,
                2323331945193744834,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7823511441712788386,
                13992622484910791911,
                7318157157104174257,
                1921124000295276055,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11996389180732363601,
                12646043357256717846,
                4335604594076200923,
                1158785087875694971,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10849487859181749080,
                18002262397676555717,
                5179652855996336379,
                233083190514054698,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8076822026253704542,
                4286937911430279969,
                16556876534049040818,
                701466644995992039,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12333830729294361130,
                11081828246515005645,
                16396905892384087402,
                1730252093513270800,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5410424609554890191,
                8922832954690742471,
                18149141918005896657,
                839366191646179011,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6898791763578949271,
                539148861614137121,
                15168590221371983998,
                1701956774565291242,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10890705141891371428,
                625361360299381000,
                10374180205646893056,
                2070715245964554328,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9813872519681908570,
                9041297237126737376,
                17737277454368741431,
                3409420679096380336,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5259931054988756743,
                1881655899428744305,
                6941858844796517586,
                1565567181530265330,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5867341069730200175,
                13269886764439279525,
                13905664544099362544,
                990234730305332407,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10019637087352315262,
                15535641592620027213,
                9914660412434650277,
                2463128548966858442,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11763087848428184646,
                10453178095533759273,
                3294371602906628185,
                3080455839427260196,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6847578087019688957,
                14958193149206065768,
                15050197264772304494,
                272790092939424512,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11351423119781999642,
                11899083238726585288,
                18390546042933793716,
                2891945883822083832,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14040037936842503815,
                14269651384334325956,
                15137578940543065403,
                172083162352393139,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14974188710862660585,
                10491496555139703540,
                548808362053579238,
                1425502164119770374,
            ])),
            F::from(ark_ff::BigInteger256::new([
                18367475931286781987,
                16598381143289309241,
                17972161556115446162,
                1521935232577841790,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10937860073763979420,
                17789668033774542284,
                11879509493032764020,
                2025610887537027274,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14396974318135564901,
                338013263145674063,
                7037321746888375209,
                1743751355208970104,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1739178760886233357,
                13669157766337958714,
                11290836385436165028,
                2789777057932230419,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9062217920416512177,
                2400210142245991148,
                5111475381681991980,
                1474198182459410498,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3656105992090564621,
                16663004615608053038,
                12318202308709117951,
                1946590300102368112,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5062869426608489557,
                9896514116321300386,
                11057282423636174927,
                2432354986999609390,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6769699469660933707,
                16037827950156700291,
                14233755435525240002,
                1266320901648639656,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1421916445230867853,
                5237321144391774481,
                5704734776130491419,
                1097210079059570899,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4259868945353192887,
                10689470057202351851,
                3704371246710805030,
                575143642662827915,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6597288910463644009,
                2298604621097463663,
                7209302884688643219,
                2633777301371857253,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10050411823674916054,
                17533103194256247451,
                14884958335836432879,
                3077458123102694530,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13454118179453098844,
                16736588163394994688,
                6120095206021972717,
                2386595256189148953,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8299945808446782436,
                5685330886795865446,
                3420977776223231148,
                1233339463347730241,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14281112647981752167,
                5740263476161478806,
                1997059912898052988,
                1906012419282410042,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9052287551986632030,
                4914899289370688720,
                2938690395995728508,
                2030659633780291153,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1883880555749048216,
                10069461876208518794,
                16022514067424912920,
                1020969105838159993,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9987356572490803422,
                9639104619508303866,
                17470229137691614167,
                3339003048300101549,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2634175975468764218,
                11870562288747244138,
                2331736113672697127,
                505595533290339781,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10207126507745115679,
                4003485015922834210,
                18260836351543079336,
                2566515916483036255,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8650000867693852360,
                459383754469039856,
                69241869077494918,
                2729823141706356300,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3201565323577921549,
                4206443082164378844,
                4469824766534685955,
                223952068847883342,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3289249922434596709,
                11177994803015895763,
                16118887014330998277,
                3040905527361186570,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9008668525446494546,
                10393535237470722935,
                3983770851871616105,
                3169811821319777185,
            ])),
            F::from(ark_ff::BigInteger256::new([
                18249688483839207459,
                11385916326182947847,
                16386144859202325677,
                3262081210883963896,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4043554387851019134,
                9333851976694675248,
                11724764117475053845,
                1764241191802913485,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8580395805872240944,
                3874191397884281121,
                3850735895448070651,
                2426496843831199916,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10069142704747257347,
                11380899578362087518,
                7795337277788038711,
                1675287817944713920,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9923148892995414461,
                1243793103326501134,
                2966730748301074017,
                512397321942351828,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7811046615073078264,
                12317176161778962982,
                5825631305211244076,
                309130870655376357,
            ])),
            F::from(ark_ff::BigInteger256::new([
                18365383396184155382,
                2156425151401006489,
                17331840425969514893,
                1258432792959836007,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16535386613739829526,
                6586431607019654654,
                12780202150578025341,
                497906684420084311,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17511929338713696058,
                17274507646758100318,
                14581799692505123754,
                474754040200560260,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12412325213237259468,
                6753197274403151679,
                3845109566838948918,
                600037831871200315,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9108223876178556799,
                9319343980882006385,
                7491066150288191120,
                3465168869283827210,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11119233983255165710,
                11998375185311265520,
                1725727478617655187,
                788139143699453221,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11783499067071869360,
                14006126051004848213,
                12553921083620262680,
                345490153044867674,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13039475731364075103,
                7580638056843208422,
                14615480795083077106,
                3373038630671541741,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14602619174246011923,
                17925602600073378822,
                5571710197376381234,
                140618358553107820,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17925354797132715227,
                6779168762923229052,
                17900551679998246465,
                3336407440978344303,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14228951914427194077,
                5667107199590120843,
                14290734883502651477,
                3485787138696934825,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10078563803540962633,
                14139729438715094172,
                13385302556909583079,
                2802801797426024253,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3173363393470948698,
                17345817983750355297,
                9482080477881718472,
                715756366571838454,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14636089677651558412,
                5532378372993429643,
                1287701112828315385,
                1083264823012073692,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15401762456388935351,
                11699182394305487802,
                8608599134432390184,
                1244576238221016178,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17866764121220130989,
                17837666698956798080,
                11677760716690143011,
                354038032412239877,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12047216519230769094,
                12576499775683013889,
                7914069010245524122,
                3256088437586968614,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13218001124693825593,
                4439212637193210460,
                46601994993162866,
                2573665010513253634,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8229101718933703727,
                7514687412441104834,
                3291816895169140109,
                3231001527939154041,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15741917012866651623,
                12647116272760039481,
                9282138941144681604,
                784148208597780743,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15273255174980140016,
                17359159824245108850,
                8482115810652249486,
                3119124844882677802,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6215166766505124502,
                7823410768143848099,
                6283987254495457117,
                391714038900077244,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14106651779670321275,
                1016136518479604828,
                16896699718435676746,
                3372036825125809272,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1146436754000045775,
                8731353703728207530,
                4311765522366503285,
                1454754771314932021,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1742368872529669083,
                5278651680466152369,
                1290972525353405239,
                290001849778739616,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14298340254454208074,
                4956261876015631978,
                7449137207143947654,
                1157251341535756821,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7451362634790721245,
                5693307366377229311,
                12035126745555761919,
                1788828121464048506,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9745697787847051356,
                18116485819794858109,
                17991935855480065209,
                1139300247245888085,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10950344773038096763,
                17451935774848501684,
                3958492408653091521,
                2688508374211479076,
            ])),
            F::from(ark_ff::BigInteger256::new([
                18234352273758736737,
                7189145074354349272,
                11967297573549757385,
                1646321155696597520,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14934090925235427972,
                18127873858534835429,
                15295319869265066795,
                3011032773222860939,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15310928365931896057,
                1972317188053195334,
                16943522386540844935,
                2976798541271547275,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5106657323638158099,
                9559472361918555816,
                13470913264475058549,
                3181497403295449348,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12094016905695265995,
                2425550833801696128,
                13449217207886421616,
                2528790052525600893,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10081806543146562709,
                14388162366542771043,
                9014993325623303441,
                1448248519037624095,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1688466522856684420,
                12872024986903244813,
                7975877133900242388,
                1617991142704401062,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9283842697525251376,
                15907556237918035393,
                180144873888643924,
                2963251379232684152,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11272606314818591692,
                82072871742248020,
                6611354323871616405,
                3392057733114405483,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11152558839719140578,
                10223519869138838729,
                11636655319552554971,
                443559306576333337,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17555758059173392325,
                3469268539515755865,
                134983068579483888,
                3378429135556701131,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9771791539308832603,
                3380769391217356765,
                3454071977952045968,
                1261053612872135669,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14306620713794070616,
                8751954222166594412,
                12072438949738583698,
                3485859582955628171,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3113386807956008522,
                6666067333282762223,
                1239299548773746963,
                320495797329895102,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5135134302730326799,
                13805223477023043003,
                17297299689487828235,
                3355030756358832512,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1071376457402591908,
                13247415990572696872,
                5346897434056406340,
                2176250451600899104,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12249213215178208317,
                14866358708433767214,
                1905785285534721638,
                1725492352898555129,
            ])),
            F::from(ark_ff::BigInteger256::new([
                259951245517055689,
                9383829278008163139,
                13016761398854329693,
                23782970979405375,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5959011712239816229,
                8166058607805892520,
                6687024426320983033,
                642410335777963570,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13710595077514963163,
                8360052682288005582,
                7438715532045914878,
                2845347385752889112,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11382960934982548279,
                9258007793934024886,
                12155393773353435171,
                97368895636811187,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10332957969684648526,
                14410161501390907457,
                7918869410055629632,
                879126146717992085,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10454286822937186847,
                9038125561468005301,
                4149337834316406178,
                124781150901674739,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15836581757314074057,
                14865288749338807043,
                13327299141070598707,
                3036328010990604195,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10351860623589734278,
                12128215007956639987,
                4222976594970709487,
                506556917224815312,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5087768033710267572,
                10210850026029143276,
                839561129185092974,
                1745104534484381379,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7786749678354132219,
                10237818439746088419,
                5181029244073590553,
                2614465618739882885,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15936799951622480373,
                7466021715322755581,
                8507746344238949487,
                2328274666290841404,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11202204972235050030,
                13799464371317606109,
                9004568117947977986,
                391622967057813244,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17586532533095832534,
                10005024358939886069,
                14103197033267329485,
                2158557230967005022,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8967593428290732903,
                13774420730023091252,
                15307196910137056275,
                2304555256584868181,
            ])),
            F::from(ark_ff::BigInteger256::new([
                500676278489131556,
                7205470368755386368,
                12730504375669351707,
                643512168989943097,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16389386048642688613,
                14882895283468607204,
                11810171316821782399,
                2031265004069860146,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4504083076128878419,
                5118441128180877115,
                8476846498082234267,
                3298328196631966367,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10147949262946276535,
                16805284154550727510,
                15789452064767214714,
                3209881938623183719,
            ])),
            F::from(ark_ff::BigInteger256::new([
                895183239445067743,
                14386476875523001010,
                17741447681015521121,
                3032465814838649420,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16582245702868680937,
                4719295655811773119,
                14136372786117114743,
                2166144343937992081,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7463824008966646589,
                15466539229934442478,
                8700237522508500956,
                2666488122285503485,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10969130866876219237,
                10771112791064538402,
                6936735620901955632,
                822821629697795530,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9130574065001961193,
                16511466144954516772,
                11477216618270919763,
                3449582760246736470,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13716814818119758732,
                465322287755707491,
                18182636708978022547,
                328592010308352902,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14493952069507303474,
                7844169830498739379,
                10439659310185485526,
                1089914778933183780,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1717081895651184529,
                8990056902963973823,
                2026838915684115988,
                2982833780756784668,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17775805331614065017,
                9217200497158145213,
                18250697919787645688,
                1258352154220278651,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6781228463872903571,
                14032936436382495269,
                8912357192276285289,
                1733647016865184327,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5370199124154976317,
                15447173411832112264,
                6544344281316326973,
                3098175054860277544,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13592536134410340306,
                13014637251472742977,
                11014623211185052415,
                995887638720438889,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11359952217393219938,
                5103079597760931398,
                1558149404309902868,
                2099164100331262192,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8828571847102232578,
                13840897166994512834,
                14001799555408705602,
                3298214155003583704,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12366145109533782922,
                10503569796059370771,
                1853909261339610951,
                3239826495733890825,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15142877083219481777,
                9496256311439424235,
                12249473134176494688,
                3412471036532043335,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12552643150273251120,
                14298629308311460443,
                10480091363746155957,
                398097051179744582,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14596795496166664990,
                8903378031744632529,
                1950740206407938217,
                2987606875317650636,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12690065054697627720,
                4293712947776804861,
                5862622097688050270,
                2107598833037637579,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8913661957092957584,
                11587457584465429704,
                17163000808013615681,
                428681903381285429,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9832599699066558712,
                1803165349577212210,
                3048978116373421250,
                2253013763121899016,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14860155127887993009,
                18195876315125349811,
                2225760069343562020,
                3257669870217644641,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15761617207541587181,
                16674320975043497928,
                5849946365175867119,
                407745429095286712,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17782572231929168350,
                5714605951186976093,
                5712480575845315230,
                351960973229555991,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8309380154468448527,
                16971587540515534780,
                323728962525332409,
                2327613877947362334,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8496747135983952101,
                8947725841171346278,
                7703700095944197179,
                2434417842702992452,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2411545334498078399,
                15464849203980232470,
                5653054514714658486,
                1370147146937072204,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14182769852247999385,
                5020642342051525591,
                1457042457425244475,
                437836784528441071,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13043666079777052113,
                141037950545602715,
                17831406874696225449,
                1092058246202095860,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9810943496440757680,
                16900401164270806328,
                5987905840707828528,
                3219248853220238887,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9625773377514865329,
                4443217323362866629,
                8315441025686887246,
                1637944020296813811,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5021513803055331921,
                18259586829484144558,
                8518160222669255243,
                3003871315503313612,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10891303680509653351,
                510180033885018131,
                12044505966349387938,
                662760332898226755,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12452544344390639869,
                15857384663039430466,
                5149510391448358091,
                338192246258831596,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16317234872763972305,
                9144897874840732464,
                6785020810402524655,
                2994050498649187523,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17213946524935802754,
                13252486445225020423,
                4647154228960834257,
                166253745392429675,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5609572952678481293,
                13663754374008066111,
                6890526594345807346,
                2498998728757892367,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2481587195790683447,
                18302934283068228238,
                3407782691253293841,
                2238111918782593834,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17887889609399731911,
                16720792061988981707,
                12610681922588292286,
                161151435266257687,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15470648047961291221,
                17943676566346871770,
                9008548372417711179,
                3065197512189344742,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8136499541303259774,
                8493556857508450580,
                11869746443192760770,
                338038855513802173,
            ])),
            F::from(ark_ff::BigInteger256::new([
                906097069252097182,
                10308876547573678736,
                300167152268482493,
                1071264773553989549,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14197161255669673153,
                465463668440214813,
                5587051822932261046,
                3219137816337887232,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12145342531593768428,
                1166683757061885072,
                6212453275009283955,
                582920610890522413,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6049661835627034912,
                1939121272652997933,
                11240943521387854830,
                3384702116388172468,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14604961006798189743,
                3328254468960577772,
                6943937041496861632,
                1664162023058545338,
            ])),
            F::from(ark_ff::BigInteger256::new([
                434336828179878984,
                9467752103246551071,
                6979294609875450261,
                1601414587754194998,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4863516691582542523,
                14062354363850265058,
                4904357816096420362,
                1015351194429438185,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9681725187590444034,
                15050195843048247396,
                2388974048303913339,
                1866477790022897530,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10190475879252191534,
                2525370625909952367,
                13667057592077516695,
                1027748920242094765,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17535602931301884604,
                8479175822886332671,
                4801419665554522456,
                3208999653118607802,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3315147529230545974,
                6860017924377889277,
                5956506242096792465,
                1233355332370693236,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2417008685536623338,
                4123889919270107809,
                14648226427947744837,
                1181195806325184441,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2609075152434982971,
                12379948446274069623,
                13957764051751149620,
                1032404314825467985,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7086671929008669512,
                5985426685767314466,
                3986977181026068010,
                2923279857469353497,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6323936188516347107,
                7113999419409619140,
                5601998296961521410,
                292856665357303735,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5551143378483257317,
                10575966208704500479,
                12937461220648086797,
                3059041577954159611,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17087362507994004575,
                7946697022606373530,
                4285609980600373855,
                1792831930713970969,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14185643239397012613,
                11472245144113725488,
                18165942300848772866,
                963634646911465974,
            ])),
            F::from(ark_ff::BigInteger256::new([
                767808177925063746,
                5926361711255161549,
                14283816120606044399,
                1019261625602300457,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9256606373654452634,
                3699799203779203693,
                17148568839753567135,
                1117241341610067388,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16678016336378811445,
                17235071902065714854,
                2641588500486586499,
                946695631294606988,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15526796977350999495,
                8592482161462675759,
                18399113963746574527,
                2285256764043294991,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10412712077801208808,
                3078346925754039334,
                6783482768224252012,
                3173472921118658983,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11916106259278710254,
                18240382638158138475,
                14894724550821825221,
                1431136686418761341,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15739113833885925980,
                22152145797742629,
                844580775101623465,
                2382308233840721603,
            ])),
            F::from(ark_ff::BigInteger256::new([
                996351385116712153,
                4992970427626877515,
                8285426498707434165,
                890862346069193818,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12138246558623790379,
                17752325241884048531,
                71205187846298039,
                2242576855710130996,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5261177161279419105,
                10607966569812704599,
                14798335463754499454,
                1769649573990365955,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3134734930931018892,
                4654231690038242478,
                8640403222416618473,
                586461354937451563,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4438563449728604927,
                4133357726690998312,
                5245114189759430089,
                1146190311034834161,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3847893292242450182,
                14748784974549881494,
                9001556681649330418,
                3383927223709149853,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5489760697448468305,
                18188412712822353437,
                5572873544794852731,
                988206376479955871,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14630688315135524102,
                17472249808403729353,
                1774308306803784578,
                3033537509596872488,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16654336817232514537,
                8282945082406273207,
                17622500211238145182,
                785097458578783562,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15072339344115872983,
                1295629130437179087,
                7677417375006829797,
                2591375773501617362,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2618396618358761223,
                16592952026299985202,
                17193142179157766960,
                1404260320019649578,
            ])),
            F::from(ark_ff::BigInteger256::new([
                28042767350926386,
                2456546352370224469,
                18002660223864052331,
                1763915492755774253,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7949657044176125221,
                4088031065768279419,
                11773658707080435529,
                2531355549710719378,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15215092944860179811,
                5231977506405426772,
                11314884151103634582,
                1104732011936288250,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2306638837948882892,
                13264059218792444582,
                3862827795700392347,
                2582878573400980148,
            ])),
            F::from(ark_ff::BigInteger256::new([
                18070655463667449462,
                14637525915374903643,
                4229074532813638978,
                3361505007305358424,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15996819985574365451,
                17313247214572008846,
                2423876318348275009,
                1653805355832727624,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17405932615576402691,
                13731878014073133660,
                4794273496616997649,
                1409789099967602746,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6165444736737481146,
                7554238263295000927,
                3515269672368955659,
                768890666051437680,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11968456337635122939,
                6190936668835268053,
                2289655809303991458,
                1590195601099932508,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9335471191353032998,
                11415828689488906834,
                4764349945829486092,
                2583753601946257682,
            ])),
            F::from(ark_ff::BigInteger256::new([
                18342930691964280988,
                8300646067504815596,
                187353040216362913,
                3396099377637861515,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15087819447289611794,
                8712727920393867753,
                3156487885742737123,
                3110340381867310799,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4838168054533929035,
                17400511451637304235,
                3222702526112561990,
                3415468275116498818,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11991373469178010112,
                2725845956713220942,
                3189230416759775194,
                608248368681693672,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8240203773462785964,
                4653144033244576133,
                6757282909693395252,
                1626952715682312943,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17035030138422248309,
                655782182133313105,
                4156078933700574132,
                152344698690093059,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10247030333518269179,
                8048377238393775319,
                7700523460977585899,
                1361314446384522763,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3988519891959297352,
                1526670867425155440,
                10102854999727135863,
                899741606204855837,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4693230854359200049,
                14116616747576205728,
                17288017150123380019,
                3469152197730600408,
            ])),
            F::from(ark_ff::BigInteger256::new([
                581450548907801729,
                11114557770390915796,
                17091551745394444176,
                2547197975316530972,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6525367881300392560,
                17111898954757984540,
                15669412731897959976,
                1495063540346109983,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6220750058060462291,
                17880997998469523039,
                4475830987331398979,
                2464354852477849353,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2807229592150517118,
                13259263254573634962,
                1870196027902663122,
                2565462861109061280,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14526141729541154606,
                5251218441866872155,
                18083193679793083488,
                134196841940583806,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12022213194605313011,
                2782142837678984352,
                9774181961886745454,
                1394325879869748005,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3840787186887689191,
                6902016133158538247,
                880386015229011912,
                765299224683363660,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8446811730725438001,
                2992236768793059634,
                9847550984789603820,
                1261338971664227046,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10406095087029383352,
                11012440243779843479,
                10690850114546110354,
                2609217412599990661,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4116079978258034848,
                17990388069971093125,
                16886581936226906171,
                3166066239390947233,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13457579932791439396,
                7614553333243813375,
                12062985237243178562,
                3237925921222336151,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8625440937437516841,
                12667025319688233672,
                6985187486877788103,
                694414164095989490,
            ])),
            F::from(ark_ff::BigInteger256::new([
                100690776922777549,
                13806602721166264679,
                11179704991116015883,
                2294682042652689724,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14622426472235327121,
                7588671397561235707,
                7108594758702475422,
                2614377490431321464,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7847155660180824453,
                11841648000559235453,
                15571242343468746624,
                1163628797135738922,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5136979547538731336,
                14515544308171765970,
                9924568640048159807,
                2675270290365240020,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2224613762551805117,
                8583603438677220416,
                10254588711932918109,
                2946316655498354555,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11487546223735464293,
                12356963491337973381,
                17166493059647471716,
                522151382342685763,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5500368119229334814,
                17905075224056973544,
                7300691476873018007,
                1496991151783627007,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2010473316615961773,
                3251336809374215953,
                5739143523923385423,
                898034506881723591,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14507766337270428492,
                16267738251745347204,
                15569801822141285595,
                279132457321835630,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9276620133866628326,
                13968455492829996916,
                9323854674740387855,
                2310701209861970968,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16481372078627922461,
                4463219153557404219,
                12467833548794151166,
                3018589679893211711,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11232292033030072342,
                7817264867969232238,
                13161346759581424363,
                1422926622046917809,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9494313194533872634,
                14538035514510785824,
                4460128942231822216,
                1749548276920528097,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5632134683047942695,
                6868542455124138548,
                12655083919665278996,
                754951444627952516,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15476982493290031213,
                7657691862663815104,
                10668182180756287472,
                1734581618176942005,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1248073855338379843,
                8691089635822131936,
                11776131340369692155,
                759541875406466569,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11778342680039816426,
                11242809207490904890,
                9798643698557313842,
                645902336920477260,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1171586749349068279,
                1431460008091800114,
                12225320502221755864,
                2451843591550702311,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14694018479861964028,
                6475991265129474330,
                3664283304650526163,
                110002795805620844,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13623957310876906224,
                3730678186490832318,
                8969421209019748565,
                530016725476745596,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3850590765685560744,
                3213099104686569850,
                4065012035545121118,
                3100403131132502556,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8797787759620356843,
                13900745001237000884,
                2888527775298917607,
                359281961801839684,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9956398600703829659,
                1319089331989940738,
                8158637145488789323,
                2730486037962269404,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13171512369112704944,
                11908363229442664585,
                10735922937308227507,
                41871922591198645,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1434156019693471049,
                9822478737706040996,
                14802139887089273774,
                2833188690622882310,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10274919360049990585,
                15398983535338241306,
                13378179574671753630,
                1466676820477970317,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15723196569880057330,
                13106905502009573731,
                9538309032825757785,
                948746648251864701,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6958275541437211842,
                4333636475694234956,
                5326583454838611452,
                2603990333957894340,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12064510455239497319,
                12412347982830624086,
                9398860893786035781,
                3194738900276116636,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17090430209043138958,
                17157158454666321555,
                7295856029210294454,
                1835105920714710137,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16504524091493589665,
                850733021546942506,
                8574677846165768172,
                1731478044822977252,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1620125799459682033,
                16143955991234772299,
                9236810009182137033,
                2018185434254878544,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13680687893310501822,
                13729378636947990994,
                9829576652988611487,
                651735693446618307,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5914572552647057435,
                10856170618773604310,
                6458724887134062527,
                1818729114940314859,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5667714663848650961,
                14404574347344239190,
                5700002488268857646,
                710959319811591500,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5323650660868406815,
                7526110513096299413,
                11312525667301687665,
                3461612068508097818,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16236163225601504368,
                6493875660342763542,
                8483446684162199070,
                1522375613391764384,
            ])),
            F::from(ark_ff::BigInteger256::new([
                471162513087756358,
                3724077876375785288,
                14304111428254442114,
                810403142656614083,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17287939446828213463,
                13012918013361493152,
                6244502770916151518,
                362986860981941852,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4151690974873025245,
                10149249399399876230,
                4274358881373678789,
                2568620957576329551,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17778170182071837846,
                258797005471283155,
                7537001039700679712,
                2099253136069071494,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12804055283424199137,
                5151080371733105282,
                7869572400204319011,
                1852277204031497122,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6645022940618993274,
                16281178250380923972,
                6617744772858381751,
                1413546190656531273,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13524907338458575909,
                16359781504306413790,
                15334033921704771593,
                1898968232242269959,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6888533899253285022,
                9856464736635625058,
                9570334743265030613,
                441884382139894677,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6629184179745511101,
                15322284711727675491,
                5332596898104256239,
                190308064590500966,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5585667636010301853,
                6004777619586685192,
                10706319394308570497,
                2436365259962826855,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11057972066363399622,
                9496931547337528847,
                190505288936862365,
                365161639386843554,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1472895192702967734,
                11655432861829919785,
                1333752861998887970,
                3332756206386636082,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13510689867542519822,
                2852748896116139118,
                13116584999801335821,
                571585342460153690,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17653372574576138637,
                3624593767862207495,
                17439379827588352211,
                933158032740929709,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8462438382226373817,
                13141205442723359995,
                17411687643655052328,
                2917312522026332947,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16996034548231550117,
                1413033102711012415,
                12837055335796479637,
                2924033477873350834,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9331162546284818952,
                16769693232210614260,
                11692846186169579224,
                2962201063378831714,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7983810463818591091,
                10736469156597238093,
                5537874494129787509,
                1013402590153806400,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9469911398498444773,
                13258501493213235743,
                1313617240633722146,
                1035994974249913031,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5493899437499539185,
                5761559763870368185,
                12152970863201945514,
                2521371986236544943,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13790897522209808625,
                3193958689495809767,
                1110507242993083916,
                145475922524919213,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8560629049594469403,
                10128526419535054638,
                8579644365615609425,
                822580098010944824,
            ])),
            F::from(ark_ff::BigInteger256::new([
                667205183315067859,
                13247166039768944765,
                18042345543280558718,
                295160214661712640,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17866308855892208700,
                3320440802269250490,
                5417265075562179756,
                2575078539650443090,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15332958455267883542,
                2699586516962423587,
                14289140829944101748,
                6737420856431643,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1561917424093291188,
                1655387573375567215,
                12936448724131345802,
                211776141644368008,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7526357336233160229,
                3880551127705655400,
                5333971909157415841,
                2244337760845592423,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11213054996751340459,
                11306929592880431356,
                1551636019507827988,
                1824873702575551990,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13751235608984005149,
                13907273515747890637,
                9658856303240153171,
                2827732806745310301,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13672366906493782600,
                7938876386281300312,
                12614346625364077927,
                2647288436605706135,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16084861297664747573,
                12977885874172552016,
                4631276929120427116,
                895775488191004959,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15565408813654115234,
                485272573176332019,
                10237415696937686327,
                2156682515656619469,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9255559221751229772,
                7709560010130687872,
                15743587567461197281,
                2008829941213959823,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1060434699212937604,
                561416806352423000,
                5485802615892222737,
                1892807890214807718,
            ])),
            F::from(ark_ff::BigInteger256::new([
                18060496628798461833,
                3291928375809843224,
                5016827298698279614,
                576768469086708362,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3206736533387661686,
                9013134898123371190,
                1093355187035756011,
                2642375027732874590,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9568247811262612726,
                9381563134593252774,
                231428036670375950,
                3122873237060797957,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16277379050755070810,
                14508929695573379765,
                580948743806156119,
                392765259351882544,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3903707463098605461,
                12529973605908843737,
                13545408149570818590,
                1994761994396902892,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9263189706175616129,
                16204004899478325757,
                7307241866332168155,
                57662158045516759,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11457041063069751568,
                9482656530725499965,
                16020012448801033959,
                3193885436491062877,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14485023447735825749,
                17675270244714471743,
                2063930304387603775,
                158932907647729454,
            ])),
            F::from(ark_ff::BigInteger256::new([
                18291378091388187699,
                12235262777669757086,
                1842170649977518411,
                1741569296759959758,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13354595767468507093,
                2296257049678370765,
                15551392319505386983,
                7514869310964690,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11697794721284326232,
                555102261785842746,
                4212601104231601161,
                2336924905365654545,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15306513978939525307,
                9954674475336429943,
                8922534375560747129,
                2846675922443835479,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4166928827467283596,
                10747474019059960712,
                2222050282630810431,
                1621496194407326699,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5491898699109723765,
                897817438737002474,
                16105493446650919961,
                1831606880475051560,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13706985220516115590,
                14308911335461589428,
                7513702566476069727,
                3002506085227282172,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6131007849712625753,
                6263910649747519714,
                11733670113081996416,
                2471605471666780205,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5751373796693398943,
                15119750916236077094,
                6319332140379338237,
                1827930051991611851,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16829639016020932672,
                14831913564934535260,
                7907507370976506444,
                1053383503179452602,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10490433991394286369,
                6092516109632400209,
                4905242863639058305,
                1843601487152808270,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16256020170280461092,
                1155340632706796286,
                11984923044161710817,
                614188693203634303,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6223868385501647437,
                12743396310383239445,
                4459183944577343737,
                1402341458559535538,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4810142721616358273,
                2668079371193803353,
                740650120673679015,
                2497464588973607504,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5069468542550535683,
                11747755320208756457,
                7593883583110031946,
                1772270479603069345,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16199822322890912164,
                4641569358874153027,
                3364550386096001453,
                2403605247607985609,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2466239096672521265,
                10988813044961365460,
                13296159280881395458,
                72892726623056019,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5660455300718323495,
                400709101715041234,
                12360276001225457733,
                850767326320238832,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1236736486243760715,
                820261313116766245,
                2379094821372873650,
                524096927522121468,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6497047390990822998,
                10998163092338741587,
                4376926754414268585,
                290163558233431149,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17111990934733396685,
                4378198863147206647,
                15833080653248232996,
                2771858313606107240,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2141268195313011160,
                1667908688792940196,
                6290539516121831819,
                2672048905864804554,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12121384157962030191,
                7955571838383732316,
                8405861863451182527,
                52790934521266422,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6521230296671375148,
                9180288730863043677,
                11642611326368727046,
                873345663359435732,
            ])),
            F::from(ark_ff::BigInteger256::new([
                957386088399374119,
                14607541147905633609,
                15958847592631174069,
                1889668094351104849,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3923375145157746936,
                7604418469499319956,
                14151346964812051084,
                3146560197612842512,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10150334548751116257,
                18339489982338260001,
                12479055920896569266,
                3348300571970595407,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9325132138530565574,
                7154777565209221810,
                1618836881625838804,
                1544109151323251856,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10613252628248666908,
                17717178868220199218,
                11366457765927488325,
                1260778035207377579,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12251052330442189004,
                10190151184387699986,
                10584259823539828384,
                803922622379379226,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13876765604202153831,
                10479641234270893806,
                7467872394690318220,
                2028875439314372965,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8013159841002780066,
                7389234187565271723,
                1449020047378896515,
                1297446626836970778,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7415133599180483910,
                14994618218283194164,
                11912986270922583623,
                880368309394524014,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2666168965251371448,
                15580625285018552924,
                14054505691200756794,
                743251393573922992,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4680963051238402701,
                14669516937609365411,
                14083591527888848771,
                1346475820521665734,
            ])),
            F::from(ark_ff::BigInteger256::new([
                518107599322159701,
                17651848738499413054,
                5190769567582111510,
                3025354533733125376,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16513053984790794664,
                16536580939448404689,
                18286045589837392247,
                2326385855504764624,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14457909562457110095,
                3567398280746613777,
                16760633977430385780,
                46940497426244995,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6183432690879390928,
                921711564936487766,
                6332726951217257578,
                1581997857221755669,
            ])),
            F::from(ark_ff::BigInteger256::new([
                548081376298168815,
                1473494993395045214,
                3976706444706600351,
                2863194914987892622,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4153348383331610300,
                5085298942351170097,
                4089501381752939745,
                1886657813074210478,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9127612178466390574,
                13194178380164089181,
                15015393982109655088,
                360078836777133590,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15242172218114329015,
                1488866706425161651,
                15469256071779722107,
                3399178109359506940,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12417455268101931179,
                16891335343462180902,
                2115690319975655679,
                2491155113056340252,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2534174862454190713,
                9804726988001389948,
                7404576769541560653,
                1216720424967862018,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15386192860617840716,
                18034017458596344162,
                10543573209985890179,
                1662445730984248905,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16446493643488517131,
                17472276713362318509,
                10278799619248144737,
                3356895416442630968,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10358824753313080522,
                1368075279143744934,
                15683848555540638230,
                3199843876934651381,
            ])),
            F::from(ark_ff::BigInteger256::new([
                914505309750865346,
                12732612038768155595,
                3510241407196609639,
                377444133075410823,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15143283764264557414,
                8106971950706646243,
                8547177641034627465,
                529019648570360791,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13803604408067312146,
                13357306532199558381,
                9351076287137656134,
                388324568126763183,
            ])),
            F::from(ark_ff::BigInteger256::new([
                853800034130400981,
                2083512912725031383,
                5977399390480307444,
                1065392978464326508,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17799273795588319894,
                12623657305459129448,
                6616886632248214703,
                2204174719912360750,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2635781855238122840,
                12983876330079159720,
                9027893774807232997,
                290555104606972842,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6001893312244132215,
                2801380095174481274,
                4414068252331444628,
                2904108547774280593,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2563717606863079100,
                2800875452114421014,
                3710782233092597633,
                793845580706643162,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5931350277934498008,
                13890217372554693788,
                8744828093970814580,
                326114193066308758,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13154553237051698158,
                8825927447307609212,
                17993891808928807729,
                2717736021164077822,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11642483599152162984,
                17781831284099275348,
                274437177345118837,
                931293447288265095,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11211732197450031380,
                12219810179628076195,
                10927525490379539590,
                3155955341804771148,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15847478172819924305,
                8676537221245130968,
                11007064387913771114,
                3304286136776101732,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4061956377172557480,
                4889247003563060602,
                5190144397903475999,
                2756964529931092963,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3306758818620985479,
                17806238293679591498,
                7210310180898020975,
                2592593101722229703,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17624012838555845182,
                17723569331432622757,
                11327256461712408375,
                785012882251036491,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17108616091900911638,
                5741960730769824706,
                13758569084301402979,
                649448646532490868,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9502396423114940146,
                6158133042570502,
                1409042266501128044,
                2889035112168662207,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9182028007105600681,
                2991401559272532641,
                7780617631829190930,
                3313382144001977839,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14882510163959291703,
                15369835511675825141,
                5897042315768932016,
                1403816706964792256,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6003935873096329861,
                166131993256405963,
                17799466199213227937,
                256721429537022602,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15365900433773868373,
                879292157961523690,
                17440075636329277089,
                3109836062712814674,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2199097427594974168,
                10916285490995606824,
                3184809768326671724,
                1315621170290109723,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11060196624802406127,
                17577241504441374093,
                10752044120314472422,
                2199604852922282651,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7252399010623525818,
                1704928182525351959,
                8270912234715066615,
                324066844017720149,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5575953624016625925,
                3860659600500396586,
                6515201826874012249,
                3280873368405309261,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8771560802343683248,
                16064067769717593961,
                9251659244739282122,
                2831534164784432581,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16716879357900317761,
                15981743590692719958,
                3534539146292820717,
                1695987323098276981,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8510666531665949760,
                10157234312936058133,
                14902581704780830871,
                499566522209340917,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9575456954205376265,
                9818637722516113087,
                3862368291831787410,
                3474163319226617437,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15553040963850422317,
                7231814355517863433,
                6516637289395881080,
                2257102662370105196,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2576438786211929707,
                11258392623492011683,
                16214464107718221555,
                3030465381769008812,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16249846991571501967,
                9170253775383542208,
                15244384309058293004,
                146532136463716158,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3705244312837424322,
                6327538810841901411,
                6017697781820257940,
                1696858507381794264,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10109425577983875488,
                9033391512740137458,
                16991473436678373763,
                3441326030606712214,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2545870929004229164,
                9083515572079222663,
                15397006266015552475,
                1520600649361259284,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7536126496860927102,
                12470342186032379190,
                5625247378804004216,
                2726390924827781388,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11469606562025462999,
                3656816480031504336,
                3467053275975943640,
                1575511771052463145,
            ])),
            F::from(ark_ff::BigInteger256::new([
                228402116245936561,
                17600696349498248385,
                1197908442720407725,
                2553438302517038218,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14247957881386179869,
                2329340223347035731,
                3928987974811241331,
                3062671688816152096,
            ])),
            F::from(ark_ff::BigInteger256::new([
                18445581365892162424,
                13889516525866134547,
                2928382046805895020,
                1284875584447675190,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1127393789535594822,
                743933875578193153,
                3028611062765981568,
                1335809384403943285,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3436621751636128515,
                5261303314525193196,
                9017747553474829470,
                2808067698487170089,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13679103983225119593,
                14718021814082305337,
                1506587091291474205,
                1554950893272963107,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11529548577192213215,
                10633719468417381477,
                8549731915463053623,
                523005167584329093,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4957292439371737343,
                16030895420014182499,
                6031844947140793611,
                603549807068543827,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15157875512190650997,
                861750510429733823,
                13182702557829956568,
                912567239606964993,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4445974171393237777,
                18063951517194766171,
                16138784914302494326,
                3271028239494881415,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16822836537035426948,
                15518776268896286214,
                17291280774335245825,
                977919593860937450,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11779953268826606662,
                4540489301761540122,
                15988888924939486253,
                2610605900762905253,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17933826198344040856,
                164143701574621958,
                1070231122634213155,
                1622446919887517139,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11858373770230075115,
                18330576964964915939,
                1056107742303245487,
                2003520124724957613,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8945012010498996489,
                6132758482477100415,
                17127378000538180686,
                213476769466065341,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12675847040339056474,
                16972591438239814473,
                6422960055958421655,
                116926473521772586,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17656583657383707566,
                2757788319645272001,
                10121282059974831139,
                1826803881306816237,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2660812709544063459,
                8008686875590539467,
                7484892556115785230,
                735955515744371045,
            ])),
            F::from(ark_ff::BigInteger256::new([
                562250715732604078,
                9348280506124081094,
                12084103543341505854,
                9327993023922046,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7945110159520031443,
                17510095960443343371,
                8210326231842774014,
                191147114115810079,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2858282493952711329,
                3448793311767760851,
                11684938984120846789,
                1134998744778556935,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17256466968983459137,
                12806437318488955823,
                3668316616536830002,
                1373828320489730939,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4654081998327953630,
                1794317929290822837,
                17687355740356763154,
                2016164461354166528,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4815242902780597994,
                7281295212590738849,
                6989353940606751637,
                2936499144479695892,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4439878028623010057,
                292531659412677142,
                9170690328467488555,
                2464070352841919129,
            ])),
            F::from(ark_ff::BigInteger256::new([
                224248855418926752,
                1461616237216028304,
                3007006259296883367,
                454209659279444223,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13312355953720448677,
                9419030792908411293,
                7719975341378353772,
                3269010940870280089,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2198876062838689291,
                850531822032740697,
                7982887583897049710,
                2752086183979653670,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5407155406948757998,
                15344378020366822005,
                12331940141751401202,
                529497562352712926,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3792235194745868074,
                7916629694732445597,
                10349456940464548233,
                711906295134711894,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1467256495502850236,
                3797149717395424826,
                13094140691555520292,
                1434817404242734444,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17734092504130984656,
                179262552135900081,
                12835747313349266083,
                418048094247423822,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8188585042589285860,
                7341361526321097126,
                17428726646080093830,
                26147640483520986,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4011557488786508987,
                2236210281175041461,
                8995818485432213431,
                2378752092427736550,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9328031172177127720,
                14211661829531335343,
                2062736790733461106,
                261068555181322745,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9645649337855559109,
                12626514606372376392,
                13300944590137935590,
                2470149683653588741,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11676477592994111360,
                4784901363916634382,
                470529289083479356,
                2863382715765471696,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3911503170372622088,
                8767440854496399013,
                3193552663376130861,
                977790146222421926,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9599411781249533744,
                7474520097980271872,
                10216008661101420785,
                2353672711602579000,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3629248366953965755,
                1159151394295431728,
                7925368999560875034,
                2527435600371668604,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17126556338655827060,
                16547826864519410901,
                14335692977225909491,
                3028477779193825858,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14932440780670112011,
                12858510798093816146,
                10256085214835979773,
                2008436912667753848,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17681989175900950713,
                13718417427154469388,
                15993697914104155473,
                2836198835923874616,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13770549935322857750,
                3169706423450274645,
                11673078700919421802,
                1603586396690310660,
            ])),
            F::from(ark_ff::BigInteger256::new([
                18229270032996063656,
                11775897196057401869,
                6279315998546154018,
                521972013622829685,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10318084412429230383,
                13979942642638259843,
                8396191604060295061,
                1199925525970117149,
            ])),
            F::from(ark_ff::BigInteger256::new([
                255906782884073142,
                14517946923618276207,
                12142030739953173324,
                3009959647715560616,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15354737924199492634,
                18350129422831340893,
                1274959722681111926,
                2539483692511997650,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7772882986632001600,
                6587319450239208007,
                7369024384719140447,
                679933123241127037,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3572167687229953364,
                15542469768670748607,
                13349719706208103926,
                3442871066977717288,
            ])),
            F::from(ark_ff::BigInteger256::new([
                232174424184193594,
                17256840985863145693,
                5780253557895807094,
                1993985047457378632,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11512189539052084354,
                10729088714602738342,
                14777592478529733168,
                7939738998681814,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15982313722663023794,
                11789548200771174759,
                10057245503569510375,
                3148571385884413195,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13323936732290851506,
                4768645947316177680,
                8096235138843722053,
                1916377729807433310,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9478032939310538807,
                7708305423095714803,
                3146208612275921905,
                2816108208975179708,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4490346117634607824,
                2303570822284515253,
                13598627158040027808,
                1683506786630675440,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1728259036266419521,
                13724448766592292236,
                7166941493077249442,
                2323264058049656303,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1905729637507640312,
                2693424616131280195,
                9638592742064073121,
                2763710422190591145,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1331643728275343272,
                3110528642950660508,
                3376291987121022658,
                1392853494950989452,
            ])),
            F::from(ark_ff::BigInteger256::new([
                18058751243944271837,
                2912810216360247067,
                7325289649340435688,
                2407809390828183258,
            ])),
            F::from(ark_ff::BigInteger256::new([
                844140213795324646,
                8129457459516520774,
                3165339742395123117,
                2920563783199610469,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11506209326660623533,
                16885025799963681953,
                13181539653418555538,
                279996693679472715,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10693315393099403542,
                7440305535729700073,
                6682801835513045272,
                1039713707546811974,
            ])),
        ];
        let mds = vec![
            vec![
                F::from(ark_ff::BigInteger256::new([
                    2002489896936373917,
                    11627380965973864017,
                    12257054355012123199,
                    3079791540599816841,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    10044094589990287046,
                    8314399418033417391,
                    13155444777092340236,
                    1966882851745758350,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    2210898060406194051,
                    10751944998882491641,
                    4348252202607987284,
                    623519555060098025,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    49262085359240181,
                    1962785914585118244,
                    10527325723873512379,
                    1122862618166969812,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    17880542846394003673,
                    4892817629059699408,
                    12672624504155528955,
                    1652846703370851112,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    14460811345531593751,
                    6190435757384039660,
                    12656849930297753125,
                    2033681060046591996,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    6264570190381208215,
                    161801634846616592,
                    4484569473894547506,
                    42519657002541918,
                ])),
            ],
            vec![
                F::from(ark_ff::BigInteger256::new([
                    1302662549384362377,
                    2849490396810410631,
                    2792214394978316632,
                    2299649141446789028,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    1150362438982126532,
                    13321296947619202581,
                    6821906295048910754,
                    920962738057553813,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    8553929507472890574,
                    1577431469761024336,
                    2805631481138427591,
                    257813320648206855,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    6752159427997882709,
                    7992539097901150338,
                    219823869189910825,
                    608306148998075114,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    330934197940687307,
                    16819360955492054055,
                    12583989537273587375,
                    1874194050101957050,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    9059637602677296544,
                    11753671816352877669,
                    14659564384408804614,
                    384606794123824399,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    9871184375315452609,
                    16678380309829928365,
                    14490481210141988328,
                    2805345170201570759,
                ])),
            ],
            vec![
                F::from(ark_ff::BigInteger256::new([
                    14795398744634499660,
                    7337663803036882283,
                    3252017149930406417,
                    57899026549284211,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    9949326171880365246,
                    7654562991912623225,
                    1738710469613651681,
                    85352556260698003,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    15937344016435590273,
                    15294173623265023272,
                    9418477521159013454,
                    845146236608339678,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    9474624122173825756,
                    16248036947907393568,
                    2421475471619853505,
                    1721755202295560333,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    5210688920497672711,
                    10004191734862456685,
                    11764449899593124917,
                    2543278179380177747,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    8856789751287799197,
                    9465847437676841544,
                    13964485642469371227,
                    3297758040669518682,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    10950551348716117576,
                    14954254317701026095,
                    8264599477715283528,
                    1614619039634207052,
                ])),
            ],
            vec![
                F::from(ark_ff::BigInteger256::new([
                    14526793096436700939,
                    7567370668424124892,
                    4970896021425924275,
                    677301459617209370,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    16082514034746322159,
                    12391493258412840888,
                    13406365703834941061,
                    1233943278671090615,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    11857674374676558886,
                    3861878853202245453,
                    3789679107737871989,
                    1870467235385654953,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    16619762468048511348,
                    10024377190760694552,
                    14074377188952137748,
                    1852813172889288751,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    7230181013337521499,
                    13259560148353080670,
                    4482287181934555392,
                    2686134645728705009,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    1993077840531438347,
                    7655615565417508932,
                    8912906509565547538,
                    58287926744042942,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    8111113496093431193,
                    252598275881643551,
                    3684684037943957306,
                    3461089014234642953,
                ])),
            ],
            vec![
                F::from(ark_ff::BigInteger256::new([
                    11801217536654349270,
                    18336322578262301064,
                    11632160195201116744,
                    1469287409602310635,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    2670310227813400258,
                    3585138223638518658,
                    29935108470253639,
                    1634763735523408056,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    9327523633148491129,
                    16138707706603722851,
                    13454950915978488241,
                    2492238951573705967,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    11235235156277889308,
                    11644287703461370850,
                    11486857721992821676,
                    2031511472285224832,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    11693898417579492550,
                    13093323202373917388,
                    14812923753363870445,
                    670023409995170690,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    6743217211679469449,
                    4848887765811806734,
                    14989386348467325240,
                    3352995704443553830,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    6856672410811368203,
                    11552844591338703751,
                    4454499666912557245,
                    2489073135105153656,
                ])),
            ],
            vec![
                F::from(ark_ff::BigInteger256::new([
                    7315884248284864345,
                    5619624932463715201,
                    3782037607853547821,
                    2399927637432777104,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    15312329385585645429,
                    13054201868075709911,
                    9745131323844887752,
                    287007913559995936,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    1967588071837308177,
                    8670319748345158077,
                    14897202488163677237,
                    461481515311078882,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    16137828515815863607,
                    16560127530035257644,
                    139509858522544042,
                    2043744391968011966,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    9442872710244045907,
                    15708344040145112597,
                    2820532032535437294,
                    3403397441616476302,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    12338185360409359410,
                    17746198705745706959,
                    13202861013944123479,
                    3404868561212541545,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    1594639342482318219,
                    12553248722597356954,
                    1126455778191900522,
                    2549987587421458258,
                ])),
            ],
            vec![
                F::from(ark_ff::BigInteger256::new([
                    18118360785952425885,
                    16136824769377637253,
                    1333734583778054076,
                    380646626893909536,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    7624544546993557400,
                    17199411251012342403,
                    6458731299663020317,
                    2433600051609679438,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    6170755763370605826,
                    14884395768050930708,
                    6347735677226594043,
                    1280654330041699458,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    10878964394349317113,
                    12506196449328355979,
                    6651507829844798930,
                    2855445578075017289,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    312379730387153122,
                    16525571913233028396,
                    4342981804057939468,
                    2053578273656267938,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    918651104921902748,
                    9996897077293956179,
                    12944070593669075420,
                    1283795754112941445,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    10516805609191236687,
                    7161777600906894525,
                    12702625666763780329,
                    122417228273112542,
                ])),
            ],
        ];
        return Ok(crate::PoseidonParameters::new(
            ark,
            mds,
            FULL_ROUNDS,
            PARTIAL_ROUNDS[5],
            t.into(),
            ALPHA,
        ));
    } else if 8 == t {
        let ark = vec![
            F::from(ark_ff::BigInteger256::new([
                8064555558842848662,
                5251536035053355353,
                1413779812510791635,
                1313242253830761061,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1847484771536615171,
                1008464887008113809,
                11062142636158815061,
                3432991264316393934,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7896127538879737132,
                1419440419715321795,
                7180895920112479434,
                2438973308333035275,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11360503857688379064,
                9212072583062582103,
                15882570516963172560,
                1912911389471990008,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10683713790680945107,
                4619437470954483529,
                13219511475115122641,
                2994004266342465738,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7310703939399294819,
                11565031100737275738,
                11676481071163038342,
                1057346977772166644,
            ])),
            F::from(ark_ff::BigInteger256::new([
                514081811944481598,
                12068498034923762957,
                16198294290921877654,
                484619751767371077,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10891223295130873898,
                4582100590776282819,
                12125565230348996699,
                850295046228016635,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16280800162297202605,
                2017752585176349738,
                4232110415860190492,
                1963556959521378457,
            ])),
            F::from(ark_ff::BigInteger256::new([
                681477520434187814,
                3288153678754442519,
                16094959855289370092,
                107818747264700140,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4798334808928399977,
                1462102870572654317,
                3790596509807580580,
                932715165626169872,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3628595090702744800,
                11135973587155085820,
                4948065307482605762,
                1791496608151603958,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14486586643156521214,
                8148847753536354526,
                3388126980766815240,
                1656114916413042219,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15362470673813105775,
                10829039560684923379,
                8579920059550715140,
                1076308326292790989,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15175034257790773986,
                7185593774488231323,
                3523830700123516916,
                272194898911848186,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1069056780895992011,
                10810583363000089299,
                650963221279331008,
                448904170216899646,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4119194801852740435,
                5773466338655052134,
                15677842932181602507,
                3239914188113072915,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1654792766569343796,
                8650896974324599095,
                16498922223299517761,
                3057515869887369428,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7396365898281327937,
                14958780987734145403,
                8292378426230332333,
                2743822608471125388,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14628297871693465018,
                4262185987225190585,
                18442180634629720461,
                2390392229107619762,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14992810064725272914,
                17881035296786866812,
                11915874940369371472,
                617427140475986272,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12340478105269819568,
                12331089313325301579,
                11169013578577996109,
                1675048199607919396,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11086866482976305662,
                13924170936760122257,
                8723290394964876392,
                332581356626120229,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2460485993512965447,
                5403115318547372512,
                9999906714658326230,
                944251583259647492,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5955569062304442313,
                1453048633832364756,
                13422059563368443016,
                3279432848116638642,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7729966355661398868,
                14929365186530330397,
                3020245555793816392,
                3077993209713639394,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1248563186769560572,
                17802754865165737682,
                311610418315970282,
                2539808892190053789,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17841683165606637351,
                309045615279745583,
                2388244086969130819,
                1783788636372438515,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6510811620969226960,
                14834741988195883854,
                17107513935727200455,
                1757820947850361090,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6466679824479067090,
                12682691719114986450,
                2877531487286399179,
                1206756450073101211,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11625774119392637868,
                17158499613604542874,
                6444940427524994580,
                1602867363137473317,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13098761323895188364,
                14045580011722736395,
                9304604566789476132,
                2964958034205427499,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8999758462505376283,
                13759575224798601875,
                5633171405554299758,
                2723263800009809091,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16505883576294094745,
                4555613518103005321,
                10445050673888759370,
                1498644793322113072,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11736555180502582551,
                17859934254931357307,
                10587693970472659655,
                1739885542926662525,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10320309475474665928,
                4826585546408664468,
                11498619189693950358,
                2405772695345514330,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15881868598791674765,
                7762508152376369846,
                14958622217927130645,
                298283700441929494,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12485097659868652855,
                15275183184311717142,
                10969341428638424519,
                2470168711239421450,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1871099761296424239,
                556839854077143553,
                14596975065026995997,
                331310223415569694,
            ])),
            F::from(ark_ff::BigInteger256::new([
                223486663697887133,
                11985360554194288911,
                8249171438363543702,
                454454211661296303,
            ])),
            F::from(ark_ff::BigInteger256::new([
                543052097415346754,
                14617438699257973019,
                6291476956442434277,
                430584710131483519,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9762762054014740918,
                15394735577584227827,
                10983382922398766454,
                2318475842000987368,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10038841677960922413,
                13327647354493239245,
                16024973798878713161,
                687414350968882828,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17560846572016806439,
                13164167198093606483,
                10311987092012258784,
                3223896178549381647,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11988924940943829152,
                8423236538762028123,
                12113346776194320126,
                1013209182213280829,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11218580203906810147,
                16764912797592506430,
                3225715566501399306,
                3322707104383195839,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8178842855079545669,
                3837880545938265464,
                4852728544689111060,
                2818395655239021799,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17878014084988755627,
                13883472263202444624,
                14617416201152394615,
                711182982583230729,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11067094539173661191,
                3583362805560517494,
                10747214107991629662,
                1838256385463290641,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3519907466870276081,
                9855724201093033399,
                17075845375583518050,
                2478669600740760376,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4965004192231646923,
                14003109876592534153,
                10265749510655375296,
                148473067981286809,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12197362493596387778,
                16982036423460589655,
                12590987151484049296,
                2030635648156197754,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15694446906631788318,
                11231435819636528880,
                6213613039898808551,
                2615241823221664307,
            ])),
            F::from(ark_ff::BigInteger256::new([
                576014988721947085,
                10126637456297254823,
                14278236134041464757,
                1190650727991655793,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5939455369775716801,
                15866882729035573682,
                5826460159645384432,
                3381177329454998086,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14200686329356976842,
                3107067808440433678,
                5405139960882706101,
                1768708579687022690,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5561166453168970401,
                6047444374069899296,
                9499384876535229865,
                424851326707294924,
            ])),
            F::from(ark_ff::BigInteger256::new([
                356632210115689238,
                5136929925079349903,
                15372886195467007793,
                2689622437093335326,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1141098740640818781,
                3122971921191712044,
                9980759026718713406,
                2085843778928340583,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14586636561220904417,
                15236076605203855024,
                2992845226479967601,
                1183085781622945258,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13027776771118467606,
                16097914668682648979,
                14405056626310616597,
                835292704662080971,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11997117902878063604,
                3867758020569398812,
                321557366772987595,
                3375043281756243685,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3374098961315828941,
                9022002632359695327,
                12041252002858125099,
                2577803649789514992,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15474617059900954595,
                17340242011339614952,
                4670560847302204548,
                2881641821808519811,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12650292126364104308,
                13108250165310894207,
                10391015043648846811,
                3252912074387498350,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17931569852107692279,
                377801530267836141,
                14873965851602808416,
                328009353457505413,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13651953347057000338,
                6908893144450409045,
                1590618382031637420,
                1012218102623459805,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17929994425390854066,
                8439153933916071242,
                6059280354066508032,
                2525785038609638885,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6006785407242851210,
                11499235556487336845,
                1865347763824025325,
                216014075393117435,
            ])),
            F::from(ark_ff::BigInteger256::new([
                108070073131950449,
                9589225175081371298,
                9869259412906734211,
                1624005369101921043,
            ])),
            F::from(ark_ff::BigInteger256::new([
                18142585414420539341,
                1893485547048914954,
                16745279803745036451,
                2922146159405318736,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15532726034589582273,
                12599059850521182312,
                12023260793344727676,
                2565188345971817026,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3949547454121688036,
                12409718437657735778,
                18216258470975633644,
                1571847369914402054,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14704708221040345830,
                7539833170638358537,
                14054591756208795086,
                1483086592374029718,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11826463148372591932,
                9494627266632275389,
                546198172495899794,
                828678791707464115,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12708978184335137968,
                18290790611700318303,
                1277588263239797143,
                841081675338732440,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4481846931625703534,
                3155090045393185840,
                7472733739889834132,
                1698500863299448544,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16917593666729230861,
                3767821131631817941,
                1200383220182826622,
                686139002996655577,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13258215934794570750,
                17567041869603234335,
                6314421355290983985,
                3177577291061170810,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17687983128137387888,
                9350298706642399358,
                12901700069552499279,
                802573312273944396,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15955891046131767738,
                16047763058896241163,
                4514794719585163228,
                3296724819775993635,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17042158854391415214,
                550145819043898270,
                12826010697118399280,
                2827568154561757011,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3286385921500115194,
                8457480524616587224,
                14257675659096582444,
                2700339803320375200,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3085978529080959512,
                7297034175476175523,
                11962393720895799904,
                2395088055802749071,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3966629279009700490,
                16501857651006461107,
                18092146133290643085,
                550428610350136438,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8083272633549995649,
                9898165869250710729,
                12181408895135640474,
                1787501610016762275,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2538110700906713098,
                16118259000885357521,
                1789031215659281122,
                1311126801300070273,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16316247641931360414,
                12119490622778690509,
                15779967142324244406,
                665703794767978749,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12239774926031492294,
                13780392486959789645,
                8314968164420921308,
                2968357216728835513,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7320197041947678244,
                9353487905235733412,
                18171627602358424196,
                2257211740029221058,
            ])),
            F::from(ark_ff::BigInteger256::new([
                18093805024190460004,
                6081805140970192891,
                13810205453891465265,
                2672828595647475310,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2829547929780586986,
                10486377015607073407,
                16910242626266246015,
                64244230397923307,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13301666946915758969,
                4534292612637662687,
                2960405952596525821,
                1759711849223048925,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12437463687032598196,
                4789977269519445876,
                16249471989577867689,
                419444097874806360,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17713676852013887693,
                4238664485540094756,
                14589551139257494642,
                2233906967655703403,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14055101733473122807,
                8763333142004561612,
                10375474642839756775,
                2923184990004346951,
            ])),
            F::from(ark_ff::BigInteger256::new([
                18288418164784016763,
                7936021226203644573,
                11161596213134035133,
                1276022589556882393,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16532634846012170903,
                12544889627498234909,
                4133090910767493926,
                523742713751904495,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1245668545614981635,
                15169076354968446230,
                12782899551101450564,
                3169178595904391561,
            ])),
            F::from(ark_ff::BigInteger256::new([
                18261334380676656562,
                3196756009351831054,
                17736315621555506274,
                3469746956806161102,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14843451804839171234,
                14644514653420938839,
                6574887294837397337,
                2220474610147322635,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3108148613075112015,
                5404118277375980664,
                3078584076969851518,
                2752959337350844043,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10057949843024700529,
                16288424449981295196,
                1476296117355446885,
                1842056381262076659,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12432656892797494180,
                760443185620423215,
                3067877267896825944,
                3434072299793417453,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2537863911050424229,
                3340428679177033613,
                11206941616520830468,
                2773103887729962224,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2686307465716542509,
                6361820990008005201,
                690298192068358867,
                3392622736068146516,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11327113520130960140,
                6030985490764779019,
                3638895253243584010,
                2324308028335256948,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4111006157683391695,
                7027764894025288253,
                8650928313101022671,
                722956747677963302,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14767700359160010411,
                369972766555418949,
                6678909827132477890,
                3384939472696532819,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14882000616404902735,
                3045423674893518772,
                12070516699021435019,
                3068862237330667499,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13892983689808302018,
                8959050735089567479,
                9586967431926083116,
                302466741558162964,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12477660093441621499,
                340048931189706134,
                687293342237083957,
                2126227762516663283,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17633678092583353827,
                11562083724968013330,
                12645676352508805,
                1934805215714119064,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9415695586054696853,
                11867153695650196673,
                11705782399206437529,
                1851835034695103757,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4568899100169083288,
                8819036721142715142,
                5260701375159704705,
                1381903528661807744,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6429249067702237476,
                9574274069500104525,
                1330446388119945127,
                2967489451964728890,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3452980380391165572,
                8104776184250765359,
                9050122485639420024,
                183798066166476999,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11813679932308427187,
                16953049193297568307,
                17997211343672616756,
                1274474017360925666,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9528508263650336470,
                5665811689432413223,
                3540037128013633833,
                1922830252936239941,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8673276876743190632,
                11472891069990813408,
                4721405705016707559,
                1863093726431705886,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9334184985202052093,
                3247618201186416099,
                6560500690738188269,
                188223044855970575,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1188417056314699938,
                15573392172827837154,
                14982163004059988432,
                2918859857355502832,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16594199016453813851,
                2158014049545487528,
                488467363118740618,
                443781556101453202,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12665885897230483384,
                15130914050116589041,
                6281433846212546868,
                2478514249085671924,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9009650107962045988,
                7132848629635088478,
                1496467270008337472,
                27907757058307623,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12426233986374950298,
                7995096267223317957,
                6725818618791865175,
                1665598044217489716,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17131416400902334674,
                7741207602472959214,
                6439450253178646808,
                2010909164553492553,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16388129843467232143,
                12164894549625687527,
                6955450295080513197,
                1533213707544444116,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6609269301930384600,
                14471357168632042957,
                18014707972319426249,
                1868120664560815111,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3045482174633159958,
                7349585945699641542,
                2615673023855362370,
                2798448603362532448,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6463226672402392239,
                6371711152156034322,
                16476870464482480237,
                2935086912323868804,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16006525252399893728,
                14911188681839454913,
                2181784535744644390,
                2773314404643664279,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7056266068942838149,
                10298024529854735865,
                90715132774576397,
                3344846763278438616,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10252508128984496192,
                17536417096141106552,
                14181242142765945679,
                1658327570923709307,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4294786302332845655,
                14364939531072036084,
                12292866199470054189,
                2397135292697364422,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12331904657341993636,
                17496473039611389879,
                8641392492978764893,
                722617042039812332,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14641598288799237706,
                11963275714992776024,
                14754632493518532304,
                3023745429167072309,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1693553385078563014,
                362611327697407779,
                1110445707026452588,
                2623510567345703267,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16528797502248704615,
                13842137775960556861,
                5554626747591509139,
                297871451259299703,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7620266952546298657,
                3910758634614193875,
                2967609883265855950,
                980042986662206973,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14179467566572339546,
                3726539564775813565,
                1092841693817478143,
                2289904639840855260,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8106021335494864115,
                6566272880693209957,
                3865057676655555210,
                3413975218890687591,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11448471359773560448,
                3919196654248531692,
                17693698279066383173,
                1885729548656418900,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11929573471318559565,
                2141416915347353056,
                11121327546514252188,
                723613105644283543,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5330755266398551938,
                2514976715985238281,
                6159944923462904968,
                2750049442592198498,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3098499334240478265,
                8871141984840354618,
                1452785116873863557,
                1850234562871674007,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7263681491098310090,
                17223941220614611562,
                9779836558476177548,
                2746689314365882510,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15189731353062063961,
                5639734445615460578,
                10366286184391220694,
                476473899396554449,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17234570702213767880,
                14478537483360504230,
                3134155076263640824,
                1598251791878528876,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17121732872933140915,
                4776348422407164194,
                7528216864226947948,
                1336798059019163797,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9002289181358285801,
                14288930140392410728,
                14093775083718207932,
                1362683169695014675,
            ])),
            F::from(ark_ff::BigInteger256::new([
                18239458085590647152,
                11226126153962709074,
                60901927505733456,
                2155404411322276380,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6103543967268392717,
                2945485124432633753,
                10035383342872917256,
                2306629754739987422,
            ])),
            F::from(ark_ff::BigInteger256::new([
                18161190744556426801,
                12782509438719821260,
                118156576775409631,
                2951799146229712797,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8589349627051010859,
                11390916982900496499,
                3412623331312744920,
                1867776364758662951,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13240706497197402529,
                12899615291550110952,
                16333660570069644124,
                422572858284993685,
            ])),
            F::from(ark_ff::BigInteger256::new([
                523426106177077447,
                6779732351737397708,
                3150969923154156162,
                2624406438208063918,
            ])),
            F::from(ark_ff::BigInteger256::new([
                18006514025020568494,
                6633498383936335084,
                1084513794909560961,
                1981269742959961858,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6416528873480069345,
                5962114221802637848,
                1594590773016411290,
                2080750319763542323,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13197551674061505739,
                6736506586591081489,
                12977920148591019332,
                3331086142763783048,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10722116439923019707,
                2554133115904186163,
                3418501460062216791,
                2274619211902688941,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6561489339836455206,
                13616386684171990415,
                1000904664331796314,
                1227270494710569434,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17060881999252087286,
                3647592367521103644,
                17255431945984181825,
                258440659595439349,
            ])),
            F::from(ark_ff::BigInteger256::new([
                18176238096920053306,
                1448948119688240531,
                11253208047693780814,
                749727352988468083,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4878616946809744988,
                6864829317252628239,
                15713709400500273443,
                2026894422781948063,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7491027594340495293,
                904360801159444958,
                5132181010789563819,
                1232496932512288509,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10474548506203227828,
                18314045001152751762,
                9430054086395017680,
                31080483172437497,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15445237511833517956,
                13617984051099778525,
                16331715104037760985,
                3007377431056537234,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4023010917887512285,
                2615513892206487386,
                678952275869620595,
                2746372978363805488,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15284265216836946940,
                6887578799690891008,
                17086277121998014833,
                540306140458637879,
            ])),
            F::from(ark_ff::BigInteger256::new([
                747767737204260302,
                2343351978108115778,
                3829888495440991555,
                1619318928583230146,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9474617197999425371,
                11489522059793445488,
                18236282538516180368,
                1902124457552689414,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12984846445247465385,
                1883543706237570880,
                7264743509503403056,
                2761649673628120946,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14299251852402091038,
                10289680010831273565,
                11841708072323592656,
                3356392151203526469,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3364373271557841306,
                6216014668416314093,
                5354164595761690015,
                2874143573521891937,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10136279729557209026,
                10983812543730159395,
                4892570099148325249,
                979613049415056525,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14819188614315539573,
                7676298723824496440,
                12697299706819544393,
                1928572910500899859,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10600642797667132335,
                962901465767419759,
                6905834531004620797,
                905506679294193401,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6520323083490434644,
                122024477649403220,
                17502160351465739599,
                2633307977817410908,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9623183007597226334,
                16088664772076713219,
                13708573949719768273,
                1905580807952753584,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17361058977327184049,
                1667507162390737028,
                11849757658507189242,
                1558956422969150722,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10181964012123796808,
                4917844520451253307,
                4677777560761297586,
                2887864022721849669,
            ])),
            F::from(ark_ff::BigInteger256::new([
                18170327303854615632,
                6694953113187507784,
                9534692706429551573,
                3074103694866955294,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15665928970207548292,
                10998230150312211431,
                2692022490142234844,
                1083464490175760909,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2794555160142283389,
                636281555569161110,
                14950963855499388142,
                472239991541888869,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7493052147491101422,
                12172526254288745834,
                16817212186535512936,
                2148439909943251998,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9175476770681730935,
                4245030557785388941,
                5795848414982206166,
                3001745842172231467,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16348488069772819507,
                13585154236936909239,
                1007128934114407953,
                2342425346975052691,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14766741098940691067,
                6456937015392220616,
                7245369348402300632,
                3380575262883912224,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10928610657171334586,
                7330805957780937761,
                7886248396626099777,
                1881212431008600862,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9286578448391556907,
                1739155211464603932,
                16641512756331801521,
                2157188024622545109,
            ])),
            F::from(ark_ff::BigInteger256::new([
                18286697379034460745,
                12435568923987037739,
                12006075611756412493,
                1154061444713593548,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1158258499861741376,
                6622947638979014327,
                9228386648381394802,
                2363941148571089816,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3856696268403567239,
                5890805903307762916,
                8786802232253788709,
                2319016604981829148,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9768984447327602813,
                2608490323782163809,
                9932499227426091203,
                1223753194821111481,
            ])),
            F::from(ark_ff::BigInteger256::new([
                18004645452849929397,
                4346108314834248640,
                15478025287499411869,
                173398717012014685,
            ])),
            F::from(ark_ff::BigInteger256::new([
                397370622073535739,
                7065442607085872382,
                12608368876040307028,
                1165283633214033982,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5395830180100376098,
                9993508291783845727,
                2792941771712187377,
                635040195916254490,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15745551467521609744,
                7757312595960243003,
                3249758542448050486,
                1238461113057885229,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3576554289858504549,
                14346128637903019243,
                17069368585829441277,
                452759047042478054,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5390996468739659910,
                14991432916833058989,
                16662986601166142023,
                846047197990306454,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7800915361364674695,
                3783038441345736555,
                15069400494297775374,
                837205221534492813,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1660954512539201719,
                3283801255608497680,
                11173395239319718492,
                746088034569260560,
            ])),
            F::from(ark_ff::BigInteger256::new([
                926980872728115819,
                6414228798579952748,
                4163409901606525067,
                226197376251990670,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6825066592735300765,
                8290227955490385807,
                11200203397341753132,
                1271867055987065461,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9163311209152025374,
                8400741090666046337,
                2774285686940358032,
                887261841927480967,
            ])),
            F::from(ark_ff::BigInteger256::new([
                18155438597129146750,
                13396917552873510070,
                1027914131237753563,
                2462797925655962292,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1691108004003037193,
                15325762766150783393,
                1986504219044339806,
                2159755356778942871,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6179208418995938869,
                17994852179670878588,
                3980930392380786384,
                2462264107588765107,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14627079280155577616,
                12394753390601481432,
                16486690101106247,
                544983848125593882,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4106588534275284318,
                4654751173668668119,
                5290654542916055422,
                3202181404966946538,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12285920927869798133,
                17461179428110942848,
                7644881672896169622,
                409306494563967556,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1280793876055849277,
                3726536249657746768,
                11642840712269519347,
                1760593863566449702,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17265008406728981777,
                5351256155867600358,
                14764728346960043582,
                2029635695796609970,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7999712498255363669,
                8968082998730218589,
                14063844162815711737,
                3445673418631049577,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9245617415808456480,
                16213150986467437683,
                12364431355146553026,
                2602989829551719448,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8143404817988938287,
                810840810822560139,
                5180846065724213911,
                6392239354015343,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12852694255155070975,
                9618717468672613132,
                1604885552208851952,
                2825024452368615,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13316704230779193789,
                13390027614190090729,
                6583555414731542664,
                3476545404872290626,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8045961033821808231,
                1602272952733229573,
                5660293391926419481,
                1482884193603645385,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4203393250105802730,
                338432052733598440,
                14186810035508377885,
                224295671982255191,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1948701346467480728,
                2942839011129778901,
                9929932600255678073,
                898606659850408811,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8943667938877770509,
                10140230960901289797,
                1705129068565953784,
                801837634576929326,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2729073828165305183,
                782639897339124478,
                13928026061505266516,
                1285156331304618438,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11608133527345340927,
                1020534490887696810,
                14463261096134876067,
                599595152619685176,
            ])),
            F::from(ark_ff::BigInteger256::new([
                18262383883066695754,
                5767038567197348499,
                14870766456721180528,
                2698665424342528973,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6992484046423900087,
                11507845106594511153,
                185596845700317079,
                769613812633064482,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14158995351042663492,
                16055400926131413620,
                18368961649888485269,
                2023898160931994839,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17175726950573512160,
                4453267142280704149,
                10932943516861208962,
                1477405307582405116,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14446297066395235925,
                11765539307351615820,
                8036765253212194946,
                1761206897384458514,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5075521165076211917,
                14669964543529756233,
                8107388051667796687,
                1052794709857631132,
            ])),
            F::from(ark_ff::BigInteger256::new([
                260009346778232060,
                774893818358111600,
                11138012118131005831,
                3327659627432048322,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7727559939119462307,
                10345307507790862897,
                3522800991546807079,
                2118440729469708128,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14375452826759766147,
                5210241350593295052,
                13861947423061367040,
                3021793315816632813,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2138492564611326941,
                15431297565441368950,
                2213367240295320502,
                2378938869035169617,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4080512527024339299,
                8150321078664228606,
                7102800469357819452,
                995777904831087259,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4098778506716869147,
                9803060615741166349,
                13382229876492344898,
                2209944153364546954,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7635700127506399266,
                16218823171660863252,
                11213057407814240748,
                12976708535581842,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6964753597724172049,
                13531301875288740740,
                5859420452278582470,
                1302888578352763256,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10449268062512225592,
                10547650650942016077,
                3559208295652143517,
                3321208826006337040,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17798197324369019957,
                18403001101694733804,
                17385823513505863164,
                2356279026029828094,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10219482759860263566,
                11047879333501095675,
                10468207697187286983,
                3023284544967051060,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14182517972247259888,
                9796025882802670771,
                7275548535315247388,
                2673308719703756353,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11987787434820644033,
                17451078413112176449,
                12310605775386651136,
                3215891169736515150,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2464505402385175143,
                11746128532722837627,
                8111963549871552950,
                1080536606783283979,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9060499860695412287,
                17607401991628173979,
                13674343945832685742,
                1432740324706243878,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5069210523329779745,
                10157235040911608633,
                10020621172826477950,
                3180815500622806069,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11814602340488777067,
                4889015533154435784,
                13551859371517977964,
                3213219196510214467,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4630156022182153461,
                8134337016292702080,
                782509698579901975,
                952641820904821816,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8712415434846738041,
                10776130300854024002,
                10347833027835974734,
                1312338032784928037,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16960993998609966687,
                6407467663006028330,
                6056092207014737994,
                1972635241997670467,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15585107912221795380,
                16043574298040523122,
                15621030187575286115,
                249826389804240954,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9092298755846534754,
                12959218870129871410,
                1313023018257117511,
                3379707282334058398,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14889723137570680652,
                10565507108971408460,
                14480667425923542471,
                1225556365905202809,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8169318000177926031,
                12557849053719050988,
                4818517366339792270,
                2165829075005037372,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2656267963586904589,
                3640797971054611756,
                3264367795519621548,
                3056840064166683500,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8428823956387562560,
                10529827187765920069,
                13021851649765262032,
                2863689985957614488,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14997503661585028515,
                14535650078033450387,
                7866515711185299948,
                3320272003853087374,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1519561068655557478,
                11530443477003386154,
                7531631218287034777,
                1196536070513366549,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5971083021658370074,
                4709327601281004693,
                16999951857004802823,
                862335019614851598,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4041089902654850407,
                9450552441703388838,
                1338375109109171604,
                1114845950560245201,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16112517216229740545,
                776280315845814992,
                4777717143784135536,
                1595926848538843439,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3193098803575351138,
                12459664309990465487,
                10773081396532139241,
                1929741807960561918,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10671229855878714167,
                12827064147960425880,
                15024173321135852759,
                3258118411888973430,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12266662428341813524,
                9249236940948592600,
                17068490559517584353,
                116155031595748867,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3065485485414435400,
                15318580886066457682,
                17296384263953607798,
                1095182442545941728,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12155209368691721957,
                9681588708627083144,
                13139896658361010584,
                3235223101869021855,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1301128099178629771,
                11699890247706009475,
                9054260948585062759,
                1395369375817141873,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16707533872655152787,
                14354741780823551871,
                5273207982106960634,
                2106815369272860308,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3488942657022140850,
                17772583266897901737,
                9045042190091269223,
                1054864729599419391,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13016664056305039909,
                10249496345298472696,
                401102874210493193,
                2220428887460335940,
            ])),
            F::from(ark_ff::BigInteger256::new([
                815970372982245101,
                16266197696399077783,
                7004724551971672315,
                783451700731203202,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6249096012806480427,
                7632364866441501123,
                11790793030378174008,
                2597523970719635902,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6750619287992519636,
                15404292149084069859,
                10333223358588927843,
                2657030812237585392,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11492308695916458124,
                14848786925342559582,
                15779336467857866706,
                1589059972595566952,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15384110561975940186,
                17808851016935320201,
                8747166181785994687,
                1974020714442902038,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2835887832178324305,
                8644368805645349313,
                12170244623954067770,
                2608995660165132052,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16445194634528925228,
                8658885919573655950,
                14614104174457576879,
                487142233373271412,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2788585380645573441,
                16198971429774004288,
                6309524975787954119,
                2396872020286009672,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9594214629253213668,
                2495889382733809166,
                2787236898905296438,
                2141494164520753202,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12015309340656549647,
                18060428884499309209,
                12613704447995427787,
                3013679716244221583,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17869214069894585480,
                12133828569049093410,
                11497000312338454847,
                1813085271538826893,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7122673283267013800,
                6409410659015185389,
                16642648356512637314,
                860211125502921704,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15026089514498044480,
                4220300769568485656,
                5948192380146104535,
                489522001761701314,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6358024303284589398,
                11554141930308612541,
                15399879341860058698,
                1330530206831322271,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4145967979724249295,
                17954003046374867052,
                17665599475468505604,
                3033171601361000290,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7899781012908602996,
                2706761197529308541,
                2691436369423839077,
                1802815063439250119,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1405706745882910008,
                343964302078928234,
                2306248124425257912,
                1250697472640264354,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3400679234933260024,
                13553255978031948761,
                7757751035863130482,
                30215165241592254,
            ])),
            F::from(ark_ff::BigInteger256::new([
                788381103360779726,
                8418648184437042147,
                10614366122511387577,
                1424949814840480594,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13324871345516663973,
                4860281116953094517,
                8339009438594861349,
                1133723579753382855,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14150382122570714108,
                7796733487198501353,
                7929478308568022441,
                404002311572892598,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12018110616448147966,
                4208972445721017943,
                1124159937861090985,
                2871143613378370008,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11206360775565170319,
                11018560411888590292,
                14337862479036938597,
                2443993142959868738,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9997615627854127783,
                3703071105728495871,
                431754954086863490,
                468756416765663005,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8172860836484058223,
                5846089443636973070,
                11803252973846655061,
                479484731271670709,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3215543352968516807,
                7112889101234271138,
                9124310473764075166,
                1512822111451203109,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13116648708709383617,
                10365131591068574333,
                2877215107698233100,
                649369230685509454,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17548428387491644485,
                18087748396314869242,
                13383160205288605323,
                3234215634603481204,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6297090250008960213,
                14640204674775917695,
                15642246039286385445,
                1950016092064704372,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8690287764314657450,
                8386156733530708980,
                6872398574624416615,
                435643097216596558,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5577746594727261238,
                17675966258764324026,
                4894810287933986231,
                304855838221690904,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5752799561335280909,
                9875884128380808351,
                16475034635829406952,
                1326086791960963578,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17356200897631074279,
                3565203452210017342,
                14855040921890216372,
                2805130542419028658,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6774463393993007202,
                12203149614837963452,
                14891262074264534106,
                2860390377360305974,
            ])),
            F::from(ark_ff::BigInteger256::new([
                18041688927926294746,
                13641714784579754811,
                1417620205829720988,
                1225667977894384067,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4078873017850937665,
                14396046052821639418,
                6823454631581765136,
                1740479284469679831,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1749662183019174518,
                14289784582387809518,
                11550069692489172623,
                2666931281304803287,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4438791450151885356,
                13541790698294327852,
                7085403907828499985,
                482461212508282244,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11653775837428771866,
                15094604350377307175,
                2837256757343169499,
                448869714194379282,
            ])),
            F::from(ark_ff::BigInteger256::new([
                462301717443144083,
                794794395614465492,
                6971311861971732146,
                447876549610006948,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14204994713465011424,
                4752751548185934526,
                14933997681952211110,
                1099393859493157489,
            ])),
            F::from(ark_ff::BigInteger256::new([
                196893282834388851,
                7714310709571216056,
                2576570985723018198,
                1085061313843018162,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10292510783365927450,
                9224681168408978097,
                4138253025641778349,
                134877860082106113,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1107554609241528716,
                2514138698974810025,
                11935690832284518143,
                2080763930921650307,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12596656162265685421,
                7432432925455791660,
                15867369561009486164,
                251955046186204511,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5398996700091213031,
                17788663034659397282,
                3005880141078975455,
                2005973288874121459,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14941434161022183942,
                14551591905454973665,
                8065492465945092796,
                1135722737431080916,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9342739352899516391,
                17090930463175731031,
                12753615424089549049,
                2832663989134982904,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9239742949395878102,
                3864699100067195439,
                9434041268990669657,
                1818031232202931430,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4766654518125566069,
                17821945475844315275,
                12514766486180834331,
                1041109138522068081,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2613149496349988372,
                1909052893598592154,
                9468903327971682731,
                2050686261977532444,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3337186034215077389,
                5639127094258346925,
                10943920630424194975,
                612559273154448435,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13058265797257529318,
                18155263704467026477,
                3052829469071700373,
                3247681289825779990,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9722825464081052296,
                2178157866443095076,
                4909507845512957022,
                902323337591215952,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8200188785067922918,
                144943032325107519,
                10502300888888278933,
                3326459282632421559,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10645501245262283124,
                5779536622513823449,
                12724766468728710976,
                1380384710562217562,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9130080106689011205,
                2169988219692595226,
                4762005529927077402,
                2423518518150396676,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17055155988671105150,
                10031261165748866260,
                18073412870112812428,
                603185376694279119,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6349678928168986457,
                5426909503588830916,
                272707893892213269,
                3129853778917891751,
            ])),
            F::from(ark_ff::BigInteger256::new([
                477320899911869389,
                2525281461629421736,
                1656004343097559463,
                2295235423445429854,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9723014048137465620,
                9422501151221768687,
                15257686607981128322,
                203271370592896848,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3361827148284492325,
                3022798929111854841,
                11219497519685114534,
                874249134678083495,
            ])),
            F::from(ark_ff::BigInteger256::new([
                51187422066492420,
                3455432420943656321,
                4237177607694674750,
                1977677003899240904,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17659123795733206559,
                5929487739677792455,
                9334227856966020670,
                2876848665878519525,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6964637684230205048,
                13358240065917080999,
                503735549030957421,
                2220011146733021845,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15094601954494842573,
                6291586790985377632,
                7611200051637831833,
                2725859608650186844,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2021551406375127588,
                13578174050204947838,
                3030787844439910390,
                1702618296947442106,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10533223966808605259,
                2570149191179763564,
                11190815138618888178,
                203590279079908988,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2895460097043961546,
                10729769856624203833,
                11988602204382494370,
                148154681686427985,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10480851902355023483,
                81561607612530222,
                18440610348278224386,
                2638155576801221536,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1499127337400424922,
                8599739963837114656,
                1874294800458379155,
                137619650118163811,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6981744595166781436,
                11005509661438360428,
                8887037439841143592,
                69587939422891630,
            ])),
            F::from(ark_ff::BigInteger256::new([
                352431351971817070,
                7952273555655683573,
                8654488110594747478,
                1009209867983911005,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12905078718602838821,
                7550360179860434324,
                5456346838403142709,
                2833450821559901334,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5750676776469914765,
                7263237052223587527,
                12403377042538541230,
                68057844582886584,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17943315570695741683,
                7372390397443356535,
                8133660475701207264,
                2408070694588974581,
            ])),
            F::from(ark_ff::BigInteger256::new([
                213677326277048398,
                7056087262578023264,
                9931214106565512682,
                1914572970468100902,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15455669169356865156,
                16518628783438126416,
                4787665621588996527,
                3288529528023524018,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9548274516907435185,
                3096978220227727504,
                14623796483378652599,
                3024057778263673199,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6872939112516944695,
                4993161936793541062,
                948422444147268561,
                367397868004547058,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14258640991807531165,
                11433746286252265641,
                4063910887478041108,
                1124852471008296494,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9856619435465512762,
                15797826332324628747,
                16455849123735183185,
                1581281414741884435,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4738900681298868064,
                5257403450868078745,
                958080375215966529,
                2236101812716968624,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8846096136434054270,
                12252090672012354481,
                9614445505205821079,
                1514878874792264071,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3758615611950696811,
                16872811311718329153,
                3439052187673686854,
                1090020013943755902,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16102393811801694428,
                2209361904990098850,
                2387169973205962145,
                1627718309143607753,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11702576643946560209,
                9201980401809472277,
                856646220832583077,
                2902674703605568315,
            ])),
            F::from(ark_ff::BigInteger256::new([
                363850499013400901,
                11615776164419023003,
                18150383460132775633,
                2846349111163167841,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10773725290240359846,
                4029176959670628919,
                3166671683990402480,
                2818875561667146463,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5209063392839064673,
                13720279059978581643,
                13367699488299490250,
                3022401656928197356,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5495676888181387747,
                8020119292029000127,
                15777080878312303526,
                2976682296982979135,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15095126220512869655,
                17282701965452625381,
                1302644046336053187,
                1471359979654531352,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16907170501826573282,
                17209920375652413287,
                14832629075144318717,
                985737434948569871,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7784365027204782274,
                1377501094266557223,
                17579562473858804684,
                2739539650817231440,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6693587171640327071,
                12530676293785828941,
                15264156469279450109,
                2914868674690037910,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16415789780399902216,
                6860010130371378672,
                18034653713821702891,
                793988754384932779,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8306809787134048072,
                17767108554972294843,
                13907492721776008118,
                362717434560010302,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8210479996304252651,
                4037693943538634301,
                7045187946038317469,
                659264949380096188,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7393303094185150428,
                3076800664174790947,
                18014499816513308543,
                929661661859289993,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3951496759569407044,
                15501789230693486721,
                7555845023005394745,
                3344318939538130178,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16131292824034342946,
                8902128688295173610,
                6322153288040003834,
                178168428786258230,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16463214499984276046,
                12239682325007752805,
                6330073430690316560,
                1646779430967014734,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2164438280976189770,
                11035969467355250278,
                5762359715261578245,
                1555805183637927045,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9199637625119612282,
                12799632786091432253,
                5554838975820240852,
                2455247279992882618,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14830065196002091079,
                6768359882317875857,
                7615272332363937068,
                3301289016126100177,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3626310467709678351,
                1575957452908229936,
                16354549283416718083,
                2642278918912581967,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17872770958359113255,
                496215632938895482,
                1037116204989583636,
                3414967551438079514,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17894545077764179072,
                16964573999254098761,
                2215927049732059632,
                2258703627970988145,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11253251119025877664,
                6731282963738361066,
                6323055657384553182,
                412700131374757962,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5240822181796082497,
                15594769706793243728,
                14651776741503515645,
                2859476226101147523,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17609189074422584815,
                7993816068343794262,
                17765690403642916641,
                1105615843043022823,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3055413940205473519,
                15506858662540958488,
                8432665339192978579,
                2104847105010179543,
            ])),
            F::from(ark_ff::BigInteger256::new([
                18182014937144578559,
                12067703225255329726,
                3153856079428535477,
                1775315885652531824,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7063860859822026017,
                741518636060491011,
                13205182858872855688,
                795700559528244288,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14826325960646419185,
                2860138377732527297,
                9016663903460837104,
                1240355729031078597,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17368987645036016300,
                6420294500696423311,
                9293616224217727616,
                2436101235933358719,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14719426834923704878,
                4903558825462837385,
                11174910018623493262,
                2903242206091043671,
            ])),
            F::from(ark_ff::BigInteger256::new([
                18207017845105931482,
                8535600061118916283,
                2747050947431195277,
                2110511169569313219,
            ])),
            F::from(ark_ff::BigInteger256::new([
                18113087314832581199,
                14418619525694709124,
                8137189420354206346,
                292910333886894332,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7584234744511541007,
                5910473126996604855,
                5360881604024597919,
                3464486038347799576,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13890151645712825209,
                3190453895915589133,
                13673040089854070110,
                249992730108221771,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2979804812860971411,
                15865125056271284324,
                6790160862702738537,
                2974136873359931020,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17723811473920092848,
                9743734841478150518,
                715772923166778808,
                2022953596440669656,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7749888671131524675,
                18271435785355849333,
                8962368233220902047,
                915032581119916257,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10398633612087432389,
                11517292728180323667,
                15162128175851032256,
                2806304912523712955,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8940265879112469629,
                1257868483826047344,
                7890497356329044846,
                697768367001404087,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11776392133759949940,
                14316948078160759795,
                3997344648605560947,
                2010871162517896471,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10485550531629056874,
                985595033991695325,
                2582641536381278275,
                2627542228926407741,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16201056239912206857,
                15116865924403925771,
                1022602640003304125,
                1832244304067292620,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9991373256947432740,
                13312916255814381021,
                10416073277866219689,
                2100406439049594115,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9520524115736228187,
                4413734350796341940,
                1810194726437713586,
                3035954177513966579,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7356907042055760601,
                14678266320196955438,
                6569609621036264352,
                188135751582592598,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3426670790253065699,
                4571302557330629869,
                7784905020283916106,
                2585513762332670898,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8744281957382291261,
                13489672272454914629,
                15717332016431531340,
                1370810123892779243,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2439018329875796658,
                5914992126319833282,
                771514910723552430,
                1332238101724024770,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6788360189276524950,
                17346249775103938502,
                1954423684347273565,
                2612536778863556008,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5320326189092008246,
                4542432424478285365,
                5387700639846815411,
                3312350157027218516,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14596764793921206592,
                17265292175471292794,
                5319625115789918262,
                1111356773801845477,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6726727460034174172,
                12102005536328517966,
                574346221730339607,
                2802026896790379624,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11081990439627939064,
                6301047564352226515,
                16487974839524966101,
                2049669154520894093,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2249936660899956655,
                18386825612456214619,
                7821464339699479278,
                3039660738487550838,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3762325228656800714,
                12642989235772808715,
                9213786270405737691,
                872274169524790538,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15292978373504917477,
                3607951309955232422,
                12618169474792907517,
                111428006349167680,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17749250865510477574,
                2267446279182852702,
                4463360742380737929,
                2500456952290249948,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2196515251405858974,
                5574973289733174708,
                2319624101552257076,
                1217827719262488494,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2499713989763609938,
                2340194877391999494,
                13536932633575644577,
                395731070661922987,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10499124980435561507,
                3992492643280368359,
                5114743024173511313,
                1347275673440445176,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15066190697199262450,
                13182974668765903859,
                15079834695057314223,
                805155283131712418,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8694420845390088341,
                16733896765025524534,
                16772273853240666290,
                827983403420282016,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2084931115380808235,
                15192884216009723430,
                9756717617138582088,
                1417238052251315330,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7662335772641192313,
                6027438619345676586,
                17757007602607482497,
                3306582885272306535,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9263811506206809966,
                1604062835499698633,
                12862771155872310819,
                1990609412762080568,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9576021416820677036,
                1347042417102393381,
                13537867142006263918,
                2810209508471179598,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4376835729300179874,
                14628176825024782731,
                13070449851876700353,
                1296779517097765563,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13895259360114659700,
                12967746919152986850,
                16761477747204290485,
                2741626437207375610,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11045255814554158331,
                17865994290311843570,
                4039263955464449940,
                417296022389185887,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2844666303884616837,
                17012633925977028603,
                12239927766659555427,
                1779910494588206817,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7451638890822603625,
                3465678081216116784,
                14213615452193669859,
                2681094366354395619,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10738057681220745022,
                5766926360828946539,
                17469004196140248130,
                94438338855564346,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2467935790546370212,
                16762011378796132874,
                1942548299283041861,
                2686650909202343785,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12063295435450481617,
                2081017415159007105,
                6603525353971155365,
                161792767726726100,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9615374832016233966,
                2516411049401560022,
                13276852707297306090,
                1590258137111365462,
            ])),
            F::from(ark_ff::BigInteger256::new([
                327656385670465453,
                13199220006516680243,
                18394494262354184090,
                862265091565892400,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3271943296847264973,
                8459501701982929973,
                18242383200860529848,
                1569662400039473287,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4616409522824595042,
                9835163973273646767,
                10876432045932592879,
                961748488438895919,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13311145638086081724,
                4926462466258518511,
                358632476509588581,
                290181792323609067,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2220848207261086052,
                15489160061154057926,
                12564506696672666395,
                3434151782930388153,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8986917071505136681,
                4588134339421930028,
                5846341988679127988,
                104876245235995594,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12390658710969490018,
                13199870405572487424,
                4911695471028181298,
                96038370577804787,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8008330317909289674,
                13853013751962850271,
                547752355207993309,
                183374951091020345,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7983716335154610633,
                8013225941749247431,
                1764953150391740259,
                3190971787054422459,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4599781714679612381,
                11081548117827661433,
                15381594345713385977,
                2302826801473212967,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12859262867171943236,
                12665794440073816318,
                1130189572961265725,
                270696815476146948,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16162147480330106983,
                11227177865528668774,
                8004406064668964352,
                2225730455520270988,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8063379284058308628,
                1090456591515393843,
                2093130343812262041,
                1793664001269768113,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12312017965653980477,
                871561049248640837,
                16259372945616502788,
                2826347434313668064,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12794617852933455109,
                14915912248824050794,
                966456239164724634,
                576294000216646834,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2296844045398608894,
                12654272909008698070,
                4130975512909459209,
                2990869138210304632,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16153541054246043699,
                18245637904257091279,
                4149486311441995988,
                62306074825910439,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13885822875758453309,
                9455244227077716110,
                3667657221552009426,
                597859425712913764,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9061548002265442481,
                10258854917630777497,
                4555147923684324271,
                2174925254537363341,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2261953684618278263,
                10885288894305791543,
                12693798446468604026,
                916510680849250,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5649109289498793650,
                12301380089043227334,
                4861282154229373063,
                2038323733425444870,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17691239689940435758,
                18355783097334136943,
                17122868871608651588,
                293792477032453663,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15029051372639721457,
                7372480008403891389,
                2716347567919283292,
                127344146432468309,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9036326269130367625,
                1786035192506399291,
                12122243929891212809,
                2535424063182945758,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16382611874571640762,
                14973543833303751404,
                1059277556457980124,
                2077690070820785986,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1795537282391140948,
                13264809039798517637,
                7031322250055115778,
                268388580935698476,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7812121299743030250,
                17342312256457407834,
                4915213513790183001,
                1624207961055297542,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10225509662780714888,
                11146530253815705393,
                7329554229121244026,
                2184797055726986252,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14760222804337718540,
                13243894556712147515,
                16532372824304850854,
                1943965748113062253,
            ])),
            F::from(ark_ff::BigInteger256::new([
                18050814698928045514,
                17003217306099543509,
                14701545636651821768,
                1240160975862074569,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14217287016499134637,
                11279077256003181315,
                3104362581362541401,
                2960221873547789774,
            ])),
            F::from(ark_ff::BigInteger256::new([
                508917862281768950,
                5973763100047792779,
                12228100889629508900,
                2362272340220791214,
            ])),
            F::from(ark_ff::BigInteger256::new([
                290614524730576989,
                959578460592176622,
                18167143679819765167,
                986421500455656473,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13443479514932596918,
                17160794275608151738,
                15941401158182151100,
                964648937335737535,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12898675056890937434,
                7961656372404257685,
                3959465430860325417,
                3193069148393848753,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1705426853176721332,
                12799920044292041843,
                6463122493471401381,
                1687335534644649246,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1098155658829449031,
                13468924346361519304,
                4998050313619722772,
                1297877688702318731,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12600975989363334332,
                7746857580333405897,
                14285523604982627158,
                2461578074524340863,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6335034799569331947,
                1221538594227103389,
                14523744921789174562,
                3370504252253448882,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13360150777194628771,
                13872300628263040425,
                8059966213694314863,
                3412641365731144067,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17284521644861578658,
                1040093075156777692,
                4274749893400641848,
                1407546087855359893,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14730652615081307194,
                17430437153586400397,
                8291919248296094644,
                2087294345591204980,
            ])),
            F::from(ark_ff::BigInteger256::new([
                570982030265599797,
                14472303568308782846,
                536007318442359848,
                3174666275890027671,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10123490566625232490,
                16172410622044543889,
                3710973158347561136,
                2934120229297231894,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12392628464187378938,
                18279809726988346241,
                6147347438765694273,
                2109612165757295590,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15898389293385825029,
                17043197532526572066,
                1028070492793768153,
                1747924677084653381,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5515100931642494807,
                6157629370232717040,
                9619790128646524780,
                3255730324972810384,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5748995444679397525,
                4090778386843977649,
                17818811341147719601,
                108942419701163884,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9659967285430470264,
                1800951941439189898,
                18347496836449309426,
                526442745025350401,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16704334570299495581,
                5281917086442561584,
                11311162510084570903,
                2282564910329777493,
            ])),
            F::from(ark_ff::BigInteger256::new([
                154342215867597617,
                11942820598481298726,
                8898888045009137676,
                2184463876583793849,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12613564303780822619,
                10661657564617939176,
                18446027162186703959,
                202384192143235775,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4376527848234063309,
                9618478229398999388,
                16678833889962409793,
                726397070537750618,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2837382738990124353,
                14176529864606226299,
                16916014933940155385,
                1639204914577233506,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3021541505347247771,
                11168255125201412406,
                6313880498199555199,
                1112293854194153268,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12126454107499898737,
                15778712615237325761,
                17682408957826970678,
                2114944692964425080,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8261356460652084282,
                626347500004158951,
                8840852234608162982,
                2926840401855036545,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3412015357833823025,
                17714435635351364433,
                2028861456892467030,
                956994785315968101,
            ])),
            F::from(ark_ff::BigInteger256::new([
                920130284663157551,
                11045659472586498679,
                1374316396109201285,
                1841245096866612958,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15262585189170876165,
                16719155610476088507,
                13725835929827979805,
                1678577003421212033,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13528711395767825087,
                7489723170648843745,
                12571993057304222795,
                3276372489702508370,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10433436135857121572,
                1978629639006143459,
                10416600426678210386,
                873685152253540387,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3070355560959020852,
                17623722948622781441,
                3680880850543164568,
                744812851256616133,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8008925009420533228,
                7871925891981100273,
                7278664212909195730,
                2311093096242104527,
            ])),
            F::from(ark_ff::BigInteger256::new([
                18005418616282090415,
                15576438160720525807,
                17726380950765014041,
                586666602765006680,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16565550768397575341,
                2020929750144492452,
                1410603232197209768,
                2668955744112673211,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4237843246993143475,
                16049208306686012621,
                16255345757249790446,
                1894568246696127716,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14800947914661045947,
                17154969448675598972,
                2850544392659101316,
                2712594106208049938,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8181027053927938644,
                18021439195250615183,
                11212235760208429490,
                567213760585872089,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13624033816227346268,
                4169601791260933441,
                3429464486350164821,
                401983439042744870,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6263741246179719234,
                10788601501711948633,
                8749921289808148917,
                2801746585553330450,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12769144857084925259,
                18344191391499884572,
                764311096636231278,
                2990535145009581252,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5305878330428444382,
                8053636284319864927,
                10138077268953994545,
                1081422994864271852,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15707650578550343152,
                12072900675030156560,
                8071569722115328566,
                1756484493678642865,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11790191611758152980,
                6343432735177281837,
                13545786186528022215,
                518855318833427852,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11327724316588873252,
                5178948202190861481,
                236513524920919324,
                116257057088645742,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12612299169294040669,
                11554739275749238468,
                17707711585901173568,
                3164398906309830245,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13837798785078447527,
                10805584042497499142,
                13074800071193501283,
                1986357644498307576,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1749497652048440958,
                15616642939589976852,
                4759284726231031849,
                113483065750996097,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5621124641520836211,
                3834061754940858631,
                8441908693733443662,
                1361470217726932599,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14931031018585912305,
                17379154952083303437,
                8296553189160346958,
                2731328283741883148,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16054887494068740820,
                8540693285554871837,
                3416127536271369627,
                1696891933048307281,
            ])),
            F::from(ark_ff::BigInteger256::new([
                18081711238911243564,
                10703738399929802509,
                14059745812023903234,
                3122833143819019816,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17259475827435575131,
                15506208033562879117,
                13934959152216549583,
                1471728044913703832,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15176346165877314419,
                173608754132356420,
                6382413708583066742,
                42000341103623909,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16506862493907066292,
                7353248954032331898,
                8377384179345429696,
                368265804148652425,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8031903935837660639,
                18337085367626370871,
                3098610599104502512,
                2056976715448611146,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15722471484612216618,
                5142387709257827438,
                6128644560829868977,
                1662662235676804166,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4251127333820774693,
                7798082296646129076,
                16039029898297874182,
                2032077195433650519,
            ])),
            F::from(ark_ff::BigInteger256::new([
                18216981324708702486,
                16564743968791310032,
                13194422549619305017,
                2770479777972530608,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1399693492489740714,
                9742692980463556791,
                4718812226427439273,
                1180883672886694082,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2757919807022821607,
                10415979907392884307,
                7034457183581202014,
                799104554032376053,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17932033664882204776,
                15159913126551348043,
                17087300705474533010,
                11222886469698168,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17655843879022311148,
                15102489663831680615,
                131005390805611631,
                246212393762158451,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5801041412609236588,
                10433715192001313341,
                2640211307484255519,
                331772082415582407,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6466164949923466313,
                17430238980025496016,
                17623936589295047864,
                1238459069938047737,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3676712089653671271,
                18038754395438470348,
                6852402990074556614,
                2534187510793578285,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8161058406159725893,
                6278662234725865519,
                1376443808153743880,
                390847645692608939,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15772356897706506578,
                13446443978764761165,
                7170316157533924116,
                1462783139136035153,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10179756390246397103,
                11757273450509185724,
                17214957577975704800,
                2099146736469805030,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17759329837394207846,
                12172812669103061438,
                3776797374300932000,
                1688674755959437709,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7280660859427541835,
                14007944100542814061,
                17113461613463159728,
                227942265518886639,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1662353633779152251,
                13639813389663371618,
                1884914318728355136,
                1148181445904581133,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12468976938706758273,
                6974111850755807695,
                2964862210755167155,
                612920642609519251,
            ])),
            F::from(ark_ff::BigInteger256::new([
                979479150604999431,
                17382370070336944746,
                5964037610929984023,
                146950327949760846,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2619795151369208751,
                8671779290859883436,
                16846287263611019424,
                684788196204906598,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9957451771757562052,
                8489187461546015620,
                1327251536917441905,
                1509477104244751897,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8836279873641853266,
                4973697061937880693,
                7653255830802847282,
                1234672297996954220,
            ])),
            F::from(ark_ff::BigInteger256::new([
                225899462800228859,
                15872251251753512487,
                1073232304173250435,
                2288185583276436885,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11212018451651786569,
                7030274873323232126,
                1445814743994325556,
                541991329424342036,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15590002396090942944,
                11182794591984585829,
                6096853301225178907,
                1812774208836812498,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17713480380931175896,
                12642083716717252857,
                6339270560976965574,
                398410426758034296,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13414564379630057963,
                7512847729101999644,
                8264121709289709449,
                3485833568505185549,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15464562358060902286,
                13797925323218642338,
                8074067680188171498,
                2843702021517365300,
            ])),
            F::from(ark_ff::BigInteger256::new([
                18127425242955685520,
                6739156278564427910,
                876488626487884117,
                2935731270608132093,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9831113689885486356,
                9163674369416562207,
                18330867672658783606,
                2988448613008154698,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10586992977359388022,
                18424491672067947122,
                2516880425417489719,
                2496797076298951066,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5262412681894780066,
                10323857882670875385,
                4286685558633910269,
                2309581776952187525,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9741354413656546851,
                9068505544060273694,
                11613630016068703317,
                1782806069318027792,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17532808545444923772,
                16766887480790426459,
                11821942689301990884,
                1328549805937088707,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9193752296378055906,
                6175958688600723872,
                1658291901661934206,
                392088376680042281,
            ])),
            F::from(ark_ff::BigInteger256::new([
                117512391887065067,
                5858566605739377875,
                6193703149316492766,
                632826120073343574,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13779548050027524152,
                14340783017810403563,
                11869792329558081950,
                513742035022467958,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17156138187704857056,
                14460893500693034299,
                7979110063105415439,
                2938244191868151091,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8891127874813479206,
                9550603586865712506,
                14850877587871392184,
                106131985723419581,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14295982843167548670,
                4362313656366743839,
                13610113569382935037,
                706520286713576336,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2862274455848052319,
                14818651028328263868,
                391271411827981876,
                2323511582350294367,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13329810362016986319,
                8569491056958322963,
                13567469303412157782,
                2492628117567295677,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9000597449561977829,
                8490204288994734758,
                1727498278415194025,
                64176240662500512,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14152428280997938362,
                12605173083585582050,
                8667254612055674961,
                2459334410522859366,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1798090268250354471,
                7123560725589767026,
                3502855771265683577,
                2228378758446453830,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9141210733337821027,
                11225291028033892257,
                9859508416620484656,
                2005558621248500456,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5004447926112656753,
                81985937198809239,
                16352173306585251869,
                2796045848858300692,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16791166537028787458,
                18391249176720080624,
                11212451197171056687,
                2150786586271619554,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6278970624191498946,
                10529137797064204301,
                5298667806652309140,
                1167995932381024997,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13889347000182507715,
                9132487987376697521,
                10087947545383188280,
                2062954890945799465,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2462485960569366850,
                438389179672656279,
                17234801439311041669,
                635477204596330922,
            ])),
            F::from(ark_ff::BigInteger256::new([
                561582991030389148,
                17797163465889732101,
                10742992354639146246,
                3267775005364062854,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4415949425534831794,
                8140444938843818393,
                12067792397960594596,
                1851840391118491823,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6477956673468331661,
                7683465989741554714,
                13482140210776383628,
                1987153096468771335,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5161270754667796489,
                14622888943735411747,
                16033249854429531275,
                3425908777981396106,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14715428634473837792,
                7628703291537326493,
                8511672311759005265,
                2565634043455925746,
            ])),
            F::from(ark_ff::BigInteger256::new([
                517318656750581939,
                18362199803775210391,
                1540522466121873674,
                1324266276882651442,
            ])),
        ];
        let mds = vec![
            vec![
                F::from(ark_ff::BigInteger256::new([
                    7698982539973574439,
                    8112196319965181985,
                    8104529470239255313,
                    1919892921552380469,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    5759157280296519236,
                    857405202803841397,
                    10296318017519522657,
                    1017618134081329759,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    8387718556547505972,
                    15862310000804313,
                    2480456814254692425,
                    876385620923669119,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    13326626156573908641,
                    6796648063881873454,
                    16240592280767148599,
                    1864614044627822197,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    11645490289399468693,
                    6494374480454944867,
                    17174207132927913383,
                    230777123585058973,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    17590889898728882489,
                    5840428776572984175,
                    17006777544747500761,
                    1820767921777343437,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    1830174992275560947,
                    10855698617160712211,
                    326839526839937077,
                    2871047704528175869,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    12858699558357712499,
                    11728824413249677333,
                    7764240285435778231,
                    3157122312181483436,
                ])),
            ],
            vec![
                F::from(ark_ff::BigInteger256::new([
                    13927636096212875032,
                    7725244757785847416,
                    5609269748423451985,
                    2687383606964405233,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    1419375049174763338,
                    63776939557519589,
                    11674154849748088369,
                    2344622337626361001,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    17908430873004235971,
                    10430991775202647758,
                    11284871003384948955,
                    999423704598732606,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    4137937508734964079,
                    17561604990410199605,
                    10653715533046078420,
                    1230829677397046532,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    6195287554501146687,
                    1073096437508537523,
                    16036752341142292464,
                    3353833346443404189,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    2364571196877341970,
                    9189693918934651011,
                    5873861549230205322,
                    2370806971829242634,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    15756370234036179942,
                    6850955371379480021,
                    13940513670713378485,
                    2603323656180076026,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    10933333177922954147,
                    2024483767041896931,
                    10546015955691150883,
                    1710610463038147993,
                ])),
            ],
            vec![
                F::from(ark_ff::BigInteger256::new([
                    9951042844053021545,
                    3438234409941961293,
                    18305221950472113596,
                    2812237320945998074,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    15269595345386706896,
                    6569554656423720408,
                    11990897114481532775,
                    1188104751926362181,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    10360969609387117399,
                    5360142455003056994,
                    1554475827745700774,
                    340419972951289076,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    3306883843281026009,
                    1342864761511853817,
                    9249672749703752345,
                    2235023094847873478,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    2588071778221988371,
                    8190775520663641053,
                    13330942698360803077,
                    2856047378020851437,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    15837749118519005299,
                    5213229682417587793,
                    4753692444392803757,
                    223784681696588242,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    4129952230553614148,
                    16410854157088223742,
                    13579655470827611873,
                    2513063824208875103,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    6624681805474591834,
                    4963875180783567600,
                    12567231208165243491,
                    2229060634115278414,
                ])),
            ],
            vec![
                F::from(ark_ff::BigInteger256::new([
                    14437681743959906379,
                    5651990412197271119,
                    17026624751267205933,
                    3169471466698288810,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    4965836181798275579,
                    7483837549492589189,
                    1871030991694985394,
                    150361428793192146,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    5923793774924811286,
                    5477851143471086444,
                    17142368143342922057,
                    2920876841757274751,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    8044283095154668930,
                    5291805815478087034,
                    5074306202162584065,
                    1981852066300652591,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    18346628644377074442,
                    18386362808315777176,
                    10881069981594030308,
                    3047734985120497245,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    4134725893027032342,
                    14074525786890072279,
                    17927622659651096881,
                    2180620914582805220,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    7982069403106813577,
                    18294153465182772622,
                    6482732170408295655,
                    2690401086264942270,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    17382573691949307560,
                    2979177317319781001,
                    3620638240125271564,
                    3246403194350868162,
                ])),
            ],
            vec![
                F::from(ark_ff::BigInteger256::new([
                    3531183157400007229,
                    8707009396208571524,
                    2083673509545323325,
                    1586967016381685637,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    10013556679191130559,
                    11283237535593372229,
                    7178358559107925521,
                    2828577489054671815,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    7772413855647242787,
                    1625771078972963580,
                    17691006133923998230,
                    2452426602102310826,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    12717001073595992573,
                    13390304832749201012,
                    6840858844568564304,
                    2018165529627182216,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    3490126654609679688,
                    11579625906529930490,
                    4075322902249924561,
                    2237786275184296644,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    15570090923962275727,
                    12950409578324982828,
                    11439967579048986076,
                    1846408495170362983,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    18148345016245708687,
                    11230132087940279214,
                    10563646026117394320,
                    736568765304732208,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    4614705391888946463,
                    15399225232752422862,
                    11258270771893861795,
                    1840083213380385835,
                ])),
            ],
            vec![
                F::from(ark_ff::BigInteger256::new([
                    13554834801744639856,
                    3029540102808548934,
                    16485076133316850173,
                    65075796687373314,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    2409752390730984224,
                    1350319031896244099,
                    13138344670295861464,
                    1707437289273247094,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    7062334080204073833,
                    199334362712701797,
                    15732756724522086809,
                    3397075356495344142,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    9755895214909954204,
                    16537965548557497797,
                    6301655826252513559,
                    3310572328773597367,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    8088069664608950481,
                    15817457957268858068,
                    8701247539893075992,
                    1584869317748513064,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    4967162706958120959,
                    1327109617665245171,
                    10881255061994430323,
                    789700983364053072,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    10182162546435689703,
                    8809995489362244018,
                    17556416940360665054,
                    832749755706758484,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    9901564790845760151,
                    1862767175757062679,
                    4190622954485152565,
                    570775339292055145,
                ])),
            ],
            vec![
                F::from(ark_ff::BigInteger256::new([
                    7169344967239091692,
                    8842385722702832433,
                    11569504801225262525,
                    340401971094034476,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    7998291772064430908,
                    9282967859451777314,
                    12238467562708670885,
                    472471567865868979,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    5797622235354542035,
                    12564245487240192385,
                    7370839608863595848,
                    235542635572555859,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    14891410244294326347,
                    10325717283148092970,
                    13670142029991803546,
                    567678547651383647,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    5797414575854099958,
                    16782337434217207807,
                    17655122302743878990,
                    2463151504249287858,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    6092607120146277889,
                    15149213248580973303,
                    13138076982004158985,
                    3261460126518429277,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    8972227487681261116,
                    6556523079122776216,
                    1053298706098154997,
                    3354733106714752208,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    15302199249017847393,
                    5531091257341226273,
                    1336696001322386582,
                    2518000713112008086,
                ])),
            ],
            vec![
                F::from(ark_ff::BigInteger256::new([
                    17563087222036015820,
                    14715779049777418464,
                    6698901448650022420,
                    1137665321783497329,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    11843035247630417618,
                    9423375191022539111,
                    7824027481589339518,
                    3412085902217520265,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    328036895794269326,
                    9887048361356649447,
                    11408405792603257024,
                    3229533228678951296,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    1533407463283740381,
                    11955504353081720131,
                    16581848506024531909,
                    2665895161944335370,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    1155332998160347412,
                    7696089458501254954,
                    4726599165070093607,
                    1344461980674159688,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    11542989011830303852,
                    9456824946761550140,
                    1987220851127417944,
                    789201955078953616,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    11579033906566466215,
                    8786689633373017531,
                    10104669292495618529,
                    2878976117563149706,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    10568123772634788304,
                    1303417413732746717,
                    6305917206987735359,
                    3128660075931242910,
                ])),
            ],
        ];
        return Ok(crate::PoseidonParameters::new(
            ark,
            mds,
            FULL_ROUNDS,
            PARTIAL_ROUNDS[6],
            t.into(),
            ALPHA,
        ));
    } else if 9 == t {
        let ark = vec![
            F::from(ark_ff::BigInteger256::new([
                1898280248850124407,
                6401424918302018560,
                10081233297564194517,
                2344350746269940723,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8370600580480330721,
                15140150009984319481,
                17599201375549746324,
                1983525818537914666,
            ])),
            F::from(ark_ff::BigInteger256::new([
                963014773771968873,
                11866108974019534676,
                1113886723914189264,
                705378678046207255,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4101360975294950612,
                16297546958934563017,
                2602916671106882534,
                2845828375846795268,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10688266923599639145,
                12437716513733529285,
                16208246540973478682,
                1708410860245148234,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12068901669225282246,
                8487323181880384055,
                11540160442885808995,
                1281063641454862580,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17277400395290494397,
                2834288396297943074,
                18122102284742273879,
                2078504862631188450,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6494075181652439698,
                3237388643365173067,
                13810716921725992304,
                3222665557401584828,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7297231816968807213,
                12417256982855579728,
                2590267141009753381,
                2268399750765512569,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13906995545690545621,
                5093860753397280868,
                6673940643652463321,
                3104413258516214692,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4601152759722155171,
                437790660401869669,
                11964430530716998536,
                2506420703684958675,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11997520749496187134,
                6444713976142300506,
                17788541300143019013,
                1918295056751023320,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5339591579661176531,
                2694256189508679932,
                14870368415698781710,
                291518106373482723,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3303485281150143358,
                13568651184169195946,
                17192291243972232658,
                1639116020749866087,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17041567071404634778,
                11811261012939773495,
                2034484089042928528,
                1172603767838450123,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7870992077652425993,
                2542614928915656950,
                15955385507816399861,
                2784097575160868360,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7901638211607313774,
                1670538514628294760,
                9127864729194276205,
                2928855551811027768,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1863971298573195046,
                18189344655952144907,
                14737854802096230812,
                1759748755508575512,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9311134550958896359,
                1566579164181195512,
                7806310051309073373,
                3189088369545219144,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3974420271517126010,
                7689651392253517928,
                13029704051020639652,
                1262769750697478259,
            ])),
            F::from(ark_ff::BigInteger256::new([
                201361090018568164,
                9324401654833530507,
                6888051258198156627,
                1406221732573822647,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3891920661951313001,
                1392138496783915030,
                8350685943627449599,
                1873589009023616405,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1024273923640418386,
                14279116071122389830,
                11341350774831401773,
                3122141631907619160,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10964155906861741453,
                10446238474801721611,
                10034600352061730622,
                2395480573209907718,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9123733731606932356,
                8775383474977339316,
                16095064321043864969,
                167819627431256857,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17214490561679367883,
                1850322377179047836,
                13622367302604082355,
                593194733425913151,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8071045148273528348,
                8912030942862351914,
                13568319874560283815,
                732721953050741333,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6051411860609297161,
                5153742057775078560,
                4384020992739692058,
                2913988199246513449,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8631796138051798407,
                5143112880839889682,
                3046469663285173862,
                47847181656476610,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4065498052763767062,
                4582169378883986449,
                3273765912543740002,
                2124006622428656294,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7235055837620509891,
                156080250125856397,
                9045380330377129105,
                2102519626924260106,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15763284217570750893,
                16744370992690476266,
                3403518626307678791,
                81373332794170281,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4855705460406051074,
                6747984285027047254,
                3901950289326015786,
                268529042397010763,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7644468798658590155,
                11759230636308262496,
                15579554339821316023,
                2406197275050803684,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11007317502569990148,
                14065784907696500198,
                2339511572330129773,
                1060011957287333467,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3601216695307093494,
                8180612457328979048,
                7626064072587938802,
                1474375269137015806,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16129824218949918514,
                4903992145403029476,
                10546559535809289483,
                2709300894119186305,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10784146844295224097,
                16949171019125906049,
                3454517017918828631,
                123163899861005845,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9066912017018174802,
                14438267467968408472,
                8589728088758901227,
                1062828700652460404,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7221462186650903340,
                14249515893956604525,
                13461768164086046232,
                3315692873932279345,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2478722623039832329,
                3530663211247184186,
                11625972878917710968,
                184318332256060724,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7624886176895760844,
                7369533342283823393,
                8513782688794015104,
                2437934002320477427,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10543268100253620326,
                2478785358753241356,
                11340468223080350314,
                3026840391751505918,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16226754079032385922,
                4470558986815767904,
                804795022192897929,
                602621582926244683,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2791175340779385003,
                15016073794176201635,
                5700183903943725171,
                129111163847727772,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5286947811468491119,
                17191425700100580199,
                11008237714036811770,
                627787772794546145,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8380771588096687606,
                12234192936866604905,
                12198527651165574576,
                194690519523524731,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3100019500969144230,
                11170268527091174423,
                11179244567948903325,
                1896062151035182767,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7650716588188690761,
                12798182031766876753,
                1877901635266315245,
                3479212366219401972,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6806544228478005804,
                16616723060695951474,
                3943449893488659754,
                130613328768857975,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10863158797377735712,
                5700287556967341336,
                5274881021470365221,
                464866895113862046,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5940343126800247796,
                9081744706811999636,
                9846239764992027712,
                1718691479068640789,
            ])),
            F::from(ark_ff::BigInteger256::new([
                18374973239301589744,
                10392628795250413061,
                5233265719887496013,
                2166525146371518868,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7016856937417543220,
                14141416118055059138,
                2603698266286431126,
                795493048253013243,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13651415067936820321,
                12474209185440654140,
                2002518856057598756,
                155015893388479619,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8112038246101245180,
                12450981787955447188,
                13159603573004919332,
                2280236461022208381,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7879451380309492628,
                11420570548266382753,
                14187816267082265432,
                2986533901544676148,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4729203641424071364,
                15800029208078666398,
                15522133199418565481,
                2312695259745074499,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13120304499425502738,
                15592760929961718067,
                11062728981493355477,
                3221741330130729720,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15598705664059323583,
                6654498340699245119,
                17505920729394717971,
                465883987385435030,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17201189045986203729,
                2168077494013918475,
                8594009881608963609,
                77335200247740836,
            ])),
            F::from(ark_ff::BigInteger256::new([
                552832413017381463,
                16901165793023636141,
                9062661925081040050,
                3434782123183010859,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4035246588906389674,
                7856722439343432746,
                15472581730542827495,
                1251614027237069361,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2837860789129843253,
                438622748515726689,
                14213175952152574666,
                239784530252926943,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1216728272244076814,
                14619464135295778496,
                1168185890551127905,
                3004860107859930281,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12245783243109613988,
                15646203440321325219,
                15035816083448107514,
                1906072230840910345,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13275460801392160863,
                6600973601423007182,
                13108328012318446687,
                2830093272458554014,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14452839525245562829,
                4147596342469694178,
                15979091299071167865,
                2638518273341089410,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14679051883498624226,
                12669920079068872739,
                1739125234252363445,
                532346853644716441,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5134638275430364430,
                16294264204391318598,
                10078375242106427705,
                509493135621687870,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12672809985151626290,
                16305807962289237138,
                7216616708218702176,
                1565983913289959735,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9284171140778790417,
                10007099939490631851,
                479639424831762298,
                955763162735431259,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15868499853182672396,
                15922377515920299191,
                12890255230152507411,
                3170455470136641378,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11843189821919450816,
                17894256615069432365,
                10271321063539333795,
                2660011954251452624,
            ])),
            F::from(ark_ff::BigInteger256::new([
                982705163619917756,
                410136783731216078,
                828815840069242610,
                3054260927461188724,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4949926074347976169,
                12162681406475137734,
                5083695430380887241,
                885369963227195211,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5908350058882954188,
                18434953735561844081,
                14000932600714299180,
                764289487002291587,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1136679333386109857,
                17294065547842071601,
                13447993189225806236,
                2454877241732906025,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16299660179655745696,
                12036509774508995637,
                15104406150069607602,
                1657932981946784323,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7180971258175346912,
                13613003916273743766,
                6698785608690346459,
                403836912004366440,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6850750953889592729,
                7639019940490104387,
                3868338751618128045,
                2571384625270874231,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15633290769196467478,
                6809779375951559278,
                10709778486632465135,
                1197625577827642751,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7491469947873866534,
                4001204666302067103,
                7577206907383024992,
                2799494018901857277,
            ])),
            F::from(ark_ff::BigInteger256::new([
                18086639502273663230,
                7237855803563419905,
                14951332274934028186,
                2660112449035354558,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14938488321574640572,
                11019967564786915700,
                12554190581211082942,
                6097585525739155,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6128259251255565340,
                9397837767343859033,
                1408916026983566612,
                3151172216687442858,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6102101683137351209,
                2134278693388079222,
                16907004803589691001,
                728587412113903959,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1169454536863129538,
                14856426756146754685,
                818767010651383549,
                759594992026843309,
            ])),
            F::from(ark_ff::BigInteger256::new([
                124532757154884310,
                3132210545646517049,
                3009644958308844612,
                2798570639682172130,
            ])),
            F::from(ark_ff::BigInteger256::new([
                679585055901640837,
                2222102070052245191,
                5684848003376950295,
                1828486964090548900,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1184653512400162884,
                166727228607351735,
                17401404950292539092,
                543070229184349379,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12750634560074478723,
                10283082787820035266,
                13290457131682129954,
                396455349347185651,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9406862819541725150,
                3072739876968364418,
                12076550467853598006,
                1930279037471751648,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10013018809057811264,
                14772061454090035800,
                2911342947477447779,
                2530026377637259864,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6349133807314560813,
                10695537838532907391,
                6902990133907501823,
                3331844000503175118,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8660639340132827150,
                1707635903259652140,
                11762065856092362719,
                681193857331833205,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4160008184093284786,
                17129574738630206097,
                11380257070407985844,
                2086786913146512037,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17838432571076645513,
                9565599369955405597,
                17688225819202612369,
                1273752202729624844,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4748942159005342439,
                17111755109693653035,
                868980713938900886,
                587996811278533098,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3328416991672335606,
                9603140900649717736,
                3482132352633671577,
                200164615583068450,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17423646853788824258,
                3894813790117711912,
                13881093985748881782,
                701994830325778468,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15908017573053260475,
                3068145271211975399,
                9351364639186169679,
                2131691429383442005,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17955593095201249003,
                3886029778040447399,
                497453832041854614,
                3135230274801478433,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17420824307303092302,
                2584974330856083594,
                16265518214548675273,
                1356631425055399748,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15348637480037694459,
                5805990908715730474,
                11851105587350828981,
                2930126772219866478,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15876138276153859240,
                878518349694434242,
                15325048389460483977,
                3158082421401961208,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4432907779086051443,
                17555192133768813486,
                13502441622397578696,
                2822913467684164707,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16189581550176502717,
                472684929004950254,
                4287954259490642451,
                1393203925311531664,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3229480047660447528,
                201674023892873561,
                7875219640123215659,
                519247376521104097,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7871471757620256084,
                7094406046413299323,
                2889513914688623330,
                538413965248755566,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5528264543316182684,
                2871950188283756387,
                4849541703760632273,
                1638124443865393765,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15392823778787865551,
                3633762542552269248,
                7535707872202511966,
                1343172069985112140,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2255054275725349695,
                11031021559973285249,
                17594891973457598079,
                2248977110764859611,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8208019173494742972,
                3454636322117635181,
                4742103386631671587,
                2726295270084708964,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1826343940499120298,
                9794850777632754861,
                3906909419726772078,
                1578151964464514618,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14674899158569372106,
                4749932295823701417,
                9132470025787665485,
                113327713534812855,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8109512178847169401,
                12747549622687660231,
                16895554259255802580,
                1751708835097979437,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14210444589365254246,
                8209745845157970474,
                18424935119354647779,
                2739787217549217877,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4407415611856629073,
                9636751773197842620,
                16143119590333957157,
                497318042431335471,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1143425321926432077,
                16267237372046939071,
                16324905084313406330,
                1404762280826590669,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5474660872488917942,
                7266506901573919454,
                4297132395583722494,
                3357313738588419893,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10601881897405588499,
                522518685124566781,
                16586027768457401746,
                1418423409394088038,
            ])),
            F::from(ark_ff::BigInteger256::new([
                18388146230079552866,
                5288956891813883827,
                5104122643241727346,
                1140247794928970031,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16085986017581170966,
                4865204628374520117,
                9398945357209599985,
                855001871779028785,
            ])),
            F::from(ark_ff::BigInteger256::new([
                18373935659122007811,
                9302854871160698995,
                11195976190407733027,
                594095238770146954,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17107352913746243369,
                3312053159737910466,
                14695730808192103120,
                624188816477498259,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12696559451685956167,
                17084904385753888616,
                5274697003777267939,
                1764035027201394692,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1985874752513688819,
                13939750023335159835,
                8435597319472031701,
                2616597946562740816,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12513002984581451186,
                18065796697305363310,
                17617995772306442071,
                2714081041938543091,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4941225923867085753,
                15644120603909519542,
                4267702068218628042,
                126739206073033333,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9055235078781417518,
                1098414224479079806,
                14632423038345616122,
                100854200955705028,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15195409712972710004,
                5594243340075048609,
                2175936612652625955,
                3093974741829095908,
            ])),
            F::from(ark_ff::BigInteger256::new([
                463383693452214587,
                6067389321173542118,
                6874210642442462097,
                1585878248717463009,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3203369169666451190,
                16084202975813520257,
                9674410043700578176,
                1381897712965121000,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14257153959638355767,
                5343301092064812622,
                12221940954141669274,
                2359230449838230665,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1593818482440972625,
                14028686982510536022,
                12345166034876365197,
                2944027334882008193,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12848097634434105040,
                772919308615307729,
                4247811971600774376,
                1872533112399977476,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2150244263119046405,
                13334171546716716474,
                13170894743628636268,
                1638931011965224746,
            ])),
            F::from(ark_ff::BigInteger256::new([
                725764914579373405,
                2953527792059844949,
                1559397672094227435,
                2289371322328526282,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9017293580644752125,
                9031522225559033785,
                3696299409137331349,
                1932621596470892982,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16212069972839008389,
                6459258889719813549,
                176647144117559902,
                2410343937551238687,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16236403649123876056,
                8801115535492677875,
                8314167204319880980,
                1254025391308150035,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6264496435138764786,
                9532219053157765205,
                2456747949879230569,
                251217169924561948,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14082592503534059359,
                13132423009124188805,
                14613490337579271496,
                2121399251365601890,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3198440986287457355,
                10823808384921601611,
                7907826059116558992,
                2415130661749263639,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1437603901252282320,
                7817471146378362192,
                11419775643960458143,
                269594114209317071,
            ])),
            F::from(ark_ff::BigInteger256::new([
                944341206419419719,
                4859189292328616172,
                4959889104716517448,
                3127054169724534778,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4158290822873695766,
                17325127769963095029,
                15853736990885184892,
                3062395171666506185,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6898425661573781151,
                8275966520477363494,
                7309682382104319438,
                2069545653739066680,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15311358734134483542,
                2830599610722827176,
                13967351473225106493,
                1613004927624033979,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3335915796189737326,
                13213795028556413326,
                11418560432345942150,
                2808728312754860294,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1990254038651672199,
                17319596640019768716,
                2802200406498344091,
                1074227841390931157,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7436849983097608479,
                9567401327110122810,
                383506048724459929,
                622930071835379044,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9754592144117158967,
                13601624286176278894,
                14186517988930625828,
                322752972363690522,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3364733574047732040,
                17713218919116356606,
                14326657079175192731,
                2754603315730718653,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8324562727474557774,
                3582723672214929629,
                15620833938400598646,
                2478423176577844962,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1547892926950237015,
                17352806113989610417,
                8687175789391600205,
                3147719756852045908,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11965209943304698962,
                15693110810620352081,
                15370329106347837769,
                1729994453360470040,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15658407315662535973,
                8672999364032428030,
                2192919932317725117,
                556947926096037389,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3407818661843733098,
                1909369785261220710,
                4204862790836286156,
                2181819226182706064,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6664937240913214104,
                14122196332614301700,
                3310255411265170056,
                2162182925807309371,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6888986840064526562,
                8175377708679590666,
                5166284986058536426,
                996807347837451111,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4081539535597638025,
                16989860260469249790,
                16197483707682103218,
                689904352707782040,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11463447113480780215,
                8688420580048442291,
                12831129807685608044,
                1458951597892831601,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12321004727405748349,
                11542959590780748156,
                10161030064382506319,
                370983095788382491,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10457200617924122706,
                166797117285679317,
                17602622746621154194,
                3287588252082865849,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16636213464644468549,
                10247415205470277240,
                2169593613952811288,
                2524715230614613765,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9272621281858492191,
                941701036521705595,
                868921353966948563,
                309435860736785565,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4516963243954098359,
                12281711106084589625,
                2569661297287549817,
                1515417894334280220,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4352799792060230070,
                1520876329277987817,
                2753809092878290801,
                1174708682506745088,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4952832670982794275,
                5091879176838471757,
                7302831303344428052,
                2677217380075758785,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15042881188787258652,
                15899540510441485734,
                15554391311373662528,
                1221547501468607243,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14734877396991602749,
                1995077422222692302,
                5679113893628204681,
                1026940285378414258,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3833917719094144781,
                9673448018877986797,
                15711953409605154321,
                1797886452654559003,
            ])),
            F::from(ark_ff::BigInteger256::new([
                494365486508899410,
                13414655044728576929,
                3626930652070615605,
                1611446621966373037,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2856753054627742287,
                14173125276562699864,
                1740178734922048719,
                2467001292924410685,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15542639786278643414,
                5844031099414621334,
                1918961846935756516,
                2379025306044630027,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12189000051803673476,
                2588545957114651527,
                11541247154560908311,
                2023125873128499913,
            ])),
            F::from(ark_ff::BigInteger256::new([
                915601605181512594,
                475534188497123115,
                14532713739857070465,
                2013089396850067666,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5821091914439414148,
                14050232671971249497,
                8249016621628516874,
                1935503243353523679,
            ])),
            F::from(ark_ff::BigInteger256::new([
                282286694555207627,
                5690079426176002861,
                11531354535472850087,
                1198204662145986742,
            ])),
            F::from(ark_ff::BigInteger256::new([
                948346023702690199,
                2162990108401158577,
                17081793964095280972,
                1917125391374774239,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15100999706680581300,
                8241470641539345550,
                17816539417761723820,
                2745339756333864410,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15097130960382615704,
                16262269044889861673,
                16185567723447992454,
                1384860250623759098,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3161269969573976794,
                5818953398141826261,
                11096606279659366375,
                199626531502351972,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2841392969497052624,
                1047325166879445185,
                15658655640929558300,
                2828278172473126372,
            ])),
            F::from(ark_ff::BigInteger256::new([
                407050648673697007,
                4489830499563515017,
                12247499642640363654,
                831636870425625584,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16965850708277389853,
                4035767439777439124,
                11249981539968524525,
                2728464321320882572,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11370765014250115695,
                2844952265207679114,
                14399804799071529727,
                2683443051713474545,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1059121475483661814,
                15188642829539024386,
                6109489915686469080,
                2676037773200881868,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3090565524099199412,
                2555779270871408910,
                16780364333867449679,
                3083209644513257544,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3519932114197310415,
                9986474988087540488,
                17999964976959435180,
                3048433310729712706,
            ])),
            F::from(ark_ff::BigInteger256::new([
                370100883992584155,
                13147926461198785395,
                10547790743636439856,
                2814326915059597536,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12590706261555859154,
                16760692823076944126,
                2830007560942445836,
                1816058364119507698,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12804009145277046213,
                4686227841880976075,
                9522731882371942874,
                1608120271845884635,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1840102663091313260,
                6126559491193822284,
                15409260962163537887,
                3441310879626118456,
            ])),
            F::from(ark_ff::BigInteger256::new([
                729103529484153199,
                6725110244645735990,
                15637600628044376008,
                1017440969902409049,
            ])),
            F::from(ark_ff::BigInteger256::new([
                467573427722052940,
                5698777400671056293,
                6769765858813301594,
                763416627456915757,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13289395146973639886,
                12605426023272305200,
                13942862540555283723,
                3325879841267987975,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4089739946099713343,
                14878503784442816456,
                2498623706817350306,
                944887473463494629,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14762320435377477956,
                4137122877800905540,
                16853823272670495459,
                834729074909251181,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7850340719432088552,
                12812123222411697864,
                9799832211146375541,
                1654426545793062135,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8373777790431315514,
                15839007362005857405,
                11970303740893114381,
                1348417885427622624,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14446271494838736973,
                17579835197605546308,
                4165949113508967683,
                2099302880320589254,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16393874254762924314,
                10801704795957502762,
                15725130855627213978,
                943027152874619848,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7061659056651956027,
                13214306519959958426,
                3432185147281471073,
                215290865724865919,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17998849323566087735,
                14412259161397388728,
                9536168008904123255,
                1086277403821798452,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13002694618083765214,
                16576339639662448815,
                15106404487589139697,
                217719419486247927,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16455177428143873816,
                2002200999302276560,
                1003137794525586519,
                2583882487944447860,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8760203648253745865,
                10674582501088307349,
                14823117362070602466,
                124138524806316567,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13516732538080000657,
                18000930713772840902,
                14474859342856104635,
                2944087565636825139,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8344013937674203512,
                16541351135185290132,
                5529784564548825972,
                214336833643063184,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11614911248691182767,
                6601855433183949893,
                13099794407601296804,
                1994282741010967917,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1699919705766997632,
                6601224967311471225,
                9766945020217295237,
                3140967311644042522,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8695908289484066973,
                691831274281395523,
                1143766372712590202,
                297123189360734862,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10764003516018428128,
                4246779166518248283,
                17724880639728100247,
                1420269231270811554,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17818808185055796974,
                17892032708618721461,
                10131948153665465610,
                3216760824956327470,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12325329199892807152,
                17362213767011496560,
                16639228018251079846,
                621333053560921297,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3968182750968353397,
                6935955906881993231,
                840969085567157068,
                357210230043781175,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6881892554162114649,
                13462868624664442858,
                7183237958343353076,
                2742149858235954469,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8913568224101495479,
                7836498713916622111,
                10010895390648373729,
                1902036525396161862,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9772419626681438633,
                12017588386448681856,
                17086663082537253927,
                57567192935061115,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11580612526189153452,
                1016072696426244083,
                13656752906150652930,
                615101512455271486,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16686078836971701950,
                12459758097817178225,
                5043439708111132245,
                3181162252133204831,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1658609215264381544,
                17491383744447896994,
                11215749114817715783,
                1360053810737008476,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3069320162356222203,
                12347467488919351210,
                17334201438184914307,
                388535974283073103,
            ])),
            F::from(ark_ff::BigInteger256::new([
                18035527885695372405,
                3358600727462109968,
                12494126183430075591,
                1715659018260401518,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8285424118852382561,
                9067753394209237102,
                11896728580509038789,
                1346962393241902447,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9221956851980214283,
                17139168483540364547,
                14689281466674835201,
                3288840979205982073,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3422070399954359500,
                4310951814744334797,
                3096812291339356992,
                1405113403955435507,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10799232081703963391,
                13339346257927488785,
                17316046475692031302,
                1375867689221625161,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10168991877859328909,
                1303520251075769839,
                1690419323659437930,
                1443584162043079241,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5878300828293946631,
                14192047430671571145,
                9286076902671265244,
                1211222959808889224,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5667583731061739662,
                18352700094269791370,
                16197395777722993345,
                2848524866535895652,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3736677267299571502,
                2265365969260212350,
                13969818347672382059,
                1986335995973367109,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5774456657218333947,
                16129996771663688379,
                13220774653201516094,
                832762592274784447,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6087241149939026139,
                4532504519639244575,
                844325242922726168,
                979957569027680311,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8028018326026619636,
                3000638502036491749,
                15434030191876061039,
                2182411627012906207,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14986186602006993786,
                11122284468859398252,
                18392091005214125762,
                3269652919585851037,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8667549060559775495,
                9240275058339745957,
                3676751134346059564,
                1660244163015574852,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14376780196455417438,
                4068838395058454084,
                2019952594768198633,
                2428463003643231552,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9854633644542281626,
                8399468549067900141,
                13385978588789925168,
                2471865518709686855,
            ])),
            F::from(ark_ff::BigInteger256::new([
                27666149193998575,
                13451987554889262307,
                2520701765254153217,
                2761997785782012196,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16328195689647677234,
                8245692611408355747,
                13836890419073874011,
                2112855908067646313,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14208213904151327523,
                5831107781274711570,
                13827905899911508815,
                2952264623912524962,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15790976854265985692,
                6201168267212552520,
                5665449163124522953,
                589516427794199602,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9751103715013035511,
                11244781677863825821,
                12815344852319738585,
                574305173570205799,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13095808985941508506,
                3144021814457766408,
                13247312269253319430,
                3337997514490589613,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4576773427385922373,
                2066913599543294744,
                6602350040934586233,
                1498359843117385312,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15234706886929143724,
                5038035970417655284,
                15717915977099324904,
                3447141468099207327,
            ])),
            F::from(ark_ff::BigInteger256::new([
                18273753831610612728,
                15784159296979112337,
                3669262988413451034,
                226738592858568650,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17987311371815837000,
                13305863443025150054,
                13284891121232895990,
                645433646476279842,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17490842725084946719,
                15694684755745556848,
                9640571434295980353,
                3010018328860250034,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11088312476895757100,
                616745449242858313,
                2386389205196014052,
                1325408327181537653,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8987355069022766340,
                5898752435555066687,
                11309310160550211005,
                2927594301621806366,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5257582237821837782,
                5499735326070595228,
                14411664512065975934,
                1917379305232596710,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13912348112158416749,
                271113204026307402,
                8060852849459064139,
                2015090512276474004,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6652160662659957698,
                14614305716705048089,
                14936506661017949584,
                3411134495686811204,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16404301460316356318,
                17722246802666214968,
                2860542154348978314,
                1227753749379305289,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3659888427048327047,
                11897686694940735187,
                11270759481589585296,
                1752395688181914479,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7827087915782692794,
                6661166766949702296,
                16089276865069203698,
                651303193484740594,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2945753252984053886,
                2537313778097298082,
                15389148806145056079,
                3090570016287941254,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15961298406110254384,
                13118911281793850039,
                2268196645667463246,
                479281610971247428,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4415863650235073644,
                18388433417601983421,
                3921969348416728998,
                3416743186713799146,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11238242966717168861,
                9747436426134311049,
                15323567630350819850,
                142362579561885366,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9073967905822448088,
                5497533265067550049,
                16607461558992321886,
                3409590235690800980,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8862155943739972044,
                6712660655438870673,
                8737460401588607010,
                1496345352760913849,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9837181014425230130,
                4374773019213250505,
                9137111998893329930,
                1842570202165178054,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17959103983611983562,
                15140507401012211573,
                16223803389418787526,
                2055096014324983635,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16972006085608366008,
                17679416995534985921,
                5402808674013285857,
                342073914516197168,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6354603668495546195,
                11521332041200713102,
                6744895260800961366,
                2597021310182471213,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7367577770873186148,
                14435468731048878796,
                11524566061517173219,
                1520854771098116288,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12739547994335343165,
                5295269781268953428,
                7250923633147239564,
                607620718758516742,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16996828408319844804,
                3038006089022639803,
                16032441773012715832,
                1705326672690610666,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2308190533495524840,
                3671710994788008854,
                7365170312509161464,
                1899168042076657781,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12873423308760429692,
                16425504235085378373,
                17939191007754118284,
                3387749727623634135,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3829640825009409349,
                8115531707880977738,
                16980956499941532696,
                1148233455437221935,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9701706361651985002,
                1127559223094118736,
                5592256854026803160,
                3316819585816429519,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5871478798904555231,
                4128540315343193280,
                1501252442827930785,
                2619298104672591687,
            ])),
            F::from(ark_ff::BigInteger256::new([
                555351208472213742,
                16578725738057246692,
                6585978574335403895,
                47330332695903414,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14614248861871906363,
                732945714901843251,
                10005808652050710879,
                2915053061213503876,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2009740228940189272,
                10040572734508470605,
                15277730823364964623,
                2422494379585371272,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17588605789516185194,
                15659444711525177809,
                4152063471002840557,
                2983232364009941362,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5798629185729379563,
                11508342083331326941,
                9518466609061061250,
                3228059547193693222,
            ])),
            F::from(ark_ff::BigInteger256::new([
                574930949644484002,
                3924606406551668253,
                623209639684219005,
                358341443678145202,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16545848910614710024,
                14765701007481124084,
                16565642087630597728,
                3315977268068958110,
            ])),
            F::from(ark_ff::BigInteger256::new([
                577414654686845754,
                17035340898153236049,
                1077133066487956288,
                608001295902094188,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17024094157804816258,
                8143659303750314716,
                16901245420709990579,
                2667031907194201533,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2996763447338457837,
                14712679934091766682,
                6285530901643477408,
                388215040023108727,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14640142334072725020,
                5053691047798303228,
                1343420152738056717,
                838804562851991060,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7839021809657740422,
                1183818131863356313,
                12240851407034661806,
                3273891303562413791,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8307191075093034631,
                4363543485025496795,
                4132129861141495040,
                3428818777888253489,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10210358464953952824,
                5300061190184766085,
                13492942909446853143,
                1478934354122157489,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7924322716381666434,
                16669273743294903693,
                11095116277232950193,
                2837897854580658728,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4691633262843028072,
                6638851839958719473,
                4483309732846443657,
                2626487696181068128,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11613363755572551809,
                15652864846711271340,
                9974624271857637091,
                1810887519144787143,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16329191338468707712,
                10583201080078463507,
                9013062056546131267,
                758223448512366772,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4062518475291578670,
                7069502798865549896,
                7338187534352082398,
                124623351764587021,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10653483767327283609,
                7921361518497895243,
                12685242493734405831,
                2148623881935725339,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11299456600278220942,
                15238728230293570259,
                3944838709973537968,
                2488658720706996863,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10297252286367785333,
                2203628582841599254,
                12950411720851563324,
                3084996205388091667,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7601700238895612080,
                1524931915925201613,
                17990354930202915193,
                3351448923452268602,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3389150302063162555,
                10503445803379232380,
                15615759600819947222,
                3255450170868082169,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8173121851933462386,
                13954898112032468782,
                8756047494652264344,
                770266258273621817,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14745075224182069656,
                3457106211995110684,
                883616262354835924,
                2835149473712273645,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4216013093410320628,
                5346050174380276593,
                13665174844912224078,
                449785674951825127,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15016158865114556200,
                4847201135699166350,
                4413998048069269790,
                2553875336174847605,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6052428171577704565,
                12135711701629778283,
                10644224758057161354,
                2791664466222749926,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1997123187598764138,
                4646058818692067924,
                1251345765898736587,
                615147176155990249,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5110019242378445638,
                18058091269089650705,
                6298599411586278410,
                211544606015385466,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12851321661448817409,
                6371408951122081368,
                5259494271908207115,
                2688242569499613475,
            ])),
            F::from(ark_ff::BigInteger256::new([
                275743409590074236,
                8588397088125835739,
                13047170706393746242,
                212826013527037159,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3476138541713576096,
                2914363306120573246,
                16927214201051023036,
                2683874685968437934,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2480081177643696073,
                12319533270721871529,
                17462555712648805344,
                304378587033120003,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14526895948886327340,
                18027479872413576709,
                5324349992591729451,
                1400920981216828887,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15969381832713705307,
                17525072807269596732,
                4878895078258233593,
                593477968744981373,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17446185251918171817,
                5504847754095265999,
                7314641143602076308,
                2795780267071524448,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4973361066362623716,
                2358096668149254101,
                10691568164405939616,
                1047501361728625434,
            ])),
            F::from(ark_ff::BigInteger256::new([
                27663062126274565,
                11127621874136825554,
                13707114732090708138,
                3103334439197498645,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14136860963822109366,
                10549776847882319803,
                15733942835766320358,
                2424453380178735149,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5381113892974823414,
                1497447513293364663,
                5771080116808985135,
                1848844063655041942,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13217823768191062229,
                13779572951899899761,
                7625833321297506053,
                3098975321314127765,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15442472196837482788,
                5364266527826859080,
                1727488910728922797,
                398028180473427302,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4501071030211901834,
                542452881708749434,
                13581216368191184048,
                289165929953746323,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7305367685844076858,
                3933699129147912071,
                12136339274607169530,
                3270887358097609886,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13958211245114238111,
                14546273134040118021,
                4077259974699538499,
                3362450463702154394,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17321450015283480698,
                12590084131776332980,
                3133636777322849098,
                1360329259292088425,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11858525699447466255,
                4446424485978544691,
                4388697916291456377,
                984530226529104940,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10000079085668247632,
                10619727308236006337,
                15736312217709927625,
                212607187948962695,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15728506364023922196,
                663021766478910371,
                10011118563972963444,
                2318759223289093630,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8541738952706169724,
                4540813105004969525,
                11402711805229103649,
                519934039269686272,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12244650157255447802,
                16783007942069508369,
                10708013529745865879,
                213889484280731289,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2645321829414551537,
                1252120000321006184,
                10144142610402719152,
                3037599103600222875,
            ])),
            F::from(ark_ff::BigInteger256::new([
                18017885816618359358,
                17715161994261476114,
                15160115278695203743,
                1824659958456849365,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9611449077636526117,
                2530211780204136780,
                2996694903078799853,
                600978124480643939,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16425212393897966061,
                15346970977870070484,
                15641729504992857752,
                3174811751399024946,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14231816494131789472,
                9188649835292570012,
                1574581905779739734,
                2814932554555637563,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5935388228345740759,
                12752856127349122939,
                15585900826021182912,
                3263847769422984286,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13166157552290326607,
                11162871517948800633,
                5720305961494556471,
                2542565660408831498,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8675555125372577254,
                8584059147997362664,
                13410509364774719884,
                2928792926283613833,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15372103919007102443,
                14332773381286772990,
                11126783267030193661,
                1881082289706599240,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10304987405380561188,
                613163326992028062,
                82177151549909149,
                3446824762781302414,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9197377228934924339,
                10833447711971957012,
                205186631765667713,
                951777827866730124,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4046038609249151501,
                6811815449488257051,
                3998590295361643881,
                2971982971935578690,
            ])),
            F::from(ark_ff::BigInteger256::new([
                802627173535894968,
                14966004800098859902,
                15543725321438491476,
                2415874856693737660,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16277056650197900916,
                15916930512473164867,
                2819707321884265909,
                1966229078913742970,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12179367387794592298,
                1082398346936150831,
                5974051032756822961,
                76661053371569580,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8278833653889299212,
                6698297105621010317,
                12407645943969904473,
                516637468990812738,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9176808827980403317,
                9471516124200961689,
                549049248492552076,
                2177941503495274593,
            ])),
            F::from(ark_ff::BigInteger256::new([
                18076149387536072042,
                11801004656194421692,
                3222964619216324502,
                2663048825952316222,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1265359081516088636,
                13551787584134777017,
                17455822800038801232,
                1512808000885601325,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7715345017512235375,
                9378492618151810093,
                8891957362277816066,
                2625947701674981063,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12212396857149709547,
                15111433682671365801,
                2562375725920156896,
                418600364761538104,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16237743594131728934,
                18422794985691474010,
                902772989523290463,
                3270292125161922845,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15024865146878522784,
                16892778474832636634,
                1326152835865646085,
                1865741661348124003,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6834425045559871562,
                6432609187593056982,
                16287068401633135732,
                2370089627554384815,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10502143762464057150,
                3981302264492235888,
                16802329449907975138,
                2760223066450215338,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16263836029223495172,
                17593477422526739050,
                3870132148282376311,
                1313008941312308539,
            ])),
            F::from(ark_ff::BigInteger256::new([
                18193438021423332917,
                12531551692765802405,
                1437620635234862429,
                977359511522815001,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4293560904501669250,
                17001995155758551639,
                9955790510209526,
                1106389382913717262,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5552074399429118099,
                10843586903145203139,
                8344468887176255470,
                3232920655317250887,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7650873635505690472,
                6612066590712535227,
                5603717453942125177,
                2021144465123074278,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10192280069615923830,
                13282877633340888550,
                14659317681649090896,
                2783121629714783472,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2704554347743318472,
                14390727469007930249,
                13377612770952912807,
                699258268151145845,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9723882731732721084,
                4591025707751172506,
                9293207678757101887,
                1809117292506813867,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5456631118313051952,
                9617411685529252269,
                14061628105109112412,
                813053022394327947,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16005151177913529830,
                13478379902945007403,
                10790654499854942337,
                2835342186958044316,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16599548004969153135,
                11484430493347020237,
                16235211874008064436,
                2296278623188062679,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6403031367505755659,
                18413156369027438611,
                8507315706555282217,
                3004678883874086207,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11704646922979993541,
                18310767207274871892,
                17022498567476338995,
                331152990332393489,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3662562929019380075,
                443717120238742919,
                1978626380253190405,
                1847104396159532968,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9664685384377300839,
                346779175775074586,
                819144536106605356,
                1311460738278155048,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5266900406604083751,
                15158154992045472287,
                10795243930036328282,
                2586344211149043000,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17135515264995648386,
                10644524604436128860,
                8221714294629452334,
                1975953521322218178,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11887186204783315719,
                17543969514373294845,
                839588689032015903,
                1634541925025163596,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3222144743068279043,
                16791882972713959025,
                10936176715833901298,
                2016163208991225069,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9819022174107624022,
                565499345448022259,
                12594245822990319138,
                3007085566748433002,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5229353124572967909,
                7241467411690376679,
                11389883435348241603,
                2071169220673591428,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9316076684012954660,
                1329854374318127537,
                7165076163448714164,
                38178687507224373,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17962978150927875267,
                9194778546523511451,
                15664748983353976628,
                2238879540056211221,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12655464008359540876,
                9002436914129696514,
                16263672063983071950,
                574560470162592714,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6826655223886594713,
                13777093273188459162,
                13125573890212108783,
                784715446239451290,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16990982484332678603,
                6825350663472880280,
                2080821020206642299,
                1564552881925203124,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8563306579353844499,
                17044742967818278989,
                10148522127506295663,
                1377217638329012196,
            ])),
            F::from(ark_ff::BigInteger256::new([
                715079069185511149,
                15607299606397900361,
                7020165328359263039,
                2005633245698014657,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13323064770163573707,
                3187215729822138516,
                13794943481067190054,
                2187153920675527146,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2206499066129939661,
                10702068064452398045,
                18114621283551325422,
                2434351606313216548,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5038599643594898115,
                16283072456507963503,
                9911168222489823803,
                3383197124317589140,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13817537085647017460,
                14017242475208653565,
                11040204025567196889,
                2898370764274153012,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8786831356456682176,
                6490853002486576874,
                5598582557877142737,
                2840316675456472862,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2077556076926755800,
                7274974249931564199,
                2424665852270153359,
                2651071890478681496,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16297896100950749141,
                17754992100344564695,
                5614880076917758196,
                1146433771055704286,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15907032994301289862,
                8680506649076385367,
                3311069393965070615,
                3390012910572033361,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12046382801629938641,
                9223720334842563689,
                5216880923705137337,
                1731408581363425962,
            ])),
            F::from(ark_ff::BigInteger256::new([
                558101868750374659,
                10901649215144583940,
                2143199482430493316,
                405518279754887267,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1898175395749008347,
                5433116702632317038,
                5721090937505709722,
                2956795275408742932,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10148847297631525903,
                14298002579706652874,
                10685371164479767342,
                1785783517566958212,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15976989346376715921,
                10431292090179264263,
                16020255897807842555,
                2231540314171358548,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16926042244243440325,
                4797570448378962591,
                10975570729277315777,
                3077248415924354468,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9272532033233659884,
                14640139051272039110,
                486004297383919164,
                2674505357088456970,
            ])),
            F::from(ark_ff::BigInteger256::new([
                667562640411748423,
                17656207208130591980,
                11894536947009541757,
                153153407239427717,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7342528179744302585,
                8261927155607775264,
                6066363681346802252,
                3150592825706530259,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15425736699765187400,
                15432182359362040590,
                9634289029898541951,
                1510375236742045781,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17717240512433307588,
                12382741681842488277,
                14400141799142496186,
                1244533253857694255,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14786420058107810118,
                2486012305902857832,
                563192991085664383,
                2358767603593280725,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2555353005849291871,
                14520680304467434087,
                3655040191550958158,
                1749650612748300173,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2725680987478640911,
                9881257836992718155,
                8611261798327268196,
                1442513061023746368,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12706355545358441999,
                709067032690044989,
                13523873994021492664,
                2674562255190844668,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1170562556926537047,
                9387873542876949412,
                9778877697885783696,
                694781280003671289,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10356317472489279863,
                198164014828633208,
                15411853293173508089,
                3377174753580593693,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12221873622886052361,
                570107660347820323,
                64926921374740751,
                1155790237333752516,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17472520561191636496,
                15649998927257283378,
                2282951208989053242,
                1551258874060752812,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8322170973091020713,
                260542673026651424,
                11921880802464974500,
                2263238434757826355,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3928168643381797506,
                12777096639037247663,
                18036450441334974425,
                3032710467517778610,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5940091364259424089,
                15199513830590322606,
                12188122968938938315,
                454856190466218122,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11499539091985520111,
                5433376839856007843,
                18333513302983562070,
                2054567187075298001,
            ])),
            F::from(ark_ff::BigInteger256::new([
                97092747339398234,
                393629131411360715,
                5346600750251632941,
                3334397277834653640,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14199978655750209874,
                5872270197926136354,
                2392397288077736605,
                29327745676733824,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10803110222368162441,
                6602100163539287001,
                15973295552626344104,
                2494487103842483529,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12049450079672973458,
                1499610372910350511,
                3701079143531548714,
                2794957912140316168,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10238845026200335060,
                16565513183515384757,
                10313477431117897974,
                1494158513149646161,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13622503974362166172,
                1338055962086679151,
                13017864921982205493,
                2521460244501781272,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9292400982397267716,
                12112715294563162141,
                5406635853293718965,
                2146550177512915220,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6864062275870400338,
                5439838338159924322,
                11840022879670841065,
                294192532230367378,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15372699068611513910,
                2746975454290987176,
                15362625370884062061,
                1427217617950375214,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6073776033568174558,
                9361861863949414124,
                846210010031077789,
                2663028392278614045,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11579813033612237292,
                18000025437863265415,
                14714710311223639453,
                2801350684497688746,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8234669532173009293,
                5076197827002726986,
                14468473905150123241,
                2463329303207606064,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16567596445862670215,
                11265993697245170922,
                4845702658682774189,
                1859206431495849374,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4960829401210427534,
                1161859362188049451,
                14657340211865425923,
                937217673847654082,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2804559793038330035,
                8967238133004860680,
                12509522929718229671,
                3321281687673597775,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3506609168511945428,
                1571298000456384868,
                1161001239080083079,
                3328384740201884130,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4631534071351995996,
                3130042000857527521,
                7798172355338194029,
                1759021899207628979,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13714192687864479676,
                4086107550219853749,
                904049295177050135,
                2708897313415884350,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5034964042796309652,
                7208172936747808448,
                6551224483724469742,
                17642268484510833,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9168130404202433068,
                14391182570202049213,
                16655240790953373603,
                2168452813240725729,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6135897820382221760,
                10932299511474766650,
                16750444163002627052,
                3244009616148958577,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16780958480789215870,
                13241066928726231607,
                1958829929889864308,
                658102400475586394,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7544644261931180095,
                9694803395650310107,
                11828977855173880585,
                2517050865884954807,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9965140703737037505,
                8627784905033058222,
                14935081094439092636,
                1977103650135658395,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17777856568654429064,
                69115876548088110,
                13895217575088440625,
                1885558710468924443,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8026667978596038669,
                18025802942949487283,
                4393523229495752677,
                529912305905988419,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2373712463551920170,
                40419486155625829,
                8501399504431122227,
                508775148770333695,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17850544268345990649,
                1356492181552164984,
                11920357435421247499,
                564543352231026191,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17732247719921346282,
                110429594433447336,
                18266614557000869194,
                72300904609722663,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12665794398351579190,
                10635176509303480379,
                4309865748825378946,
                1606793181056336615,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1602564693740289452,
                13461778573439634740,
                11654349746564054302,
                3017070625289219887,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6775550742932848520,
                7232556459338105813,
                15293727634322071006,
                2870359532045721037,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2191676411163154012,
                14188980846783184924,
                3281642779824646717,
                879018984202020912,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5147910321158089171,
                6963429670419143582,
                899944629434382653,
                931354045636885758,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17823736764521457230,
                5567840710469921409,
                4762264076921975797,
                506135577383394948,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3742808427586199014,
                9215074577166815191,
                12158014354339839913,
                1256313198643690184,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11771559823122742977,
                11725822275091475159,
                15005891642203852019,
                3066611515293784144,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2275885222509263121,
                2658410061976671521,
                6630994101438701983,
                2679334614567922470,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14452144478598879464,
                15258512927861436852,
                6556841453139703520,
                2588829211575860728,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13288896296502296830,
                13519727335076679735,
                16443519493734049416,
                999505619927415346,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3258586430183489165,
                6513823038984435880,
                15779294177455016929,
                2538285663097826546,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12932926348027407177,
                8514996673744468188,
                14742974629169415338,
                2808920225655275626,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12347946810488985688,
                18126665109205789080,
                914282573409017720,
                539684789278612774,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9307649253991068192,
                2905124209976625822,
                5113927732038103075,
                2512360699419480521,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7606065258693432245,
                3043771768029574017,
                17762971383133858470,
                3349626741112857379,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8976981430838504933,
                3084328002413046650,
                10752093861878190199,
                1868446705115834274,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8827872760601147833,
                4089516512360622309,
                2706367354646141825,
                348023135971428985,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9918589918186487170,
                2753290213154606904,
                9700807425053021240,
                3304839196176196147,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3544286642378843821,
                9604265384376166547,
                16415649860362341863,
                856785452797530508,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4878682579464919950,
                10778373429061253107,
                10419692577680640222,
                2880772293715695269,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14883155388835490337,
                7660014151827835463,
                2589246578787300225,
                1667004843424721762,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10485074420627133608,
                11231105035140347302,
                607684178599359814,
                38890595478823748,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6350966913586422618,
                12596422782786001948,
                1980714960281370335,
                2855570347179357473,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5302801585541741638,
                9334132306806162880,
                11062621426954455228,
                495100050319256472,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11317202534840451224,
                6019764220341142411,
                6213695151964069976,
                902616973878655896,
            ])),
            F::from(ark_ff::BigInteger256::new([
                18104240150867935764,
                10081453304936785108,
                14545202406243108775,
                154103394113049024,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8091082856067597007,
                14989440673909469261,
                16262403639626296808,
                3093775120579470188,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17196814124969570959,
                14889605650480971044,
                7998952524498276490,
                1099546268937149061,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9915191052169667343,
                12594816010492116385,
                2445084432947846999,
                2185986527551798803,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6591555586579023621,
                12111489287731424472,
                14258936699048990026,
                443561634051967224,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7426387145426791003,
                4038644305682551116,
                4197907320832298186,
                1577857696048729256,
            ])),
            F::from(ark_ff::BigInteger256::new([
                799619197229694178,
                16501438978644180303,
                7091334841749902859,
                921216165744037787,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5802599454348156555,
                16851946975710992407,
                12953597727726236953,
                1729194632653474785,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16062296154908162534,
                721254555189727103,
                7381847896885463466,
                1141061449608519303,
            ])),
            F::from(ark_ff::BigInteger256::new([
                893739939826528105,
                10638023624623270781,
                18208039665865191294,
                2319710769210649225,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8456613849313255892,
                2169755747834190015,
                12173007617275162466,
                1835553320805049434,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14618207698175560088,
                17922238372432825237,
                13452517108661373056,
                620202089506163163,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15353062850321088866,
                2722716492429541163,
                8667916954799316926,
                3183438995602932885,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7909271452390794359,
                12944245046300690101,
                3488186216606598062,
                3140741450525589529,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1816162163787658485,
                333139484213668197,
                6626143575809120839,
                1697699458193663712,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4197096931400891070,
                11536905259249700002,
                900000357823757163,
                2472469665416042475,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17760641771441987284,
                9020795303588943198,
                14810913235611952793,
                68378757716634850,
            ])),
            F::from(ark_ff::BigInteger256::new([
                995781693979615798,
                12800305588932104595,
                8384143398912930446,
                2699194671310359187,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9162589584059986287,
                1965387952315885466,
                364989251829032492,
                1127792594716573075,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14622967807748399917,
                4561779121511535809,
                3070533965386331084,
                921963924097010617,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14318786544165781509,
                14555911226851084199,
                6504053097259470017,
                1314988225872363469,
            ])),
            F::from(ark_ff::BigInteger256::new([
                18190830066322282849,
                3029094381716487353,
                766155626608100897,
                2456187414414640619,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2698292005066866234,
                4768360826388361270,
                8708966900101009037,
                1433136053592505769,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8913958921745102851,
                1211133257852485155,
                107398366157874840,
                1422782893004039972,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2624505532233875228,
                1552175927697057520,
                3725818710255277418,
                1435849026411326064,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8112573807588553287,
                7805883031253776673,
                14387940292180444386,
                3457082890330222481,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10042554985751975998,
                1224228834033945620,
                186698906867442698,
                3057370570898719914,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9678782878642274872,
                987698603130135759,
                14588526239320112144,
                2871136154085890622,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3119482894658991229,
                461957760427518778,
                13554937489569671692,
                2207050396109831137,
            ])),
            F::from(ark_ff::BigInteger256::new([
                983351883926888436,
                6451706951891363099,
                6341998003920461870,
                2119404891936958599,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13105759929006020581,
                3342936913015902524,
                17292064486203182477,
                1985563136412271054,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2382260878105180989,
                1630171151453192137,
                6923376875542412787,
                3260839196196725674,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7923320963099283595,
                211683229560694007,
                105567340194908812,
                2657619617129485153,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6053254318775648333,
                12397456178866244416,
                14617557948459927827,
                2596592278107493772,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15082096257356328158,
                12717060698662334087,
                7684123570446517134,
                1025816642553881313,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10443848006687343592,
                7841038406626714944,
                1855420743339121200,
                2633617301657806616,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13019430682531144173,
                1307971122034593138,
                11419429386401709557,
                2774178745499950457,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12502869881272974009,
                12853660623054765300,
                10457005910627005486,
                3159582172700391380,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13968011204481146644,
                1129186964027585381,
                12527381691140933157,
                2071848465666560178,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14239275107685945020,
                12705851650919200075,
                6889568234612109052,
                2458732045465872665,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11474086194157529863,
                10213910615103222600,
                5031146393216703980,
                1795763923808154127,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13711110004233165387,
                16540137009740368017,
                5400870080641314024,
                3401215824596759159,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5271808335155952414,
                11359644512603412654,
                5706050698199646616,
                384699067374061035,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11757012680677466756,
                5846668706727620642,
                4088258381987369840,
                2216052725708580933,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16315452228880627791,
                521445222465788807,
                1894741712330745906,
                1604867640983190201,
            ])),
            F::from(ark_ff::BigInteger256::new([
                696187471195312976,
                15984249149071537188,
                8376366754568567739,
                2497117073446745642,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4648572097470589751,
                1894280351529304933,
                12660652777428855990,
                1916600356694312282,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6903011292502104745,
                11927826650371398113,
                4118493911112270001,
                1060708054471511820,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10410024420893435477,
                11852402974287219450,
                16048618655430510852,
                1029264983922683733,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2784964571090799462,
                1971389803885454054,
                17971561464950582689,
                1720298940675327743,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12926193118535396001,
                8032329903615359937,
                5919603748155112538,
                1008732655425760121,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16339852437123318544,
                16711728421966113745,
                2568234368800615335,
                3428520657384991583,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15565624692727443772,
                11370006853679227681,
                11599167924102892472,
                1823007278698849258,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2597661896225567695,
                7367152409442465537,
                16601050728498221548,
                1650597873283513531,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10280102296384106210,
                17039614165409150140,
                6793655455999555743,
                1292265007774637937,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13246573318953665845,
                2097934423119114381,
                13818768401173151786,
                609925876087662849,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3566231433179458677,
                12134777001044029797,
                11879843673886426458,
                3348658724131071862,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9872059900586187374,
                11836825698013245170,
                14081307860869009337,
                1870935074916552115,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3881152259475899646,
                8848705593362905767,
                7438184830517708949,
                720251632214931535,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2409778980671408177,
                15355351532485591863,
                8806229566705696861,
                386805699293537774,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3068895054196807720,
                17995103652595355330,
                13157565211708551254,
                425832287643995451,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16156871274148997391,
                8442696600114723735,
                18070527774628944176,
                897098786884600434,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4394690781955887290,
                3809522480138714561,
                16363219799496572792,
                1616862328688252077,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1098629843567738702,
                25160524043743204,
                3095349112791341768,
                2914210236684139137,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1554813274715572984,
                2484658829003483051,
                6805483055891296510,
                3416045189210918409,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6438909455072867107,
                4348970995185717025,
                3393890651460158356,
                600344085084117114,
            ])),
            F::from(ark_ff::BigInteger256::new([
                117326108189125341,
                13335187345106126440,
                6566534871715994508,
                2178995837842256840,
            ])),
            F::from(ark_ff::BigInteger256::new([
                476799418849928970,
                13555127756175981838,
                7468971945336619413,
                2211957325414238777,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16422992482188253630,
                13559605637535572724,
                81958123589033587,
                19626270175203672,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17429456561100405354,
                15457081321323729477,
                10715781501741310513,
                178718896978857619,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5616068083159266539,
                7088186523385153661,
                6267973693654858568,
                3336146571822654693,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14391738556052877661,
                3351315959242097955,
                10819794276450821223,
                393639123787717027,
            ])),
            F::from(ark_ff::BigInteger256::new([
                199640097937667293,
                16951430656471498901,
                708156101060086564,
                1721575600515404498,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3872220638258216515,
                5390514784083892419,
                18118147940789894972,
                106410793382096271,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11236765843945837235,
                2338364673821931169,
                7376395859306182271,
                2933894713956027188,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3946263779421520726,
                16189576752860152836,
                1094306221573185696,
                2256957534191035079,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6254416550793095694,
                12065791723764112190,
                3720068691930334422,
                2968518946153285725,
            ])),
            F::from(ark_ff::BigInteger256::new([
                382722323696192734,
                17923982136456373634,
                15518057872026694011,
                24346055816689991,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6179808872146004182,
                3443451085052057667,
                1091597071115187324,
                2204154362474743125,
            ])),
            F::from(ark_ff::BigInteger256::new([
                591930442443498393,
                5257615547987756874,
                11037371462176627111,
                636271720535160103,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12639088818501662370,
                579170741523076931,
                975541331305637493,
                2482394740327272692,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2703932587013993287,
                2154620529178804272,
                17187781344644714452,
                2407199614836169698,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9504768339969548768,
                7327157111420737056,
                10161411287027315003,
                813178646448907503,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1704725592881940649,
                3865441487247587995,
                5912117591468286761,
                1757958856118777412,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9459923155825245217,
                17081573254317680803,
                6373082383777837721,
                3443367890667502712,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1429431556460817263,
                2616537794181182293,
                16750347372798865270,
                1897327274477059596,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13196972724674877986,
                12885149298859652407,
                3405126947689867215,
                1540527448468882069,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14944889994045952530,
                4806034896995353654,
                9143468911143639184,
                1394281104644865859,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9537883584623772543,
                9095144602577064131,
                2935325370497559036,
                3295392363422053376,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4523617413248078585,
                12660811206054587446,
                6444276763981969080,
                3221098374490409689,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17149686103071006884,
                17016814295160082616,
                3291389779750802061,
                2913419478483938617,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11487486161351906071,
                2477603976070788845,
                2122663512359110895,
                1616359240974539008,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16930366170056642401,
                8693495037658785737,
                8789389521987493096,
                2145330693763071608,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5869514183245501788,
                4744001985503775668,
                4469343821658370028,
                2240250482568321366,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11495246387437169429,
                12536166616651920928,
                8052159393478390047,
                2945077963461729456,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13444762648182296502,
                7041104639380930738,
                422245866830067236,
                1777343572664300833,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6322046557747853723,
                13681741489133891203,
                2398103744045423235,
                33402618925715577,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2808031462072383710,
                13631890619901918602,
                4390793184507140811,
                2744417833297643269,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13640882160799274628,
                13826539374248061980,
                11767125246212927189,
                2492326970447184932,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9461589199233317971,
                1142263363921395876,
                3748737942682022588,
                587792869476178320,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8481286296855758715,
                1693529869687675137,
                12029691786103255084,
                536490889962613213,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4920575837043325301,
                2489653716427734241,
                329429872547614532,
                992069400043108881,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4313200691729817505,
                4282906869703074982,
                15778987924818225826,
                1172186018462553017,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15345891467945654893,
                3867014859245537740,
                6544332666179098721,
                1487472991540947280,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14507853539352007658,
                5544828906900945802,
                4927495602091536046,
                2335278840114094672,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7029133902335862086,
                4592914295985775880,
                471869468580136130,
                270962790436696923,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17538260852303490582,
                17379532802567591582,
                9290863634015243642,
                1341751056055413182,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8020089758495392088,
                14406137712554159906,
                15973293606649919767,
                609306385831714188,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14840217095542352558,
                9257294275206954561,
                6015799113528881033,
                1198162776567896624,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8821551820927401003,
                17118889602510849249,
                2458757982524350652,
                840878026586061792,
            ])),
            F::from(ark_ff::BigInteger256::new([
                423681465565856690,
                12505880622437525040,
                6314762058836724097,
                2324421931386574912,
            ])),
            F::from(ark_ff::BigInteger256::new([
                18400984629131561538,
                12143802040470890047,
                12113847916775282339,
                2288556128529881689,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2149109936479176567,
                12632494227184979611,
                14716550656962185950,
                401425749729011208,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5842377851745384923,
                2575092191492760443,
                15592792100033735141,
                2817995356729886433,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14300316099252299090,
                11316437533194542113,
                2490508111113931302,
                2641994582893817951,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8198265461767770669,
                17222443192590757234,
                13105491890294860855,
                1947532565177006933,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12028605294609681203,
                9557402890629176083,
                7647275937091759385,
                2176851410734298828,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6845239842344780313,
                257934448267221984,
                13737825285853805065,
                3078522170768255463,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12299694883493967462,
                6835213166117307143,
                3917787577448271324,
                2443618388449531090,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10880088138996362510,
                1578291237384558021,
                10759731445612977185,
                1764566454723654295,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17251867507657044882,
                15068375376608099817,
                12159834410614334457,
                1788772962343760790,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6547140530132431347,
                6636829296379413070,
                14110957344627256184,
                758727309671882371,
            ])),
            F::from(ark_ff::BigInteger256::new([
                836247177599233996,
                10771028478503903754,
                13602943462912597061,
                2912593535212454536,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13169689525876440850,
                9917397314878791393,
                403884544422869460,
                2039671962440489810,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13855625559155747517,
                15390022280426872997,
                10644398973516118981,
                2815900379107188902,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10036812317736338942,
                13100999690685781066,
                7671035059899275521,
                1577165619657543387,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6268216597894018609,
                10509301830233207465,
                17053055159630948336,
                586989203767460391,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11450381281227015642,
                6810346924851226400,
                12453458541704533982,
                867173391814298161,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15058298508333922042,
                12405486710207695387,
                8138628854390102255,
                1808621426456265923,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4956070612760402497,
                17623559151472184313,
                11461283454738171250,
                1567083002667922443,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3666958996342846389,
                2318565848962116848,
                12839180896431798651,
                1309987783776311305,
            ])),
            F::from(ark_ff::BigInteger256::new([
                18102666697454920809,
                7647663123217252673,
                7299896866439765887,
                1299096081683677336,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10160322165579122808,
                1999299273258127606,
                17302675634000015711,
                3291493305901079757,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3461559254994553713,
                9264726923733682682,
                3989763752333015565,
                1455593751335809451,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1111791571522786348,
                6603763170510413983,
                17857980655345955894,
                694988478712038996,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12402746641956106587,
                6695126419343296071,
                10213147788120760016,
                328358078057031762,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8289567966431479798,
                12176128190929587849,
                18379357245180882634,
                2327137188658280263,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15044732851677238030,
                11240790778642562533,
                8371716173311688759,
                34306678713413061,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2304405773171516282,
                5660265790539089949,
                3484298613789327169,
                2874579330228239138,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3838037586359098714,
                5145117792904450490,
                5021022559885686311,
                1537728501551285582,
            ])),
            F::from(ark_ff::BigInteger256::new([
                141855476178904414,
                17140063748180889425,
                5402380732015242314,
                3344561839889957604,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6289602781623270565,
                603601919977935428,
                12795620304451704210,
                3306099302010454055,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9705560606905292274,
                17565373170207392458,
                11511999084953356008,
                3322716373058547232,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14274012509273510760,
                17310984786976687319,
                8182604277245181066,
                1534389942454422373,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14775081822666847165,
                11935951900107415749,
                8911565870330719922,
                3018976601629892062,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4035033462945924080,
                5786231448198629337,
                15535443127979214465,
                228894145309184058,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3237073343415081530,
                6757650222190987472,
                1179443769863312854,
                2890712934310260595,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3738550490196902031,
                13476507247695708561,
                4792221829781915626,
                1116460253774265009,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6629665090184377779,
                5138641738438903975,
                5607888232607482261,
                2876182075370001786,
            ])),
            F::from(ark_ff::BigInteger256::new([
                901384451881878059,
                15246281788819280296,
                15759853972739976650,
                392005943950245373,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7978949502183222270,
                9893012150221179381,
                8693094959948645837,
                178588717308719140,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1664558812333872406,
                4851866144027328985,
                5289096332349261106,
                2148466077358587151,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14040181210896372290,
                13204243756531071493,
                9700180600906216421,
                2307606929022154080,
            ])),
            F::from(ark_ff::BigInteger256::new([
                613366017847684278,
                18392539816612936000,
                4862478213326683812,
                2817325068929932436,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8544272472521490318,
                10191117095077605377,
                11136312168624110407,
                1972716081242986509,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5405594875702820362,
                10790920899813540216,
                11409238778696425444,
                827094649055027578,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2553201372643380169,
                6456888553591263088,
                12358102873760872771,
                2930850555430155077,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7795861947490417556,
                14166281746664921100,
                3428135938416006421,
                3426522826147320061,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3381279351716540862,
                10746597433565152265,
                15200695679029687559,
                3044136541704433431,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13369159122267775216,
                8587527162063758962,
                12836500819954448079,
                581416340199908068,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7010506858912231941,
                14428849206212776208,
                2826468867708169356,
                1894799849752575650,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3282443314623992254,
                2666208772490470961,
                10080084936909272620,
                2118269537692646280,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2595954976238247823,
                6216191076310657710,
                13359699903787763552,
                2946193713949156582,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7138720511377844840,
                8678118235303098310,
                9760032187384370821,
                1221861136382815539,
            ])),
            F::from(ark_ff::BigInteger256::new([
                371312643306013400,
                4348610161396044232,
                7372477583989173770,
                3085923251941609592,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4368538721263135873,
                925762416559672625,
                16446358012686184079,
                212192728806352073,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5167863119358043726,
                8426643867399244311,
                14643401095315632256,
                2308211800383520357,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2249003809888964726,
                5011447387945468508,
                17038447948674346909,
                1077070729918921311,
            ])),
        ];
        let mds = vec![
            vec![
                F::from(ark_ff::BigInteger256::new([
                    12260085014100487254,
                    15115852496701773162,
                    14918758887244068681,
                    112863918757284477,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    212627258154859181,
                    9941406968500905213,
                    15689897025534483386,
                    2273537833634474802,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    4139066349783842292,
                    6646296118586528850,
                    4768592091888506564,
                    2525509219902385984,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    13275196821702353716,
                    3924364462601223371,
                    16544301244798973336,
                    2333889771329696090,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    14859162044754297967,
                    494666766479801700,
                    11319720503231673470,
                    889265636650563258,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    6656801688331807632,
                    11235623431567322139,
                    17600703193002326225,
                    2838438575480021984,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    4869460402909364901,
                    6558544306104037705,
                    8424099473773029610,
                    2647542704999703089,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    10019494238491625102,
                    5433848205853062365,
                    6103700914641928781,
                    1733826587084767459,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    2789741066657820347,
                    2000351555797916587,
                    5990167486166164287,
                    1185130391945779656,
                ])),
            ],
            vec![
                F::from(ark_ff::BigInteger256::new([
                    14604775297975993369,
                    11333041108180549894,
                    4368129399032342086,
                    434541907318180215,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    18294724710747151737,
                    16714724681736668022,
                    17531844691882409533,
                    2707041657238828501,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    13989884495871779905,
                    15907254440357433650,
                    5070011504557080197,
                    665687298071522483,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    1144148549877124806,
                    318685301192417182,
                    9068147522069265273,
                    401126969663586598,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    2940511423187860440,
                    3710362899429185222,
                    15673773457594603349,
                    3212338219223160039,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    13241678661736665230,
                    11623257850286707321,
                    3347705666949192880,
                    3181557296524815683,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    1206411094816182673,
                    508420081562384361,
                    13065410396642280725,
                    2486223202235162374,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    11002189009779021479,
                    14736713925489155043,
                    8483915817979050437,
                    1953359451293855382,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    14564664435037463764,
                    9529957679984527877,
                    16325016653844097698,
                    1657661488995581114,
                ])),
            ],
            vec![
                F::from(ark_ff::BigInteger256::new([
                    7951398568912335884,
                    8869889641676338874,
                    7908872753950712538,
                    2136154073861940357,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    12620429430556986923,
                    5315028616013574827,
                    8759846306086720903,
                    3149236977874173893,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    13919400371396645727,
                    5906186376145044328,
                    7719143280340493866,
                    2668411370749341238,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    4074771145058991633,
                    4328744037481570619,
                    7240672733911600015,
                    1964538780799688901,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    723933641554629106,
                    12273674172600272882,
                    17165812576361957987,
                    2512199483780732066,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    10156595408889687172,
                    13903794940656720998,
                    11068829831367458409,
                    832736587020437255,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    12001192427075678964,
                    14773276099687617068,
                    7244381738219993810,
                    146416836477816494,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    13962747142309686913,
                    3597772845329568066,
                    12519615695732795415,
                    2845059109816345910,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    1057908941480297537,
                    8034323442509105705,
                    6816716784129085671,
                    3473766934070763712,
                ])),
            ],
            vec![
                F::from(ark_ff::BigInteger256::new([
                    17013839005187141116,
                    10306047814235021221,
                    6341551380358509200,
                    2518425537496777190,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    7814764238085776783,
                    11717404640892433013,
                    1473878543557880397,
                    2043762831195192648,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    14503315042416009405,
                    9425097410919368559,
                    15262305795779391410,
                    3078471832307640860,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    14427939661590488226,
                    5613788301320464843,
                    5575330075073640834,
                    2802524660993500226,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    16821378841011070166,
                    6071149294907532,
                    2436739437961746697,
                    841565647215087523,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    10817741983298643189,
                    15879691037545572910,
                    4207688021718599990,
                    2551038900218189318,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    10876499511372734195,
                    15532190071049058889,
                    2120318911993323039,
                    147899719125705872,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    4793506377511301550,
                    7527586725053773301,
                    16690433456368553987,
                    2127468591815148132,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    15668683898831043406,
                    4403681684765559080,
                    6927969149544677259,
                    1398756311951539481,
                ])),
            ],
            vec![
                F::from(ark_ff::BigInteger256::new([
                    298776373186473812,
                    11144794132546739112,
                    16782119216958255089,
                    531289420655606669,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    107779541586224633,
                    2188817057285529712,
                    5985187415402316856,
                    1681255778740181877,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    11421195526648282702,
                    16073598818484365593,
                    11572295587436462522,
                    3010039075799045496,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    10340459013552791056,
                    8915174067938836394,
                    14261895264569898096,
                    3219940486337077708,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    8011759723686157154,
                    15051507490009599348,
                    12393330635546438325,
                    2664204170478106804,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    5984391267147583972,
                    9958597187001860052,
                    15320005350091977939,
                    180317699959160188,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    5780663385327427484,
                    10297839776632880574,
                    17967507082495608814,
                    2769879282045866037,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    15235782546781028938,
                    12032357491681637126,
                    16145289486311297584,
                    2187312093427751447,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    16980895921157827241,
                    6174250601249817367,
                    7160056597323061133,
                    2526768101745054657,
                ])),
            ],
            vec![
                F::from(ark_ff::BigInteger256::new([
                    15232215744839381301,
                    8852783932308666017,
                    217202872842800055,
                    2343207276248673720,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    7577911754728241123,
                    17441644548141931398,
                    3487460111499581435,
                    499087109183731258,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    11175988146250693047,
                    2529292052392802744,
                    12956170016776401423,
                    1379320064097340663,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    17457782376955745843,
                    6839391313470892858,
                    16225918042888364559,
                    3355813548273508935,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    10291519874243211819,
                    5858453992320594716,
                    15418926230808856778,
                    3302638857538705,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    5442335102949368264,
                    12614469331880209397,
                    9309235909116840519,
                    2738481315502734148,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    8608007240073245977,
                    5888564880127623687,
                    14862208948961780554,
                    527383658599047765,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    4487680833707105727,
                    5592101375766193125,
                    3579589876170908863,
                    2941471875664758523,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    14035316669023563268,
                    4828086879756041258,
                    9765563927160468101,
                    1735544983949832231,
                ])),
            ],
            vec![
                F::from(ark_ff::BigInteger256::new([
                    6569184174354737442,
                    13176823256242748016,
                    15189447931767505429,
                    806201825492421522,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    3259354179305690830,
                    9076276871397890644,
                    3146473653394032030,
                    1444773273630386075,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    2886478710045159407,
                    1628715579611622330,
                    16216154090391887018,
                    2096621729053454978,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    5097369541361116041,
                    16522276673597798602,
                    14387510071356665547,
                    2217674474520284976,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    10704383844230613435,
                    1883165267800084058,
                    13553629956915823935,
                    620351822438362297,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    16386707849697459815,
                    17197075862236908315,
                    5360559559101197730,
                    2850888766107913422,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    7895879872466244476,
                    11897612148816835788,
                    2720477890791011647,
                    1557724165172537199,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    2225666905890601642,
                    9995206516027266886,
                    287390480265579145,
                    2631993910243445152,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    11523824358127049488,
                    1134734470102924096,
                    13618795702764599756,
                    1233700508269411854,
                ])),
            ],
            vec![
                F::from(ark_ff::BigInteger256::new([
                    4470576814709902154,
                    18161291680640610107,
                    286385760453393278,
                    867395680136811378,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    1259315862594998688,
                    9116480613917662871,
                    14741059164592314452,
                    242806030114534449,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    1753360292221749901,
                    5275628017591269977,
                    2903709650471345116,
                    212617589072758769,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    7438105167744434947,
                    7582704639746391305,
                    5991429845823359808,
                    2569070662563260138,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    2915781354158865964,
                    13104001237896876071,
                    17971332699480897107,
                    1010972157656519933,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    175469971343266546,
                    10429584802638678258,
                    7087125206064719068,
                    430752183736384777,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    9582907956989028300,
                    13487882234365314823,
                    7642891907811108625,
                    3273903686526516573,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    4152557576435169894,
                    1526388561284955961,
                    16705746783885241302,
                    2391823427372880799,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    4575168928825116650,
                    6292629709239964905,
                    12328628354823796301,
                    745927692889904686,
                ])),
            ],
            vec![
                F::from(ark_ff::BigInteger256::new([
                    8109607047099671432,
                    6272337147882764663,
                    10548629457743429989,
                    2044591375865744793,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    8173165631775990731,
                    11737432706522732675,
                    14049946902660851537,
                    1341798675158782831,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    14019120476174267636,
                    11215208935417343762,
                    8047340430417016421,
                    1487821845853015406,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    12824507385883461471,
                    13490877029205551184,
                    8793594004137491082,
                    1456377848217190610,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    16884136202939890647,
                    13814094470836839074,
                    6330137220727289278,
                    2920006927039655167,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    6480092164666689389,
                    16407080449071650421,
                    10566246501330813,
                    320140088416358717,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    11125368731662206055,
                    3256781314650167737,
                    7537492427726722025,
                    614890429263355182,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    17972927106381872067,
                    14999209284536743104,
                    7792001330985679698,
                    1359757369273735742,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    14404906450189228718,
                    11415607189383684172,
                    1709323994527174222,
                    2253988366937057071,
                ])),
            ],
        ];
        return Ok(crate::PoseidonParameters::new(
            ark,
            mds,
            FULL_ROUNDS,
            PARTIAL_ROUNDS[7],
            t.into(),
            ALPHA,
        ));
    } else if 10 == t {
        let ark = vec![
            F::from(ark_ff::BigInteger256::new([
                5320684913217557238,
                10159823466695120380,
                13063491808611449219,
                1015951777167379051,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3276852821979680593,
                3562673401694027149,
                10001598297912310950,
                135658144054277142,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14481282408826518810,
                6843211571699640687,
                6008241540698228597,
                1398348733595039561,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8787548541948450566,
                13629945303440035844,
                14661690383685179340,
                3456140255839320618,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8430697171189628017,
                2295949651248397699,
                15567052965080135701,
                1525748570047895818,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12715139918265977207,
                6704387983387170137,
                159399664321068870,
                2600555947662863861,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12032781244593610582,
                1245983813249872060,
                7134775817107067364,
                1559622551571365294,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9622079574808941976,
                10987209090707444979,
                9788106399921857334,
                918688415987890371,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16806961355696186820,
                12405115185963052132,
                12585025889256845536,
                2284895419890873593,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12527706813978077931,
                2646465494294258212,
                13025923782877559942,
                408795357098095041,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16007314816585715064,
                7923730155794199078,
                9079563143526141939,
                1395978657097472095,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5638299962601488458,
                16419937443392867889,
                11448456739776462181,
                750115844064890250,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13635205866785946896,
                15458917350481232547,
                10693291279244692343,
                757555437935839611,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14083654115361701862,
                13993119288994567500,
                16210538693886358160,
                1398967730214869079,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10646890314590534660,
                13309923822070393057,
                13964257513101966294,
                3053608341001664737,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14474194360695469151,
                1453794495172947119,
                15712864182960342472,
                1972216130457826086,
            ])),
            F::from(ark_ff::BigInteger256::new([
                772141167609071584,
                12247878239557644852,
                4009127999718108699,
                617060179271516775,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2780165798532339736,
                17317073839346576922,
                1346098843095273648,
                268165490362286938,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11686714474718691964,
                2386255410924419667,
                1248509236938436334,
                442459405190610195,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10384102667945096646,
                5024961188735250038,
                9214622293182278989,
                849124532364736100,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5075758552584040272,
                2043699229848839876,
                14540749021327635909,
                1887709042206413025,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11816369913071142290,
                1750780657344970051,
                8847955931748203811,
                1191184150753682560,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2813298336220479589,
                14759824204691411930,
                5430449051953439990,
                2996883262739036998,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3690998129090248803,
                7302983062133612420,
                17844695578583410009,
                2492258808645008710,
            ])),
            F::from(ark_ff::BigInteger256::new([
                368778244770833881,
                694950861027439738,
                10559478936356437089,
                2401216784096954870,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14163717920367915055,
                6305666713944972982,
                4724864186560556874,
                1176583649310749854,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11574446033158810771,
                4741625949894659630,
                13314494873038603170,
                1689681102148485930,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4522682876892300702,
                6920020025985580622,
                8753176362275539551,
                54771827522536317,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3824942046204041822,
                10247059088298810933,
                3222214150285525468,
                1065201516784551653,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17533189039560202342,
                13284447109406414703,
                17030015526052283779,
                1485664740386359459,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13535024300462063838,
                6894223768379655833,
                2398851627063646779,
                2108832574993700807,
            ])),
            F::from(ark_ff::BigInteger256::new([
                683652665202562259,
                8206054993903882408,
                18155159172373671218,
                274631990261729366,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6335095302595679097,
                16877994458325173273,
                11336094111270886071,
                1090499177456332226,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1153063058988700960,
                3995938397138121406,
                16736281819115198003,
                2298766751799946120,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1205969607895763519,
                7477927863790825668,
                906706865685283027,
                320618603002592902,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16102203308988857567,
                279520315017144712,
                15303397173909045118,
                2415491697208303180,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8884899099400833678,
                17222926299248729905,
                10831076369351439276,
                983183837139229388,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2753248038262258706,
                10416359063333884752,
                5738781120083663596,
                1841671709437666667,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11824150499556006668,
                11641488972685963571,
                4667535544859452088,
                1267287194254051219,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8037844740537532739,
                659986188025479905,
                17686300850421243654,
                337905320973140258,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11859386260570704364,
                2614475152038659022,
                9902861103568768797,
                164088897884068272,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3300663616675630544,
                11341615298550221573,
                11437568094281161993,
                2654746287556208628,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15929424893357622875,
                7205611026514235315,
                17698427681093064519,
                2584665029065503105,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6410634620366074883,
                4767194589891511495,
                16095653853304654860,
                887020224714021197,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4841952491973431371,
                17998170324826093638,
                9023082265628308986,
                1682544541595711124,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2700795707400041751,
                8413539290979704185,
                9243387301429276238,
                204153949077155098,
            ])),
            F::from(ark_ff::BigInteger256::new([
                667517897588983672,
                4464453552378964715,
                11893392189591664578,
                1529479801438027566,
            ])),
            F::from(ark_ff::BigInteger256::new([
                619137118390884674,
                1355267554487770152,
                9060122050181173516,
                2547247746323205033,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7058887346127246834,
                7073085387340447060,
                2451008887586966242,
                612742274344136836,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14267232269414845465,
                9006369810752136270,
                13682552313042259728,
                1590069489873353008,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5584924188982263775,
                15473626157460692608,
                6266426306765075138,
                1373818083802442328,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6158206821455810092,
                8300412899157338997,
                16689007809069745468,
                3362051614832174795,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5347596329839097021,
                10182090342506557820,
                5193720307762392224,
                3009816480542427630,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16203383768081571327,
                4762321321133414220,
                8496193366899606164,
                2730147681944688480,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11422957825082965107,
                13155546652293211912,
                5277779141933252083,
                3251886516674655448,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2131614757615552392,
                15248798236125499089,
                13670877325429492849,
                1788347107907513156,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4267549390285672282,
                10754153274257613600,
                3204539892913303510,
                2928173441252330237,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17293181683275383055,
                12225728601706734235,
                929818839057970200,
                172086058870029055,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15384808309060188290,
                6252379217408370514,
                1766642010924191309,
                2285832388911376000,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9202377928412372965,
                12109136250623707399,
                16885703459602510338,
                611975960667774099,
            ])),
            F::from(ark_ff::BigInteger256::new([
                18401639816196630490,
                18142337242473021770,
                4359905796761037701,
                542959273807642865,
            ])),
            F::from(ark_ff::BigInteger256::new([
                231003825425094094,
                2828718074226267333,
                17929671173194705292,
                2043655414123296135,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7147597153529894002,
                17792040114160834346,
                16221559984111978313,
                873619380704633063,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1963436857375782921,
                4974445123782600604,
                574571866478754070,
                2492558372307659833,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7074162247170853418,
                16149409287505591238,
                17134619481195132341,
                423451383050000597,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7669867501641210135,
                10554807563091437407,
                53479017355779571,
                3105956909122939941,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13341019285433116314,
                16827686983172266917,
                4604925957337534620,
                1847981710277380320,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7839892049568030137,
                13089111341487182506,
                10551165100014501457,
                1996882714002945453,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1598892030884482359,
                6288483176336207385,
                11230290807472700558,
                2265904679788960047,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3085823148065900406,
                12576735821052425020,
                8302432424334618028,
                3157913954103367849,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6121946092044962353,
                16811119199600958493,
                5812198360796475532,
                2907173637257832502,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8171047582433660127,
                11765609586678973214,
                16944504349085035711,
                282376804517290619,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17328796128503531910,
                5610113472219749273,
                13539760843440331731,
                2023039750542026106,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6117857172826900295,
                4540841926049075703,
                1397216742186918291,
                335705343024053783,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5482873138948883001,
                2907172531346357039,
                3321668610928093641,
                1329769180629297454,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11534713701678643943,
                7616277543028834337,
                7345066151740743818,
                1202264535544442315,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2129194437360429052,
                3868331016449123629,
                17619160699565683531,
                1168902178978587929,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13137326631516609939,
                18106141887853791717,
                9026174002442683316,
                1219915766287466571,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3501781109321985333,
                9810123791820059188,
                6968590337357317598,
                2092147239785944924,
            ])),
            F::from(ark_ff::BigInteger256::new([
                18323419801393858941,
                3804379949908066700,
                13451280542753906377,
                485252840447994146,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3663322081435853653,
                14348157430408201150,
                8099865770530994900,
                265255257122253187,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16918713942398964680,
                6645504880620446640,
                118069759469369893,
                1139682795432959036,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1342878358175459063,
                10007714697087460524,
                13942305655583357206,
                3263618484430894427,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15371375146457662255,
                13340466343963954550,
                11228033322562065894,
                3368828034701207919,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4523809835882484598,
                8609623157885817560,
                990016313774572032,
                747619405958352724,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10348800972209776776,
                2478551057426728765,
                7094046592613237505,
                994103420664144012,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7561954958454905895,
                17477532894710786901,
                145336703163287814,
                1840907094571328920,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9190387948239587310,
                14710182428802413408,
                10249073789853883002,
                3068382322222977891,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8345036194808273096,
                3464136999688471447,
                11138217237246055929,
                3260980460755920591,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12848381915621783513,
                5062432739484174949,
                7803874975678961757,
                197009277034360339,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8207468327127871316,
                18188261998971970784,
                6622468115515509774,
                1813643365703968500,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10840313196754962178,
                3518715939688972054,
                397053691064495600,
                3151972232795338419,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5023436114363393578,
                1397073421493638264,
                10794884492879252858,
                676509909012263676,
            ])),
            F::from(ark_ff::BigInteger256::new([
                500001630646080719,
                12632947711078923731,
                10845420542304506564,
                1585561801698016202,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6105676115665264289,
                15045249499271123967,
                8863765396686570981,
                2614446384903755527,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16529595388442961348,
                6285496260713322472,
                2454306091105305473,
                2943724303492995115,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10438375242316473226,
                7761515144355828349,
                3808689647773406395,
                2103628159492098140,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3755929195171652870,
                1616081572094789269,
                11588597168852090086,
                1033292764678598093,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3033363005362658255,
                14466637809856649209,
                9275458035115948783,
                1468125608808408515,
            ])),
            F::from(ark_ff::BigInteger256::new([
                313908064544502590,
                16344742588406852275,
                4321736050589188241,
                821630166179326563,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13001921150528208116,
                10542335839630817432,
                5919802583176700971,
                2839316813684390545,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15633712052795092257,
                1711640808039880049,
                7788158482043199034,
                1325838557779889444,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12870396304379989133,
                14588274777588154148,
                2660118712183143625,
                182612977956944248,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1314822883739579244,
                4497851244336216128,
                11762923084437027440,
                3187210595600552477,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5199024614512686885,
                17700898170485551443,
                10458590641030561693,
                2092082972042566509,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7381471240285259310,
                8153098885673395144,
                14122369230227696813,
                369900782649264221,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4451288473596760918,
                13752207878405951193,
                11248149052929322536,
                2815586594642470714,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13102625975640873644,
                6297342182922721693,
                4776800028357901289,
                2841542082840796320,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8499533256690790242,
                12306851350731450593,
                1980470518197886052,
                3108046433968078502,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17405271503849163785,
                12316619896751433832,
                13299312234636129998,
                3403643146020066609,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6145464303346900839,
                17757347011342262527,
                10268533628050310805,
                389316264949593866,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8549335019353747775,
                2509299291777132956,
                11715953878830005144,
                1015301119227255732,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9277131906707666440,
                5337820696356372222,
                8894233738772195948,
                2794910645242345506,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11765452062067370158,
                4175100728399342295,
                6869074322877358211,
                2214538139754972370,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8109111110274890376,
                2765678204916333132,
                11315641415445637830,
                1590322497366576382,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12771017197544863291,
                14130488291717677285,
                16882328605322186032,
                2621873254110204334,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9688043365386117778,
                18436050670943046441,
                18397356359450252784,
                1096466779026115653,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7959514551665003068,
                2672792256823595438,
                11009266074848448881,
                1044826370021630637,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5739375739946151247,
                8016272340615078310,
                12298615965757894961,
                3364540724207196787,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15840221382749269723,
                6576283272228362397,
                2270989126823652451,
                2496135843348471260,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15800803425746084381,
                2346231433750217274,
                1597812541105746162,
                2828465775646355422,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4791702582456035760,
                960592302224565706,
                4389690827853804020,
                1829959175993283227,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14012389707217288830,
                14890070660115662615,
                2667898646407351705,
                1198198827225016796,
            ])),
            F::from(ark_ff::BigInteger256::new([
                751407254966811409,
                14925419316381355522,
                16487646840102943498,
                186134547256943562,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1034925290580275517,
                10296094941077576713,
                15992039170238734790,
                2089705881329017958,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7939806943820323781,
                7528019282374931046,
                15481723886361761513,
                1234587044912710207,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4772378728237888239,
                11124158373755729325,
                17509789343828914941,
                2724589656828226182,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5769665008832584981,
                11518787113629573097,
                9367306491612107180,
                1892264610342238282,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7948818110159103231,
                12566028298219816746,
                8490271449937156649,
                1511921182410016306,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8335119844173435492,
                5848267196560294973,
                4894087549041579190,
                1623535578165312670,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4796785793557037401,
                15974621383989912678,
                8882017696382549165,
                3270944015157791119,
            ])),
            F::from(ark_ff::BigInteger256::new([
                495706033483985218,
                7944507076141314133,
                9929158024073279269,
                2764675682435661716,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17373562145503864833,
                4238288457759099829,
                15214227766332733924,
                1234224966721359562,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2341692019985351498,
                1949417115991917130,
                3448794886949758054,
                1395025235456318468,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7776769427290637914,
                15082077446821869286,
                14215924539918805914,
                3430958266792779775,
            ])),
            F::from(ark_ff::BigInteger256::new([
                18289116426839076340,
                6960920498189974610,
                1711234431891100920,
                3148715541108380403,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6319233305192191291,
                4086138863758263220,
                8545172916858353213,
                2620858426691303047,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5261193288792708030,
                11804311525176762376,
                6749952254808975137,
                1586529580051741807,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9793268251899987301,
                6885183373191667312,
                2460105626162914513,
                1380808336397727665,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8962768035812488377,
                1355744318206858331,
                1044691400690485220,
                1553985836827099722,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8561443462201617206,
                11820604659101873772,
                18446683559544186059,
                1872759374758833149,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10574477373893175369,
                17006182726593240179,
                6954169432729165038,
                2763003373975474648,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5546379633782631345,
                17039250226017610916,
                12837424043541352343,
                160962567671286705,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16141025614599162323,
                10019711630182451181,
                1834903717031740518,
                1979064306422747531,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3827799552722902291,
                4443505784425344039,
                7401103538269054564,
                2127082946976423608,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5986868076417763763,
                5259153351518106076,
                10965966047917129462,
                520740396646276638,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17374945597948788609,
                17711122945960174183,
                17178850977598560461,
                2714555235251822122,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1039985317656202289,
                1983043995291806553,
                17465840852216583340,
                2026353484933451177,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13556293483410932043,
                16163068281858319621,
                11662459575811360580,
                1168483771776820276,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7264426708072045672,
                7696666075804806434,
                14435837947048900291,
                665485777229441633,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15587139822291658574,
                8308569485725958021,
                413107683748214240,
                1356861269940072731,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3695261736689836852,
                4412412300330379129,
                16991759803726197699,
                942527680379397231,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6549575851965756584,
                17933605350411236255,
                2732371684839312198,
                3367199269675511109,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11806723337907065952,
                1547659948191001775,
                6459950022055664811,
                1668021883373593304,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14681340594215857328,
                14356945697602532327,
                6176490184955854228,
                1492393184445289270,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10889999467567146561,
                8297201963253352321,
                6529412243062872051,
                1462730558827443643,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6737460085064448585,
                5908811914885784721,
                5453112462324137860,
                822334920128668275,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3861147518941420506,
                13817437540043954619,
                14856238953536507430,
                2253365378969550907,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5942937079106284759,
                12861950095081220645,
                10534780348282524843,
                2792447247255327123,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10539148611274491328,
                8856828341028380838,
                13006652484892659532,
                71637975160695727,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5878718454377346868,
                3053242096271139599,
                5911656348917276349,
                2656821661551323878,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14577301036073783480,
                1381375052644068955,
                617371571569709461,
                845495121563110062,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14210358171459496525,
                10250024856165097263,
                4862475739794744332,
                1125751302550053752,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4165717948494366960,
                9712075916735115372,
                12025470810451038199,
                2782330647635792746,
            ])),
            F::from(ark_ff::BigInteger256::new([
                429981273402200958,
                13883344975303927333,
                1455140832540768012,
                291149396384760237,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11743510479190210067,
                3675368819561828455,
                3214179667627759715,
                797378282878797880,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7764837433299577504,
                9291437890071469227,
                17683141427527514811,
                2192344564378709393,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5193977836924262155,
                16499735276786048622,
                9446803809721555194,
                1175326694197203475,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13391044788749047828,
                2575889016422992575,
                14484933319053068329,
                3015563828023118994,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17250588962811781629,
                16067516333347861871,
                10672814606379429374,
                1637365925181514401,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9039460022821003168,
                1509619585417630886,
                5396943474469440406,
                2883603145626124011,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2567750854904990534,
                512971195458750234,
                4993640356603782250,
                3153142549329210246,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15916527106940340722,
                14231258120337684831,
                5615962979061330377,
                1046493111421631168,
            ])),
            F::from(ark_ff::BigInteger256::new([
                780708460969559598,
                2241351769395568047,
                1846800471219478604,
                1546693273121251240,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4867732643925657585,
                2705203244053994163,
                13550778846573674086,
                3345364837202974584,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16005050893215595157,
                257205413612115220,
                15510692642500385881,
                743681912026398852,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14354625832602621074,
                142780738219141463,
                11429706162862975771,
                1634538859705889099,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9549686432915331756,
                12170948040227090181,
                15835687644578474895,
                571737429432501980,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11608163590013280981,
                13949538399292666736,
                3147736167170436975,
                152242626280553900,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14750224870966843462,
                17719334994723364747,
                5761152803235685702,
                653490462040585251,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8327299497311957306,
                17483778309482368505,
                14977673976926529522,
                2055364021678977886,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15273572417678183435,
                8410077049178954018,
                11345172542009797574,
                228194567528840197,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15065720658671876176,
                1978697325333892578,
                11693842798444960195,
                2142442723011182003,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2358165565431997797,
                776816408706273014,
                10845256101131497409,
                2978886617313969967,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9783753537233647400,
                15067980846082313753,
                14468388380281472540,
                974057056101895282,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16793445763904650816,
                5767003928246531454,
                11242202339292479985,
                1320742120702302002,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12740623625502567460,
                7084176855098016103,
                3992968551388332813,
                1632264993022978356,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16292908960434082649,
                1007533033004065675,
                495288021591312291,
                2586396691517889045,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14571287780963795279,
                5452035469855615737,
                9237382480202693957,
                971184384542710775,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5809925360487651361,
                3849454963037076068,
                4680816663166031941,
                2412026449085525584,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15057362087004131494,
                239359248420263430,
                12326846055869997574,
                346670596384073839,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7497716002761445924,
                15847005709697323903,
                5323400409211363010,
                2169757643386289781,
            ])),
            F::from(ark_ff::BigInteger256::new([
                18378859499854610388,
                16502246520137172947,
                5467588641797139920,
                1323945900241180647,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16596592033035374258,
                15177779765415058118,
                15092222034465749073,
                1232793471047938725,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2808828046428386074,
                3380809016577373240,
                14413299877549965120,
                713600028641984613,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8130259331725609740,
                2074390533106262816,
                2541894748609990646,
                676936468096618657,
            ])),
            F::from(ark_ff::BigInteger256::new([
                186135329744242605,
                3663224836157653291,
                7180118820440779527,
                671929121899565560,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12991146680226358005,
                4080519678657310853,
                1037384557740505880,
                2095441066832968290,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10149028884660194628,
                14648559294497358825,
                1793227493808241835,
                700464686436133386,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15530720748407425683,
                5369810594873582891,
                4941299042745576585,
                2665949231670646600,
            ])),
            F::from(ark_ff::BigInteger256::new([
                315763338415812291,
                1709827414544856716,
                16692364844811821528,
                1436277342682784334,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17031520090212077808,
                16924703657714989470,
                15935608831658741131,
                1252011926863609314,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17155253083555408442,
                11670631965192583755,
                9088096702849477540,
                3203892031668536270,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17800284542182911159,
                13509811273627564271,
                9633354175559544655,
                3286253004624233697,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16420241050165216024,
                2703229143873683892,
                825412994299531767,
                1725231034258346881,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11336246242677718786,
                10313334410545017532,
                13671839340928983028,
                621450020772775426,
            ])),
            F::from(ark_ff::BigInteger256::new([
                143492899117710076,
                4803266283024939381,
                9550602089340646900,
                2844562978572923813,
            ])),
            F::from(ark_ff::BigInteger256::new([
                157357476465929237,
                6444245216664401053,
                5515131578836441715,
                564854160324278371,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9409038426957685224,
                2507210848182859741,
                11133092179807725591,
                1957122837845896870,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10366850508278844091,
                17967201195469252801,
                16411284715904239680,
                1882963418116246169,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4011415773222084577,
                9328047981735561889,
                7500501532983049028,
                2278372688694187475,
            ])),
            F::from(ark_ff::BigInteger256::new([
                624301097049622438,
                16208007863809257600,
                1862745849622264125,
                568994180259105857,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4618040996345255042,
                8705813315908922887,
                10648101141539841621,
                715928626496764917,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12793741039736379153,
                13041978115235694145,
                11334645572563858864,
                2909294235252114214,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7236895362876853692,
                14175547757518772088,
                13452853524769052097,
                1848560710226796943,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10629937869375209936,
                11461989524076471505,
                12246405742102965883,
                772249363060317452,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1264017289492600844,
                4569425644927114245,
                16723399310275845248,
                648347877942116235,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6395247864481810601,
                1121617048943183538,
                8203677540728963799,
                3390442284144146017,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4793763725728634056,
                11870464403382579405,
                16233758596110650801,
                1708099863147422817,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4996673691014992369,
                12294301029788872463,
                1006918584581666399,
                3250054858059696424,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4433831258253625282,
                8156763213639821532,
                15331520752468212121,
                2168522877455597783,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7237426402026846899,
                16672975400967847560,
                1991073935392126106,
                341248937634763412,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5330452260056477786,
                15917024504200797801,
                11836162028831103490,
                1085544744522137754,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16643263864025063543,
                17631343093386448712,
                2266444294804946220,
                529064751942570991,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11262995445632537674,
                15286400055532263625,
                698788786999552675,
                2294783421352300496,
            ])),
            F::from(ark_ff::BigInteger256::new([
                248764807559227185,
                9179247838583624332,
                9788614124121709601,
                3098429317964342646,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15628661596320577650,
                12064651334829019456,
                7390315997649469830,
                3441140765837519227,
            ])),
            F::from(ark_ff::BigInteger256::new([
                276878994583852499,
                13863479293982356229,
                3371646802218022936,
                2568527681351213794,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12654715897821710459,
                2519371942925736428,
                9583867403454095418,
                607609429412469613,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11072344359286403810,
                7245399981607618260,
                13241565343486694545,
                1101289917473149329,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10609811301949860955,
                16432673394772994394,
                1219683245580875241,
                1432350933782824844,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2886603682788745195,
                18353961840393078088,
                8005873340574304730,
                3153464272717307388,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3546662207751239084,
                7627737707177167308,
                36181404532753329,
                410634386546380875,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10092461332019821754,
                17754796708881468362,
                16733243272282355775,
                1680073165421810136,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14793164687279360015,
                7222513692115575730,
                12838905843561702083,
                1996557116752657314,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3644210772372102861,
                15308632014197919771,
                9483702828910660804,
                3048526644666070610,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17390733308525181055,
                4994811688079240319,
                10768016590783495631,
                1853658556543283771,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3693913035949866299,
                18345413324532588075,
                12124921617807497272,
                1229587708289118770,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17659958574043529376,
                9174259899016368807,
                17474447808336339664,
                2461252755247754138,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14007410478637531101,
                9619047059775191435,
                9721007013570249917,
                2356724459123911105,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8438481828220216822,
                6661243093071771576,
                11354670441384228339,
                1494301161999798975,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2902572830914643318,
                4929745644896131103,
                406946560736893522,
                1593344555571514811,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12404627961586374469,
                4692554470981314411,
                3445529435135681737,
                2798204096431322086,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8394380532175475189,
                12063700779753357484,
                1226849080726615555,
                2154923903998302728,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14729608452549724864,
                553720605072134218,
                5691863473465547894,
                2990407401196937613,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8069123934516800293,
                2251534446859379621,
                5782541799544685480,
                2815438028384146083,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3462581178093306364,
                8040944670999179994,
                799508490740370195,
                1614905450583826740,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1084292095319688504,
                7790394848370215668,
                10909096514931965443,
                1859053725986710523,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5565463102239161647,
                3317682195274358755,
                15270243763900990036,
                1388723196698443398,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9470412395628358900,
                9701176065837340751,
                5639967231430559628,
                569480929721168325,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15520706661760066609,
                1327787885686374677,
                4254340669104493779,
                467803026516726241,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6318022265972929711,
                15200832174734360642,
                12896295843701428385,
                270781533112904729,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11793000444576857015,
                2651581757954840764,
                15819917452851053686,
                3224328601698106010,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15203707300013380091,
                2461397409349218260,
                17126841041224972378,
                81123776642387981,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15061894858534094254,
                10598512320430802773,
                8323427056287998616,
                1985086040705477430,
            ])),
            F::from(ark_ff::BigInteger256::new([
                18416314565513043286,
                11435113389365120806,
                3306396204304194963,
                1461241562592555034,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1168679927782588711,
                15311800015279845237,
                17967334054546705743,
                2393512508204435541,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12678461504202107264,
                7780164142491650708,
                17366329714583977185,
                3084757670438215827,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11930822853608354688,
                6513973227452874652,
                10856457777005336009,
                1808559638511634277,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13677934688965338040,
                9496889984557645139,
                16946906310133416117,
                801818689774485521,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4844866327198801034,
                238544089180154374,
                12498228865262718326,
                1488388923324290806,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1309015865317891005,
                10782349996766000970,
                13374472365919339037,
                1807109031210346667,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13134308413577348392,
                17831271179665566920,
                16564218003788927756,
                3229458194081426406,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5580301500358243416,
                3527991165951413085,
                7963213015144937474,
                2067692934007902855,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7394475012537436525,
                1563874742646209992,
                4726115930661090142,
                415645575192632981,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8234900694196909271,
                12754957542549194903,
                16496761303133967582,
                2650608090576041111,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14341103924486490176,
                16428516099879493575,
                5526569645202135481,
                1002078782929006583,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5861335536631089396,
                2697033905069629061,
                2545108045983340434,
                3480412571647168918,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16082490977242032305,
                13402780311668050132,
                12077590564311421165,
                1096263148996729177,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5068367271690573929,
                17254560044130344027,
                14410009924887850688,
                1910412265111184490,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5907311540013914860,
                9873919007363073126,
                610787533261548536,
                3278452599000503982,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4862342631755638401,
                2957517415064417917,
                15967250622189856734,
                657619037903353458,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6686468253234317631,
                10018090352634241858,
                17815614763895480682,
                2027439781287550915,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15966648662835403937,
                10871600850515824596,
                1285435979587946389,
                2042793799613155204,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16825469616018171507,
                2767754860792323344,
                11495950734130943102,
                561690766036614086,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6293658467228556601,
                11991665648077570308,
                13119529208147463190,
                881747305401976096,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11182925915684633005,
                1842973901210800012,
                13419071292253213241,
                257424731382040844,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13358485818929834111,
                1317060850972143859,
                16277183880753697639,
                234459345574384495,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1707170010782764418,
                5480128901833633184,
                11295507229996631628,
                219099395876429568,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4486676303350062612,
                1724038357839496731,
                3187384430494161471,
                3045181028333290969,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6085462763282367658,
                9265105557272547949,
                11185292653492764946,
                1533935041967830377,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10577105035158737940,
                13576786725207154294,
                5629328806481513093,
                3481307001189203309,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5349975257593008138,
                17462311935702638745,
                3708984901134164394,
                2035243756711107804,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8379246946404626931,
                4355001188185505918,
                5592525989077542408,
                22286156727396877,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16273893510234739360,
                14482235318400357158,
                293462009248623765,
                1382575578718846211,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17599073137202579543,
                886175917485900279,
                8050705084771454349,
                1871826674098200968,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7744446430712717025,
                9724501449625192524,
                14409130940529191902,
                3269492165167108826,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12870241860521551642,
                11264981311403303323,
                6609806211593500847,
                3291526212262905574,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6971043659679186059,
                14346984134550309494,
                10525672903819844193,
                272749460552175648,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14360884855283240724,
                12712332207416171599,
                4861845991373181144,
                840228270998776705,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14401800544959017057,
                15048196747060957501,
                14387887669357188490,
                1073409050335604826,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8943294126666026850,
                13494732854207077179,
                84772637023712559,
                1242996152966416836,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2888877307091940858,
                8641694838598377346,
                2957388609923324101,
                1688498839659166070,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2534777427211274218,
                5379501037959839173,
                359460946156416211,
                1433703360682348400,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13517141363193368454,
                4352274847377278210,
                14655029355027088341,
                1617650879160540166,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15997038888010446117,
                15315416857564022537,
                17832659662348099314,
                1611638767320301269,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1663211104230600953,
                10418489262101002775,
                16676320419147526762,
                883411932082061799,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13325239476003181246,
                9011982809231947544,
                6718115429630862567,
                3382629115903937034,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5893370118900415955,
                18133121754784683091,
                16171491155378283401,
                845516331720484790,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9530668030145224357,
                11646492221884545656,
                994779995550685787,
                3405284537967262719,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4316806679672774353,
                7211563491315157351,
                1535350518054545912,
                401123466858245337,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9502517860679051301,
                15215291922825634871,
                13620010823768222932,
                2287735046917873915,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3307064560718170117,
                15217582444051685825,
                6262715198244831758,
                2716742538292937857,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13198420448742998638,
                3611359478421428684,
                226288457745757904,
                1578212819643720781,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12877058455165877239,
                8091639075024181724,
                7794653201682298429,
                1649583431735131366,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12252006081647542508,
                6324384394583913505,
                14615920037777839470,
                2134570488345648001,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17212764544101276926,
                4610991412578983598,
                3061423534563151845,
                508379444473547564,
            ])),
            F::from(ark_ff::BigInteger256::new([
                18149133555332128167,
                2611107020605584612,
                16588036224304813417,
                747476243184981260,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12128289173360956430,
                2220525254069461339,
                10881957116753796428,
                2842729346477400917,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1612030774595452580,
                2405192891404167157,
                5992091618470890679,
                101262647486910569,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13892067556046652245,
                15367323058050912686,
                9244843040722940747,
                2012216607180038292,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3309988843119317848,
                15282740553716985556,
                14285345225708391644,
                2252706291887237113,
            ])),
            F::from(ark_ff::BigInteger256::new([
                637756690274115094,
                9486424257047723040,
                3660185996704150317,
                3245156022627948603,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4782934080121319866,
                3821586707476955922,
                17253121723907839494,
                104242248166518103,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15344326437656011204,
                5920705910324218695,
                10389200053682020945,
                813704591488045843,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12025656678433041589,
                17499245454495319550,
                11734328611961291633,
                1943379261274112063,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5073449047472581053,
                18246686422430347758,
                1882447872629820089,
                2588636855094425125,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5969440173838634016,
                13980416635946352626,
                13412559397633189752,
                1764222498176056876,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2954181059170686451,
                14972635117275195112,
                10046106465192808290,
                2741287938093310873,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7990326107767638734,
                1512949672097992239,
                15928744677100510445,
                3387325573650730060,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2617552892023680398,
                8764981697456556254,
                8290299278315049332,
                363344944794206335,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9926589379768285569,
                15838740704237702314,
                17985676523146955902,
                1633556967400178640,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1476890841596120492,
                12026048581756270769,
                4592296438285156963,
                1236232802230979733,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8560304596792034932,
                14682229956116007563,
                1223043211504174430,
                2264471331262362807,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16005460241389229688,
                6586401216291043949,
                7088378327389855335,
                1449003788542944179,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8408937453312030982,
                16482442685649742225,
                6782944916147286620,
                2015226665020527796,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3291709288631797329,
                9834906004491708273,
                17827524283157192266,
                2655362895948800827,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4395551545039891820,
                3364075993794499866,
                2367658643447775850,
                2928608523427611923,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9852844957806044703,
                14562274344549238982,
                3602558078795299452,
                2330397395399591239,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5473535769730359744,
                3693926885638502107,
                2229045904005784545,
                726681899311395428,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2875085063915539568,
                11201392661623107244,
                13808628330885666528,
                1957874671474198767,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6138430016780530602,
                14871317937221768065,
                13912569227934654086,
                1907738265275382724,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2706136869267960851,
                14396746337239051413,
                18289061279609678808,
                2167893818435921751,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17392231022364801046,
                4472893912357422169,
                14066760901436097611,
                2993209008847681390,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4292367108304704278,
                7610533443641241808,
                17300723941195512708,
                777694574818761579,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6569259099186342243,
                16907390017247367992,
                1920468890975689146,
                2677888056010448125,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8100525766818637476,
                4519410654121836953,
                4556332490403615643,
                1856745467139297048,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5322580549642275209,
                10137275681302001698,
                2241897591621021190,
                947728162045284241,
            ])),
            F::from(ark_ff::BigInteger256::new([
                561714592401628946,
                8397191741689843132,
                15166654560626584393,
                2186747680856192186,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8848211461635843512,
                18138315717904706938,
                1632525232826964529,
                2781169541406875451,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11196414224197598765,
                14736744527547711788,
                511818890539116655,
                2034094549517034772,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14157814787586717100,
                7397437101067335301,
                18033641419350607274,
                2965565728864363635,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6716386819041410033,
                6531215290063477300,
                5994913707493726884,
                2714007942534270317,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13966578697784568212,
                17097071832951323956,
                10960026899419876237,
                1139148003425923351,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4632144137795392337,
                14608862030649783066,
                9026925007021734730,
                433239940983183621,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16827527735750519646,
                17513800294097141657,
                4448149685810324822,
                136249838613458862,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8146477909989707914,
                13675597659675026487,
                5486816267380758735,
                2850924347404611410,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3936301414497513780,
                7089452375636336728,
                9143802081998915543,
                205784686201625808,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17007842146798176700,
                14718267753861394014,
                8439368573927692810,
                1817055415039811072,
            ])),
            F::from(ark_ff::BigInteger256::new([
                18312604482617593786,
                8137020431363824191,
                13300010967662887719,
                3242679452894624169,
            ])),
            F::from(ark_ff::BigInteger256::new([
                774947356021050382,
                8545399006630555375,
                14830234433842903915,
                1709175592915428371,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1986422961930150908,
                15929899505554257796,
                3230096552189577861,
                1955488202537637022,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3636387160282568614,
                7800218813458841663,
                10739972774229937308,
                1551597812875889240,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3970967614159236276,
                3979805348273227198,
                9522956076123631210,
                1076220204860773397,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10521304518834758734,
                16250600590330785317,
                8761320476005885444,
                1931831928887565845,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17925282543383421115,
                18022366697445898683,
                1549578187728179890,
                3409037955769507532,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5997926456734282428,
                18263080560557156170,
                954950026412402519,
                442080969388345523,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16098937997882282475,
                10879233533746387927,
                16422779425641117664,
                2057216258967578004,
            ])),
            F::from(ark_ff::BigInteger256::new([
                109625769667790487,
                10177660912585691885,
                7620629859115586467,
                2230717724989794308,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17998775710381519700,
                16091563460282392364,
                16271769598873995357,
                513294274682974481,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14510007296466088619,
                5885497245162641179,
                11326187383867237299,
                533950654428226095,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4603966610796948080,
                1264552365417372523,
                3764395704339219560,
                1400542314439839691,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7236706541850774145,
                16900998030720357183,
                10190585107573776549,
                1450241359475109836,
            ])),
            F::from(ark_ff::BigInteger256::new([
                18041814621950938223,
                10609061816056010176,
                16845368643278846783,
                1816246247944165617,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9121223106842283992,
                9316450759329932475,
                2112249067401106225,
                3457943453751400367,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16859916631575380740,
                3396720343470029583,
                13406777187610023216,
                6897047578886994,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2952977584216582735,
                2460629629044618188,
                357246951444737678,
                2419837096225400842,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7287337762801158673,
                13245570163408637422,
                1890686643069551196,
                1406474237056074280,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15538978296801112009,
                16225515192601057017,
                16858809684036312237,
                772384273748428681,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7746923444744739435,
                43565524678517934,
                9738625111465324160,
                2009631651932700996,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12332460084571065812,
                6186586151632965760,
                9097174330567265882,
                2272761492845913127,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13484556206457338606,
                17987065715105930177,
                16888942767324113452,
                2755224155388573270,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6979512789522782369,
                3244635963040679744,
                8236597963891154764,
                72498056464953837,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14798750334045210105,
                15402993546789370523,
                16395129462780149725,
                299980565984919650,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7140468367525234655,
                7043949178028820684,
                16421609568112212165,
                721667841179238574,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9486242326537986346,
                11519106020349302010,
                3406877707525777004,
                470776189451273026,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10743065501586390079,
                9345182571564805208,
                9670918849083478925,
                2065871398847302284,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2267985951999553338,
                5708178903652238631,
                12950365924070824488,
                399097513522272154,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16269038540528875256,
                15018227640096505810,
                4529629287227796344,
                583811510215036975,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15638824406901909795,
                1724824895662683256,
                3029695892471642286,
                2094854578408701222,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12702271210024572893,
                7834696180120696110,
                6595619709193662348,
                1860035896258537667,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3016796403861732844,
                6113185620517950094,
                8553708044953073830,
                2197230896819315835,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16309969912758961315,
                18384017703155528819,
                7328437002664604451,
                1629353771602608388,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10339283192430256571,
                4383584376058308581,
                3609972945130111933,
                1944461083892044861,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8654507631747366639,
                6817704790099488168,
                13626043979198605867,
                2422802479501735342,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4752620261193740440,
                8542231947206919089,
                11700454572821532876,
                988291198686101811,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4719626968046388548,
                14818607377899311349,
                17378727921351300437,
                1396184452866773708,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16604084466117001427,
                9134158680273095806,
                10199876369845994421,
                15133907293059230,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1359199991194404138,
                2084697241228908470,
                17157345030994231491,
                839512684483249247,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7606328419098384263,
                9174863938054217122,
                16508817657403655463,
                2590825009475234548,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15860938931758755863,
                7694103720431692695,
                16279683637039778122,
                1297362670935449068,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8911364687587022058,
                10062134081936395096,
                3904918049525506228,
                1807484391773373315,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8474934829427409957,
                11236750583121142585,
                10004850539440494629,
                1805610880382961834,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4921695242445849008,
                8540512122958526372,
                4147889599999832726,
                2657194706186903212,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2807312230776604157,
                6789314148415220339,
                16716298159025968691,
                1388742195633262555,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12960162674323299964,
                17291436180795489685,
                6153285439528082625,
                1268390873582664666,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3237804030749473609,
                3703530019723997928,
                12424521906352136670,
                1836120194996923565,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15726646027320347411,
                17559915673900340800,
                10333257246280723979,
                96617161756773381,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8714404397396400892,
                9475593254428030404,
                11764568392139904770,
                2583487602083952784,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8219269056942909470,
                17212789783128433533,
                11857307030924200118,
                2541428124850648251,
            ])),
            F::from(ark_ff::BigInteger256::new([
                18198702980691943557,
                382516756402793560,
                6206807096329774706,
                3375157407852825293,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4430996105788823085,
                5282036463019155297,
                7108708821708546403,
                1106356768472105932,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1400933633639236840,
                14199245472478742095,
                3597890354498306340,
                1622710917368256259,
            ])),
            F::from(ark_ff::BigInteger256::new([
                844686324207360752,
                210428536419355232,
                16773438295977878196,
                454974261592344241,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15983965153803492907,
                18035219414301887901,
                6385394006560212815,
                1662109276849155781,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7073383656816906298,
                13926204750097411377,
                4134698537451095234,
                3094911156452481371,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1157258267953695002,
                15353475189584908894,
                810252316757396467,
                2492267143934714087,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1969992804037479266,
                8346412490034180056,
                941228716329574068,
                2728630848870711700,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1324982387843455800,
                4945868538681235178,
                12548008289250996797,
                2293869164853678810,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5942302919064180908,
                16195988626434318170,
                5792856711437625793,
                374842664952633187,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16532471358306371621,
                13203859463164328923,
                13016201431997644800,
                1620601371325356495,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7003528450975696632,
                17446452173526770068,
                16162051479705402944,
                919563647227841857,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6410997178516303378,
                14154883514413855839,
                9567350801939637777,
                234593294700689246,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16615310882618745985,
                14046373123477535624,
                13247562191676254457,
                2075153937747154108,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15396364473965689736,
                6009459073496075476,
                14458514679978255563,
                2798227392271687030,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16207373987078894275,
                14757488432885340775,
                7280632722281153326,
                2613549259250862889,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12865108493643110583,
                8022781547177121075,
                5133604593417532750,
                587857014392844682,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14335640993857513245,
                18039185882108564954,
                11840191379873747934,
                2792871857431930816,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16683682085873457828,
                12706168197009277535,
                1666902587065401064,
                2097819732825010748,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7647423994970460305,
                17087309576876871828,
                1913389488286625682,
                3001310725630396722,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6145330752429863813,
                18356716319439375108,
                8971076456430201519,
                2499576054156943904,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7336727936106845544,
                1853268557981365085,
                6680474449679448309,
                777543275261278527,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8603483730144565017,
                18057289454055063394,
                11664713290056011481,
                918854023274802264,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14268832707743464516,
                9408812184841754277,
                8891695564532099610,
                2433429731027805377,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10946950893716373439,
                16482990972588193145,
                11226468550715721811,
                599310756471280689,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13680620338366987321,
                11952206053881591641,
                13210109550956806728,
                2720413862842083940,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7714446701983537502,
                16167554334577791682,
                5680666745603947450,
                2545329373362657836,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15705505740750067555,
                9221348427971034108,
                5390183668082701481,
                2646772144154081297,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13022229338118639419,
                4579117288780774092,
                10044620301918712593,
                232956353531239974,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10756278587784045747,
                8849843491831746012,
                8305604656184603208,
                2435122780539869084,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2972438593342526733,
                13327770716585683963,
                16940278760905946738,
                335841077345259005,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8346946587406103705,
                8104751121844270806,
                6929555650987058823,
                3311313851475717062,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11993651772497106951,
                8604147901935391733,
                12943637047459303879,
                1700447751216833485,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14507815843835924323,
                12037679107682628931,
                16710740690208874152,
                868980050118873171,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6814847155077778088,
                7167293031728048793,
                17331113353968339517,
                367753164243353834,
            ])),
            F::from(ark_ff::BigInteger256::new([
                18262157157052250695,
                15249345858891115774,
                16274385556734798207,
                1198197829900848642,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1384311794737115827,
                10888419599471124169,
                5001588228366489548,
                3277471815689309029,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12107466070894414933,
                4428032295877115224,
                3936174094712194674,
                1437136149290208181,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2020873306503895256,
                16507933686061054706,
                16565352338227911440,
                297195246744062103,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11574275659557261361,
                8434949023849522344,
                10378105716866044105,
                676499989382515154,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6823823844362315199,
                1129482725696581855,
                4236726669571258059,
                3094938797322925570,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1382191705711889568,
                15110190679979565802,
                697009387727383852,
                3429303717809487937,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8357728118408881872,
                16261311023281733128,
                4932638570872192267,
                2579520316416692562,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15568857908776614800,
                16863407256622155670,
                8707818041216853287,
                3052151750223504229,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9159578005786562316,
                2431029271003966349,
                17235653865713138751,
                2819829388405783682,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8034212709699392706,
                10845800863206688650,
                15684617933067405343,
                700640719757969605,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9447978458675344907,
                13691353159423604503,
                3232320761857735010,
                1168269416952105303,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5449496540373878581,
                5544936383364028698,
                14319900367027753878,
                2573287271949063618,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13546392235077722425,
                12561380634957156897,
                16274986955104879732,
                1767398161028703648,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12620796759895126621,
                3257236225961599807,
                10041724369589267266,
                181993170123568660,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6968683851461034626,
                15607167380492522350,
                12440809680709168448,
                1266070475480335853,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3853334621396247478,
                13062424147649449218,
                12977634574185018455,
                1614553123528332760,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4195681637547385399,
                14466493340341887330,
                575283069660697921,
                1733840087427546750,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6749144444313944498,
                9785160589746588309,
                1879325468447606501,
                488548565082231689,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9758081498979659183,
                1940815519888266269,
                12009827077897599276,
                3199184500619154291,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4313954292140944327,
                6337938223499325842,
                14001639457938623868,
                980658855986885140,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1254774432202001678,
                7366652610728234568,
                13137653367431173282,
                1356604174807711621,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1131830178607158763,
                8930073348078325178,
                6768481974295129064,
                3203729904715862736,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10795450777624282083,
                10149465062271175212,
                12220250787426708139,
                1711388958423190300,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5947333646749427449,
                14077078167873147090,
                1961490608695017873,
                243091462705180652,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17090660945772886824,
                4386802863128691284,
                16284307540573250014,
                1471098790530615298,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2526464184132099228,
                5536191702982527739,
                4146325701066988222,
                415161121734889968,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16762559351206104391,
                9737509168102404870,
                12299521222963705219,
                1879332317621420111,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5713865985200543901,
                18011972373648705409,
                10601714443133346191,
                613559848692235186,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17647142387750944341,
                973217103321406870,
                3347903763912882715,
                502588052451121880,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8028382569875017110,
                9941278702109384243,
                9668313997298358083,
                877860588048246637,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8951036314526950977,
                10207142074437431397,
                9667759907245414238,
                2689090773583121770,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6034291944395944039,
                1662366800182565829,
                6304443631934031413,
                2753603580934453733,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14673390317385099606,
                882680550716395423,
                17647140639306463946,
                451327771666422012,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5352398058199805385,
                1280192620747233458,
                15124348437831055667,
                1155266773716828532,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17542401173442348174,
                1854225047864979659,
                7501500490387499348,
                1525481216928111982,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7361150585466038278,
                3685501508700047586,
                8245168898652841716,
                952558456763062830,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3930977642975088237,
                12531180890531477836,
                15625840903974911657,
                1104593726646691382,
            ])),
            F::from(ark_ff::BigInteger256::new([
                29795898817710282,
                14726164291672205181,
                7724644629854769184,
                3053760159430800020,
            ])),
            F::from(ark_ff::BigInteger256::new([
                233896372914408711,
                14621800039646521484,
                914418426185668298,
                3159011683421167527,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9293997367304164254,
                12196548462367865752,
                10844893352970623283,
                1720782633931491726,
            ])),
            F::from(ark_ff::BigInteger256::new([
                797613176224768817,
                3778227876289478178,
                12793182002212261368,
                529644413896841967,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9402010003441624636,
                9975916336444387768,
                11727593088662733117,
                2494785154613264543,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13947964581345483255,
                10257284257798924554,
                16779299441483343777,
                950633178450889384,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9026904312038007612,
                17770925641087684272,
                8399489904040424533,
                1840405544986393154,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3263731566493026276,
                15831824293334247277,
                10556016045473418736,
                3009176442120279005,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15581579924863630797,
                10899344780329465063,
                15062203913151586440,
                1434215102077344457,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3570074910373750889,
                8598722002650029871,
                12300181541584113034,
                1760096181232604616,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2513497459780651116,
                7602194649157242280,
                1355834310024810481,
                2613827313052417849,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12644386194757921730,
                12481823176725423029,
                7870940861726967056,
                344806002947215296,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11901705839290034335,
                17098477270788935559,
                5487217020515493642,
                1678196656117840508,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15306147419241747759,
                8005000281983361055,
                7122409444492145999,
                1699202928893946264,
            ])),
            F::from(ark_ff::BigInteger256::new([
                18284161254104877585,
                6554543150615837174,
                9010004779786250890,
                169806595051974758,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13709778161912677703,
                995921485038670642,
                16140063959756772447,
                1273946806655144923,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6065795976968944027,
                10108897802538462031,
                226113317467636916,
                124175235243294379,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5195648543567131322,
                11512015804825565687,
                13803568250883583691,
                2887238093090637222,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8307592909239091385,
                4514146643090578845,
                14555252884897661357,
                2625995289823079980,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3134673678122293736,
                3943184885205999707,
                10259148142290333209,
                2524687003595703335,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2126145026545739337,
                11625521594878198079,
                11057808876283882152,
                3220038546582807443,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13991095838555463297,
                8666147667684060556,
                3223338398511650830,
                1898302599327434350,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7339834818426071623,
                776066998769892302,
                13288832677900637854,
                304466972211562357,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5764773712186860723,
                2278671978783757933,
                12215743153630658909,
                401408454703175575,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17245063187400256722,
                13334240170931002236,
                8787596042026346681,
                1676816636676916736,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16376425124124524231,
                12218107920773310428,
                16803357670374555217,
                160470187108236422,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1362511929136114584,
                10518953020858757343,
                15320097113120830217,
                1420815123325489552,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12940362287228009981,
                2784781548077058233,
                18084586523595027458,
                275692946050918581,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16405510872871279086,
                4138549827650703559,
                3837922452437793617,
                428894178133184888,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5967608796398183935,
                13785947725668591461,
                17029492918399004591,
                825660181816180673,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3835528340003423895,
                6348307284932017743,
                15146335361462925861,
                666903846652490202,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10807380458931490356,
                3316495385649559491,
                6698454391227611459,
                3297417717695390769,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3421208268009036381,
                6767125744524390352,
                10381396097290255677,
                1250411632748596265,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11686696973148621730,
                16125902197189615468,
                10357810438278176573,
                2217892262007794966,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15915491935072794246,
                9760318699320649794,
                3482737465099759929,
                2647067700109662139,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16713946579656841819,
                15339157545098853759,
                2094040929397935953,
                2849137172544834398,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9625247182414167220,
                15454659827346330217,
                8956766636563239412,
                1222099134814947447,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10749401471577683315,
                6699960339534189663,
                4680689955651034921,
                3272238022514787416,
            ])),
            F::from(ark_ff::BigInteger256::new([
                18045064355288300830,
                5827023049133985555,
                3601533314146712242,
                1734225774533424310,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4366414653241656722,
                3071169164214914625,
                6491493923703173028,
                290254454254352669,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9488849672055048722,
                1570087533790919032,
                15567768240012231362,
                2214400865839566279,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15713703439159509262,
                17885508724271639071,
                4060476064803788774,
                893966925682226087,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12383700684734133678,
                10439629737658261939,
                6609774993163136087,
                616793703382935623,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4518219635862837428,
                5869312378156237684,
                16753267872093563750,
                1379758348230466730,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1318187420588364782,
                5693189194239068855,
                917588267771987441,
                3404338285229038487,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6149482228527046448,
                4927711946122921449,
                17613007972720009770,
                384324082551410648,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5082598304491245589,
                8349223473330979167,
                4941138886458250395,
                631241282633367298,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10147774887223549347,
                17038666762852479067,
                5206549184688619151,
                2389833917225904530,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3880760234007074556,
                12291097113638003843,
                9149678103108664082,
                2370628138526580464,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9483034294588700435,
                6798119550763099444,
                11937451593777751533,
                290605144135076625,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13315384419380488611,
                2208257409710099306,
                14957084556185235958,
                455503269172907763,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8459174394033994324,
                16580571837143081743,
                15888180979491627416,
                823062637595070048,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4986941731615899799,
                8989690789788375363,
                16321499967657974737,
                160502489211003981,
            ])),
            F::from(ark_ff::BigInteger256::new([
                18161999587139939295,
                4472861309500451573,
                4170824958441250665,
                1968588758029311200,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2587937920049035309,
                2627213531625646192,
                7074003121220665680,
                781985365336109224,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15238059403193275340,
                7686297150943704394,
                5160305194302628585,
                590642297123509018,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8245235260970184645,
                15393134154106322432,
                1173420661056770824,
                134400955885218443,
            ])),
            F::from(ark_ff::BigInteger256::new([
                858894361248765790,
                13496399131071137706,
                17306445682140744787,
                2508397986981611829,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16818748537427140955,
                7795293337936182687,
                13124916705997258528,
                1721418907934710545,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12360239975038749139,
                7287083212552188517,
                3151902841582312344,
                1922224217546669146,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16431737432987460563,
                7377644638041249878,
                15042756679667235830,
                2942518436852290837,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17292980646579529862,
                15835141990772466591,
                17127475495438050628,
                1541934819841023726,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9085663710694859750,
                12930684609165419178,
                13433976493798361408,
                2260401565634181645,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16977492159480669736,
                14216021080655771310,
                17644458808672983698,
                269153931480704434,
            ])),
            F::from(ark_ff::BigInteger256::new([
                301060693199762953,
                16610516720421311533,
                3933245461425294037,
                1195877368785174878,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6850857092677304200,
                5100140339634766230,
                14912414166392280362,
                1414490470732292799,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3231631824555512709,
                5807638494804805312,
                12253574084995371486,
                1094499266579781027,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15968280956891188121,
                1177227335905359919,
                9827114595379925538,
                2193829524129651351,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1244198286588513631,
                12105395464569471667,
                13643552284916514876,
                1398792079936038365,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3791881730067842534,
                16816713748151262930,
                17209969806815528026,
                2576328215397640191,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4492443431015281173,
                9601356847686470323,
                3847591642936110166,
                913646012658022116,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7861471545670177825,
                16727137607458230608,
                13900196493856636774,
                2107632733221644105,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8931010470622561775,
                18088438397982172419,
                9771261554776443916,
                1127236188506184881,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8913041265853538915,
                10998897075582127874,
                11200183766644108552,
                576958075329649503,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16544004893353283158,
                16033681610458377245,
                14973771236909729052,
                1419293352152092428,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3164149492949341808,
                7459948903608818624,
                2308221762359015534,
                904815267888838673,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13887051535988033295,
                9730760954750749563,
                17546162519828345678,
                1609863885025601267,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3139754104767937728,
                15260228701663768742,
                16405616457294736448,
                298718984456526424,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9555843542630445061,
                14073564862361598364,
                17325209054356040390,
                1191273809817501885,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12855470874024983568,
                10955224784259330979,
                4595291271332478669,
                2610585472283938487,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5263065232163439487,
                13671454642552424223,
                11758929308283912909,
                961181311155525401,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17062312177556448516,
                15469293304727765349,
                8940415241151305512,
                126021206832997590,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17571795471803287475,
                17854263945619409940,
                6124514303325574192,
                588625763911509119,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9492804046002764408,
                8865636310439463602,
                2154413629961118507,
                2131028405965318809,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6396699993216622977,
                14565830957031612855,
                554885605320575997,
                2199371564937574858,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14700229086249581108,
                17121875672110611119,
                958437380916082489,
                2730042762171422878,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16403189776115141763,
                7350155719768786629,
                9857863719783801499,
                1291985343138389724,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11832390632332173760,
                1274682255747570147,
                8368335574892992594,
                1798003280523034450,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14438960778245459579,
                10294311554749836072,
                10456071965287742750,
                411712249807274034,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10015411793162767,
                13571273353855627581,
                2474753816534563395,
                66280418425167661,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4967003702739486861,
                12929026064975396078,
                8147206878033891329,
                2131950236517733281,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13337660629747298643,
                11518871815628743157,
                17902785659068543758,
                2937562213392376645,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4496422047429941016,
                16536333507420485509,
                16510810727247797221,
                2843551515327668151,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12774800512624688783,
                18254417481987535740,
                6225151683234826306,
                2380838516665502846,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16785158967790606778,
                15585091035138680282,
                17737639533503469225,
                919239350270210464,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3589465416959767005,
                3181518846821550418,
                7092199302903254521,
                1428169722991539233,
            ])),
            F::from(ark_ff::BigInteger256::new([
                792843591933390345,
                10190207576941044055,
                1291871669712865549,
                2029815925199607830,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9787117065946087989,
                523774943087130150,
                10135811090706921781,
                2733052880178015894,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2211215205335271840,
                11792968764994350466,
                15774880758241021272,
                2525550148973439801,
            ])),
            F::from(ark_ff::BigInteger256::new([
                18427817609464187679,
                634043006553941619,
                16683504454596984714,
                2970065902553312331,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1858024903776146149,
                16332625616108642760,
                16617786771099703293,
                2928373743173441964,
            ])),
            F::from(ark_ff::BigInteger256::new([
                236973080191463542,
                5117087538089955192,
                7744944215246778931,
                3309077133648012450,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2581759119736898533,
                2286912547850280607,
                1213213783544578237,
                1045901599234321218,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16711890796712524145,
                10534553120698211144,
                6629033735730241832,
                1238404210871602808,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1424066390557296514,
                9351412306612725643,
                9233729290171482485,
                2281744046701649884,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6657585293531122833,
                16887183035399176260,
                14713874495123719686,
                352356552328040183,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7226141694718873697,
                3503596840569495308,
                11977297777409357214,
                1905235953471047135,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2530010666814439402,
                15399645096794058133,
                11901833896615688240,
                2490315627091147496,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5724085554844549226,
                14942435620471384669,
                14162551832813463017,
                1030722130991350682,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6523923131913195169,
                17750316011862085947,
                13167896888550759565,
                2273850717721282809,
            ])),
            F::from(ark_ff::BigInteger256::new([
                18085670245289995384,
                5981052287905167729,
                7807190490846044352,
                261634431948800829,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6791382560563561094,
                4300230946592486731,
                5299854039501768894,
                1980925780523239591,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4194872785796735413,
                10200456224281805297,
                11631882033427428775,
                2908720369079141217,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16294559096411830790,
                18394159096672374948,
                10709208664417253127,
                502523914340060440,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5328534775898090463,
                4365678734103553387,
                12066163986866884323,
                3416054921086068574,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6140518695738857472,
                5254561827198791,
                12050921947098615952,
                2012631424859555334,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5533634766505898386,
                15268109978370908901,
                12350459032332085285,
                904244685714581777,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17690106528231722325,
                12350287352526507397,
                10046353697385574797,
                1270011173154062572,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7861403368833220358,
                2688314612843743345,
                5267003719208145414,
                3462439659386247163,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3200526845397631361,
                9985747725396122381,
                9717310998344545044,
                356950687796531296,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12809402549012389461,
                1652987080556345278,
                8072265568900186636,
                3093586483851219555,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15184789447056930548,
                16922815592536876478,
                7519819724880648558,
                2813237587980475871,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4855869600410882431,
                1729220705783707986,
                3602369249993956848,
                839063916053725278,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12106231242025895960,
                9496996657201838340,
                1209887424066194629,
                1772951459795933469,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8356327095591578357,
                15534712489614512687,
                8022844178366588162,
                776802781933629003,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5485850753988267262,
                16496494301044695092,
                8493739532011402449,
                3374465768183463050,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7856181044708584424,
                9700952833207197069,
                15914451590184810530,
                1103009924051169393,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17094860767449258611,
                5967470290324335152,
                4605903571493621745,
                3404409775978820540,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1624026608910039872,
                7750341024231366273,
                6338129154427586339,
                749406011339493532,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2386080875104160292,
                11686270882885873989,
                13792550688697868005,
                3080334182295239112,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6406326517503005300,
                15589037810098638873,
                12658156768292737766,
                2165750189594116382,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14426577535744037903,
                2572813630037205041,
                723258479810517335,
                1436212832958598226,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2484416902942793233,
                9141062233777929382,
                17250691984177691868,
                671021785830729817,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10611613326592273607,
                15761331748017599250,
                3432092572837428371,
                175602179900323593,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8220556447368343526,
                353688921389876194,
                43015510194956119,
                435917791372576377,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10459669841367791581,
                482351028684350014,
                14211059011304573971,
                770391018924673028,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17838339341475069203,
                5246586671549137074,
                4071131229229408364,
                1431338628347718115,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11936156971883568449,
                17880599749201623088,
                9138910626311297600,
                2649852650117516327,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13224154851765259360,
                8925993164798166901,
                11687837516533180358,
                3027083572307234502,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6610819489118752276,
                9240987235351839098,
                476260377206049801,
                242103597631805112,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10252646625578405144,
                2930121187702538791,
                4208375883291832922,
                3398358469182706610,
            ])),
            F::from(ark_ff::BigInteger256::new([
                43277972444457383,
                13144256214382940417,
                8483702904184032240,
                1419034365786736396,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9313153972922545583,
                13154811792266174963,
                5750768893837328556,
                3259638610074916169,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15820234318773595949,
                3499986905487470096,
                2372019017205449676,
                930527630404029069,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14608307430609425361,
                2580050218027733557,
                471677735117592515,
                982932318666925357,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4568041038211031662,
                6472560526318270936,
                17924833723716032697,
                1772076405687783321,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6716439790040003280,
                5477852616018277051,
                9535137907174693494,
                941654736185201206,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2857048480651559700,
                14718652218109037255,
                13156902157280099543,
                2441612160628921877,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9440149837383681320,
                5772300028444126882,
                5256721269105905056,
                1890858011626184794,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15858461261087961478,
                12688594939359744472,
                2226778439033777362,
                1854707865183359637,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6519566888014766977,
                15839105993774727416,
                7455978607479708753,
                2769428184013000781,
            ])),
            F::from(ark_ff::BigInteger256::new([
                553995615949359978,
                34516275743084622,
                13753401765896647901,
                2057459863758719988,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9435625824954296067,
                16281465183953627104,
                8836007580412863264,
                192536395124650116,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15832616321610057167,
                18304434228925853716,
                4074503474959614693,
                370125067461815591,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9655474409217401755,
                11454273124668638809,
                7044868035009346736,
                2792078138607032148,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5111236330774276180,
                11651680435746149320,
                7011556983259658004,
                2554966022470698732,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13352291827564687964,
                5542277053187195543,
                4471754786543804224,
                861420169061643239,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11406199665149309380,
                12786781073697224224,
                2816714525010392388,
                1051651462171542110,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16575106312229758736,
                16702542211614148575,
                15609239303750733985,
                3430702322880746385,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7872011711441183290,
                13198315866315858557,
                12428096698614659472,
                1814704410559075771,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5386810689378881830,
                14654812190273269024,
                1204544122331647415,
                3168449787190488271,
            ])),
            F::from(ark_ff::BigInteger256::new([
                414890550692069802,
                17934366980147885202,
                4950196648246504805,
                849528593135707250,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7718353054770075128,
                2769586506503270904,
                8188833947472912282,
                1443277575519422004,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14231389472075669521,
                13681935716550290007,
                7601287883746249866,
                844601190483204321,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12565757863191359541,
                17863828447305014442,
                12381180903657521899,
                2401088348931099040,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1193481223463011572,
                16554071441371717014,
                13796624700709758661,
                69423454908574640,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5529736827233948214,
                10410163053750326458,
                16473017440268274649,
                1155498012157392565,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15632284688633099764,
                13634348556713921367,
                11633886303856520010,
                185643262244895785,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5035608847148348278,
                1148545569616164677,
                17672800618367393165,
                3129193491737651478,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6330847598454465880,
                2175341281332687567,
                9490427535641127046,
                2954559437294248191,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11035228939284832681,
                11185032724295782230,
                5466599982080382507,
                1202421519815125430,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17134550818278575889,
                1351460758523642115,
                9204584147576419372,
                3467438932286906593,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15000899053407633169,
                16216276340133079728,
                9693593846109942755,
                2878225888338748825,
            ])),
            F::from(ark_ff::BigInteger256::new([
                18148259790260495535,
                4232209895649877582,
                13737256298452488389,
                886913770179856080,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3659817986158853415,
                9560928932457067721,
                18394243184336822372,
                839265550313943611,
            ])),
            F::from(ark_ff::BigInteger256::new([
                198809382986218176,
                14450699104753899839,
                14259151757144777881,
                2433477330851911713,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11688267864062546289,
                999323363213911081,
                3634265290466826827,
                347283254098822937,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3966122317031957967,
                15600221649883636337,
                5694726266013936141,
                736232425179171452,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12218474764790109984,
                13170994874741514492,
                11835571911343543018,
                2943928627523501867,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3337789251576096195,
                3868839389349750752,
                3171746921480639053,
                2956305950926458959,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16083510608721574339,
                10112026006332743988,
                437654723010739094,
                2740221243229161211,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17472550580532261431,
                16799762800794665246,
                10885991536524748432,
                428255031141002379,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6568965696853760351,
                13201565397814414271,
                1814540409348745279,
                2774516609224927340,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17530585830387503439,
                916204777207536021,
                7732868842491311460,
                1359794541569356969,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10787250641789512123,
                14871478997295382366,
                1282538341839468228,
                476071658461960522,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16686155460527965670,
                9164024136021102070,
                3032803498653118835,
                989408937714913983,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15122202099219368440,
                14098605669423457820,
                14553545271226022172,
                2917110222202371468,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9556586293774058729,
                10871332832168046595,
                2534095535088102793,
                2199878809543998654,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17518027714610307825,
                15469751761530463030,
                3305989967268191240,
                2057334630698022388,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7633982607299203980,
                15050702243666650439,
                9463994334625432323,
                526668713469946184,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7528724032885478525,
                12025039087679427087,
                2251193292572086651,
                2839208781612238603,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12618048613930882356,
                12469631131594947200,
                12602743353793248318,
                2333818243843599595,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6517237512116726598,
                17932042201814401534,
                13970371314062726769,
                1291855083676629434,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17459262941707224129,
                17237358355268609366,
                17574451529017688290,
                1733887369466075224,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3468419270119395092,
                1330616894996475780,
                16293712814498264300,
                3079513874781644066,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5746155178945555597,
                11128436544923939607,
                11193686843306408890,
                1612942057074874411,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10720153138662703287,
                16778209923248186717,
                11972684438010750840,
                2005173977522260666,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3349629964980670008,
                9465501307476492932,
                18267568654226717961,
                598550940107520998,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12474990672933971015,
                3310315539019440485,
                9742048565560836107,
                73665423015346840,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15358199983673954033,
                11567163078890245291,
                8839578862248605690,
                305915156703588182,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7609892942813727355,
                3444827687748185247,
                18242744584373580253,
                2617096584901173139,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7814133197534754497,
                7996512720536215716,
                5281929621123755413,
                1797917437918113692,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10930085620359727488,
                8950976772441161991,
                15673021599879353806,
                2015445965687179468,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2277240803613110682,
                13873429301893559454,
                11715246777694232325,
                862914669617225528,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7669810420114757770,
                14046464352004680918,
                10123192002806337880,
                1869553641093049032,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3566573202380746404,
                16378317869737513492,
                12547204807840998301,
                1943820262246823848,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9038254975816647742,
                14819104709943325938,
                8536777439066451472,
                1961930103575508539,
            ])),
        ];
        let mds = vec![
            vec![
                F::from(ark_ff::BigInteger256::new([
                    6272925521698787480,
                    15434569038537626331,
                    10152136084411682209,
                    801211369788478904,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    8838887931598016039,
                    11521956688518703801,
                    9395158794966823885,
                    3450929237201577499,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    4779945039114057360,
                    17050124148077879594,
                    3703597303049799647,
                    3347591138979887368,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    9060001057653629311,
                    2997457277061682864,
                    17852291491524599869,
                    2532001030492329855,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    11753692653892119173,
                    2089912240986142533,
                    18249319035043909043,
                    1570890428086326095,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    11994623227217008338,
                    6896749388650020970,
                    5809211150082421293,
                    1661875347580273637,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    8895094120931891218,
                    13756153853691787321,
                    2054163674706036841,
                    742753828085767841,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    8321040834225536143,
                    1482310456055911746,
                    9949879493710234487,
                    1063274019261635652,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    7332004413885899797,
                    9565965838819778397,
                    1404761796488991239,
                    2691544243528191389,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    8247912798193446702,
                    101378222438319897,
                    5929315797652723102,
                    1301334026556125507,
                ])),
            ],
            vec![
                F::from(ark_ff::BigInteger256::new([
                    1033516247935415686,
                    16524057513051858678,
                    5829174925046121885,
                    1926297878359327648,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    18371200659931502883,
                    11388633542258069809,
                    8191221308388242009,
                    746296045302696192,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    11640948487260055294,
                    259764780737598140,
                    3147069759456124769,
                    2659426112521196409,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    12602162178986087809,
                    10256022312406798360,
                    4719228219308913159,
                    1216874181553266962,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    10747585743231634592,
                    1065759342890615869,
                    10707591292389803399,
                    1610844724376615425,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    519446674312890819,
                    15446339007699743534,
                    16708604472316444258,
                    142849794341006836,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    9419355631780903674,
                    6356684710114689548,
                    12320415023284985105,
                    2009309326310969347,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    15573000482228635497,
                    3199467581206820091,
                    6170464057539056058,
                    294994700144907651,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    11473960196913325846,
                    5911443361531773233,
                    14338972508960004490,
                    2251399018923899528,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    16578147555902749756,
                    14514841960325818597,
                    3222244167111953068,
                    77662253471174307,
                ])),
            ],
            vec![
                F::from(ark_ff::BigInteger256::new([
                    13774306726997599274,
                    565332894485838757,
                    3466577631885736069,
                    1228674808538233002,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    12003134843014193730,
                    2930281062576332948,
                    6097609538963548598,
                    3233268724406746329,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    3025129870387114220,
                    7947043895419959375,
                    6303709444100230696,
                    60252161404394824,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    17292077337766632518,
                    9930477971733556871,
                    7092268231346501283,
                    2407416933152994970,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    13779467219391364335,
                    18344384755607751867,
                    1654961987437469146,
                    239550432499672028,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    4568628600342515133,
                    4794410159729085049,
                    13909813128856495148,
                    2084540820157327169,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    11384992419464627158,
                    9262937348738058803,
                    7780723302208314473,
                    2782252685629284833,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    10707148934678917318,
                    16056947203409705329,
                    15165749896984647376,
                    1056019699361259577,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    7954555273557982021,
                    13809182640791918584,
                    12473466083922125789,
                    2414133819598615356,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    2341303142736968695,
                    16364318733816143001,
                    2181348783459125415,
                    1787107103824599763,
                ])),
            ],
            vec![
                F::from(ark_ff::BigInteger256::new([
                    12535608550931105160,
                    13793702140028972618,
                    7542967763859260114,
                    1796288437204767671,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    5119260810582421269,
                    4558213581501808735,
                    9414274574786515073,
                    3212805400489987130,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    15123348035696104261,
                    11387908014933047593,
                    10208787226656503056,
                    2254437073950020760,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    7765118748213309343,
                    13560499551928174133,
                    6194156176606787919,
                    2749733391490289088,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    16484014216992635554,
                    13333130084936448993,
                    4907475083574590375,
                    1981181600508216294,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    8415791220040581647,
                    13935190500482515524,
                    16570129284804493915,
                    1911969562431240754,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    5331266033934317949,
                    15861999694005546165,
                    8154613857034542536,
                    2540161204167143678,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    12260474737004632663,
                    16189198091110821595,
                    4406977474611337218,
                    1345223365947101634,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    14618771982584000030,
                    4511141125390838372,
                    4077180571104887997,
                    2073517916844623736,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    4944956475703829933,
                    15419243350646164488,
                    5983967132413473506,
                    2580838110221962748,
                ])),
            ],
            vec![
                F::from(ark_ff::BigInteger256::new([
                    17639676468645473319,
                    888693410042224555,
                    14177576109517589658,
                    2185346627827744462,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    10607355115540624468,
                    15774427791184772893,
                    13149331261545044480,
                    3154596821303160759,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    6543863809062994531,
                    13732333187846480653,
                    9952271210104001802,
                    1315950358235830656,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    5954909155541952390,
                    16102404983695120323,
                    9897837726895240517,
                    19853293672108833,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    10863821035493121331,
                    1871197509128646774,
                    4173388152322705691,
                    2626830779975508358,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    17561286655688524601,
                    17800398173999086601,
                    13425280108638453231,
                    2761852970889205258,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    11810076715844233486,
                    13946497197096041702,
                    4818928063215157415,
                    192818041124382789,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    13600152882828592882,
                    3100787321998877309,
                    13958442074939568224,
                    945250254364857967,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    3611725845111605616,
                    8137492958489745362,
                    4041472523250775420,
                    816929964837233422,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    5716819708311147616,
                    3035955680349782149,
                    15503885738525137040,
                    3361571168933697381,
                ])),
            ],
            vec![
                F::from(ark_ff::BigInteger256::new([
                    4558970207800659510,
                    15993193242186600419,
                    526468997608638413,
                    1224055611437574647,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    498660970270635048,
                    14430880431421000632,
                    9917135655581012924,
                    3334085529816218776,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    11992110873255512505,
                    15941967633762947605,
                    9885627033058253738,
                    3405319243587616211,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    3889367010600291667,
                    1733092886180089053,
                    5713798896780769445,
                    1380177921653681437,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    811247083455867042,
                    8940613026096036282,
                    10321220332032078240,
                    1462926486851077567,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    2065976595929808747,
                    6564265560069087493,
                    10009703696548222366,
                    1326621636940721604,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    1604590050622970084,
                    7558241162584225359,
                    11213827884792987238,
                    394246795101350288,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    2030575369586631870,
                    7050103533348723434,
                    4646538798586690506,
                    1361125340590821230,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    8111655984621691951,
                    2391420669143636106,
                    7449131261115475744,
                    1252745541779700555,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    12238636937339192587,
                    10063926441592117082,
                    7011731630867042963,
                    230823416734505605,
                ])),
            ],
            vec![
                F::from(ark_ff::BigInteger256::new([
                    13605372029581974424,
                    14499090432198117563,
                    3838112490161158096,
                    3106793326832331865,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    18212322872656611853,
                    5658479006109341116,
                    3755847017970885545,
                    2720447388881729374,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    13517669622348007518,
                    14342851297718450885,
                    4416932281229105975,
                    379343315880096473,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    10856381515450970214,
                    6871366753546401071,
                    12695681108182803011,
                    3301445434575743148,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    6493570072567596789,
                    468716296047898565,
                    3291775223530899627,
                    2781824561212686099,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    17951783765070101648,
                    787097698163824860,
                    17074860153427546571,
                    2365586196433283191,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    9981676994633596653,
                    18420723120411946009,
                    6897046263162816563,
                    3378502521721513731,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    4557630297673418983,
                    16607522508087199040,
                    5168374507508251591,
                    494703244129490024,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    11471583109499573842,
                    10688825685889139319,
                    10669244300181267114,
                    1396387571285979829,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    13906906598902072872,
                    7906495638633317657,
                    533659054527014908,
                    1585129071590507177,
                ])),
            ],
            vec![
                F::from(ark_ff::BigInteger256::new([
                    2734208781506096631,
                    6679726827865885825,
                    6885249432451213766,
                    1870247168557016941,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    4261906686250341558,
                    18217348172049853332,
                    9876866800129954696,
                    864251051311246875,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    2361576234042156559,
                    8338407785888410177,
                    1512784787385852567,
                    2323187694649890921,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    5366563045341593124,
                    11250129638986661671,
                    5203005962418926028,
                    1986415911247432109,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    14239816425716805658,
                    9615690093945014723,
                    9678147020313371736,
                    444917640559041385,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    8221740887820546911,
                    6560541252738176090,
                    13326826323666965287,
                    1408504781754472508,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    11924879267030565349,
                    12501102868796553984,
                    3438152000219859348,
                    1249917508356707113,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    9616607317439755647,
                    14993306982954323569,
                    5864753244935338021,
                    1709673295134358908,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    17128019135641325823,
                    4974677594578461538,
                    9973440025986223386,
                    891779813493727450,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    10699840941518754119,
                    15434571860541369675,
                    5210718876410493439,
                    420364239778944658,
                ])),
            ],
            vec![
                F::from(ark_ff::BigInteger256::new([
                    17840816547742862593,
                    9136537853115858247,
                    16004631864106780289,
                    1139193335759254418,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    3591923788557647258,
                    5643145322169059292,
                    9114012152402632856,
                    871497395380667565,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    2787307423441729706,
                    14502528079983083116,
                    7237062310328461719,
                    169481796829284671,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    14372621672614784799,
                    7537909270096094049,
                    10295271978907244424,
                    2311296528427679600,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    3954968700651345390,
                    16989504925197258067,
                    9007310064622590084,
                    2369281494869445167,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    10090234748750667818,
                    1040413653919394397,
                    13443906812789652403,
                    1517133711550087395,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    1778238098979188820,
                    5445356098900644849,
                    3246644265667548957,
                    1681848350219474720,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    6920335689745114700,
                    1492719641696554617,
                    8037576327871765753,
                    792375583169338367,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    16463727122929319271,
                    1225906627841726702,
                    6788682643777896468,
                    1914469376100771396,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    1718981074463554572,
                    611396712845732785,
                    14402002591497214333,
                    3338882841713746700,
                ])),
            ],
            vec![
                F::from(ark_ff::BigInteger256::new([
                    1850853417574402082,
                    13399939877898838634,
                    10841193241000738325,
                    2571604869515719816,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    17431396625558030186,
                    2467225601615303522,
                    8616322008584043557,
                    3455616304831693414,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    426124312020486855,
                    17185729430408511630,
                    17210746215785456637,
                    2128372353103547996,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    4508876780039186319,
                    2269617129010043605,
                    17486120874545287107,
                    839592365774701267,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    17576002845669035015,
                    8234311209310641466,
                    10698037333171095413,
                    418435337306901288,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    9493757741432966245,
                    13213096263531373290,
                    327682181542512598,
                    879445493962259641,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    14676882960401942516,
                    4626147780757862085,
                    1181108528433793705,
                    3440623426333506352,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    4824513410436531322,
                    10705259804359619317,
                    15125633568624993168,
                    938523825037835281,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    8303688589961191914,
                    12164504190673927410,
                    17567468780317093831,
                    191559167249577573,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    11431490657029837894,
                    6056100482032374753,
                    982000750707275487,
                    1580398295318777601,
                ])),
            ],
        ];
        return Ok(crate::PoseidonParameters::new(
            ark,
            mds,
            FULL_ROUNDS,
            PARTIAL_ROUNDS[8],
            t.into(),
            ALPHA,
        ));
    } else if 11 == t {
        let ark = vec![
            F::from(ark_ff::BigInteger256::new([
                5675361832794080601,
                8403179272078432182,
                9692439538697923001,
                527676780465408755,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1775757101357945880,
                13905455119187098750,
                7550150135178806017,
                474289545226116933,
            ])),
            F::from(ark_ff::BigInteger256::new([
                127783581768523926,
                4405270859941778920,
                12900820918815849312,
                2720650409281539319,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11527415293499744110,
                3251949135550295820,
                2864647163660975692,
                2582450797010596648,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8766957523995242186,
                1499535894504420732,
                7709084319184083675,
                1960609338539373239,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14484278442285627353,
                4334556197460847669,
                2790623819273423471,
                2948831638984597092,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5444522097576537837,
                13635084388899186958,
                5175218480093644976,
                3213426439710351483,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17190953482254179016,
                7227384865464623290,
                2541141813675658048,
                2264869754654078885,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12313848567062015965,
                14613151061300341257,
                7863763216072466343,
                277874178708994215,
            ])),
            F::from(ark_ff::BigInteger256::new([
                729937339077167223,
                8766446263337918107,
                5160097554444447307,
                1135808553766370731,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9089682816921873572,
                17489269669026227282,
                299660375475254192,
                2228481480481847644,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8045073319415103039,
                579087480806431907,
                4590810485072351955,
                2695613242600862772,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12527219768420737840,
                6513279408565151263,
                16053809871893187112,
                2139119540454699601,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3353314698607024953,
                6983580535616645986,
                5536757057717894266,
                788794555657836282,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1287661385781369884,
                10905406258610456826,
                17567230927749156309,
                1391634626396372492,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11633657044753340110,
                17496596673043924313,
                12846475038786346112,
                1833042000680672205,
            ])),
            F::from(ark_ff::BigInteger256::new([
                415683226543883873,
                6813938130134094259,
                2637869501768956455,
                1889556304103956396,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8528056405029778300,
                5149125509015648661,
                3248776770627286105,
                1725002281441753858,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12603484418053471817,
                17461018461835864279,
                11200375741449954478,
                760931332592060793,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14919327137529000162,
                14395510373645550666,
                6611828675911227090,
                2650903582476304799,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1385442591688237013,
                13396371638152385863,
                15858489299072596003,
                10139466196129104,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7716194553714129099,
                4928693566921585112,
                8130407538410772798,
                2209944521730716850,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5425183003653594368,
                10940614514370643772,
                17142205659085698228,
                462629760736904545,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4302055085687339586,
                6568237794014148081,
                3505711265838390551,
                2225868543146449008,
            ])),
            F::from(ark_ff::BigInteger256::new([
                319380783892703971,
                7061498171355445412,
                3014466432973299949,
                2209380220337396702,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2889840809098635725,
                12806794962143627384,
                8447879222580762908,
                1374592073013368532,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10425618866066975277,
                17747526197977080172,
                11108648357158446088,
                1893095243280962401,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15695644280291497135,
                10368825808237075800,
                18043483943655698648,
                1966398931474780857,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11770347989394119600,
                6696040286251230109,
                6606826685199333754,
                2109324452377631120,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1490073677450055311,
                14480150151925877492,
                8384491778465241996,
                2744183782224396377,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2336940259402674958,
                1235422743284702041,
                10669815197888974151,
                3131280283433944523,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14648195492967692120,
                7928009844466975922,
                14815892592662018933,
                1703845617700069338,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12878868166339302630,
                6843230482942804265,
                658572740582286656,
                2780873685630849599,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11456517989674636425,
                15697686726049651472,
                17991289046595132728,
                698192994057124939,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12487533000231754270,
                5365416836609383957,
                13567329405886159610,
                476283472044827393,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8771054630440616649,
                8258720898294156868,
                15372453053310650553,
                1880662606881796978,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17501479651070902919,
                9635180410011021005,
                1408112354858205708,
                1777189930015049267,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5874614697096547248,
                5081727952867451480,
                11067874884870208225,
                3316149624893803083,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11972115426776786947,
                7463743144919246254,
                11423676006336183234,
                2192410890479237309,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17111933416964245853,
                7019319908296620233,
                16749359903936181022,
                657687356010158878,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4156526749851558840,
                9156045890390784909,
                382641367371659871,
                1453610302774968152,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14873052988111195521,
                4921221017971747467,
                12467075635209265522,
                1448168794557305821,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6691628233586148017,
                3357812465451318654,
                7306333606933484649,
                415981475718890730,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16785394070847708639,
                2183426226214483625,
                17011904627365313632,
                1244229559212024094,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2423456768856325877,
                2572305056536360323,
                12705770370960650354,
                2429432174881159411,
            ])),
            F::from(ark_ff::BigInteger256::new([
                18015296346051409281,
                10072738433559656149,
                13575848595572033460,
                1580596115361991248,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10102858361250168809,
                18327454787245336883,
                7544536708230865148,
                862590063932626703,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9256879979117164537,
                6333583720181110823,
                6991943335571741935,
                1765956491596314616,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2699137262193379790,
                3930908606681796628,
                12513320855078705029,
                2330027262100034428,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4149058045846409219,
                12734458678071277915,
                1623625530078645734,
                1853311250012337851,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5333840071467806118,
                4450046411833351997,
                6856359391218941307,
                135556139183596749,
            ])),
            F::from(ark_ff::BigInteger256::new([
                18035785984776418391,
                7399864323205974994,
                8234948789665450209,
                183396166787338977,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15346895557267346822,
                11922183391817468073,
                14052174055526969521,
                2283835146649391412,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12609742448234750009,
                15548196854162779687,
                2476845397651515068,
                2575270645434406485,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13969162251185983028,
                1428752407543832228,
                5631889631037555027,
                1020868969122748052,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8252888155131555082,
                884957584695723739,
                14016663810145503399,
                3412893251885546853,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15522988739002268084,
                14214686195900409242,
                18201102115894833434,
                3410844401606806113,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1327034283157243203,
                15435433869392301374,
                17093921149406869375,
                1767767803862728190,
            ])),
            F::from(ark_ff::BigInteger256::new([
                18207122081862660226,
                17910708925142179931,
                10879529426951811229,
                1326251604028701722,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10644293788096723202,
                7812330161287323091,
                12849814491769879887,
                3046052139601812017,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4874986427142787862,
                9355727026390469270,
                6922480380417305934,
                2477023346880250787,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8547662959327092971,
                8771902310484255507,
                8664737448832468962,
                2948875086383856417,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17555367918036502960,
                6308611871433541358,
                13695878955861042288,
                1648657697488221118,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16352768047318739925,
                15021728107895293684,
                561673697658387901,
                2488494138173506968,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10241847738050556809,
                11158073214004046302,
                16472274113557557776,
                3293732693564634924,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10299166946279135064,
                5837709139842524100,
                12184211851741395995,
                3444754131298883059,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4925185212093715887,
                12510865829802900901,
                8105408723686036678,
                3199225916599892801,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13183556200512655358,
                13946117953721231096,
                46855737918825304,
                3019164529865143772,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16006032401472713411,
                3882865706263219029,
                7257509737061673458,
                378884858287082990,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15823265021164115771,
                8287514702312771510,
                8252603889855927504,
                1193047645719854257,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5308026580689752398,
                18375597781880821905,
                11913112214973268879,
                1151161960005689753,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9577286166048365552,
                4723567999352704388,
                13183561425768771456,
                473225144076218580,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14860686678716612761,
                6754805246163845573,
                9335923622481447325,
                3462489071804891631,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9008000976416400788,
                4806260849324658581,
                1817192298618499299,
                3024644928150954752,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7279620161928115196,
                5855967390346497712,
                18102663412475795151,
                1503654423751076607,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3504834469111128301,
                13283348850727832678,
                2927282465111611870,
                314391964531941097,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9851176953438376096,
                3615610770227739887,
                13351728097340790870,
                2197163362329282222,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12668441783973352347,
                6482481248400478258,
                6538834043510035726,
                3171448778186239592,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14683790716707290699,
                5659277325193158485,
                4694531233932407303,
                1260636698818536258,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14605307255997176125,
                17458260463338494315,
                10033028884623279138,
                2992897153361101356,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1460958525309968373,
                6268408055753763855,
                18157256324687632854,
                990411013317338184,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16335595051012093277,
                4611945831376234709,
                10109701320294088656,
                2224569633900988588,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9548753160884991153,
                4736050753494897068,
                5934022560077821914,
                2396766848669398589,
            ])),
            F::from(ark_ff::BigInteger256::new([
                81455292879985816,
                3537708513909285134,
                6531629534290358558,
                2607022018445174147,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13733414266031184250,
                13741023062309206599,
                9933458140784856533,
                2960356997440759049,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8668626443250597131,
                5794492226357887195,
                18125235993088663888,
                2725290566966848892,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11735128014617900486,
                2743403802507774186,
                8557242485154449033,
                3396823226320378889,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4175423485352514752,
                12724481505488372000,
                8115038138670613999,
                825348992131763838,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15397162928010146768,
                1379657468421857156,
                5675391371913752422,
                2760418501990893259,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3777941686819933751,
                8849321917634658011,
                8602669173732360206,
                2038397141120145464,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11095724774558230573,
                7902878711465083600,
                4999820382405440485,
                3404847062564804782,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17167194491852848128,
                9607775593383627751,
                7145634993303188753,
                2761194701357176534,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9661301312123256495,
                16503266813415467777,
                12323915169512552963,
                2420952739702624447,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4160718004587242504,
                15648407256652590481,
                66900627821567760,
                1789270909899309562,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14974577223999772620,
                15153326635129182475,
                13349289236395927717,
                405002657062264518,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1335412173172720978,
                5470369564390149717,
                2190552457610289121,
                1340490909599309378,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4915536545884970774,
                8473498782296150779,
                7737862059688380534,
                3327032358476068063,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13269759485125298633,
                5004333317343823821,
                11851660679556857541,
                141926597254885099,
            ])),
            F::from(ark_ff::BigInteger256::new([
                80059173411082123,
                13294088996397127965,
                848733365701377689,
                1260367758995022951,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2530406804871510175,
                2917682432471976699,
                16056830409405384815,
                1288254770291203831,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3978868902480024112,
                9908137796436211398,
                9728023919275638614,
                2719975038978940412,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17833134685644418927,
                11825625587807639762,
                9065643781457416912,
                1248611732823607457,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9228987841295965055,
                14834828868627073388,
                346471043153667589,
                296091032856755056,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1255243140138097981,
                4964844670234880410,
                12581307221239899005,
                323368318314653188,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8397577663835019278,
                12572809059879316623,
                10270495790890844781,
                1535431010250163931,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11858803447392081442,
                17476387216359516713,
                18085620593276186781,
                1965889909379835480,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11575386858837297965,
                1832036296238686188,
                18284495585146421378,
                1358829470257072982,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3705871798666373520,
                2332011617684460992,
                9119470897969642235,
                667700700185584474,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7333269030300596020,
                4911660877301215553,
                12244080769159743054,
                3349208052598288259,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12254675322809936914,
                11318900431747801853,
                2844312786318885609,
                341324117983987149,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10478473108927546187,
                56358674764550385,
                10366159201897626276,
                730439335021084998,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15156991237553581172,
                12594577576856146877,
                12119285504508433048,
                1594797895674869871,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3786997158752905418,
                15010661363762423321,
                984700735429012640,
                1967593396422376941,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8250050255938307993,
                10896656485345912536,
                11601035171953940349,
                1906041158612831900,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10677722193608899986,
                10103397110057396797,
                14772564845564852228,
                2965621352216286822,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3539825084158463448,
                9343021139747409825,
                7371286434278546724,
                2279791013476925866,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17649345393535143409,
                11645422831432655807,
                4998748247196253881,
                3060311677790282522,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2173522336127371353,
                2286402468996686561,
                1309821222826421393,
                1522765347487305886,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13518166287799456087,
                5505501181694587928,
                13962762291404270457,
                2606167605379890137,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3248467689668208509,
                12657879227853516001,
                10481016289053687046,
                2072514856740454219,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9915319109210960735,
                18374979706488559204,
                10077745876958219469,
                2575259790953787213,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15403173109373286668,
                981596476765725500,
                2301243935337480787,
                1517620000451392957,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14623013424358912122,
                14353217746239153083,
                4594499933057866411,
                1780114481020138440,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3074864090029319832,
                3340026746008074110,
                6955981017106713487,
                1421803685766461552,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8922430232267598523,
                13417574844033185737,
                6881589189113628480,
                1925459636215821993,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16781344569815162806,
                936846856842466413,
                15398105589771693221,
                3269570510551091960,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12508829377953876937,
                10892597380696847308,
                10209916801397601160,
                829626660446344225,
            ])),
            F::from(ark_ff::BigInteger256::new([
                18130758550202691130,
                4947556049846526919,
                17637024638983647898,
                210789235240349420,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12471077872193345205,
                171118987601647965,
                12887027764831769481,
                2546659683901959473,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3865427421291489174,
                14546474597630343784,
                14394846857043210351,
                2076081040119455240,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16782302515302551507,
                9832435800509285237,
                10606127213666825608,
                2021191771922457786,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1186504043627815907,
                14064787542729553652,
                8867552275786334609,
                825255733263934455,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9548945635689285170,
                16841264769090826739,
                14138233664009554748,
                308239014117231513,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13353758724986042112,
                11795956932526469541,
                10577874082458602350,
                2106336932398773342,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6719219241635744992,
                13646332354357811742,
                3057521619484224407,
                1172265305607086680,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10346861015926641972,
                9239858906412736470,
                15047383666686875589,
                1734971221917030660,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9430396554511094350,
                6487682614635845921,
                8631517892475655726,
                2884578854390054885,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2133109543463108625,
                11194938805399450613,
                10970275666709084533,
                3290232066910231151,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11524136525632045673,
                17985041902990081948,
                9093934565768622626,
                2376525692361872072,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10287272752252821274,
                7605470764437589972,
                1838175154612788034,
                2005402481991062141,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2821694497942529645,
                9573810917072001760,
                3538539666527810070,
                1391914321793517122,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12140242654867750572,
                15550317727175334932,
                755658998968659405,
                3215047163449125993,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2914582703094625636,
                9613260064435397611,
                6823165754390787998,
                1887459433197513193,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5710199614189156360,
                17639687129229348511,
                6612461705961652336,
                1878630179905822373,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4132351673232322422,
                3951451460773098116,
                6818682656778543089,
                802766490240927975,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4136415824071766804,
                12807708126678047349,
                17111925832512669343,
                2054196546360780216,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16799049154137174896,
                3838174443861491581,
                12609919334815747087,
                126031051988409676,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17102123462627958764,
                7538214815920629214,
                6617267413793370775,
                1587114010454755582,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1905308098560721718,
                15396069958544842317,
                6094568475482895716,
                269545005535193483,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4970671490759029442,
                16811401482024710821,
                716062910021458630,
                2092972695532152742,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13334069003775560136,
                2437829396833370866,
                9502944523544833612,
                2371676115601106008,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8210018052278115045,
                3683752824013319770,
                18379630065222674849,
                750375732597373343,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13147632387281607817,
                3631262181295001861,
                1116428729582554560,
                605673747693514531,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7637113847547216900,
                2865081815453786684,
                7779717248603650097,
                1465363096659430645,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11919410888280012228,
                8069924161318700761,
                1350551887538789696,
                3137803247640632650,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9059302946038065110,
                6821964997315241466,
                12802156619622467296,
                620306862513069456,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10402614744764202899,
                10309647516591901746,
                2759781008301989944,
                2862837507513246842,
            ])),
            F::from(ark_ff::BigInteger256::new([
                18344988138431919904,
                8920447801448267078,
                12359638782242114832,
                505598220094845366,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9606009799738441932,
                2165566700805717615,
                9841268943664446985,
                357417004672924536,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9309029068244648534,
                14210190280965617674,
                16375118151880771201,
                1563311888065359473,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5969046422152861043,
                9744176531640836033,
                10534175900604003393,
                1196555936801521595,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8369653946396764285,
                14241359946461251370,
                3582243004636486862,
                1083051434706316983,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11688992527723480340,
                4259971711132035017,
                17801203809927106701,
                2095091772075951209,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2919956484514351888,
                15756221032331185701,
                12864652273674146371,
                447473325929939923,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6821713791899412825,
                7028659653209441931,
                14198639077216933045,
                2005388446517477620,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5357140928307164573,
                8566443277367924750,
                13030367244167959881,
                3321983547271099202,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17948299846892516021,
                12180139659092662957,
                11851287997253346505,
                3392769344941487768,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14286366940559791398,
                17888679936082959727,
                7403857981712654727,
                2827621707956688980,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15486088895817189922,
                7591127097976062075,
                16964232836486766928,
                2022599390476253276,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12989753270087349081,
                6022660375776619019,
                10444020944067861235,
                1630470797576694960,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3030070741157783731,
                14879975166787215429,
                10241226015032420947,
                2404080349587808632,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3306819345127825185,
                16798373164363739680,
                17116656165157166841,
                3233543886087291033,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2938566807010022210,
                16449928060177393788,
                2135720649797485735,
                668463391061175380,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11987487309641973312,
                6668137340143608999,
                3186162341137769245,
                1122954876561625052,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10093381973781012100,
                14716416668525431323,
                7031573475073111578,
                1751937301592099611,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1057295475371543349,
                8939220033110992606,
                14329530068936617106,
                2654145535248990107,
            ])),
            F::from(ark_ff::BigInteger256::new([
                121776532933713175,
                14555716822300675026,
                17215032737048821165,
                1195940515111318819,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4140751869808753800,
                2435601884884829272,
                10429290221166214009,
                2641813924768450179,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5382444996034348659,
                1760996819538508622,
                16918392561491868915,
                1344321235238967744,
            ])),
            F::from(ark_ff::BigInteger256::new([
                18423838256676301590,
                5946018736757920252,
                7365175327470608508,
                955517309818858218,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1621294376758823594,
                10596369518613889595,
                16558090288879178512,
                60184010351278580,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2498589177803991508,
                3028526821999255808,
                473810858671718473,
                3386283007755095621,
            ])),
            F::from(ark_ff::BigInteger256::new([
                219153374313691187,
                8055482956551774179,
                17896022063200269659,
                2144687885141204004,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14389269649103927140,
                18276564729771839318,
                14989106015837312039,
                2607889120711894146,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4791907660650933873,
                10768994833899468028,
                50198962268982064,
                3083013486113117410,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9394781868089852085,
                3855658406660526227,
                7927113087100926246,
                1637149261322791037,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10678306740567157364,
                8015991908079908113,
                5937068213289407607,
                3017310595639371295,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12316950544597092019,
                15536801292305226839,
                4250508443845227430,
                2706815508004217103,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6903043776978040134,
                15848621534658269334,
                16567129907869821661,
                1477450518667784001,
            ])),
            F::from(ark_ff::BigInteger256::new([
                18199102126166832568,
                578638860113258350,
                18080481801152100017,
                3176323037186102491,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2692264176011862705,
                12025199300469392154,
                6298987442080380687,
                1744002249849645523,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3228224817773029171,
                2293771214175493102,
                7292894498361005304,
                2797690164016182179,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6264909217960464558,
                566446319448634914,
                10280614791852027415,
                2955064010921292175,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15779299151409158915,
                10671117547239304387,
                18103813498885861193,
                742212745398775112,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6011791206275254020,
                17025901847242661282,
                14720633293342867868,
                2287478072384817289,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1441254791276185190,
                659573403204511527,
                8659226838203707811,
                1956517413822105142,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14430918711883435753,
                4898333950074987325,
                556395650228156390,
                648322567775971464,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13175905896996256804,
                10474776961869025953,
                12056150857489193,
                2452140940716252746,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8003367699337404149,
                7640911205543756260,
                13194713500615276441,
                719644097867494713,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17236386182439344335,
                9144369039716879826,
                8214357905562386116,
                1831264937318609166,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14073452282254196613,
                2575491378061107482,
                2892570787191928511,
                555734316144875270,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9278653598799741989,
                7345846432574673999,
                15721087863311965190,
                714291459703315690,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16277607698574650615,
                10370329608644695366,
                1383859722789054797,
                628057229702880048,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11896784674130764154,
                6266352953609453483,
                10659052841172078496,
                1816383402840763966,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2061419501260375056,
                12491797383645213647,
                11304540370006131973,
                271519293143025683,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2518184849101891020,
                2957188302308249346,
                11309269498229544512,
                976005083805473642,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15151275824540211662,
                7321153288264190943,
                7707079767974056317,
                2167570707456096615,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4715756378372307678,
                7107384190836600677,
                1917144876600648972,
                2415843018055774505,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9862432792868200853,
                5410871017288278871,
                6081990887649785740,
                533918262527216451,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14057981969074780634,
                13691283821798850844,
                10008131668946260487,
                450178030262537535,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8083842089851188255,
                6142545741100956045,
                3707022048373958131,
                1693490745005240411,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9282261063102365074,
                7350249057254071461,
                3652380528253593518,
                2734772441879279846,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15919934046097241813,
                17244034061807981947,
                14873105795075912794,
                2482790018983724619,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11540672429315856129,
                4589676569635040104,
                8902900079765239503,
                612181219759323049,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9527272416663695806,
                13742228405449395999,
                13346322340408668906,
                769764754998888573,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8055667997378394773,
                138927670095847324,
                4930385485564226045,
                1210887888827393595,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10505883530867463374,
                6624178497005363177,
                13136390846054943245,
                1686078717844191787,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1685558044098081829,
                8765157073062572470,
                7541172729490760209,
                1056954023291086636,
            ])),
            F::from(ark_ff::BigInteger256::new([
                18068127651301378030,
                3420876941352609735,
                2298964595009712849,
                3377451925839008169,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13777925435240000705,
                8328986013987593632,
                8914271562242789943,
                2287953093993372979,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16434993221268600930,
                11435370175696285834,
                13750990283875415658,
                251040454054100444,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10701051327508383651,
                16569697092901803542,
                8194824614487274463,
                1693186976148127142,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3923116906259277322,
                196011652686425332,
                3079056113819255435,
                492154156918919212,
            ])),
            F::from(ark_ff::BigInteger256::new([
                128460004221083045,
                17532575339712337,
                16674544538302243024,
                1377354692266403667,
            ])),
            F::from(ark_ff::BigInteger256::new([
                724056037554736284,
                3725599836099396481,
                8271309726649573999,
                2552599129064334996,
            ])),
            F::from(ark_ff::BigInteger256::new([
                931049322475501554,
                10859952348632454003,
                6415362963791853622,
                3452000706303170132,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9864953197267135857,
                5948756153124000341,
                7233763207980491963,
                330208796970774509,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2754561830385451908,
                17519888461590384486,
                1766391504979241360,
                54852115972459949,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12096577743685300874,
                16307886907977053054,
                5847178174706724758,
                1309536089374230829,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12800200708236497221,
                12955236432545401229,
                9413162385666631679,
                1287401950733734116,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16148703827180153371,
                17170153040028515708,
                15305626323828518258,
                3417061272232868595,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5169575376334747527,
                13673590267683572960,
                6854882267992071689,
                966610157849966531,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2380444191947893548,
                10637640059548640834,
                10037397760876982450,
                3349657861096366759,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5722187340933532581,
                12929782807667619399,
                16051975249292481471,
                967076657886881866,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2922203171710428564,
                13257715694011973921,
                13200414980721826575,
                1315118129314718992,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9228644245089211515,
                1673123892278905407,
                9721058996389398295,
                1520873961190271357,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1246647621769973656,
                2128150282658269181,
                559874318026244567,
                3031932382474389038,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3121621683723069894,
                296552366312547213,
                8333621231921182433,
                2723633184029788696,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11780885140991280097,
                11309669012963431656,
                5126135769995756658,
                2292757891524340793,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1998623325266479931,
                18331240037272927411,
                2482606352857743595,
                803706084126348493,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1124958750808741375,
                17370896559451730757,
                18018182828933260469,
                1917993842643825318,
            ])),
            F::from(ark_ff::BigInteger256::new([
                18036033447411428551,
                5128601923504734145,
                1147377122258701599,
                561394476703347727,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4684876039564100781,
                7247693259516423694,
                15750059123949920694,
                3448337073847501449,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13227486786783296116,
                16779426029896468306,
                3962593294568617531,
                1535019931109557924,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13320344527299044590,
                3833336638790235147,
                11586361706211916391,
                3203532094271885471,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5456584029913612712,
                4605747952155345067,
                4717904379376636630,
                1463375269768356607,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11344168448098191491,
                15076047455581113158,
                4004024743173436336,
                2505845741243275286,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10743196070401709979,
                4492018003984308016,
                10555152884958855730,
                2867166561208298634,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17533566904168824480,
                7715680864214761664,
                2612610801255560144,
                245891213087205889,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8648435927039671610,
                4090795132100565097,
                238748733824087089,
                2606877317337527896,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13652101638522982130,
                14581556822104670472,
                5664673866650898793,
                1820991729372218402,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9576119551060331992,
                6214907603365727462,
                2243876741977045392,
                1045462527039041111,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12811885386982775884,
                2646155501590157067,
                13072836739952359527,
                587817216446576921,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10766377335993194064,
                9820844941449026152,
                11865036125357155634,
                150732214235016253,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1419824589114280886,
                5960347089941298553,
                9902140493557100521,
                2831745245530107118,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5276071287115328227,
                4903959376163972384,
                8617995859553232159,
                2869731337984369135,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15423766389562064762,
                59644267768653814,
                3887482315609072254,
                3114550083405628715,
            ])),
            F::from(ark_ff::BigInteger256::new([
                476985513150278492,
                16012131162619586464,
                3217902909789197042,
                1973457124592556233,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6913258699510026740,
                16748911780229374579,
                6897948395014080884,
                2487683946265375141,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17636263135521398442,
                11899561314825264280,
                15870024731739476323,
                1777472795675125985,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6733427199949603674,
                4072913364230529902,
                1580265135467513600,
                1136027183593054119,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9137142376373234226,
                6182618624622625143,
                10547055198099557589,
                2083360171461238856,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4518488109779318567,
                15049979138197459178,
                13491286003866784893,
                3325558115630408629,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9244459011847214069,
                3072293281278265745,
                8050179187177052806,
                1518111632673659348,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11138219651611870102,
                16601794931644709873,
                12285424991267287568,
                1775097785746256520,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4394667622879747195,
                11204885353919561621,
                15851488581812757062,
                2510655699437756022,
            ])),
            F::from(ark_ff::BigInteger256::new([
                18242346782518442750,
                11091345278657238280,
                8996556269370427736,
                785721216445234960,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7468620671439656705,
                1423627031494430409,
                4576325610622262750,
                1661914042671305379,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3933580697646221918,
                16271085632987226780,
                10893508411496479285,
                2774468814107032555,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3090160455437457113,
                12771383576600588861,
                5671047862764169815,
                965175123440720952,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2589656262679419123,
                611007820351517177,
                5105825515911807390,
                786665811866091670,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6487223511928913720,
                2439571330880859256,
                4916738583573086533,
                759470969631117286,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5487944536068092860,
                71367144631779961,
                344558320522142014,
                554281000045354722,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16261934857677164213,
                1243747196452541795,
                3358249060550664393,
                1265437238393070048,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2844489435181281314,
                4693772037592029140,
                477064539873855327,
                2834089074027299315,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5368522105610689986,
                18038996024712807106,
                9472888573790248725,
                1802390517102053140,
            ])),
            F::from(ark_ff::BigInteger256::new([
                103709925171904339,
                9796388035459018066,
                14930755143931216033,
                2461537100135882597,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2600855924332330069,
                17513955184996583121,
                4415709387981302054,
                1197570206680993487,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15301655133229062960,
                9822764699548970724,
                17262666854312114288,
                1594620901421077705,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4238297394832562464,
                4807166993821131168,
                10001304914537467145,
                2594705220322339584,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13030456076958467486,
                16994179646306975339,
                14351596545052270499,
                2626729724934227686,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6004283887429652843,
                1714481064452129407,
                13844395530141539562,
                2127531024750416701,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17658880077435123630,
                15867028719163724609,
                9726669885607961855,
                878561782608433992,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14633651751711396558,
                12852091528540134696,
                13006094473847467682,
                919230458525436177,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10063653942400227756,
                12055348040565681470,
                8321290911296134187,
                429182683178900811,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10118321225194546263,
                12519083682224699129,
                9669560879933012358,
                413624060583585575,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15989453874095046962,
                8938294364522069020,
                5662973779229611311,
                1440109606364938735,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2790986336801541891,
                8541164305838324460,
                10727382016495009320,
                1530150242453298815,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15082156225915494680,
                75405029144940260,
                14792497832329394932,
                1656477411268897768,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5784629535394287484,
                18161503998065545854,
                13385743676312455472,
                1536912541034466885,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2405520072334493121,
                6608631936617309284,
                15812059180672103972,
                462175852701825542,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6185865584105270390,
                17770525521646223486,
                9157202402654968926,
                3123210691752159117,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8796369679931967251,
                16675669121413262352,
                4577249624247881971,
                2200708917044359974,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8967493370525616775,
                13809883661224478089,
                6369919595877124070,
                3057346828386942507,
            ])),
            F::from(ark_ff::BigInteger256::new([
                18024939381752133696,
                13620693708580668067,
                749745947815551285,
                693224088323166061,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9150153206764403703,
                4431585614276071505,
                6223252177506398843,
                2197049496962417171,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5149385296310986169,
                11030496434789598824,
                7989280332346145640,
                3346242675082196312,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2698419186848700749,
                9398263047057827656,
                11115441568015549883,
                361889700390081173,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8060933332487099308,
                9400429142938316341,
                14188632430817909191,
                2797294547748916607,
            ])),
            F::from(ark_ff::BigInteger256::new([
                929616428136222986,
                8907942013641936100,
                15538765452389290830,
                3113176745308254843,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6660302544801309012,
                1073847081737752647,
                9656295853522834347,
                2181387551850688736,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11463251410331306432,
                15836567207524747102,
                3796591803656253029,
                2590663750895913300,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15602920679246799267,
                5242255698431932693,
                7991249859049758606,
                2129127652946728681,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11611541677852802347,
                821396253247899182,
                16108122389376287859,
                2170105362794283820,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10471931905703253063,
                11031441556256952223,
                9427184900388158178,
                1245219097682068961,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12967991123329847481,
                2098900704346394542,
                13811305899177021929,
                858292343262952556,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13097678739478455287,
                5415138691288973411,
                5312502758781778263,
                886041048674542582,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14561580074508439022,
                2173111979029789033,
                9732660690346830671,
                439536507243651672,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15429580766032135069,
                7466100949724427801,
                3185355107010289529,
                2227323234907766180,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8275078904178284490,
                15980661304513247303,
                5147925851322359631,
                1272921519513059860,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11695045144239128152,
                2118581279898594200,
                926521755988569846,
                1891976333756015345,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9967616968124096860,
                16798559116608869007,
                10815555493318807564,
                2639841091648469588,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10299952442476472178,
                12839637780985592863,
                6461896994548390113,
                463931007003098538,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4156179933554654259,
                243144576026171457,
                12805075079597807371,
                3393717766593165252,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15645441032713242273,
                17491558249975962310,
                16962467746926103480,
                1961445268195087419,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11651178335084790782,
                14277046077755863011,
                14128815507472291976,
                1077913508359217437,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12036573856970614376,
                2313254383129001897,
                867301656327192762,
                1831336268721897469,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8464217730368972998,
                8689875120241399990,
                6235854198911278423,
                3315489499426156518,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12980817536431182529,
                14301976196708928579,
                12317549990406158439,
                2559095631221693846,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12042985564101295989,
                10548401341235924810,
                756347998890026609,
                2368379750846774944,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6796278704446806279,
                12796990999805072854,
                13452592159356467662,
                2241611959119838093,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13469164826442688327,
                17402242065960300037,
                6710718599092825608,
                395065427831380283,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2682474979316615765,
                12361005910794458370,
                2689484061963121512,
                2771819111600001835,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5463693829272284992,
                143771750424516862,
                13315855540106463121,
                96999549816422142,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8335590175113516825,
                2541900780777751541,
                17615144864118040852,
                3041820485551219633,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3060003751274248717,
                17447640664075809871,
                3095071211581132250,
                1075664803455500452,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3665835477223019301,
                593792821812253978,
                13130652074470611650,
                3168370218852488853,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12491198079991646777,
                17040173650262334040,
                5076429526916162499,
                1606202423185588678,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4572474922215446334,
                712638967567671180,
                887682165976697222,
                2328490656128344443,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17849855839571755894,
                1207735307040519377,
                14961989028359924060,
                2242296146215697802,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17509508118137113684,
                9358888201743201651,
                9314455265825302791,
                29722181313609077,
            ])),
            F::from(ark_ff::BigInteger256::new([
                18335102228316693395,
                4887605355637461920,
                9881209774307806666,
                3393277778459106132,
            ])),
            F::from(ark_ff::BigInteger256::new([
                884588980696232599,
                13464589628404563865,
                17740029895293162921,
                1604743726331117441,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11084298394430011564,
                5798973620245176126,
                2430844605453128148,
                224976196784709617,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13781518475943310362,
                10217673786580807454,
                942540662397237090,
                1239419786517541908,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7775633527496673555,
                3306250609779545267,
                7306709070515725000,
                151066109635711261,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5075662677007219761,
                1764873012088061504,
                2634104053525070489,
                3386647780716138356,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7366826414008541462,
                14233456545878697846,
                11003227459100744425,
                568932640869630539,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2558475285871730609,
                11152514560859932040,
                15839293523417182123,
                2989862077123041235,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12731151168443857713,
                478997557572641595,
                9076987704988343179,
                1481709184322733516,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13426160436915455311,
                3428043413372346696,
                10726327680257918226,
                1125471243643647245,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10883825360777411464,
                11593457101066201729,
                8173208383111652010,
                1888443820306502323,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16054839571714538389,
                12088442089868328858,
                9955555975460148303,
                3149841730082754373,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7045244600874674546,
                2556430906077595107,
                16613508185524062641,
                2280813580013026881,
            ])),
            F::from(ark_ff::BigInteger256::new([
                193300538317695043,
                13034309808600936721,
                10804387540785580857,
                2313067430592099815,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1955974717446950106,
                11066525889140295266,
                111535304018099999,
                2473524181355231994,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9882772859736808161,
                2647586071722398113,
                6036710798356206994,
                2083487417987823088,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15629055520548533985,
                16313281370835829263,
                10886054962468676710,
                1313593385308925042,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9396225950282119589,
                10578497581626166703,
                4923136700651912143,
                2230859057032624038,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14894972938282197921,
                8082689937070416702,
                11224777772629030334,
                1867900333128378159,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6184719072945125957,
                4607319271694591628,
                4091991606417404339,
                2604847243139688655,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14650403420830951859,
                14770623529746580239,
                9338231299506183340,
                1021343361590876987,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6812345391103961838,
                3229635055251146895,
                18077229160167151679,
                3348384758345669475,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8129942219414841142,
                10899451166057489213,
                5190605374978790473,
                1809501189397333694,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3238755459508935012,
                5815424813151785835,
                14133194165885206920,
                3037362437969710226,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11174795738838995184,
                2550311916373614230,
                1658493478528686351,
                408256562175921740,
            ])),
            F::from(ark_ff::BigInteger256::new([
                850886550370590853,
                16012636750441734577,
                12420256500767488956,
                553014286627949959,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13614619912485047305,
                13827912616199734272,
                5514071892148936679,
                408310952774990864,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14328819102780972505,
                7289077674492785402,
                16818278732831156921,
                293192158243586043,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2834236076274535710,
                9668295403736970272,
                3012618226232565439,
                2168015389745919709,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2188577138625637236,
                15142487864923107597,
                3350424336351348236,
                3127161883406230411,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3940856883897991455,
                4377405151944594203,
                12813828487862277297,
                2452467820228130663,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8506857880167668148,
                10513586558540147380,
                7911660635906081591,
                1929538051681312733,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6194319247724056171,
                3515390467413446085,
                2877170599391459398,
                684706534493431737,
            ])),
            F::from(ark_ff::BigInteger256::new([
                748198141377596782,
                18112719433302149949,
                8274092387075101755,
                2143151549154867967,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11739700016288500120,
                7540221532929602933,
                4486215716760851536,
                451137751654114028,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11888585183426922219,
                16376759220376696519,
                15797915091352869991,
                2618299822037563940,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5864188095167460686,
                15880432778168766956,
                580436222693280178,
                657523609934570938,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5282010335286828622,
                5439377409248655835,
                14936470411646342514,
                3053861477916224826,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12177804516994885836,
                4369677886366480813,
                7840433765126238625,
                1369805166533129891,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9939052122450449067,
                10789568847316095039,
                16189502750868128071,
                2130270446698919259,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17715303425785493360,
                10327380668544975138,
                663906819086559878,
                1109881752969251773,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15605770757034799729,
                16844395355745264924,
                4037336773365007068,
                1172523752901552333,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13988254250123529003,
                15959140509663456310,
                3312242311766033222,
                2942883517635736413,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13035073172456119633,
                3872705669854347067,
                9404435689849652335,
                3252825282034955055,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3137749705421024929,
                8975443249021448380,
                5903581959192615098,
                2712461968866239363,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5009362522707946461,
                10674650136528595372,
                157761788323602783,
                2009719064707699860,
            ])),
            F::from(ark_ff::BigInteger256::new([
                879159522165862922,
                15065970572549522717,
                5346000732752010570,
                1974805698957997441,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17880878343560402370,
                10448962982773760071,
                5327635855714540645,
                481156321888210503,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9432849983018975513,
                16565960825279687951,
                15989976926204332161,
                741830085678292915,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15624265395647892627,
                17854477814596495569,
                1101178770039432207,
                1059657853923088847,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9901635858043951486,
                6798844060515803434,
                8605219727158042648,
                1901744045612458703,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14892870051315323585,
                4502521014053599099,
                18407933354169742959,
                216951552517202521,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6314365244603115340,
                3412149953028834371,
                3515581337940474972,
                3177255142894450998,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7024068369016001996,
                15051311027126095609,
                15888652879636355444,
                796634496223273539,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2053345634587189947,
                16578763918765984051,
                10176285255270119606,
                1341636441039056606,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4934921045035435263,
                17208286466926348400,
                7477162800240130134,
                3233206210676528235,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5790731692513064773,
                7011894062097668052,
                9019114714563093554,
                3086355670794174862,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5142240064988156406,
                14144976727743947005,
                15667719799042056463,
                3074919868410639802,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11252756330228556427,
                17435523549735615900,
                18297396428185646465,
                1721027286030790029,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13372469211289236118,
                6782172798055081460,
                10501106978604348140,
                3375833064302803360,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14932653066923429642,
                13911833915635811785,
                4888645588644752595,
                1593401519350081172,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11100721511856121019,
                6521420993585113917,
                5171747305704988445,
                1819168891469168901,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2088056403339048929,
                9719553196019638052,
                6265920992324036017,
                1461879369634737723,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11735181380676855762,
                16713345935762004092,
                2314876768493726558,
                2333125164024322865,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17234718842505702174,
                7456083483391428424,
                10848751877949183844,
                328126017639771216,
            ])),
            F::from(ark_ff::BigInteger256::new([
                18035185638654276616,
                1446570843544404930,
                10495678173441942370,
                217012713186541293,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6953974297899798171,
                17118507006558522888,
                582427744334579680,
                281502626166431525,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16378323852561773632,
                2459613518825733709,
                17515683227369206435,
                2041424176892166988,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8500593942408384751,
                3355713955566571316,
                5033140189438275198,
                581541040466878825,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11930559736002123875,
                4096228873116124074,
                7548562179373054162,
                1370860377376751697,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17414540710958548892,
                5716382538246106142,
                9006631058549268050,
                2587798323207286979,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8151171623354876862,
                7238153392909219988,
                313550636576870955,
                2907980290316092299,
            ])),
            F::from(ark_ff::BigInteger256::new([
                491319104012556364,
                10097713347087923528,
                14429201133436071254,
                2691461483408925184,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12673696306947659835,
                17839104871787649562,
                4748054894418913783,
                1023380575734144727,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4954201927094064278,
                13977408016122193372,
                3621553456581778991,
                368522364757252010,
            ])),
            F::from(ark_ff::BigInteger256::new([
                389331136035238216,
                8856761037898951117,
                8657062331076243644,
                2286349572177926023,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9399636925695847549,
                3841752025515567734,
                4340859099672188439,
                2030969584424107301,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4095172026187905366,
                14885045870539987511,
                1749461207350471154,
                1546819220203762268,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10168748425458173943,
                11402310494799924666,
                15975279423419997918,
                1143666350557331524,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17167902390102132912,
                299507084117194120,
                12412120546925161858,
                534213456039930910,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1960580454029650012,
                7903755864503333967,
                3816254829665208913,
                3398545708814558891,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7933985222514238148,
                13773477358867791349,
                16000161470703557333,
                801007758158449462,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7079272701248375588,
                15855910103723030483,
                945061628304182227,
                1706539701378381805,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16166690395811255425,
                16632688285736497982,
                2529491645097859446,
                1527072849260997870,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6560540191115170218,
                9234557273586909646,
                8900473143046829500,
                3092040969448903707,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5168934001083505493,
                7185468161711146871,
                8967757307813914814,
                1117317410265738955,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5433278628617322510,
                1536070301033931391,
                15343168730972974517,
                2261514592439686894,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8014697535605544373,
                17300795350579806162,
                1441306561897805328,
                3447920407406316852,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2868029885323684689,
                14699994568566265202,
                2378873346816504423,
                2779513848504868365,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2702790378681376832,
                13036042999720102254,
                7530001936759762997,
                1137906065009234434,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1104122821498802636,
                18086237923015419495,
                8738107438771379080,
                1906196013721438269,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8797284395602575875,
                4801329715029236590,
                11726117452312700156,
                520827537497108146,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9102092884139956582,
                17480239253280622281,
                6710744562454650563,
                343967745071999816,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6790248542920738858,
                6706309451904098046,
                6650584873157779573,
                2105444565610904991,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13042936831878723921,
                4757198070909297329,
                9863302195827250428,
                914367884675336496,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8836068027695610395,
                5406698969764235775,
                14149790920414102845,
                1616430171900207059,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9434513977472052956,
                11273108517216877293,
                1981316047968527678,
                248440969016063491,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5290269625422936884,
                8820140978919708614,
                13130539113826681237,
                413598143730769094,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15399586085702337051,
                3048502335398878247,
                14265689598834773812,
                1459580905172112220,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17952080062004830666,
                6862895445595858736,
                14518498648558438325,
                1040305990759190332,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1996189038382902606,
                283137673513321141,
                15247428758438137436,
                550260869934044439,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13833600401645222047,
                7114245768736906775,
                17725422787907592894,
                532810730897384308,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13414308543279265010,
                8537144314728368837,
                18036180011071871489,
                556932079189537908,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5833283699856413024,
                12697516524679686131,
                162975307247181135,
                878865248686564215,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9897120423699759729,
                7278602202473061211,
                3716458756771973740,
                2422325040148697441,
            ])),
            F::from(ark_ff::BigInteger256::new([
                998310377488233278,
                8523237539668560628,
                1949293537513177053,
                1059107186075563342,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16241646258829813738,
                3941596491098088508,
                3231730029662193002,
                2523626256625294421,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1459300710374422914,
                13716078743254687369,
                10424627005729674702,
                1908595087345336163,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16179531414644306725,
                13205272366699185595,
                624115520404445670,
                2190946265226872722,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17307642389887389984,
                933936220187440286,
                8089359410070818779,
                71992970489067401,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17824369865114876077,
                3409729442343924530,
                6056219292853570111,
                2996121479957159237,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11867380687713616978,
                8152790512200160868,
                10079577775792099798,
                1858649722170627069,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4762881307184847944,
                12524707015370918655,
                14142448838836772017,
                2909281837759172748,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15874161406748833006,
                1331628783681354161,
                2086534404438637964,
                3434768568504526876,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13016162934366502220,
                7021411748671073203,
                18151218539807698899,
                3442812370708844070,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4679965604810239125,
                2101986934198600745,
                6637181975523617010,
                1339483376071430373,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14007683259750806056,
                12269593469221122172,
                3353189221863105614,
                2423151943537393103,
            ])),
            F::from(ark_ff::BigInteger256::new([
                18022531615905880759,
                10296655121272138114,
                9864259453279460762,
                1931929842795452376,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17923154558928680466,
                6089448531730426839,
                5175065456004227281,
                807647064760950404,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17630662141355248496,
                7267169234642878469,
                3653397282001743658,
                829453284790548619,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4701536714233668583,
                13684737575441485905,
                11613649004554537258,
                3209608224464662373,
            ])),
            F::from(ark_ff::BigInteger256::new([
                48728241872826249,
                10129350144449937735,
                3410703213336614946,
                3448847351015256336,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16419431964557872396,
                13674915305457684838,
                2444183072787670159,
                3210357981279301868,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8191929567075887384,
                6469959015472779363,
                4855551162233965215,
                945403628072281766,
            ])),
            F::from(ark_ff::BigInteger256::new([
                158727652995641563,
                10981641446340228199,
                3827039644440875991,
                2007922165019641265,
            ])),
            F::from(ark_ff::BigInteger256::new([
                728784893390683799,
                2429170184364397387,
                6253658929049610185,
                1597293683371203800,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7465653234797784746,
                12672611700029084200,
                14166540675144886728,
                3343955368954575724,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10737848755691953105,
                13316441068514589202,
                12114015327777982540,
                3444166646711267252,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16065224002444883104,
                2610292721900147734,
                8414640648123719262,
                2213068532440828439,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2965283996920927783,
                10930504495486795691,
                10027837036331890016,
                1675415783177996419,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13487334092573522449,
                18191381323587897695,
                11564851716689201555,
                189247250050000943,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13166697574174597318,
                3613485117305501830,
                7195182581535895826,
                2895641587862728537,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4101304008878948025,
                936114174667882546,
                8090209713823255013,
                1989108584852485617,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8792967960115329756,
                10355009663928605703,
                1287213603576625012,
                1799368811807218943,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8484139188861037188,
                6650066542508141841,
                7555731475083182360,
                2427174000803988627,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6185705598891718839,
                4208846341820754388,
                15894259355308069723,
                836014533714758549,
            ])),
            F::from(ark_ff::BigInteger256::new([
                755401648633660674,
                9356518793073386461,
                4955056939082453152,
                1963920665251152015,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3862962271094256082,
                11024808495302379558,
                10370483221016350260,
                2645002441883743624,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14804689572845453591,
                8686453399215119337,
                15420331755542666498,
                3103523229877512666,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11087850425353693406,
                10076978978379494418,
                17944068919312937278,
                2315546681985649986,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12612335875000967262,
                17666651784149908707,
                15166144646152517240,
                2341548515443325719,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6550718393883710749,
                12701346330421642833,
                3581105642195612329,
                361444693152523124,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1367288016682497866,
                5332923614065051104,
                11155565938567078054,
                7736695337681445,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17603670072141334606,
                17272987085072847091,
                17243585277871168983,
                2216218592561511497,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5875006885783709952,
                10537908699619434821,
                9842934652457217535,
                489515664818160031,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5574105164020813187,
                17743249821744169830,
                13986810838980994141,
                2815497573416473367,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1936331356809149186,
                13479295850360343411,
                8145578311573976077,
                2124864943408231632,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2179849664422209133,
                13073865840319168427,
                9652681991944791021,
                1730792633486934941,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14926527869842170170,
                15957602368679205452,
                1250417037737698939,
                2035374361313081079,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11366199833667737100,
                17593405203784534243,
                3333872771637624616,
                831055112462830330,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8411212302695934994,
                5395126445412535751,
                4989662535904300375,
                3027384805640142213,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16643451077064277308,
                11010614999127987826,
                13159786684327854190,
                284037651980240089,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2044253535304399310,
                17673218083379829559,
                13825073454542711142,
                2238183787255281570,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8414097752612522828,
                5218448262272115230,
                9890709863323489100,
                95950788485343531,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5772011299254848266,
                7861256438297254790,
                9875971108525937544,
                722024770176709895,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9037300918473413852,
                17930435831445037005,
                16574607749442419358,
                334519348306792384,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8317309235287166861,
                5244161483513822088,
                6208466319124860051,
                2383825947769122657,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14950961608534339719,
                2863757536924850998,
                15183018196698340466,
                896692597161622927,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16188815797800060513,
                3015531224607724030,
                6553532869522470935,
                3284476244270774140,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3292331670668400666,
                745068775961048232,
                3588349405011755595,
                1505429734967334614,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7947939973283569653,
                17910499471393221949,
                9493933000998386233,
                773594788431230978,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2903295341753097273,
                9512627340380481254,
                8314848857110552684,
                834146288776109039,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1532440907033886807,
                1539966696146594215,
                15291781636328373517,
                1406982344740637119,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3886656838401491861,
                333492469183378979,
                16471851410495828437,
                492067665823853620,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10040235385583508017,
                9401180139399466796,
                15854077727072583198,
                1536906754033601739,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9743704801867282716,
                10362670637618729156,
                780543059336170310,
                2184171379946903529,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4140838421508485303,
                16432000936275510575,
                11384121898712887944,
                635792993076439414,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13220453023709936470,
                13441282295845064612,
                12423622948972089424,
                2259609362552620084,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12033658891163799107,
                6150456349560472140,
                15933316496982448147,
                1270820986969916485,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14871475099881315431,
                15163123464701963618,
                1011741026995489296,
                3065283277764938657,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9305039230556471093,
                14903202239060658208,
                6978082672141715529,
                311457066374660350,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10326890588496644542,
                1242052159224273344,
                6167588387276801881,
                3046585433170322036,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5905197260352203661,
                12593626885636827164,
                5041914755169550626,
                1755266360274201802,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16124358592229202134,
                7963899203137013483,
                14233454459544771707,
                1067060953489644245,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15371655395654469786,
                11052520939812251711,
                41065103871028284,
                1587716653541489708,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11779597873964172810,
                748863275696144731,
                14512910473145976657,
                954587699166730213,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17554949439825886561,
                8230232498292775128,
                7463911311438047778,
                688715411350431658,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17837064305693691997,
                3530348748280365640,
                8224624396734537169,
                3472012396750697334,
            ])),
            F::from(ark_ff::BigInteger256::new([
                685338199690090978,
                105771468943192620,
                6060494378374284634,
                658776239665496041,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11457314990587881605,
                2572166296638230922,
                15370065117212573386,
                1673132778465257225,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16968199869992981169,
                12124210755818662549,
                1350650475090089866,
                2365234301667637721,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9550353722989348614,
                1489657301012644192,
                3767648750007374330,
                402154862755408281,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10864867534192029686,
                16429941433609750151,
                8531089017994542574,
                3237743715364748244,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4325170822353990770,
                543940286539606168,
                10932251212543973852,
                2855191626907848189,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2923526124056823054,
                14012665406190429826,
                6335591292371220879,
                188404865885717439,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16538479672646287440,
                10182853911109017959,
                18440633563809518155,
                510966701254593958,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2380291261184753325,
                15484685569385840258,
                4431354562970234090,
                1333289222577826844,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13062348620693048316,
                10520220833564366487,
                6433550769840812628,
                1916643762290570303,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14363549225407436041,
                3752250024921313295,
                2154136483334210200,
                2233856011159014431,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5187471298036989962,
                13971614387928625740,
                3524391305217028944,
                289915685740067018,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4693604066481880203,
                1043297075837820965,
                15141308371971226004,
                830673637967973491,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11913074726434383664,
                15236413419703993182,
                7668198814174950338,
                2889616392746527937,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8228731494930629951,
                5588483753607145369,
                6110850688168422007,
                2515020658871572055,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5252222733867207964,
                14227935210092357532,
                12967952655993612712,
                130924228517585348,
            ])),
            F::from(ark_ff::BigInteger256::new([
                854788594404532978,
                4022634903246063926,
                11782321113509405397,
                2221670329557185530,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15600335641020578268,
                11876391402234698815,
                4901350092871075477,
                3435570396643650141,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2245901484398865121,
                17081038454764294727,
                7943144947814511176,
                3123955947234154605,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3672139406424358231,
                11232963244788823105,
                11432294202698546779,
                1760711069876412044,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14574378628720512278,
                10069380338278660087,
                15214381821310016571,
                1938211679974003349,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4424738279794046860,
                7146479140298397597,
                7102920237850366852,
                1490656876585103233,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17651206228545864861,
                4871655596930476230,
                1891205931412525206,
                1929908882608392967,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1055815450626962089,
                646149709530078203,
                4204756359975840789,
                3297367314478176692,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17767946897506767634,
                16596052938929158848,
                9626990262887682666,
                3380180472134091448,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5934781701171781305,
                4383591089371751864,
                6779069139126337607,
                2769011162972467813,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11539225362019041837,
                14466654151820885683,
                8007212568409987926,
                3336436107135190007,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13651291784760489216,
                8691265465401801540,
                14300064488795457395,
                1158052845132601357,
            ])),
            F::from(ark_ff::BigInteger256::new([
                990728692129487505,
                804335452936146959,
                7005734491698692024,
                1824643006828634820,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13488617484273047126,
                5844065761670603257,
                10355600265990996779,
                3348383899531536067,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17714191312078195480,
                3182805701613058605,
                10472957917313629538,
                716853859071594178,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8325036693894798935,
                5303506174715396602,
                2795016575861427737,
                996245519341826083,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11717693106902176613,
                3202222745033558123,
                13419099615401769290,
                1840374348635745662,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8550147697324645447,
                4996326856548337037,
                4000294313501987497,
                1723625826060326585,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4505718082943030822,
                17010325537069923358,
                15343425803867006552,
                2286070906890983401,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8412947293930038544,
                4498481673652635953,
                11375087978902546135,
                144893325711411960,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12034168488788897923,
                15352204262747033732,
                1351842026330002459,
                1246274376166152763,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4001825577369285447,
                16267618405919148288,
                7325778700093165847,
                115637735629793847,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17830100208545320449,
                9924556572381252604,
                12856777262701729001,
                2688641197723503935,
            ])),
            F::from(ark_ff::BigInteger256::new([
                167357054947272450,
                8585904632805429027,
                10659591190619360907,
                858157511174487802,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1572532437073992610,
                11917422447193140199,
                700906942979378664,
                896629000291060425,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3468108035051658734,
                11707870961300768996,
                12202099696042518,
                829575785892031669,
            ])),
            F::from(ark_ff::BigInteger256::new([
                18359025234418955489,
                46738909445102358,
                2390176944250793864,
                1678415708648341926,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17403323783777520533,
                13093899196655706975,
                6009160433590065822,
                1694230681305446490,
            ])),
            F::from(ark_ff::BigInteger256::new([
                755660517839586373,
                12414514318088895658,
                8412673267164985748,
                1305511847956204085,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6503556014470594331,
                10083053601041220690,
                2964627020838402168,
                1131188958127353815,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9109041194190405620,
                14051618183311002005,
                4033558293045538114,
                2022992511812060163,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5258020618567833686,
                9559677891167695103,
                1393294321762588031,
                508595265249807069,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14497451329056542170,
                17566593112912518503,
                9296901957209393014,
                774720957029937794,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2724638477631493492,
                7589903188524044946,
                17219834399294151977,
                906808431377856480,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17587806316351734879,
                1207161083120991433,
                3434028003078838818,
                1613232130541252791,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5544343103915140845,
                17870822478530181027,
                13729871010679450386,
                926672708177104919,
            ])),
            F::from(ark_ff::BigInteger256::new([
                18281148647327728754,
                7850641541704781715,
                3557801043351595465,
                1046915773798353341,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4861196639177801783,
                10464755375812922928,
                6320935973102048458,
                2244657437647272958,
            ])),
            F::from(ark_ff::BigInteger256::new([
                282283810953678733,
                2221954514194472406,
                8225053329857422806,
                2493331084189430583,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17839849339818886650,
                1856690033332010676,
                5937904532814911490,
                2427001771377909238,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16180247985411498710,
                6240546519146869663,
                16354508469922244541,
                3097312312692397659,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12006724610430381142,
                11615513659152510821,
                6367734870042344114,
                762728033415035404,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13600738679525611945,
                17840678034054227335,
                2224972071140282541,
                3375485579428387426,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13606693566665327029,
                1909122995265796257,
                9965489109344127373,
                1221920711501971005,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10375877057857352834,
                12625662567941673502,
                8321264279065877926,
                840846023476747546,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5791293549416751671,
                11138942597924872648,
                17782913851898369074,
                1085232480454249650,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6451563359216739658,
                14710189401669258103,
                1462990935681432935,
                2311977799615565483,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14495340088050189014,
                15067344989131662812,
                1620036980455333466,
                459394982779697034,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12635479783442866408,
                9937367338066031525,
                3148937602214607977,
                221444773525505663,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4817737749106805048,
                17886567997014335607,
                4501424148843745297,
                417645705176654332,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17751407434149912605,
                6716721288790115488,
                18213068608280928528,
                2586869981514207410,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4603902366454878496,
                6587659520477791492,
                7539937163643202229,
                2747221143194515069,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11008060223564739568,
                4092871733345256324,
                3564017518314962290,
                3085378357433054990,
            ])),
            F::from(ark_ff::BigInteger256::new([
                18371112212092917619,
                9346020485016845177,
                138877133813720334,
                9705545211764536,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11102224599077682640,
                11480483807249072619,
                5163171560975762547,
                1816853194572977321,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14578126523943318234,
                2445327973886378508,
                18067220066140818049,
                3074220577191341759,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17388454540812282995,
                8815729669748230392,
                16241477381004590407,
                3186259921076138214,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2703432405615403560,
                10675863278398571123,
                1267204382603128799,
                819680094850551768,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6017575098148136980,
                9186822172574443801,
                17258611118985063861,
                2030460168678941018,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6347666042870170654,
                5866713522877462585,
                5193382286745801931,
                977309570742919901,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5300486144822670614,
                7113883553325786964,
                15449009099715126680,
                782939338275657253,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17384707992512930,
                10315021296507218687,
                17412775307026495205,
                1946902283191148794,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9784670519218936706,
                2558464695479300568,
                13014814317986788993,
                1852540996620439999,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15514974449865578449,
                15304990107729078622,
                5499401520467795116,
                2775645249938487742,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9340545441302110644,
                15559251961615206481,
                10111973955741361239,
                1428319154874844598,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7822632574475342879,
                13386026058993496867,
                5718978657186557847,
                3220260187913029856,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15635352912338426062,
                909425887705220433,
                3807561090376916495,
                2234068206787090633,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6907656952802123964,
                4067036256196763033,
                15521206525267764385,
                1451900301417085667,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5552481663500618334,
                1113651052683660662,
                5534274154367597675,
                87328855787230796,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6900715074173076267,
                9539434746587871850,
                17066667570999755618,
                3458102527267014727,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15199628870393917379,
                16815436337147488581,
                13326721918125644968,
                1975566881000076619,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7856555322951848618,
                10216929133815068136,
                12298881287389479331,
                2978031811574671810,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7447388003911996788,
                13742920297862619490,
                15739308674468901240,
                913482971011876251,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15604419326871954578,
                6562605095973266902,
                9649357474907682929,
                3014867509606460119,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1119568273020521595,
                4898252784797506596,
                8127212000056865382,
                3475204958576367335,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15432190837464268934,
                17486881410930124124,
                14357590274030151650,
                2528953331756189306,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9061219104468281512,
                4208438922984278950,
                2106255191982748369,
                2490114490884626544,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3192435671551876068,
                3698929794062580989,
                8341504427229996421,
                3374537713801423736,
            ])),
            F::from(ark_ff::BigInteger256::new([
                220315471028976518,
                17641997000201047631,
                440809894054144737,
                2139145201425221596,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7910039391642704698,
                9776607814536317513,
                2946163033199873056,
                1738257553985445729,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11350215535856746012,
                3029321696039946978,
                7049854148791929592,
                1459090191969193973,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4123422635492237459,
                1591639978164001768,
                9388281857476540601,
                487761395309038875,
            ])),
            F::from(ark_ff::BigInteger256::new([
                18017536088120444877,
                16328219282430046353,
                15958382101951733606,
                1510902324057066876,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11997769442090096030,
                16471647838015362786,
                13366240106870618708,
                2428579135320324271,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9397067195725659052,
                15513451028145688715,
                14843371107922612976,
                154031658555077746,
            ])),
            F::from(ark_ff::BigInteger256::new([
                855193194769420557,
                17872817693213152872,
                13994872873825412939,
                1826272104000793098,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6164069890302406866,
                327429843626790174,
                5112320091728802050,
                685312953780945455,
            ])),
            F::from(ark_ff::BigInteger256::new([
                582721743536215149,
                7397369447688681165,
                8106748236136255847,
                80146100259648655,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10508608581706110538,
                11685352655594282388,
                17331762072650948152,
                342950255585856487,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1866978664098276694,
                12263071710471493563,
                15333369414652233412,
                2439310355942867312,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17920009130297498531,
                486866905788372227,
                10175741945944161244,
                1405310385925085382,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1135029214759421794,
                529049569660025601,
                4703829120353270437,
                2391583941189481248,
            ])),
            F::from(ark_ff::BigInteger256::new([
                18078699596485289300,
                17580308447487148870,
                942236211975119509,
                610568494350560837,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9158212389508350846,
                6507511375720337621,
                4837429949228209513,
                787865573224212802,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14615272308521399919,
                3137755380356057730,
                4600694111005360845,
                1012030691696087915,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5014180968338402958,
                6716434844964639102,
                17217723353861130326,
                1670744418534831948,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15698145022203774136,
                9918206559364014259,
                4499874673492753916,
                2807424607193273437,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3178468896423781383,
                9739286070844293799,
                17786885856812674213,
                1216063379663523919,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1471276239647949078,
                17586966051939161556,
                17510229834183923967,
                2234253296915045969,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17706487729348522899,
                15799432757173550711,
                15560010491587093084,
                611600771873464249,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4005582733124991573,
                869221984134096820,
                5233504636792289228,
                2277388676694265101,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7029346236758552776,
                13626859162656336945,
                14254622499457293056,
                1545195026685974720,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8100250773221955640,
                2800685621648497393,
                14926721701423239574,
                2531865158341248503,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15907696791114638585,
                10616614122729282351,
                3819666024228086513,
                3217989126630202139,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7788698537794175048,
                9314093523303495722,
                15305250582109117842,
                1911920717225759776,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11303075884140290495,
                17507317722522397468,
                11402808312383313568,
                2247747072897431362,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14506383115542620102,
                1230463877358889397,
                10315383867284749586,
                2601166812269534537,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4734019910764418968,
                2349702905097976657,
                7161777814849947512,
                2256669700848274353,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12922262997291702813,
                15566849545793410515,
                6226036346082889660,
                2883230016581987404,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11609994158287109362,
                18372858320528634325,
                446491861798356640,
                2792998270724286212,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9618658151190296799,
                17221328584160194266,
                8518665867972850897,
                276190630107722584,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1131573606821806535,
                2428621147237898135,
                16786396382765132177,
                2451915964552355467,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11211338687657292903,
                931874726205900704,
                11524526154831477601,
                1004959779798872323,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4858551750955984462,
                7211294243511485489,
                14745666142052071413,
                2898780552702334053,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6643033284761502826,
                699552333580691028,
                15896859283836236871,
                2786362913212928057,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1146147225165446286,
                18298596591076997406,
                14223244270891890847,
                2454699235313181225,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5137854008973327625,
                2626789741342557346,
                17133525708385840562,
                565776796549473463,
            ])),
            F::from(ark_ff::BigInteger256::new([
                679995492983110063,
                15088251567886766764,
                3892974243718107384,
                192953848239552553,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7836755956175664810,
                3627647897538205176,
                10539942639990820490,
                2772961695151703523,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11956475242149612714,
                13799078954611351636,
                10389594931891270234,
                3262565199526920653,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1691194943394200874,
                155003788650916203,
                16597430673211552869,
                2354976272604433125,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16550545283864828148,
                10319523939580974668,
                11935412714763254615,
                147382015990450064,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10464493301947887778,
                10949501129144360583,
                4291642485420295083,
                1409839822621742891,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12599197423620422744,
                3159288060776612374,
                11614025512960135891,
                330798258260669789,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14212162172348812499,
                550018133805262955,
                12614553095292831875,
                2735838178297163204,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11376800068810574793,
                276225321570001654,
                3451656603458464463,
                1318842278187321208,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2177356295843285416,
                8245722538688578476,
                12525921878093685903,
                3417955661366124753,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9959532780390587218,
                1772710826291316866,
                10352249145233096850,
                1423928147639240227,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17371367584453775012,
                66824904851217629,
                17599209502334283237,
                597888099614193051,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6854081990016051475,
                1507540698345531935,
                11776199111705923691,
                376994867109034844,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12860862179644936802,
                12073396942597776339,
                5224790287993834439,
                2575289984584042994,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10090522016142174622,
                3920452920400418530,
                15586453824569259448,
                3125487262479933836,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13440185984245597620,
                7470012140403341563,
                6807168182160567175,
                2951956412565800201,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1321279988328613957,
                3633495412877255481,
                6222150236632892104,
                900558497742600050,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11174762584101053491,
                10752158307833031478,
                1675188112590489363,
                2162663703991883109,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12501561062039937142,
                13217228960784872895,
                4674229449522310557,
                2079222015569170660,
            ])),
            F::from(ark_ff::BigInteger256::new([
                18107750336679530010,
                2637121613737674065,
                16064329783236311932,
                2656631638894173847,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4959442890693494821,
                291866076990833646,
                7416710153604740665,
                1143071421532737346,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12345996029705128581,
                9667816489525462797,
                16452579147005594490,
                223190012773238924,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9167170176004415287,
                4534871764832926208,
                3747487374090526015,
                551921414135466234,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1059844831490126458,
                14225262472171489267,
                17819424801465995500,
                2902272379039536880,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6054996223866504342,
                6411664912839409984,
                1369971595339472280,
                1339166876865791409,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11893941220706032776,
                7585254802103582254,
                8809938484087424909,
                2829704288310473473,
            ])),
            F::from(ark_ff::BigInteger256::new([
                57234985312655794,
                2489796375691480137,
                18414301965702849578,
                991494554102186208,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3802159801216112697,
                16350043580423640925,
                13942052015781984085,
                682305401516016284,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13604171394503760045,
                7410278735575665199,
                17814457051524922296,
                3384629714164410912,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2833651469701519956,
                14187934479902763352,
                4577782467801994203,
                1112365688062826156,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10093703133586292002,
                18281896565499337667,
                11688261390726488820,
                2326603698667899737,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12285366075689851461,
                5251451036212963523,
                3054923555710701412,
                704014612631552875,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6554781120450587236,
                9062570081298569253,
                8195831800130457464,
                2110196517777957519,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7149907324670256487,
                8309354845545798917,
                15165189837001625845,
                533318386542681272,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4005907040157594771,
                8021410468997087426,
                8164128836033668867,
                1768007373712000820,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11441698966487736798,
                8172566600420746354,
                6356720869393765981,
                16543143029651260,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12263792783052640938,
                4922914784977040319,
                15783857594401406589,
                1666133112593509111,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4235283860653882016,
                987216875280266742,
                15640320342014394117,
                1736513294171438873,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5866423949542860800,
                10531938095368939267,
                11928564632654585317,
                1577671292047144883,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1054239732833083753,
                6103700247670904905,
                5415570212524415309,
                1148058015824962423,
            ])),
            F::from(ark_ff::BigInteger256::new([
                18199943526656693117,
                8266732157722765705,
                131989530318531143,
                997920187098657184,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1939885535548615864,
                13466751372042613576,
                8267736382387663817,
                2422854800835179641,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17680267406110154911,
                15506478309354411019,
                13124828629697909619,
                2294036181772852979,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14002357292218261689,
                6436860477649242230,
                12072219967381118202,
                2273482111012228075,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12425116934835476204,
                6703964773877985281,
                15082100408227030625,
                366326579467101079,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8370176753320702677,
                16790348184256140286,
                6933839869011005072,
                2679839128506447382,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17658145342095021237,
                9908154785898334639,
                5903316674011403618,
                1586085560031324169,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16020579513254857854,
                8749100717598328382,
                12454832249094419986,
                2537926967889916603,
            ])),
            F::from(ark_ff::BigInteger256::new([
                216001193860781640,
                17394079090495223103,
                16812591092418522088,
                1750127475594122184,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13954418694066913699,
                10375557610539161846,
                8378858714851679845,
                1873578224874020994,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2686955467226622227,
                4492680062441726698,
                6150019724544314092,
                2065406180885047752,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10009420822571946913,
                9309271989716247269,
                5699009423282500276,
                2928079662043900558,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16635727393310381374,
                8300597212511365459,
                12247000142159898442,
                2436938798781692268,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2082612406067846115,
                123948752638265284,
                3645590729497458159,
                1761211962042784409,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11318681382726647916,
                12985208220530044355,
                735787064625710083,
                1283705691724082791,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14885889703880576869,
                10686217926595013933,
                14561783235796282579,
                1149806791214493476,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11529505619055905061,
                4572496908229584432,
                4940237186508827252,
                2320354671658586753,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7341346663983130134,
                4142559999372578478,
                788033614870032745,
                2697551718719503809,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9803272930282320265,
                5771137064966167401,
                5694294712910777834,
                1462442143659313624,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13586058431297107300,
                6534523263084028212,
                6017006349075466221,
                2991684040986320177,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6157359589965934785,
                3437987561958725700,
                11690681739701975977,
                1786514952952163772,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5460035966269912641,
                15667121648157036752,
                886939765682713797,
                1822973958663818687,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15614450258614962914,
                12902917730329553597,
                6242274410061050257,
                2592171351650442152,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15870097061112889350,
                317248516201075838,
                18057691883178557651,
                2725465680106737264,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4961556619899217934,
                14730874306754037706,
                14401085261261276074,
                2094318275069725901,
            ])),
            F::from(ark_ff::BigInteger256::new([
                323113805947077839,
                5695329345604131089,
                6341060737753065617,
                1820485520516104166,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14850767776466585287,
                18272693936814067479,
                14699437753615977437,
                142265471174589157,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9961003746934904696,
                15527037611620820517,
                15347046933708378268,
                1356851562893366065,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5283103231719058992,
                10971881154596118954,
                13284285325116824497,
                2856035783308695191,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6262282883344752112,
                2642485790024737270,
                4710103504680854276,
                463729102892614852,
            ])),
            F::from(ark_ff::BigInteger256::new([
                449394879555176978,
                17488746434879468623,
                17936872115958352828,
                1649048596087463686,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1640900858223433524,
                10817471451884559639,
                13435039876745189000,
                2022304497396928002,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9371050462617733747,
                3358659620090099515,
                13946677483302243675,
                3231271879626562285,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15340116002320363807,
                9824211133622889947,
                17109101986038313652,
                2538089319661050521,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3574869418397586162,
                8753590792079190410,
                4207374612803298585,
                3477277375938555299,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3526047188594721710,
                9549059154394070863,
                8645364919970731148,
                353049923121264109,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11421893890645802607,
                2144153652026948284,
                18049281099636718994,
                1811888031015800891,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7011766053167114709,
                17651897724852473945,
                10176108066286256722,
                1961713582904335534,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6472557420872111792,
                1337549735095571222,
                3705594196306503165,
                2412333860334130494,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17125160747553435452,
                16075554759240982033,
                1000920168261102048,
                3172025036665484802,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6708876106857948393,
                5142950392753679779,
                6004491857206280890,
                2272172791002466254,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4345045306155692495,
                8779519293416012721,
                269622867226962281,
                2327616192003797218,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2097074397778064565,
                840948937425749789,
                4237403834229950487,
                1057384692454500983,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5782076558322031919,
                5679320185308563036,
                1585410375742395828,
                2469556538893105861,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4390442735361195966,
                7862419968382998748,
                12332035331832883841,
                1977969763978762529,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3459063862431525012,
                15319330286226487340,
                16267705090745107890,
                633584387723783449,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4188858863594070477,
                13200418348214088319,
                2638911391288213431,
                2871065945828071547,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17457902518847207930,
                15161300156913747214,
                15494743156526187389,
                694374817529805276,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12172425741938188214,
                16869485103563037418,
                7022653980326157519,
                1935698206841343860,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10773667477234439936,
                12335853345459541623,
                9226422518441702042,
                3036363874751212448,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1990833208781019757,
                17910630045023677351,
                2984676544975762550,
                72932287477908675,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15132754218264180119,
                16520265591452338094,
                17173238265470617690,
                830326972282094516,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15699406257689489798,
                682164582558953151,
                3506937201820282842,
                1731207242298297766,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11234916579861193179,
                13174538966159520061,
                2925395645051806036,
                249701007770206461,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15303853385210339191,
                14601144017725347531,
                16004160934948534320,
                261525678708856444,
            ])),
            F::from(ark_ff::BigInteger256::new([
                18139687006067226497,
                15326184836936438296,
                11007735588312753793,
                289115464447267005,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1008089078325235096,
                15812436369103480969,
                2749005537092770347,
                2975352650336268860,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15968871950239862565,
                6879193289208153175,
                885295523024217016,
                2323160593409979086,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15890229741388111608,
                10571824169675625673,
                7290985395385890670,
                951267231555300127,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10923845253483449248,
                5028495879704325537,
                2930276972049124448,
                1552585328945465194,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1305409575878418109,
                12114654465526951283,
                17123648613691689599,
                3290468536836876833,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15562418357762757136,
                1425882119959583372,
                14726408511028211558,
                1886859825860852833,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2877838621009487210,
                11133539617060386578,
                12258235599473939574,
                2064763399545052981,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1687614646143979,
                15441754690555260955,
                9194065553734097701,
                304753602204061434,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15690846003029046434,
                13993109016946849755,
                15582457762601570828,
                1075540711063739757,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5829885310276697843,
                13715608996490795322,
                18423146137062361382,
                223898408859837526,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16390221367664797520,
                4060143039647417131,
                2843193484086311497,
                2761497943194275703,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17318348797889887192,
                8928162993197240030,
                17472566145188453226,
                2640671824084697510,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4147149246499859010,
                12951012237376766266,
                17201508756145910081,
                1946913937319759723,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16181740376832494623,
                4335115700026011067,
                12199372375683373340,
                2957632265917180397,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14398275813985268568,
                5537784905792341916,
                1500363935521417232,
                3026750105598641609,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8954069393115894911,
                3328807672401268592,
                10299914442150349396,
                131982885120699851,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11512214851863995048,
                16817324518608626862,
                8871293120782771779,
                380090106203900789,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3828710574829455136,
                12322493743033440229,
                14665593138843461781,
                1872374868878930607,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11345936842588859750,
                1564362188469278814,
                5381826182488616710,
                2437007102086991463,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4538086061646822167,
                1935944517516250032,
                4307677527042265688,
                2780248827126838994,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4689079623689624034,
                10826284928960342711,
                13600404362577820173,
                2594937663151175829,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13532432155810565363,
                5144521529217948224,
                12428219326652387213,
                2767549561396562696,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10929218600584698904,
                11514460209530691525,
                11656069798861040801,
                2407571758316897242,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11850569974172576781,
                12034611477310407526,
                15315035093506179512,
                492270336873979004,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2805471650420184058,
                1925702855566862211,
                17572506625767236535,
                1207561457456986282,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15964102278699526147,
                17328112604986780129,
                11971039496125537604,
                741843320009763146,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2983057610387617727,
                6232933822357195434,
                17933729163862834266,
                3105895303823345465,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11020673015444562774,
                17561045298050402658,
                10308943038546826660,
                2199315858168839958,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1762919944572042451,
                16177212091963317424,
                104797510676270036,
                2897552063492515067,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8529890089731214660,
                4902261060612707427,
                18383853342426415617,
                2327799694902080300,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6853293283146109610,
                2175523491815964915,
                2363234443423517526,
                1739242309666914075,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15966553104732240541,
                13317559191894742694,
                7469239104141309885,
                2225904866687578605,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2459206620123091984,
                7263163422177333900,
                11456082673150354798,
                2919899005371675721,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1339511610158290837,
                14856458256078139845,
                2495577344344853804,
                37775879282348624,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3092559293873344106,
                18441246422232342664,
                11307080507652345068,
                271450522157127325,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16532028774492549325,
                16697344712169415459,
                345593512252746758,
                1800688698084961912,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16291557521752336894,
                15257384749910382402,
                14277755688605797340,
                124394211331054147,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11838001280807199466,
                4044340370618451890,
                12101885147986879115,
                1197013646269811825,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17605012030106573818,
                7759563180728942133,
                7558425824832742090,
                1123669335181054615,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5551605165185996070,
                7537427905283250605,
                16266006155533323951,
                2451509929006638701,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8824301355067163165,
                1996397662186530791,
                10355447782094818673,
                2966302788963908093,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6569829578586920768,
                16082095426623200379,
                11780680957492183365,
                1090305168790641006,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3567283126512847440,
                194497088444059304,
                7648783911461107803,
                2394013255439819027,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14234051551981501369,
                3072938323551244140,
                4560687208490471898,
                496008839138135469,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11333291521220565447,
                8894996036145239980,
                2999424925770259554,
                2218042042189088586,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6211665243877596528,
                3656319756378592672,
                12939655673310595404,
                1040152495259207135,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10118933986221437642,
                5352425876695884239,
                214326833854851554,
                1628825094902607230,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2352759454188050832,
                6979808492955684045,
                6007456829695102442,
                3250980465751016793,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14194482735970344005,
                13726137848552200170,
                7226427086179246929,
                2272706815834045353,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1114229836527831848,
                11009534750999843832,
                17559673134668805235,
                998362558896662465,
            ])),
            F::from(ark_ff::BigInteger256::new([
                538747704244809601,
                3226350144284412913,
                4559650380947399759,
                2131911283764787488,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11238270075133078461,
                13370472111937606513,
                5437562997607864740,
                2743180294405077375,
            ])),
            F::from(ark_ff::BigInteger256::new([
                18228601196870467984,
                1542148795884727291,
                809896088270663773,
                3212128412103079212,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9773339946375088120,
                11070006510323101210,
                8500385761544739177,
                3067637251986336251,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7305954348383564516,
                4143452010802215201,
                2294987717145386513,
                2386290843320580183,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12898780154942969697,
                3798637881114578152,
                1795201998414468440,
                3133902983075212583,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10388038404368752860,
                17603430635235751584,
                16068446257337572023,
                1709768452075034637,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17951301515671634702,
                4210257216786664480,
                6796109861603278901,
                2729510554004699497,
            ])),
            F::from(ark_ff::BigInteger256::new([
                910546733942970032,
                17703179365907219707,
                13060747720749589488,
                675199321035145303,
            ])),
            F::from(ark_ff::BigInteger256::new([
                777790492103293620,
                16778670497181685428,
                218107541946153607,
                1230237759464788704,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3420442680473887571,
                12664390542334278294,
                4140963366655262996,
                1975198413275550414,
            ])),
            F::from(ark_ff::BigInteger256::new([
                163311987901076099,
                1954643451491722780,
                16732282382224757205,
                1784386573764180619,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9958977275430760240,
                4067338553256495777,
                12546706450625413826,
                229119965367484222,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5825833563432051897,
                13097977489626570872,
                17646897339775367399,
                467982032913188415,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13149408962193025225,
                6343146717324346651,
                3541019442971243714,
                1759372664379145063,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12967296636482810276,
                16635028243863885866,
                14306000953781616259,
                2734017065058792206,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12718894637785034634,
                887341186088974110,
                4708026632365055139,
                2966189059447275319,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10744684478324424059,
                737814649429747225,
                3660829839858007919,
                1334890357987088325,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16913048478668856150,
                11736762322939877088,
                11000959950351580795,
                710771000113505886,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3697337508237615631,
                7952395712658613569,
                7159402136502105378,
                1404275361698262635,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3958747600390589927,
                2011639790305718868,
                9723475923343830869,
                942487543901519121,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2502521494717711891,
                4338505878426622935,
                2294883434579403826,
                2142729360485538990,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9172742281672625573,
                9425386650617655952,
                5827741105217612705,
                2598673402069221034,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16318134147160761145,
                13463428176898907472,
                14036200280567299234,
                561390613285332415,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13904969747015744709,
                9807977476976010400,
                16059121614697875005,
                1583779375024230955,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11415872433942961664,
                6738391707428117457,
                13813196467855423668,
                934574878394684855,
            ])),
        ];
        let mds = vec![
            vec![
                F::from(ark_ff::BigInteger256::new([
                    12752381838601681454,
                    10313698059184076678,
                    9868686762743109243,
                    175000850574921121,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    16679733335612088211,
                    7193281848976548519,
                    4770363499436317140,
                    79396946753143671,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    13667532428494685098,
                    8129502624220455994,
                    14664662400891032885,
                    3033362699531025939,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    14945937640456088350,
                    12634927548548549060,
                    14090505059841516709,
                    1651650645126507875,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    6438778006746722226,
                    1158795113106154897,
                    5729756344578144578,
                    195886002219414995,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    12351086545807060784,
                    17574156571474965612,
                    522144851590229194,
                    891231016008258869,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    386635115885155085,
                    17946070794830349297,
                    7173009089188999547,
                    99612104283989197,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    7713281306969421854,
                    931899404179025018,
                    1964289461136937682,
                    2466987187800884541,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    3656407693414789798,
                    8416465712572366679,
                    9538112320156666875,
                    1936654370447002889,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    5199648366305253571,
                    14471481993446339312,
                    16884542083016062778,
                    315693613623884073,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    14439534471752072768,
                    16520619523435361604,
                    7192262017065680856,
                    1493536271935697724,
                ])),
            ],
            vec![
                F::from(ark_ff::BigInteger256::new([
                    8762439462518443248,
                    7012797157938455884,
                    7588263759301058930,
                    42805946241483939,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    13715507997017584931,
                    3344781560019095164,
                    9723519800304277731,
                    1483284083408732360,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    3394387598427068412,
                    1339559399098433843,
                    3668130348313076766,
                    978944472100812140,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    5945151533680789291,
                    11370969924495024632,
                    10897305751354660092,
                    794306308361970026,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    1659319063296738730,
                    1009869852192293679,
                    15509833662553627746,
                    2525219980194135113,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    7604437937489633571,
                    818158092591462302,
                    5256587939318568681,
                    1572345209147388944,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    10777004743910446548,
                    13685017882115641474,
                    13942375566069361949,
                    790856443562653212,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    14247855098489882249,
                    11893100800009772755,
                    13302578972930706594,
                    2136101382405032673,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    11122387590819045713,
                    1773577213506034321,
                    872022208850435586,
                    2909512632139486844,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    555823025127668151,
                    9159357047476722730,
                    14909122710483638418,
                    1657100184200745499,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    6899804233981368859,
                    10079790688276527018,
                    15574615811477280522,
                    1990846353730053808,
                ])),
            ],
            vec![
                F::from(ark_ff::BigInteger256::new([
                    13486323879402122592,
                    2954657403507794929,
                    2444707753970986363,
                    9052777313183165,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    6412993668704406419,
                    1119980197852058430,
                    4870709435889079450,
                    1266431534043754498,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    14633468273123641459,
                    11548928949037755642,
                    15010176926184990631,
                    559061629115127379,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    12248589754610216541,
                    6589958608476848405,
                    14096870545719785145,
                    3402835644010210563,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    7889180777386265112,
                    10431099271918648280,
                    959053935474220998,
                    2333798594864725449,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    11630019723829543222,
                    7640329777811656813,
                    4639114832529838487,
                    1931175667878056319,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    4842411454975244563,
                    16590756942108209167,
                    6790441630967042095,
                    424794186965863054,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    12102289696565370152,
                    18081839290384560291,
                    3883790251489491596,
                    3346328089567476824,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    14149944987448013437,
                    8273526038126580775,
                    12810698711115250748,
                    775251359371975594,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    4503593510812859648,
                    8372957406986952503,
                    10556458181992873448,
                    149541934900871097,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    12753792408379407387,
                    906363289054231583,
                    9931202156103607336,
                    151422415826560294,
                ])),
            ],
            vec![
                F::from(ark_ff::BigInteger256::new([
                    8061052764911063037,
                    6865796273107530213,
                    12074496272184831956,
                    1310651592819452964,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    2770324394608724857,
                    4370035425562120121,
                    6202996480697860212,
                    47617472024138080,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    1429361595053113875,
                    11210906333294863793,
                    4619113080147773153,
                    1112789694699115020,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    12396283029428271093,
                    13905774269713830507,
                    15384186987547711630,
                    726625609309196581,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    9255793241060991680,
                    12284957690967418607,
                    268318268142971333,
                    2456923079957434031,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    5273787104845674476,
                    16603391092252939120,
                    12104551496065216629,
                    3430566013623351431,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    1658380318194625321,
                    12254107408469875832,
                    14797984581530264217,
                    1507368989565954525,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    14037486331557013814,
                    18225738561524712991,
                    13343380891724254743,
                    1440628230362373029,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    8728386585182664336,
                    4371635864595633504,
                    9178803609453876770,
                    229581263707713285,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    5734751152057363119,
                    12058734457378132018,
                    2168017451230722957,
                    3093370386471409325,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    2009220719041834870,
                    4069731473894794870,
                    7309561065916051086,
                    1313468161266295729,
                ])),
            ],
            vec![
                F::from(ark_ff::BigInteger256::new([
                    4134176964081944057,
                    8113199522919274464,
                    7819505745744360992,
                    2722289280546677244,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    17831781587133828156,
                    9323380082968777549,
                    15669239213796846764,
                    2052861407290209741,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    10471561448416286986,
                    13414789430847260145,
                    18116599163983411357,
                    3230927850826095685,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    11405659026880763996,
                    13216384247237800687,
                    12232923536409400101,
                    2109206079886215380,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    16205681744575491878,
                    6028689623317534835,
                    5402378906625979279,
                    477070309065000461,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    13180213127255116173,
                    8292398788780189496,
                    13415895960806295150,
                    489433044340673613,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    16648326403269782638,
                    4729693857040763454,
                    14318418437734471576,
                    2165693130153305620,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    5215310777255875204,
                    8960471118211906432,
                    15440580254646003567,
                    2055783915072886279,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    17958985084863520762,
                    2011138862539219176,
                    9419628639415818131,
                    2403810409605873076,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    17766632042262954694,
                    371581519758440443,
                    8897600053311426961,
                    2252358811364041395,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    4115923939372683878,
                    18010614578139287009,
                    688863346621738362,
                    2147713472420897968,
                ])),
            ],
            vec![
                F::from(ark_ff::BigInteger256::new([
                    4306216465398980927,
                    6438130767707640047,
                    14355900155466133871,
                    2462883370654486312,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    11516068583303337159,
                    9530323188829290392,
                    10207730769701293832,
                    1367837136756623830,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    16832244910990573582,
                    13640872461975221938,
                    5076593427945836791,
                    28755726862557894,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    1561669505434638977,
                    7889871671552001461,
                    9809878569273210647,
                    762413284523064419,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    13245670271700460073,
                    16065244333786487260,
                    12910119596593731202,
                    730252917673401301,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    5138274673490850351,
                    12108661279224369808,
                    16753689599429326282,
                    1971917176339568006,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    3803319236751573230,
                    3428333255904189678,
                    5740864355657799574,
                    3433816521208617843,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    11384422630185102528,
                    7506184234697544513,
                    18062553543174745843,
                    1336640092383857604,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    10024575285094172333,
                    3498991381306843970,
                    11786729760138513404,
                    371266257157542958,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    10918550985480863857,
                    16805170640915019820,
                    14148577550785262741,
                    2509039687132339808,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    11030439511889341224,
                    358408419318723454,
                    9569780840266282685,
                    2679206871616191861,
                ])),
            ],
            vec![
                F::from(ark_ff::BigInteger256::new([
                    18307715327311454956,
                    14215979999287583960,
                    9730764343144559985,
                    1591125426894762493,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    13018449678407094888,
                    500762363885573781,
                    16225271180618903465,
                    3417192019904529691,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    16674226611057399947,
                    11172015256945330414,
                    4261213575014133840,
                    622041444737165547,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    10582038905405290894,
                    2636423376445058709,
                    12249548778591929262,
                    3417972976818116952,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    18035183974064208879,
                    8337616834407055083,
                    3331318798892459586,
                    1715506600990371283,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    14782847127833337277,
                    16612026800088556439,
                    16297279378921098645,
                    3088306458282158503,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    14468626850544145792,
                    5164719158688605617,
                    8086562903565159785,
                    1495163401270143286,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    9523649178599210290,
                    5436588932836132062,
                    14879199512406501298,
                    3167477278626266656,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    1556881237533413900,
                    1943554125460645870,
                    18025418855781246113,
                    232942146092924858,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    528356028777987270,
                    16777388996531938355,
                    4854147161757506954,
                    2638899854584755498,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    15250360923433908250,
                    13214505687504051757,
                    523071175519289047,
                    1849017482578731575,
                ])),
            ],
            vec![
                F::from(ark_ff::BigInteger256::new([
                    9254491476217356977,
                    7840150923060947664,
                    17705286847839137288,
                    87129550910804401,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    16287525234099724740,
                    17277612251243992783,
                    11405712581355413578,
                    2019520848305625099,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    13908144091806413862,
                    1504504508296958352,
                    16753780425029465862,
                    2617319504674157165,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    1426453682754561068,
                    7007211965312519667,
                    13588411810623359444,
                    1325215817676147303,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    3644355414977619567,
                    7875133302608335524,
                    4626346478470049670,
                    2954693339264396977,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    16215015021407506252,
                    18159601410385283864,
                    9400042715980962348,
                    1912775100973108104,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    11807348644557449316,
                    16179365322328926744,
                    372676108398827,
                    1723217910378422746,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    3609294899637597056,
                    6315237970395231208,
                    13844435241450670207,
                    313750701620463267,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    1971914961395291249,
                    15195306407467882837,
                    2772440777477495953,
                    2688150682891299929,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    17778913966393053804,
                    498018369538140895,
                    12427824740008562078,
                    1344771628291551265,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    11792769075047534207,
                    16519380444721427633,
                    14792365595916933783,
                    538160292548399631,
                ])),
            ],
            vec![
                F::from(ark_ff::BigInteger256::new([
                    15704368486627096392,
                    6766045213836664060,
                    5096783128153588822,
                    2026628540980617880,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    11645046251754383884,
                    8458644333489064210,
                    11762681199714635625,
                    2384767843536324497,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    15338167975241875847,
                    2597313278134988258,
                    5869209160135086183,
                    1361412276259152921,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    15104834516796307918,
                    4599886965975053789,
                    17406422702068288044,
                    150972267828398881,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    14285962283737095374,
                    10231111540383658559,
                    7356221836690550655,
                    703884211859170386,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    18401257541618136001,
                    5760541657096721426,
                    1864491435509793764,
                    2890970201216750089,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    13570700633055582619,
                    5950167815873708604,
                    17930113975444949801,
                    374898986495561069,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    2024965680552078192,
                    16507705756380139184,
                    13385762103264062818,
                    2903923941931567100,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    1842476601477014601,
                    1182222458095007862,
                    6136816389432085136,
                    358693818498832348,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    5979288591422036232,
                    10570286115403136097,
                    14522013538885062572,
                    3253358868148689654,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    9555943792902956014,
                    5167988565260370775,
                    16077395593194254735,
                    1304415945361445724,
                ])),
            ],
            vec![
                F::from(ark_ff::BigInteger256::new([
                    13295751895242535447,
                    7942838326471729341,
                    17584798096480522601,
                    2518811205700124225,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    13848557014209914477,
                    11807441000393029618,
                    13480673178046159543,
                    2689825488297343353,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    6288607733196010235,
                    3555543635687280401,
                    8078606066121285674,
                    37357604550498699,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    17174348412862029748,
                    3139527949233197815,
                    2823215319957169772,
                    3417567485028274637,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    10430602731951894179,
                    6558199845110312609,
                    8674826502084503920,
                    1730149159102528137,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    2095024089878073212,
                    13770152610428854792,
                    9361801816488856540,
                    23868755818960277,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    316708242408025571,
                    8187291821416604254,
                    12281970448878173461,
                    1830743219928899112,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    8116166292897769557,
                    9996792471806702084,
                    4645714978206204745,
                    1927510617877230237,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    10612875043329104404,
                    3897444033380581057,
                    11656645071097207928,
                    2099373860660751344,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    7498743214887906688,
                    14080312811193200561,
                    17336098773079099431,
                    1091117061503848983,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    1484981442296336554,
                    6339056462422742744,
                    8535027087408111476,
                    3422957908798176080,
                ])),
            ],
            vec![
                F::from(ark_ff::BigInteger256::new([
                    14015793867349134237,
                    17267980837269381042,
                    279191435206710476,
                    1824329734238029124,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    18045308240394561103,
                    17161997133493207679,
                    2286933835785728984,
                    1348160264445747408,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    11153694772782670703,
                    17778295140190338962,
                    10179425683374243738,
                    3177375918792113462,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    623017227165364416,
                    4143245308103002872,
                    1201712554144572450,
                    1656278912917100592,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    11324491743372275660,
                    10869565337339386644,
                    11256589850131192,
                    3292602245725299616,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    9099228271639437445,
                    3857640538663072964,
                    11519185860296927166,
                    1660708572992200028,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    11904926138870230615,
                    12375615430342791293,
                    6784465398987997041,
                    2213509206659700161,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    6590721476382035098,
                    5679443540206993765,
                    9454913703409585949,
                    1445966044054450956,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    17653782086652343685,
                    39884861576644156,
                    7654164312247028130,
                    3262493503406522565,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    8451972540119506515,
                    14386993616391008237,
                    4910067333464801421,
                    1581624103269011174,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    13952860174605218149,
                    7193654172708436485,
                    5432581994464013611,
                    2227400294965981002,
                ])),
            ],
        ];
        return Ok(crate::PoseidonParameters::new(
            ark,
            mds,
            FULL_ROUNDS,
            PARTIAL_ROUNDS[9],
            t.into(),
            ALPHA,
        ));
    } else if 12 == t {
        let ark = vec![
            F::from(ark_ff::BigInteger256::new([
                13019163725912690271,
                17831376289416665647,
                9939568708494062748,
                1518521220666386725,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5201327689120485784,
                3847632679997179977,
                2086948159225652623,
                3103150431059914806,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2288124371212402484,
                16289830897521045974,
                4415872311721111881,
                168986058962173781,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15185091319137251132,
                4287772696479250882,
                12875107881903751632,
                143984824209007348,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8569431135466702300,
                10048523239771522672,
                5536326580599592620,
                1949125165966490173,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17934750335151004621,
                500727850177208160,
                12069744396925580668,
                1138140353514722583,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3838479562438580564,
                7542838836713492866,
                11308293534486440777,
                3297406749840511715,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6632123291405465188,
                12717209208036823284,
                11847773751162976446,
                409222693511072141,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6383083348555236767,
                2079609236074772170,
                11060793795834251289,
                2434998681265917354,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3103039038941609732,
                1652089228634677079,
                6390683229414961741,
                1829082691448496368,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1596606255434555074,
                3269477287062455364,
                26461698434757373,
                1690613672420459370,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9199983853456890520,
                8426884712122366168,
                4136011947011790600,
                486785820201774744,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13569952092635001947,
                8861734460263521840,
                2004409017960593738,
                2505034527323854602,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12712616013909357818,
                10862891274858110563,
                8629655053773214401,
                2267476165258200456,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7485936846112146758,
                5041419188004827325,
                11249338596514000974,
                2480425657959081777,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7585926579092583969,
                7550999264494973039,
                9829872746470036106,
                391527172870662374,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9259333622189874559,
                16254619569992438750,
                673423930991618000,
                2982515747170819085,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9759312870253197387,
                3347831116860901250,
                11383110645416811506,
                2306090081521967344,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10212161184533359921,
                17750011337964402305,
                14328262281680441039,
                2485766433618013508,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6856229459014797940,
                1844231901217844375,
                2473650410793244649,
                2595309353193811351,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17288852478687885951,
                4761099491198031076,
                5448092524045735692,
                248986248047353523,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3127976846365383005,
                7144376726989541522,
                6866584050182691994,
                1795255409678927506,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2700185375660907665,
                17979198879946379356,
                4332108341894270033,
                1266967234627388459,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5479456121635865924,
                11495838665913817058,
                3825714563088014228,
                1350563336562560544,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6852119073123648344,
                150091716570430364,
                214743821376574010,
                2587328428763767903,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2578543755128017774,
                5688632603921384929,
                13661593136081912659,
                532546582120844205,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4725275122015140103,
                4955204452873040177,
                6821993897038897747,
                2856684942018874049,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5207837753913899117,
                12499840093067284988,
                6115537736704176193,
                3309665077824419174,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15126911310477538133,
                2256886443191070963,
                10481898030810152481,
                815522965049711394,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14764988249427343723,
                13867635491428829875,
                9252088675256473559,
                2384342008111490828,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14988018745263193694,
                3595384524743051894,
                15098815798144883351,
                666959979335068888,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6448160544647491217,
                13101503175482035414,
                3394333004476050109,
                3143647159292973788,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17247980068724703345,
                4185945208356448376,
                13037794400958474490,
                1923018873933595926,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3555458663283957165,
                3432426991624372543,
                15797761065534958400,
                1820383098113577957,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9590191759044272699,
                15817811050169098924,
                16483102025909657236,
                528356846889234031,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2386491357909606447,
                11858340784913991547,
                17753959856721907610,
                1303331703253746874,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17989246100742793154,
                5495551385538572484,
                621678748147206581,
                841965663379221437,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2029923905967509165,
                12864292761373077209,
                15523293352360536979,
                3326289884270785958,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3766186261547875919,
                5535562068375203065,
                12065659635572683217,
                1469694732073487040,
            ])),
            F::from(ark_ff::BigInteger256::new([
                18351012538378601548,
                17646348089256344266,
                10494829776734061301,
                2530221342786950231,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11786646519630667382,
                2419103579128185330,
                13637644584050418610,
                117926806518375535,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4061395776992406329,
                17003814006028878419,
                11185969289244516264,
                28742095688523966,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15830724022637821107,
                11572918145556973872,
                2788769829471483869,
                2258095235265788271,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3037914776009902113,
                2399082500697869892,
                9550355854334373824,
                1368327760515455580,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17631519174462467343,
                14417859195318677545,
                16283441721284497093,
                2964517809931395419,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2948271302040944460,
                9775386746722164694,
                16256682100295158715,
                2111732996389558281,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17488259985431249899,
                15660570944876835641,
                12249092366620873941,
                3192624214436384302,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12392370480513468806,
                12820188073653191248,
                3270451674698987209,
                577305746836958039,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5658589769425778894,
                14836088031558447913,
                18335723746739412836,
                1817826955021939163,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17322373179179039896,
                2938243413524068771,
                1258938546182076381,
                2729839116359662838,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7395928410734318870,
                18008560205463441733,
                5070181280128753267,
                2165547078063334994,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11238017290808467385,
                574905265282093560,
                8763435883858028713,
                1410104253044671892,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10938094325061544229,
                5199586189859218554,
                9294458308941992132,
                1351939016311524107,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9936447788448127710,
                17508642377676292172,
                4548870842549071729,
                2349410756618292915,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9901474076275674793,
                10440827206237703260,
                1618950814518206220,
                848837311793420765,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14427502139200772708,
                5175482788410067640,
                97155168138507831,
                1906257175485064023,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6471783537212519277,
                4757202762986354225,
                16999952832319738448,
                1019253631168768987,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11188648902246595068,
                18216131958012365032,
                12771764631511400807,
                3074416442293310873,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7969751881146045460,
                4296462969814918559,
                14600658186949084316,
                3189283238142722099,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7544138999144795562,
                11046179603032531284,
                1349934514201777740,
                79475579032914521,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8928667553892101134,
                16927856157280012105,
                13304818068789357843,
                2497004112076381301,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13177536599598249919,
                8081654100402711172,
                15151311437493529072,
                2491113747984257831,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9811137294856427158,
                9586629711807839818,
                12867976261165346582,
                3480802618237821850,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1191271935403227620,
                7628626882193324352,
                3596916587098268649,
                591123334541902753,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3175306332209349791,
                8792145365143429538,
                17842406001831487481,
                1179032732189330147,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11772783939034260104,
                12864631318913559110,
                15804231635479592519,
                3042098768683518125,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8464840578818886074,
                13360347847962817112,
                2665693351833425878,
                1186910630387241405,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11079055135222890029,
                926097125962168876,
                6723336609896997510,
                2009902559951517306,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11173147192148979519,
                17136339671208613070,
                13580211228702875015,
                471936972608497627,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13362919348799985437,
                7611543536105706477,
                16374852467257082441,
                3281304673810998841,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3193473446178030047,
                15984390227371366004,
                6474167855139803290,
                954272103343926015,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1005224563613932477,
                2300206865679723197,
                1425313126338755199,
                1491437433055336904,
            ])),
            F::from(ark_ff::BigInteger256::new([
                902586607183510925,
                7510599342120142173,
                12127273623086651261,
                2508479911054069625,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13847736550367767183,
                6671343260332858164,
                2214978972602766879,
                1936523265283467571,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14932662484947160950,
                6563287638392341024,
                6907185787488074923,
                2833242292074607034,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4658994652445256609,
                2350466688136103586,
                9390207052765144051,
                396194329510621249,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9482804325385405342,
                4173622813013988860,
                291656816890837052,
                781179855611955612,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8440840487542669043,
                6606193803461918454,
                2730490683746562869,
                792847915395822894,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10150099731159153540,
                17405630550310501464,
                1888871428278148733,
                2852247631492866201,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16933651603219668523,
                3431948653217319996,
                7489053882481212913,
                655129816260893134,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8105186424296659523,
                524185293561765795,
                16513159110167729280,
                2001707471927930620,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17882696859299670961,
                14600737266765421746,
                15012574690245340347,
                3399393683065479426,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15398693449781905419,
                1116526947867956889,
                3060846491448586372,
                2403793163882625333,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5890171733376771915,
                18161085855620666170,
                5876752124395683764,
                1038980400072210781,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2918510798517457952,
                6925990962208544988,
                12119042773607401339,
                2821846527322989450,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4249054071679366183,
                13932576356676131266,
                10626678709985391969,
                2791190817635176444,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7591213297055465469,
                15072424427900484024,
                17228236651557888978,
                3245184526107847542,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4049595423751016068,
                14176200924057717611,
                2260087431360198876,
                177252330350726203,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12331516743121345962,
                9663126934837820146,
                2548051947704919451,
                2222025368072517772,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10143495842601867500,
                10479195425117514144,
                12345575876845498511,
                3185026253027413292,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4507664678269931674,
                10973959918802948364,
                14508591522064625907,
                3260053301960840324,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2876965509475453971,
                4711099610096368787,
                16965795020046176382,
                382315975374723791,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17689484418215178134,
                14454939890974158104,
                522128429589136946,
                164759595267979586,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14002633818020669809,
                12363084550631266083,
                15946875537755931627,
                1822645146306694096,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8345104848235310083,
                9377376688796649380,
                16416635655661624842,
                2260591197179902213,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6696883239868217862,
                4275460195611540412,
                7516438306879282682,
                1961945453886997018,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10171740262556567350,
                12493951458398747326,
                3444884820913032326,
                3140805120770124329,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10019181046859379063,
                18063320767903342949,
                12038348026363796212,
                838547134755140468,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4114342199611947602,
                11684779599955119599,
                8405411075909202675,
                2726883110535084331,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9997148282268220123,
                4084871554752648494,
                7063212518167717373,
                2065455759573945626,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3857374733856884919,
                16889715205217722025,
                5910187492053477981,
                1914263902597327862,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16916653739387409636,
                2260366144053348918,
                16269241474577540944,
                912809843562754225,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5988960945770560729,
                14853235529173646233,
                15125351741525897992,
                2058026456100016233,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8577232912537659648,
                1518656429402138041,
                7905484491010769915,
                1931273650486051991,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13554066329338095364,
                16088163712946797691,
                8726763066008836956,
                318838395623738471,
            ])),
            F::from(ark_ff::BigInteger256::new([
                18036289184409043631,
                3011170866590272257,
                1862168475016967102,
                3119591357501963290,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11278467870023345607,
                12272246308697235380,
                6999882905839632339,
                1916846721771125738,
            ])),
            F::from(ark_ff::BigInteger256::new([
                470879079137618799,
                12456937125425193902,
                7438364929846319152,
                714768352081150224,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2405497206479417252,
                1568889950604943559,
                8238822188849572342,
                936868465133218916,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17439906110533562926,
                15900788533871218643,
                11719819734697421208,
                381619416396402689,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7399289389971265979,
                7143669485758083969,
                9801047844132233244,
                2063210635676681962,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4618831152072536539,
                8311336707752204521,
                3688691585703860705,
                356440635132343291,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8556922520036447549,
                3932444142288959612,
                82189459292121961,
                1799868619640802522,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14239266725826310466,
                18188233433840795397,
                15218394120648974735,
                847087730030204230,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7543915527915087384,
                183922624408380261,
                16342818783848353619,
                3326885974183207696,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6044619497311714038,
                4000478971221014514,
                16886738801009652235,
                1303506285378586864,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2292059012866083330,
                16627619972736745633,
                3229732644666895643,
                1528992367334486868,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12614259387139636175,
                8544245432163671079,
                15966070060059392434,
                2714076473730992696,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1720387988937817660,
                724907257981518501,
                10702993022883104977,
                1850027498654343312,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12186604795369091842,
                7674139991321472873,
                1435743240634678450,
                1423176211146541103,
            ])),
            F::from(ark_ff::BigInteger256::new([
                281497657298520605,
                8548767143289026388,
                4668148551919171490,
                2301073111134001394,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9930222223532900006,
                16644351394525612344,
                4354026400304863750,
                224506951709641316,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3847361107511362289,
                2934341347993541610,
                1442403214944662167,
                1904118877897413443,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7850393126439752358,
                10600107973410833350,
                10235378019301864065,
                3259373417002899567,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8532714036988860079,
                11689676736209807351,
                5426013836500747619,
                565861676459867842,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1611489547450675196,
                14300257597000093352,
                2010430289398746748,
                2762209724835819981,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13157688435101287122,
                14029499687313479430,
                3742872479969227179,
                3340449771061831108,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5769525802629319303,
                15748384405368309072,
                13821848026431498502,
                3458983959234551524,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10124550939825922131,
                17226414155214592773,
                14941647028675068968,
                1216320481113832200,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1551232793299186825,
                2498569596445566658,
                17203610517008271169,
                3162275960161298020,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3364449133804152236,
                18422174524146858251,
                14434248632915041577,
                2391695179774587500,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11382751185746569189,
                2655038596506209726,
                2331735927676160991,
                1502240697144603970,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13884328440352908254,
                2058985530609902694,
                14857691639821495098,
                408976675179429889,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3101674720376146394,
                15009903441546329190,
                2588804984953605750,
                1315049312295370086,
            ])),
            F::from(ark_ff::BigInteger256::new([
                767722224391191287,
                17206388642261913729,
                15970293511602769393,
                2753540653063627870,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12708003656525674624,
                15197043567457669717,
                7162782977279791603,
                2379323510362009269,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6290340772868714660,
                11354536675670057257,
                7639094286557282891,
                2516964157225899314,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7996201791695830060,
                842951575456168942,
                17764444358479685930,
                1028356078790295858,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12725541089191302712,
                7654797647547702622,
                10048534855630345954,
                2160492725985236707,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13741689809422375958,
                7580738616736566055,
                865094875390809707,
                488066633839938726,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7010416429741346424,
                12482089659495933050,
                10259733637746328829,
                83396874995500820,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13638883078317077293,
                13127142279318276713,
                17996971094586872929,
                1427925858277465208,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2986562495481209224,
                5508618566019150338,
                12177019925457696832,
                449627156684211674,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3484950629007002168,
                9674249766470370956,
                3627030870227546977,
                1577314403322704109,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1109826969105219056,
                3284903016395546716,
                16179003917891997813,
                1350901585928695058,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6824780188350452935,
                7245766136057302943,
                318322434784165459,
                2611027276967647829,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17736047982276445157,
                14527801364222387013,
                1538968978936094296,
                8489306112180934,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15687166933104559804,
                4381444524005565522,
                453002443444112770,
                3085488722791030884,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13971473897951628674,
                317279560854096508,
                13006693576816434053,
                2527496279658122177,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14322485125856393020,
                16357696007560316955,
                2456875793322436434,
                526762440876267150,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13000724490431024373,
                7094885989379471185,
                6514183318519692768,
                2499790616870874390,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8882335501553230098,
                9260686678042329071,
                1968010383949404310,
                871728281809119905,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1861964233949895542,
                2693584492705116869,
                2385640749334976738,
                3465649128101040726,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14417671035535874626,
                3152844296867646341,
                15145019333757972530,
                2021948902353131088,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4227731946697554169,
                4588148433087065751,
                7655323851083034607,
                2913435433441375892,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3559468912050353455,
                15786522100724236523,
                2252419640448359682,
                3130727865682668048,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14019434761204012794,
                16461756368644007953,
                16800966673959881268,
                2247567879440816792,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13266621737153536371,
                2494925052216662923,
                8695102881428015635,
                2708148783000035673,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14309637886920457707,
                1065376924049028076,
                1839855123529523500,
                496347191491361482,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10457203391999642594,
                10583014352253810910,
                13784926965581196424,
                1220983524752623480,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4232070724788605187,
                967324878026896701,
                8409608263866777430,
                1693293338775125186,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14232340393711425040,
                3385477924939120093,
                7518016016587794026,
                2248425595646820493,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15705402474869205825,
                11438587598387338385,
                6766466695169595340,
                2782328156168151647,
            ])),
            F::from(ark_ff::BigInteger256::new([
                18413092378763017318,
                1084540603256882824,
                12803171561811584264,
                526713864773177295,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6744995174328326469,
                6531884074846416641,
                15159148471951284599,
                258661196671834803,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14763718686057373967,
                16261275669012489344,
                3671843517451159722,
                3315686989696577631,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3203351309334686352,
                2120107836807247181,
                11059858528400336623,
                3091889667338566729,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9189791904809559696,
                9785083376846761887,
                1223863583683642332,
                3170216258196419594,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13995668115004037632,
                16683921985269524370,
                11523286797720224529,
                3121192016911769373,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2800181423104048062,
                1477051462423293163,
                7123681082215551605,
                2653023366224470931,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17047266034831732593,
                8288237099256011389,
                19762144298906058,
                1529487701891195138,
            ])),
            F::from(ark_ff::BigInteger256::new([
                18242355701659628840,
                3217716657491774491,
                649114048540241850,
                713923743368084999,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8738691844355536915,
                5906413834583138758,
                9369522098061612435,
                143692390323716482,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14482712644631812601,
                551751989585068940,
                9916496675821917784,
                2103780051461579159,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3871197043079059416,
                3757472271544951968,
                10472735256334481277,
                1397447697983371708,
            ])),
            F::from(ark_ff::BigInteger256::new([
                867814861130018926,
                6767078882582725263,
                5959609345553428475,
                824421106975086364,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6500751108449096215,
                14353533206793456081,
                14978072472749936268,
                3069969966525792140,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1265697007289861413,
                7574254581962565556,
                15672244287706842487,
                3370851194445374161,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2391735750778117604,
                6285624068427991846,
                2492336081636077169,
                2025311411635364420,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15942742257972858185,
                10853169955159967223,
                3156207681706210575,
                2296741585443939597,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17802218053543331942,
                10516055117308072394,
                11874105148613491067,
                921249363378983127,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7446313370550159379,
                11491491805900426341,
                739335412930104171,
                713141020950595184,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7165717753972486982,
                3386990207436243689,
                14315642372894630289,
                433608748737019284,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11656602607832156496,
                17344200884703848081,
                17668060866732812471,
                3332708298673102367,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6967194133246601223,
                16287968603559407924,
                4956438268169077430,
                1324455844421545809,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4098366377249422511,
                7956529343751784188,
                10797541791886042202,
                754425906230405881,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6528034642871296250,
                2726955193961050409,
                6580680638111058915,
                1146665328086673588,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11600890074375174685,
                3682184456253987918,
                18034949030832229680,
                3349175979013721969,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12779930620336237709,
                11799121069576611533,
                5269674388080332729,
                1172783971786843759,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6426728181823542385,
                11787327797864277525,
                16059266651803865063,
                3342730995160851076,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13571830255025223718,
                7167567893532595025,
                1637435361351374214,
                2584517070915169765,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1553400458539877447,
                11327292735339986885,
                11406456811582736876,
                2685164425265299671,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16092651807365070480,
                710989027038245414,
                9111546189571822187,
                1505135215064551601,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2703426347855585190,
                122653476869013949,
                12544817232588540384,
                3318739277241684185,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5434424160646098411,
                9291126488178724674,
                1979121655330399928,
                1376378508449755040,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16677458889360759545,
                2020534325022413390,
                10059106811417011620,
                2750435972558650289,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14139592174071232383,
                13990840552713356244,
                10855208567260008373,
                2843809287794131074,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1525987185674946380,
                12110813713267139865,
                16152401838971402949,
                407102463773860967,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5843913314913349582,
                17025118863174608068,
                3212448687238437656,
                45784422093024507,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7230209054839154082,
                11428834589687791578,
                10429995232722325034,
                1354511920989531394,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10191693442950565943,
                15278589947022501123,
                11115973796748854462,
                2711267759835569346,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10011666499238003928,
                12709686854927959450,
                3500221401110737235,
                485621136109531790,
            ])),
            F::from(ark_ff::BigInteger256::new([
                729193546685968315,
                11647296630498093251,
                14952804427764317303,
                3410911406315862770,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13720021554425436416,
                14308377425486364395,
                3082354318284616293,
                1893352532018060340,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10550110970707988802,
                16247520254077532957,
                17465380369477758383,
                223789766758151249,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16171094022839996215,
                11775898686127355643,
                13701588649124281338,
                2952493760821624442,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2189576861656760843,
                2230806463717356080,
                4647908073051276522,
                357768113063095939,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12234535318766248293,
                5656637554701620835,
                11178100677030530668,
                1141418213218038525,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12065826255699982211,
                8773243827696166301,
                4595378707556749576,
                1043123522889308362,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16885589697923616605,
                11634552637350884178,
                15664644428926863428,
                3329909039483257167,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13025030947557823881,
                13909356583907617208,
                12661164674206243956,
                1071486407627715485,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5397019271223342659,
                1278895403037743868,
                7185187639155806916,
                2370082059019701508,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3280188907264719318,
                5511963215258495819,
                11179265156384669574,
                1918737588225341221,
            ])),
            F::from(ark_ff::BigInteger256::new([
                18205734026942846383,
                10280190886116883177,
                15354529516243955231,
                3123293516966345837,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5749965055647716277,
                3503971074749701565,
                6001437186334598714,
                1227205690530794562,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14990822588886777932,
                11556314439363936570,
                16826046328887315404,
                1377516720468585376,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3992255055472802662,
                4718801417732652437,
                15892431274448373429,
                3462942723656776885,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2729294205105689885,
                10914817454091528003,
                9663944292913059192,
                1769388116895722908,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16558658559943516796,
                1765617104941955775,
                4451721037385319390,
                1716752739634815024,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17533171481335628304,
                17882845972043063027,
                648595363194366391,
                1524715412939505166,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6937606606567072641,
                6764113897486346016,
                2872488320430977823,
                2816565042368587743,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16010071174089668049,
                15337143268126849828,
                15742633128349521620,
                3277864674362194582,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14124117701732643822,
                16330574361591910746,
                7315065374604240215,
                198487001829225549,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2954149453660429168,
                13729929323828080532,
                15721885519112795138,
                2231522009686912090,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8288459094380742246,
                6140055792362468766,
                5877997478096056301,
                1756988686842850504,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8858977986457642469,
                8412418504215210742,
                16711377784808391787,
                247956833671063596,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10072409588780220675,
                9902858375403893512,
                15322027005718875127,
                1175439485549303985,
            ])),
            F::from(ark_ff::BigInteger256::new([
                863915323085650660,
                13548020277829362047,
                13004113603212820992,
                2336587371608153680,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16242631378735794143,
                10065416347186713997,
                3563278043259377259,
                120086500453244505,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6602088132686153977,
                8639258814369842083,
                13885462838253699743,
                2254792101464376307,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4127356144581739238,
                7248520062335629357,
                10885734554621063868,
                2810704322889276109,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3748511485130899527,
                13967527233951082,
                14851099878998979849,
                570157071537896474,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5567076549001842521,
                10133046269878474718,
                9754644710633874944,
                2472552432989548112,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17717877124780890014,
                2095626487478257284,
                1220727256011748642,
                2741956089731173053,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9460923294997782310,
                8986612017718187031,
                10553601499061099797,
                1561549309714121284,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10208146317694091674,
                14610117532763325951,
                13815134924209677541,
                681485524008444916,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3234611012585054810,
                11586566521304840407,
                4572833109592081546,
                627082460851375297,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2996087805348131525,
                8253678548503362941,
                2808523553340777034,
                1834860665372162586,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4311518679212439785,
                11568138413173524043,
                15009701716215378219,
                357364506709141053,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13030156178924430093,
                287796653419313006,
                16694231217631451567,
                3148634356726220132,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9041620436378929539,
                11661758237277531116,
                12563913921135453270,
                3415621194826607708,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5797755735548992835,
                11044301290621867682,
                7202949685258329138,
                3005424818578201603,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17232340527268833102,
                3348966431533782730,
                4208028098701623673,
                1921736331789849871,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15918640079409900067,
                15016500279574775556,
                5670021999207479106,
                1155549285411705210,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12085678400663392746,
                3408907555165010645,
                12655165197404458009,
                2817905624506234106,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14906238167926963062,
                17484884940452149173,
                15264932589616332525,
                1162468190310108080,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9539455980700196378,
                645567278060677975,
                5104782134756908103,
                1425004466717946048,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6921052298818010271,
                16998422313048130201,
                3087824602288133118,
                461216128892722128,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11831017419484492185,
                691798465595563505,
                4286929886083702499,
                1600918563104181862,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17772677096514475670,
                13179320502753806736,
                8170174353071502049,
                3315159052784577105,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16847912727429365623,
                18199801285219656883,
                9244462352320807893,
                406709864935235935,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11914585535276496050,
                2105725554308759959,
                11991583343813895146,
                14341731984714708,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3583540885398087641,
                3608992791399267414,
                4407085420732776772,
                388459874229867645,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14729950382775212029,
                11360255875064753245,
                16932878167932030352,
                1639425289092647221,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3723697112680987026,
                498792598364897463,
                3177563406627917495,
                2129397700130940221,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12872676253804891620,
                7121845962942644154,
                4894394701552426636,
                1918993166900705656,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17965997204929668828,
                10997941017697413960,
                12035168152739226989,
                843655520375470098,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7985392367367306302,
                15540999139315200138,
                5424236751902388813,
                2612680918567969555,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8210543869570635732,
                3861373567277289894,
                1115187736305779657,
                3478399597560481161,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7920429618933451197,
                1978581140215615519,
                17125591247194094595,
                643036428621599742,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15649698692431910319,
                14504215126926216262,
                5357025892113041448,
                2669705129272637858,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17970036748285311293,
                1578219187167360147,
                9305217512136618718,
                1091478850819048217,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10188751573375741197,
                8782545993383910750,
                2155244715302414748,
                3384081035481499715,
            ])),
            F::from(ark_ff::BigInteger256::new([
                460303064184793873,
                10647547840954512386,
                6249691814357990366,
                2199033663980215776,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2286398375379392270,
                17559186424703167242,
                18356935892245565455,
                617625392568657632,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17688857697129612154,
                10319890466048426937,
                12710061816124651325,
                1254387462854472973,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14497357397228606630,
                3023505892837487418,
                13422944157992179986,
                2834316106956839642,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4200158495418382021,
                7079116391811670070,
                1409513788984222926,
                718337893868526200,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2160403415694912843,
                17512373221810656554,
                7368986637451829972,
                1170946298772308155,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17200210432169592138,
                5359304831892142888,
                5436602975282441283,
                1832545169740161969,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7171594783822389447,
                11891317563255614582,
                12164409417511191701,
                3209529992088224850,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6041009570941067669,
                12123932742225656294,
                18043222788049704430,
                557687564853946284,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6845487624250603887,
                13775152644424720034,
                10456452563185029164,
                2622398450998945645,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10979473477563311269,
                1343704646760931383,
                4660609191256935700,
                2775082226066977860,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12437474514675093419,
                9054288178739247637,
                15433221988514504013,
                1645677414571516125,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2666311339061747835,
                312905169666855130,
                14874157345694400014,
                1636313516146891228,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9723697925050341130,
                5207197621523817702,
                1645261588626518182,
                957546114450573084,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15256931670044309882,
                311444627630199274,
                15408124601567432602,
                1945476963147988322,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12570995596805459643,
                11149274981579221903,
                8470239453706483420,
                2871743644035476486,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4601754973107108361,
                4915402816346040687,
                15138925186236049799,
                2873787003269142326,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4360024720377283362,
                3098506859488112855,
                11799961391881830724,
                3436771117826692959,
            ])),
            F::from(ark_ff::BigInteger256::new([
                18382871894102835174,
                3567728104838004546,
                7825313658570972942,
                1107618093533025030,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4462136958595915774,
                14810114459566142325,
                18317263619154849338,
                3156217776130076919,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13412079460006817821,
                7932444474770482454,
                17357467289728435445,
                25989336782775194,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17385956994244064428,
                912343507081337701,
                16790126519224943947,
                243942745823652655,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9696200603475094057,
                14297117312723808356,
                4542312832116843324,
                187938988460511673,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1427443448057557388,
                3096630355780984697,
                17102596803012664341,
                434867990274971000,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2472788472058201326,
                8464797441197002100,
                8092642845228268369,
                2272713577967511605,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12199201232131306296,
                10917098099574065298,
                3777585884665073411,
                2393806942442692028,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1082435150667080236,
                16936515571754165393,
                9942459359698485611,
                2986209768337270257,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1631550222238205870,
                17419860270663527957,
                16161349789992263964,
                1114080830147201121,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14727394956503333655,
                7647870402948474876,
                9591579021877168078,
                406738958261869393,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11004306178874754209,
                7532521393471089674,
                8909983218022978829,
                1548635119740528569,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11778394824823611055,
                3827396956034971627,
                13014360416302622033,
                2844765996011241052,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15710870102957380517,
                13372793903978759496,
                10161539164611996916,
                1484227563375842943,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3253511989144041208,
                10286268901108823964,
                1339698026467506711,
                2309179906467080026,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12328309678359299962,
                3884547837962992234,
                15956897839862471548,
                3343981952072249648,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5229559445618777402,
                15725649080842846051,
                14149848485872983426,
                712415511845129650,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10758298649673452953,
                14562898195868898774,
                1591981623782131587,
                746261823930370615,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15715877366808832964,
                8783865296633936367,
                12497359636483158935,
                3457360763666719390,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15548034576979335019,
                3297104575359684403,
                10628683522169580741,
                2250308858570155141,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12769938454405880616,
                76225158030503022,
                761225897499504178,
                590039234078093061,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3887740163165070642,
                18131098009126138055,
                7794369256329987812,
                1869156975186646287,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15679728339118244772,
                3734147062098369598,
                3905823991291953215,
                1901110971459799984,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3635294780327039627,
                8443869938811549213,
                12989100825836492131,
                2416879889781486743,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4070604354993349627,
                13301885588638811559,
                12083362586041164849,
                1651954561739259737,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4934454092369498028,
                14292724424964139597,
                3889543856243567183,
                2138898029795333684,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1544086589505137944,
                4125496705363123297,
                14282562843706395961,
                2740569884212127893,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9742853125488434741,
                8685215389013796598,
                5879003788720804118,
                2071499564458427089,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15770236629826447775,
                3152165360517460650,
                11921281393258066471,
                266454689742976142,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5369144059914395503,
                15234986482447818198,
                4783195278832372929,
                517724067793184085,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3814087779062663943,
                10653378927824932307,
                3891442322239802139,
                1728927785929341714,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2103142843902381341,
                4415382364252331227,
                3978080766098024774,
                3374240500093746184,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12123296026613472281,
                9239066862879851099,
                10565200643355211784,
                2803556302972550102,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1896483564964614315,
                17071919949795688933,
                7701739518138809739,
                1682373624618072200,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9541933334390366677,
                3859855902563935869,
                9003628556310053270,
                2539505720739495093,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12174098858613991726,
                17655873709339953065,
                16270844721466458507,
                3160809275912779253,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11973315841119458777,
                14136629926455489801,
                13624377912897592092,
                2432755845368592646,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9994603232746054649,
                15249216246537290772,
                11617054891008399424,
                282668720352364985,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12684915833459204858,
                7866155350713353533,
                1599453701010207590,
                307701540339946743,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12739919328243281450,
                17700503146150705348,
                730045007988404541,
                2554921757162266084,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17923459548791072931,
                10172350340772267478,
                16848557177881262425,
                1777459597802242006,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7755327695639459171,
                4718602153351211936,
                10862822451749221215,
                296470801433996780,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7275862037782827749,
                16155972845048747739,
                1940768291263385806,
                1203518618680359787,
            ])),
            F::from(ark_ff::BigInteger256::new([
                41659021722799083,
                6064176715142999722,
                6704325887188104634,
                607421915751149961,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11766305504056836847,
                5243407376660635611,
                17636721136614182246,
                517477280969091665,
            ])),
            F::from(ark_ff::BigInteger256::new([
                18157635359882527863,
                12042913665971733587,
                13273726932242323393,
                1431825098449417058,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16783599404380946564,
                7619167545952888663,
                13291444271924191444,
                1115611826459687989,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8610438642278015107,
                7557387813467466834,
                15994370624468543128,
                2590378820744646105,
            ])),
            F::from(ark_ff::BigInteger256::new([
                347526550964668653,
                16318537857757410820,
                2689866228591969545,
                2756258807680777392,
            ])),
            F::from(ark_ff::BigInteger256::new([
                670215735976435045,
                1763802393625382207,
                5665702536634208854,
                3049038550530001336,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4462117934834499493,
                1522967423905613136,
                7907004837235700596,
                1304178798921407946,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3344362628338557240,
                3063813522004951167,
                5215433178348265956,
                3140825937925457903,
            ])),
            F::from(ark_ff::BigInteger256::new([
                18012269977969843166,
                11073700855329941255,
                9820241912702954279,
                1625171505550716116,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2857548155486491705,
                12314222081335857623,
                7997779506310500011,
                2972339876445390107,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13816341920992218720,
                16518642002354731973,
                15314165747342537096,
                1192681094896724414,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8105153419687919766,
                15943343029860366689,
                16836452957570023396,
                2104570331702619035,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4259938984896393289,
                11318660158182337050,
                15960177106017416158,
                3273472819590373138,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12415034806875598162,
                16070759600277501487,
                7356104559126226342,
                1781863562875431161,
            ])),
            F::from(ark_ff::BigInteger256::new([
                458174099081721577,
                12596874126928811758,
                10443313816219205769,
                727155680533396473,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2881027286703133436,
                14593474573394488820,
                3241085289412390504,
                1137702525576413375,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14214433896923210444,
                13980247396492317767,
                10125087832955657660,
                2891924525192877027,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16567048319152331590,
                15943078287817350631,
                8173220727296186176,
                563647643921893853,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14758470906778520204,
                8346201552513086235,
                4486200343099909056,
                1461852694417830502,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14686469209534660557,
                9638203160769538747,
                10607694516788992078,
                853686627467949395,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3658344974613756417,
                10966319766657374605,
                15718317336942552244,
                2376457186412731974,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5667487200807133276,
                13458749672516036775,
                7974179390021554565,
                2781421296267552139,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6259647769554817237,
                9984479371104607734,
                9438515009739374668,
                2917648425728945416,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4332025533533283082,
                12134068119045318261,
                18107853130181769861,
                610469971050173567,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13732275376307726314,
                17853918926833803162,
                14022829219289338711,
                3331620067071824146,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15901471780821035331,
                7106386931886938446,
                6163983448551858415,
                2173142091002269882,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10371076274221242246,
                1788283161743712308,
                8788593971262168401,
                1881800292581007552,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15996617127520677256,
                7379036678904306471,
                16182045186045323134,
                1088301387308420281,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9646073397720150423,
                1004915380194510810,
                2086894228069354988,
                3276412240752781377,
            ])),
            F::from(ark_ff::BigInteger256::new([
                95788382886206432,
                15111938178269082074,
                11372391320432716809,
                2014477211156611530,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17227520441770065861,
                7111588988112528708,
                18353620068918789153,
                3002741298553123359,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17863058366149521320,
                1674420319640171984,
                14473572683246237189,
                212988737871020548,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11278488961618254494,
                14048861847294156346,
                14245809398355871352,
                2047213762762205922,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9427536102679051552,
                13221324940527332183,
                14382285020785777157,
                2723428953566991173,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15155612717606921004,
                12267970903497198589,
                3421804177604393217,
                1565675640532090618,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12542975747974277768,
                6199503185103696968,
                7059433141563172877,
                3466085974376424385,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1894942336855634773,
                17136099853774303689,
                5846855795542847266,
                3356807842751976303,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13511896327097018481,
                4672738145095059471,
                4232481178648689524,
                728951859182663597,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2340608254296009809,
                3491757906340440658,
                15910829081798096845,
                2809877853425416592,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14870274573374594379,
                9991159363662596893,
                15933142552242518571,
                1805573539037903212,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16833987079581305635,
                6360920570196336352,
                1766502222159976462,
                2108841585721468794,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7269122656464150818,
                3330532546143194496,
                8108781523710389905,
                2810961406614392974,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3202674504349133907,
                876541164052331906,
                17393515513042222985,
                731478686148497401,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10097258981451001815,
                3311019888661647404,
                16089547308101094404,
                2689180390019645493,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6141329201983172780,
                5478501406074733286,
                8215759084119897496,
                1713092158569480242,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12773629162001079422,
                445542855641321552,
                5684915712872607806,
                1002698857244388385,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12814670247616947756,
                16218990532192607242,
                4567454614242945000,
                425133529587594495,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9828017296736807984,
                1595227336819090492,
                13308741212777108718,
                835541712917744427,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10484606797393533242,
                12349236623971234759,
                4849743987809891776,
                973885362291414352,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16606645470524292000,
                10979011632535460452,
                429462351597493134,
                3158228702191258016,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13583464631475535668,
                657744468647320550,
                6801738386309386897,
                2690656553429402147,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16909959447587401071,
                7688186876465270634,
                7817410688094459893,
                1275672154942451201,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9626033216054662861,
                9871624290782477488,
                13258451143900655519,
                1600097151674619750,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3415259849496568375,
                2905835102081392114,
                6012003419192156936,
                1723069034072238371,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17668821815219953040,
                5121291023181252831,
                2609582957921964801,
                1837623122173790410,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10695062578055633273,
                7279589395947202005,
                16244695071364365960,
                3071908031975018446,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7391195739736531309,
                1372472704524045302,
                5213998748953870698,
                3344532961852090087,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15748099095829238053,
                7767522010014718708,
                3065605651407965031,
                351617303256543818,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12902286880647834328,
                14383410146620430425,
                9564214541622294641,
                303671405092171235,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14774526633216479380,
                12356664508418741225,
                18048462923775226326,
                56715650488262783,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17565729927386855239,
                9498327325887651450,
                785544719335553725,
                3388053577460744216,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10835807036475738001,
                14226638786046860009,
                13864690468063831177,
                1515193945539851725,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10805166221539079950,
                15218038873498682190,
                6321401636855801013,
                1391111876273717944,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5469416444157877998,
                2080320345223924230,
                12655415858868028694,
                1662218109497704987,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17302219204846354814,
                12310799815801218398,
                12860108267917043457,
                938769747538965011,
            ])),
            F::from(ark_ff::BigInteger256::new([
                18313792873716947104,
                13035068332758556943,
                16299153804092884180,
                967353085553389812,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17775209929385858270,
                17355007006944120326,
                6734566192053467418,
                1997324934149768795,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8538537165963680165,
                8849652852960683322,
                17388457191238611044,
                1102002618389685441,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1510174788221356177,
                5994670188747632235,
                3086314813159287010,
                2791489935408229742,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9916053129454219083,
                6316806119567709615,
                985867364003451411,
                2141288741280138944,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10364410024519754456,
                2496028017854782980,
                7545095394975568861,
                2603629135832631264,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13327035344587132124,
                4504870341388368854,
                3992529345127548712,
                853950096400107822,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4714851799746695205,
                4026992862592362476,
                16392127452327387783,
                2760232743714338191,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5194655813313004709,
                5411839204352674745,
                3379391776674032085,
                1885020195284297404,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13921617479200714261,
                3271559180777727183,
                16685108282415065836,
                1486757854351828683,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9644673545256794051,
                16250348303765837726,
                16984502772570729350,
                2961503161907582469,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14229745926831666326,
                958577910011463772,
                916617163655747591,
                3165112560235847018,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9290673087431629950,
                1097202185972398085,
                13757872415731404603,
                3365052896619721147,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3354740865081395390,
                16451889456348048832,
                15972031325697878402,
                482231569753243328,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7020104380723724197,
                518611838615307207,
                8797464112895771577,
                2977256028879737573,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13720255149859909274,
                9768515502467034019,
                9016260643735740761,
                1117052400555316878,
            ])),
            F::from(ark_ff::BigInteger256::new([
                18059906263330326055,
                8442722712169033036,
                3845852325466901666,
                1111384962141811010,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15252790451622364187,
                10722208599313233026,
                13356582887826422016,
                2761167629293298041,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5193964353392335460,
                10541388452294306400,
                2504909801267508765,
                1873477007315669181,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15773147603407397647,
                11625535580882979532,
                17817484620810171896,
                366186923400762143,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13682414027140675936,
                8012188649301918035,
                12747241279808204349,
                2860437700201228167,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11409805855218065865,
                9724694225136344053,
                18211995190016023421,
                238192740858260729,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16278880332341285434,
                2176624303158464902,
                1287754663681093410,
                1464702033804331470,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17397684255390046150,
                670415526094212642,
                12596065686270090347,
                1254264701462127299,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13407132904625225113,
                6838368778535529520,
                13032761858924939620,
                2671227922868464040,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17755314860539806544,
                15883684533064610664,
                15939986585644148353,
                3463481222692234486,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17714726601217648542,
                14614081016742467178,
                291820912198143638,
                2112409769041481751,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14716913464848855722,
                16779152652012277196,
                2890737026020182684,
                1571135809400172946,
            ])),
            F::from(ark_ff::BigInteger256::new([
                949334377827793020,
                4985007328708578400,
                16044387769285209304,
                841757691771636577,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9669969524706920183,
                4167905843415978707,
                1736228257684630689,
                677475833965597437,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14823924769922025575,
                12255034722054638582,
                9491903672781757571,
                2066750900893493231,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10751741175472635207,
                461740621761024989,
                17157160439858236544,
                45822712418179212,
            ])),
            F::from(ark_ff::BigInteger256::new([
                510238599448930244,
                6858501535500269078,
                14457521557846037506,
                1447855290861854811,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12074941859770877826,
                11619854855480486173,
                5635466035912495747,
                372772308163772947,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10214215107320852173,
                2740933514800230856,
                8933342887997946248,
                2497467905196003420,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5847848218104416060,
                3582017521864744155,
                4419290280435838244,
                2033518588647088228,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5048311308627593137,
                15396400939219580966,
                4540430857862861017,
                770787076451700774,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11312870262099854028,
                10347348814567028315,
                1174772606550950936,
                2425899082479619330,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2449394744192273358,
                7680498994442021789,
                13948470847498688234,
                793915589385654194,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10309730602524551910,
                88984995856019897,
                8284493631077759372,
                2088375344595251219,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7074764404295513209,
                14423217285010219328,
                6561847296016098359,
                2045645289793538703,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6450970106495278199,
                9648433558541262035,
                865256895004760058,
                470388520192059949,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11822485625250096483,
                2142787904014306841,
                15224255059870949926,
                975316853631046464,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15855130464505059829,
                1293143481486329546,
                12820992990019276680,
                1751012839594870448,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11029035122586288143,
                5349399387025877083,
                17605058390614662589,
                1274198550116071178,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8438553997196975747,
                13318822366413304268,
                15882409471241379403,
                2985225634305548462,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2920959158959470779,
                6933416252510527240,
                4933435602430695051,
                839207296715428549,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12323820673845032671,
                17910211736092609983,
                9416566015515234934,
                1837078669933614051,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12855863177851115054,
                2273752546714791521,
                2806558341724854140,
                2991842757968854177,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4164098346467414081,
                17682125954602705158,
                11503805914982265268,
                684198796247223041,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5149956992014959611,
                5886865935802635182,
                11898808571454781817,
                750248261392408063,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15830279162999819990,
                7383814100035026186,
                5909833990931480168,
                2370741752373469526,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8577246516941033648,
                302945499247022605,
                14512898134866186748,
                2848824947449335723,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12542495337056503409,
                6901075479736663146,
                3195208111984347565,
                1864316212463003669,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3755465853682114409,
                11304654678064580816,
                6042041178231995726,
                2239865716942102839,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1693329657337462731,
                9877965695592725274,
                14914403831577400702,
                283598113109555901,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6288376300406334384,
                17897703157490270965,
                3714208640438468487,
                2075596081667315917,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7204574796223275937,
                14344239538346961280,
                16162559391135878750,
                1208880427472827537,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5734804445424249372,
                17809018914259376530,
                5226474330914939906,
                1536101531504655278,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12657591677502131210,
                10570506908474216034,
                6808121553371122353,
                1625562875230132116,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7616695898022820461,
                14254079099131810937,
                767829830291258615,
                2727382607699603151,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2390397017569281375,
                15014700178436979699,
                2885813906195100730,
                672634049512558001,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2618259613787026236,
                10898341438921729483,
                13625451087990462754,
                636246517942405604,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5731604312728611688,
                13852010820082836058,
                13249591540432188736,
                1389802712701637683,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1774240584365365705,
                16593051814443545920,
                9151472242961528195,
                1547316042734208020,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4285241145586495638,
                8895453484441153718,
                9528000264395778602,
                439778654995635045,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11311892868949259842,
                10357956132379934094,
                652022165813926797,
                2726912928120479345,
            ])),
            F::from(ark_ff::BigInteger256::new([
                18156200413843655416,
                1394755542941085928,
                848921325864248929,
                2522509210488395090,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16440580783929799971,
                16115505468535930248,
                11502561122469742435,
                3272308176090519228,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13448046248154173476,
                973844596368950646,
                16999724509184200977,
                136904705053187831,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5091788240945144552,
                4338131101576157927,
                11070588226539810907,
                473130362581837399,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7240191098061685223,
                1392321535137315701,
                956905164060756807,
                2893884561726648789,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8999224483785579890,
                8207409966839333726,
                9024162352347481344,
                259021844899231760,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7036596077319099607,
                14175233781271595137,
                12293890038916827829,
                3260092606160781407,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6478657434195737394,
                1584032046788264856,
                11277197509344481241,
                738899074323012336,
            ])),
            F::from(ark_ff::BigInteger256::new([
                179974311748839169,
                1207151303001972880,
                11244629614578836802,
                2256068993855425033,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4349513836577685039,
                1987847161735596340,
                5666416801466330014,
                1329783214747354310,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13046675260962944782,
                6012654335961526891,
                15244575646475408663,
                2519207743770759644,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13067067388421919465,
                15396397153976147759,
                6271673940342633036,
                2530114996478023325,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15110193975803284658,
                12187327169975602666,
                4142379011095833566,
                1137577881681505963,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7683624015269497722,
                1494398916246871381,
                10964049287331024368,
                3028613863704238904,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11325237712452076839,
                7491728421078842105,
                5343192903026527099,
                2330070523538952203,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6692831571025559889,
                1836048073311004645,
                16289084954898439246,
                1241749923929750140,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13712120858502950070,
                8574882981177046124,
                1606411886793526832,
                1395930928304402697,
            ])),
            F::from(ark_ff::BigInteger256::new([
                561021569563764482,
                6800255848065987545,
                12397362515886676172,
                1368748093691650165,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14456598790441342526,
                7882977165602651718,
                15998908508613995115,
                2663989947701092439,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12476851128466939379,
                10761224849887874905,
                13936314014288329686,
                14252457365346395,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6670837706759988580,
                363554771126219862,
                14934171788183148758,
                84013832114922281,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4289120085110694938,
                7786687414906545122,
                13670372531797148897,
                399522274681119678,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9510569947914100685,
                17317182589407286294,
                556709903712631773,
                1848075688901390810,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11546396471806161658,
                4434640784438538025,
                6277179395645230410,
                731011405126221056,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13972045051660920229,
                14451485418506041298,
                16220688464115086657,
                1690925631599510955,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4811005007095409844,
                6915382555967230198,
                10921648816176386609,
                1461267673942896950,
            ])),
            F::from(ark_ff::BigInteger256::new([
                44200578190798774,
                3593742180813310776,
                6697418988264301467,
                922823444456185506,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4329267083004002237,
                5226473586033711167,
                8827274671845996139,
                2054591696309029912,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6596506945366931826,
                17348485796461245630,
                16312749212603282537,
                1827072014178768531,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13657756730896903907,
                1178820575811161120,
                12873065430524307253,
                2617986919929158665,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3301730480803302057,
                5790190113587085918,
                9499331228491806540,
                202941350629694422,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2825845822746933647,
                4083672960881367358,
                7143689122330833623,
                1917436843180550407,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1242413701301550602,
                5839866523253998113,
                2774308528311005914,
                252420381614659867,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12155090307682506601,
                14445759665145355107,
                4421001587608673828,
                3407032181566180974,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7311081270123289271,
                18299352213359180863,
                10684671941660264254,
                1330832509288203947,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16954759133281662264,
                18273173689868770537,
                15547894975993101979,
                3417403224340468124,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13912470163683252757,
                17382382368072636582,
                6903293014691060499,
                686719320416375372,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16940859723946566730,
                7750444201158853135,
                13740731192033776240,
                2351434596611888756,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13649086015585712653,
                2728794397538123566,
                6954352832529225602,
                316981043182118938,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13074385429320387422,
                14619452600264629395,
                5502304177255937279,
                1065222210408434509,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2783209612023331880,
                3095803601190932117,
                13996163339384083932,
                1705044249993818907,
            ])),
            F::from(ark_ff::BigInteger256::new([
                289293896098377667,
                15429902405062121097,
                13908108838471592962,
                3428147788075205638,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14437450928289936610,
                4455515938556810152,
                10199054257901100366,
                3010104872396794891,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1984800807313219797,
                16506337062254361231,
                4156457232925780269,
                2174742591850703552,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17618680955289686862,
                2111999560654753954,
                7708634848373819875,
                2071883544978226101,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8971188083835816037,
                15070092247885733956,
                5147440465301996001,
                285403792462882230,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5905255224377718019,
                6596806830009180069,
                5724887013531749332,
                2760515480021137072,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14367556630219798283,
                13904853626676530173,
                18195228092311507634,
                1697911585302352682,
            ])),
            F::from(ark_ff::BigInteger256::new([
                500421037076267445,
                128637893978014872,
                2999885859978442821,
                218875874169279411,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12642042561927122917,
                16486178274058824439,
                4951335866534819138,
                1586580590559102767,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10365974799445515111,
                16707414502899764260,
                7486042909045798204,
                1220112222479061906,
            ])),
            F::from(ark_ff::BigInteger256::new([
                18209102255223293925,
                6139802999438037346,
                10164953930572474343,
                1363105040671206639,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6518356325714109360,
                9055934033131053835,
                8208361424058256275,
                889707519064705978,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7651878367792964857,
                17667121176636917194,
                8758076938098410299,
                1951605852691549631,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11650797090302605030,
                5684882498546945293,
                235823497990204897,
                939061891346391042,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15698720631649681748,
                7985482783898016014,
                8692269835459603042,
                739194682988814542,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2092387014417574116,
                17113657284062369857,
                2699609665304564886,
                2263665853339672602,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8031575194210634931,
                12406655467572233913,
                10839608275068662465,
                2046686587925514748,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17446956831545774866,
                15688367211196298735,
                10289360580560148678,
                2343082181563421214,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15528285366807337060,
                15702500753104067487,
                16688075423343131332,
                761010164117305723,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8580887038097391623,
                9532514526603288543,
                893312880546656499,
                2763319002757842750,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10191145428715404616,
                6058671224386986398,
                14918125125132814056,
                2235162442880489783,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13318934264039227462,
                15438168381154362849,
                14521278551552768506,
                490928575581378840,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14685808997270239569,
                2615783508545915965,
                884046397857887259,
                1103302908080296729,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8457488935122243621,
                13832449927962978517,
                12662570723691765899,
                1025446784524112976,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14759462780307699761,
                14320555371777904444,
                11868413213815087691,
                2520744755955916363,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7703422394001180445,
                6397953920840613188,
                16656365290598181981,
                1817024161422634022,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10616043955978883918,
                1673776512593557169,
                5506689080390751890,
                1283689877294922194,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5425676872736768680,
                8613305785695034309,
                11572505885085663012,
                2134854841308755437,
            ])),
            F::from(ark_ff::BigInteger256::new([
                18011105194954945398,
                1257642958935144126,
                5477058415955104460,
                1906328530918708929,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9225142885808008387,
                14209927661509871952,
                12309639487827728680,
                2457615589870706123,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17482628749141486555,
                8755162813942719093,
                18162408185402137919,
                1001252599476797718,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14438792011780264636,
                17698291751370131287,
                9307153994700739701,
                2157281498775668901,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11988507673941768132,
                7932670140067359007,
                1595139117199019178,
                118965809037825994,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9621020558564505901,
                9477888585389566765,
                10781822932238262850,
                2791750974592492227,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12668160742798260530,
                16092030604649129696,
                11111114394143178695,
                140112768846556762,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11386025362751550047,
                1212611069467186262,
                4760005119499902088,
                529263863633285539,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4214485444279209496,
                10055043642469021098,
                41405060786146519,
                3344426391216360083,
            ])),
            F::from(ark_ff::BigInteger256::new([
                431616080067627328,
                10719094117797767285,
                12807805464642341801,
                2436966472020668590,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9632137979349018995,
                10930471503204391975,
                6367493514769417280,
                1262817192438462718,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11198040870413698814,
                18249649852860724270,
                13034722786857540217,
                1795599043158161921,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9884041871564554670,
                12940854658026539673,
                3107055592703639112,
                2915202672310636779,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12290498456267507864,
                3177488421097070925,
                8439627335302449100,
                302133587956078767,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5144691426661523221,
                3727920625024847287,
                3552683226954238656,
                2053550674325371238,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11784053326304482168,
                5099360224099613973,
                7568082401442831757,
                8074427007132761,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8114064112526671874,
                12156507599033617534,
                15091109975647651756,
                403979986570750779,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2874362516722088613,
                8958619311795075012,
                2208711769033597977,
                3384434317116392423,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10485601699839528312,
                17793176874639236649,
                4038582459685884776,
                568125304074650596,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11890357780550792471,
                18267324196293504843,
                16697634171084013753,
                1104771145294825590,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11049416138699307747,
                15998305278256410236,
                10784381344942799102,
                1707315113693379595,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1239528654565021659,
                7030116039967412181,
                5282251228756715592,
                2554189359011938575,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15793290561559770025,
                5585635653035115260,
                17449418647469368826,
                612313803918210940,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2760689414810367277,
                8291892071270984816,
                14840400644309132800,
                3211546157106988021,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12776631098258242510,
                14484094302857573445,
                17202948241545264460,
                628666925664155056,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14917335562347822061,
                15231598377895623569,
                8129915552733675710,
                2396271210277205165,
            ])),
            F::from(ark_ff::BigInteger256::new([
                515785180926058829,
                10462041153209159691,
                2641452603961482941,
                2115434353014544231,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8423673294380830628,
                9728493753587177753,
                12378406186287602081,
                1497824541269642746,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2267441780447558452,
                17188460418028152752,
                15989345254334302447,
                315941605472482750,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17448982546560231451,
                5722701130474110934,
                9777341093023970653,
                2294758859105278211,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12273206391206738941,
                15506106533746421228,
                6552553275495068423,
                360859850371125333,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8040825456157400245,
                7679409573823114040,
                11773121182243679398,
                1205459207139616419,
            ])),
            F::from(ark_ff::BigInteger256::new([
                199964647170694245,
                7676747812699517067,
                1751366223360990407,
                135077546605358836,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6809361214987304125,
                17088240951950267494,
                933456480554565006,
                1905817730933256286,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3719645316459988625,
                15640872473216766787,
                3956299691426268507,
                2768362694557562465,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5475083631906503000,
                6269865010807572446,
                4900714182397275195,
                163411973095671476,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10743244975362384422,
                4433563654463834047,
                7325992652614989552,
                3072121548688413483,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2253941063619375063,
                12358409111428225691,
                4240982266737814314,
                50055603885225530,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15981359269342790863,
                7002038540971958880,
                2509722463834659752,
                2625457285016512512,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5376792042794941344,
                18060636623884485586,
                3975685503171810467,
                178182100785022136,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1804952626058405352,
                7997375753359665144,
                14380147204043282213,
                3127676443511507976,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5015932972356789076,
                626564941243683184,
                2359616548778927879,
                623160622428394590,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1062769753251486205,
                105092768443308741,
                580082380936936381,
                329265490418998207,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5421640744709756012,
                7915283618462895134,
                1807731003614997581,
                846210609010266207,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8434369352407599120,
                6336407327135449839,
                1693096303921423660,
                265824241127925898,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10882395081012833005,
                7189738399124025992,
                7246275184733238980,
                1293196673480400887,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15047489707191571630,
                766474947572293926,
                11329071988871478446,
                3392608347170606326,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16798283173851477675,
                5020066564471314472,
                15093671440284244686,
                2770129325835463204,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3878863259404210819,
                7521141655939557515,
                2585209981697507984,
                204738590471182583,
            ])),
            F::from(ark_ff::BigInteger256::new([
                799396805893396417,
                5086101223869021491,
                7595270690615334035,
                298715401708262535,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1540525995074807127,
                4957004482682426481,
                10907175162316986674,
                59950689070306086,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16651204772587276646,
                15252007719355179235,
                4888339255020683900,
                2855903592041122038,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8463514264417677007,
                2093599897852391197,
                4943230212254224683,
                2053242704609557985,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2126545638529390934,
                11897432474795056196,
                14182560997415062784,
                364364735927265729,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6956530850150486987,
                8168470133036169916,
                8092230848340423317,
                1926250426667043136,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1844715589455478948,
                9118471246652610130,
                17627605653279442927,
                456587841755504504,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7489388837545569461,
                789649215528439091,
                16946707153904501181,
                1885435699159566122,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15633832649315617062,
                13780823152356288598,
                7407535108190393755,
                1556383084583810556,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14469982814540413474,
                6564990245899570613,
                12614632876498183263,
                1059839032637327174,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11471459007232252004,
                17807240548355108533,
                12533749058682685857,
                1864955309598432945,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12921189377820400970,
                12461819463134578110,
                17626874892653558900,
                2310488800512681713,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1725623834094632731,
                5783761648490859082,
                8945257079499659370,
                2360892637149943797,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11575679268194314548,
                1043761757196903680,
                8125676764964912037,
                1701498943157348767,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15710908396377050497,
                5579557277542960835,
                4809809235869326301,
                1408702463170630697,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8257313422990114254,
                3004433504964098003,
                3066007962005595246,
                1040406602569038278,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8557431771510241633,
                5853573862528420035,
                14633621467284039033,
                1453273951391497972,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12571517897470934960,
                17213068176167919903,
                15181498744093729027,
                1318583136838399483,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2604973925545768006,
                6159874461286230868,
                2783371478439112111,
                1459129380623867531,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17038022523765221676,
                129267632921144664,
                3885752255396421309,
                2623713904837237348,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5044396365081854194,
                16403857158896604275,
                9845351959262484988,
                1623011406285762654,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12323439742899512382,
                3263577542617216072,
                8104868419007801394,
                1989437717702124638,
            ])),
            F::from(ark_ff::BigInteger256::new([
                18228587118726953864,
                11162010593800460536,
                5320439150635514583,
                1135690928859518560,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5039766532331449416,
                14908920726975367654,
                10263896299420469281,
                896031827524206798,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1495389026814976832,
                5880746296862505525,
                2460838898153310352,
                2618295948457013386,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14286223883643035719,
                2103544815793329491,
                2044430404037653492,
                964316096618726435,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2089734381670056158,
                8997031550382806219,
                10943237225894975642,
                1936448330993517540,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7103589841865172250,
                6796097999212345144,
                13446240818132813015,
                719488009504823669,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17700117417968749048,
                13589015748648428166,
                3876729744492732645,
                2018104731184446656,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14157479710517028777,
                2381757118591647798,
                8394774191170370739,
                3313466564080310886,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9635556580553047369,
                11153877654366788585,
                10165564898984635081,
                2862395815971633094,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15845805449766666121,
                9195902496564435727,
                4038686483787930145,
                2843304904605892870,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9152204943597704609,
                9008384664019557613,
                1075913242384141367,
                590483616697964060,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3649436059438364324,
                11378283305196075661,
                1960852795725257662,
                1266736120182234250,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16741978750059630484,
                15156084890269821149,
                17440682036186474063,
                2875628196002191017,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12631163823656990013,
                16935015367734712394,
                8813761416191260020,
                1223691088073266013,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11622558814555604346,
                1356519105840927188,
                4205552584007550911,
                2138427402389710538,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17057363237467153455,
                5248184750830510414,
                5442031053605108221,
                2610310776927984726,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2841947515545414221,
                6032617726342164488,
                2697759964821576225,
                3164412436917619671,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9103804937444885980,
                11205945312553997231,
                13853043939170979453,
                2965359687967257755,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9947072838215415088,
                8082213224525851444,
                5576007229320371053,
                2601989281238816653,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3238136684065025536,
                3447021349121722775,
                14487947321253748340,
                2418874422959708276,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17285635237181422419,
                9380110307281375382,
                4765282646149435379,
                3476067997842833049,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8787128915678898243,
                13465110958522751003,
                17035984793366730906,
                2238060471843482443,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9881322148429094522,
                7775931734944883288,
                11525207562790681340,
                869854550714366486,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14829171229626466428,
                8924331625194005822,
                7216621506277409998,
                1244574456378397362,
            ])),
            F::from(ark_ff::BigInteger256::new([
                186371153141365274,
                11663623964983114218,
                6046001344309736018,
                1093027895385392300,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3388994516802227517,
                16957492317901971254,
                13103523606809674896,
                2445396213377861238,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2620457162374518861,
                6601636829829996800,
                7098992745636012076,
                2169423058063736002,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2055304909912861862,
                13340065741782076024,
                756991081760963935,
                1191129864590572786,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11198638398944154385,
                6490417836042600255,
                18165468041447157594,
                2362035479423173232,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7060224227397578878,
                6550774422032093666,
                17649778515927618387,
                3263006303982296151,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2403802821214490460,
                16457281479908745367,
                8709484321801517806,
                3307570761450254801,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4272388462370912932,
                16264861490783999393,
                15526624425280845349,
                61083680897261148,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13264708457767614211,
                5812799891901677153,
                16814556062337299897,
                1449086523672439759,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4009196664802848296,
                9490499852380326507,
                4624370296926765826,
                467303718134365321,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1106054667971984903,
                13985196426801267819,
                15135368855526722261,
                1481466562117725127,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3315250373071521103,
                4611623002931349338,
                16275129789349207525,
                2148777088338130510,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13530490748138155764,
                11105102511259092107,
                3103361131903973007,
                1565862367349847936,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9790732158413752107,
                7180953806357322413,
                12007427266996233713,
                2757045154356375418,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15272629773671941034,
                18272572766652238769,
                14355233570669370323,
                2722187084916963848,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1107291121742743163,
                5714239268143590609,
                18183405353650362522,
                2064197344033289498,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1878561315496458887,
                9112427282449665828,
                6509647151686188765,
                2079166186547673144,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9786023346293000367,
                3627262962433089977,
                3256311209782995328,
                2691752712120564093,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11414280261791887302,
                3414249966158464167,
                14138485048685422897,
                2068277170945632325,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6092559347295479070,
                6072364960617148867,
                3809389637867452577,
                322625451450345383,
            ])),
            F::from(ark_ff::BigInteger256::new([
                18222488666961104504,
                746107221614258992,
                2159642506726402398,
                690653899980869872,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4746496969103039640,
                16588175367301087726,
                4651989705006311389,
                2013512439056419146,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3685741603901624863,
                11657206903729328773,
                14823130032225847305,
                1373732433222123626,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12017635688764922681,
                4533971877861970224,
                7940057781257373687,
                3266847784543725791,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9887258116888868137,
                2519979745703003714,
                13646116723337403840,
                3385883067292454959,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14005210833736769890,
                12107009215901204150,
                2168687205637998568,
                1225147088081593121,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12359243120370084563,
                14799012029030947974,
                13090633693581827582,
                2353646619811561853,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1964606715316877494,
                6434491782407218937,
                7086938741228526088,
                327780170331671813,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16251449791459749152,
                7693109677155356231,
                4293341609883081618,
                1838673300338161226,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14735831168962644216,
                375405510987177946,
                11920892444252528050,
                2350013135307229096,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14836901068521378568,
                13742714264859286600,
                13121572796972362505,
                3290344151728068023,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16652685000784146265,
                12688076404069773045,
                6134870970863461795,
                3472082816105161085,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4230741053330696254,
                17624925599680639509,
                8660795879711835471,
                1965505135930426412,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2563023601653298420,
                1886226661932626103,
                2792123902165626928,
                1517781491253295862,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2196638578969865938,
                6325901502568324680,
                4607871669381362652,
                1541610961313295419,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6692431251210435678,
                934505472902455496,
                7921358270712802728,
                2185476415646729636,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2218657095104282054,
                16505255850516237993,
                13179974473172583539,
                616100379605958600,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17210954442341006533,
                19107046061756523,
                7482304695394388540,
                628503041246761028,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3545420197871028294,
                10175810706084443331,
                8422540760641808262,
                1509185236051702202,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8819942368920666506,
                2355678648448716726,
                4825758938965227538,
                1592516066487316177,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6712989248582687013,
                11882979717586821655,
                7799926593352380874,
                1222200256461532312,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17913140959692290358,
                16148742816791789166,
                16899465059288702916,
                3400063733705930476,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17288198527681614863,
                5829515822068368312,
                4626913585133624451,
                1318122301427495541,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1713620665743242617,
                6780404582845872449,
                14552017219896668127,
                2574837419171743884,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6173644837491707187,
                10303751405136114793,
                13151604550492712925,
                2135349389777829974,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3718640395707410032,
                12009419948740049300,
                14992270561426406640,
                3447544294312069478,
            ])),
            F::from(ark_ff::BigInteger256::new([
                429755288162851775,
                8322087119690137389,
                5667196702754170115,
                2087475949015424982,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13849983059529422768,
                7318624078540186264,
                18034570199541885589,
                517916736053721216,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10326193550727319821,
                7248649092727167106,
                7136322739529147536,
                3093473375258526023,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15896122584900190396,
                7618193885780678452,
                3030858985236255569,
                2551932397972391710,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16058385416833170123,
                8769758352206618289,
                1061454989244599516,
                2779140385915626880,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15913684392884189354,
                8079906723834108263,
                18294071739483589237,
                2410122658444039819,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3574869603773914642,
                2646141408344922288,
                11784510626013803534,
                2271118671121758576,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4280621433310003813,
                10757457767735778902,
                1519228374103808505,
                1452970094562284415,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16834689807127656355,
                5774510605505117185,
                9259253075909438776,
                2680315665976547731,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5684292735106318115,
                226634946924564647,
                6166446281518023658,
                2240890323232483068,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1605906498049669530,
                16964315273229336485,
                930424501330638757,
                799281868094937152,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4520032187304611862,
                576959968191003472,
                4544283502153583241,
                996368447309530692,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3764640115039238350,
                18085265327616963752,
                461686623194690707,
                1289423942402018805,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7035962186364559084,
                8703687053085890452,
                10947367521313362147,
                570093752609559834,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14839474613733441987,
                830571112667439348,
                6386276262928245052,
                644666390878533563,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5490260741036810818,
                6318021056515336857,
                14797663425930475804,
                1186319743128927518,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16241410239983919179,
                14992072506612526996,
                4264532595949667689,
                1970149378927733646,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13601637126161964127,
                546827109306068354,
                10824805350192675105,
                1630583203859101013,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1556209918890510764,
                4882198675434686023,
                6970175514215308779,
                3434339651401504913,
            ])),
            F::from(ark_ff::BigInteger256::new([
                18167772776279562526,
                9092302813228135540,
                14859234182096832618,
                3453217096838936504,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17779701241779635166,
                5999908837008047656,
                3277564115643436609,
                1013689137017617907,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6161133329164992228,
                9987800167767094579,
                12054989969609057844,
                1013742282780064578,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2115792111678018256,
                16031615325508813015,
                13119205608473918259,
                1135030369850340101,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5567375917748477892,
                13332984829667704064,
                8328798906304779056,
                1147937036323739658,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8194221596014993713,
                6921234589022739145,
                5208417394104231233,
                1470217420493792412,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13873632842335874007,
                14443471296047474681,
                10607608484098633936,
                2806100855407984843,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13851577623551004601,
                4409871093845832375,
                2174317026204583023,
                1022451518373959224,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7120969075357470300,
                15917089995975725953,
                11665231856616406687,
                249132720326526921,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12721723267109781687,
                5276084066760814989,
                10301391462882797369,
                369511733469083066,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7255657130903771695,
                12986570461361758054,
                4134965321058633835,
                1900203009247136892,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1707328586312278804,
                17345159236382927822,
                10903950248393290443,
                1839198609858908087,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9446586622215329134,
                5436976317071900105,
                11148168823358641067,
                254027257319761923,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5909733165479109627,
                7329785044981310128,
                5640024084088884511,
                492408607649863543,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3329142138483098123,
                17896713730835665052,
                10603907349607116095,
                1186274461975569536,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3044981523111830322,
                11199564503194505274,
                2341347985391722211,
                1159421157887786312,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1606040013762900943,
                16872424956023094860,
                8733311871361291202,
                1074080809487586340,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16026269644307420733,
                6074423795613479295,
                14760366574981565946,
                2810108480959297359,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5103908900335244823,
                1132726524029765697,
                15406021326436967432,
                569857565670775880,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11652685602852223069,
                12511714850711184891,
                7365262321964121657,
                1340819700569967569,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3658049251238213258,
                5320373850090280048,
                16889404381687265552,
                970894092261450846,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7770993797971184135,
                17796265609738393093,
                5993015252056205892,
                2747957710894952998,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10704671869037283878,
                4561130443534957008,
                11961069877631596029,
                2694010802100671463,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7669013873649851226,
                10223915813488451995,
                14842228319989235558,
                2954994420635439144,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8972326816662074477,
                13455406329646855945,
                701049678000473454,
                3172054962409979554,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10373516181125079071,
                15471758081928867557,
                15092497262132245015,
                2256334202888424110,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2123303505118935949,
                3610622440811258071,
                15848238442287033968,
                1853803384465349205,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8050294847701582381,
                15746279349339642145,
                8133867183626829968,
                1852418913317957954,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7105461339935643189,
                8660311065577242132,
                1809742064255042611,
                1393131800646379089,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3913989683591058215,
                8865761774013153707,
                17557072237749427236,
                2469695357551139179,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6540310869754830730,
                12427657709140469416,
                10906101035081888141,
                1011554654114613107,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5236249229448775550,
                12859613973934190153,
                7551745072248547216,
                597492567541557771,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12663041568401047401,
                7763602800843700893,
                13931106752194659445,
                1457862156339151179,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6606479803950695714,
                8466960965921605651,
                13692871985826395250,
                3070220067322693409,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17605356296006240215,
                1773149446805998059,
                5437931929438154283,
                768777049170235398,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11932500730746685011,
                2489039454498111228,
                6285769468241546649,
                1609754992577837764,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17580478571938954461,
                13211503884621376383,
                3966848978979085406,
                92933620365832930,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13917801259601324011,
                7996719109669157725,
                14760356699452510875,
                1530260555709815815,
            ])),
            F::from(ark_ff::BigInteger256::new([
                148219147315646293,
                10051840283551297400,
                16546025814517632939,
                1447634316898698136,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9933502759224981881,
                17521119924549620101,
                9474943938236258211,
                149605211191431562,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12880190657533531263,
                8739429681530005283,
                7888650611426086258,
                2115865821889868145,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13886050162693954176,
                4477900786860712108,
                15385899800165966542,
                1400612808554469004,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15997967743551280228,
                10310654727974833172,
                9626010695272139848,
                3403230662041324411,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1791295077453916729,
                3538428323691136903,
                14519148677952751559,
                3387677226053183309,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3854837812905780653,
                18089374278081862436,
                2947054540991592477,
                350709625841085030,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13932946461494683195,
                4911148716623585189,
                14323346769114134703,
                583305893521072062,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7105212386034753205,
                15918415100306008040,
                17075941779708593768,
                1248853525640086980,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11168163837182462720,
                1588705996073055106,
                12581457074808550600,
                217344365094117700,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11903880734033379892,
                10082755681868747943,
                9254464324302578201,
                1397534636143934894,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8067976093129060963,
                7557628763171476699,
                10171391080161929420,
                562089708172547249,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5836347179014831410,
                12385604578965660778,
                5736832808917781763,
                1956892576751490155,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2732823438041790271,
                13741601975734549840,
                3262731116171114554,
                2042440131297500134,
            ])),
            F::from(ark_ff::BigInteger256::new([
                18391408142483843758,
                2418803064216561318,
                18417262940816918055,
                2074189297746532714,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9897313736444237682,
                13308216632931670050,
                5146403635313568882,
                1430347973349670612,
            ])),
            F::from(ark_ff::BigInteger256::new([
                433939093651025320,
                14165091444967624457,
                7058502351169568250,
                2425048399292116942,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10130386507322038406,
                9730604511502073227,
                6600780753739423059,
                981160797248298864,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7005380063294784617,
                1455061686945518334,
                6810580135143161647,
                544718894502166405,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3082864372081062203,
                5976888632119591107,
                18391090339848583152,
                445675319366820899,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17106647327075515515,
                1481454024819864129,
                7645385397277111769,
                2713633444312350173,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12696838798454011244,
                697059278880371719,
                11596055197303531733,
                302133885432054579,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5368807469711198144,
                11191489739806336802,
                4189818195124935700,
                2455089427350849066,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1528016004302732095,
                12338701991166231707,
                13676443366104818753,
                3117391748246812865,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3304211590401822160,
                8693811854478416222,
                15495474388204768961,
                3245419440163036255,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12617439625148876467,
                12642418933709480438,
                13213742360034982967,
                2998753011785747150,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5726673009252352248,
                10285482515419588005,
                9261185158844885131,
                901071141601394469,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12211740205047064888,
                10984779587318567245,
                10780251804252324397,
                1022993565929346730,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7195889718027731849,
                10505869930405023524,
                15750547725403251945,
                1918143955887908626,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11342572370317907010,
                11387849081219383947,
                4878813802317365815,
                1556810808123569759,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15972508950449686379,
                16011990395421909588,
                1733783414269965816,
                2628327646544429560,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14202557316870784989,
                16516731057595078916,
                6850440706107617361,
                445727931494939438,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15983218690942753365,
                8119060920933233724,
                10240569746351524466,
                2492245429079629036,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4913341369439560628,
                14972420694659352438,
                14300002895247946136,
                307387936634339142,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11571529568204595080,
                6260300971080690085,
                4523650342561279546,
                3158959095961583645,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16048191753704917307,
                4388632687492121046,
                14647848671308286023,
                374309865291919058,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9890668558008192732,
                790011793992751506,
                5861289109593733365,
                1590267153927097899,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7342264948401996975,
                11104123314361068353,
                2627788400945158653,
                1478875012903284564,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7070528262135123458,
                4308531784888309755,
                411265053824823712,
                1151516239748822043,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14546103775631581593,
                6445246442816384689,
                7319467220167634948,
                1490520497558643937,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4683007972084640920,
                10310114722270533014,
                15642486938527023441,
                80548211561548656,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11376321509400241396,
                16515670760626850183,
                17583512002019677191,
                2288670692400087639,
            ])),
            F::from(ark_ff::BigInteger256::new([
                18134389172652843125,
                8949069884696365493,
                15517133613792887448,
                332854194173696292,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10556759552213821121,
                11989482388219141108,
                11406586164472544228,
                111328600104066012,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8482517958682250013,
                9750800572152486458,
                3386268701444095957,
                2755213297203831777,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11221297270752647927,
                12748189758166845804,
                10991826215636998805,
                914930916666269039,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5007830062743224359,
                7221826426675850252,
                12995004000213884759,
                244118184002643771,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7167716697226504779,
                8918071975382894392,
                9229943949662597624,
                2458026014694576504,
            ])),
            F::from(ark_ff::BigInteger256::new([
                771897354090110714,
                2816275350953473554,
                37994563705939797,
                636657707496117836,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6090278849999283552,
                6305415800696991394,
                5737021219904061393,
                2207379004441882691,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12273584864700537383,
                17468067771234812652,
                11144938667938868238,
                1595850391677703095,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13078344499077758173,
                12163837801771969666,
                12691377691523628037,
                2693540927818033010,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6468192827557644036,
                8233192398247245607,
                10764650681598968655,
                1194669958947732477,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3125815738319337605,
                17561312487146574005,
                11048649656570828657,
                3020203830476157896,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15379673120928697848,
                12831212594059263172,
                16027792360954875200,
                1687481429855480163,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3246753492829020355,
                15763556268742685622,
                7131946343616921118,
                1630360062585604528,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7318145362773654466,
                7084301623495657799,
                11518804328835198080,
                2868061388739799482,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5280129267377911645,
                9164876876187018920,
                9238008508546067208,
                1121400187615915672,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16340904754457104892,
                13094222270299993864,
                8379509023480637836,
                2332707073708425197,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17983928159688917089,
                12203987793492875302,
                17985048207259580491,
                1993014529659883397,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4530801401606938348,
                6402886094742992317,
                1400482794212869681,
                1742509158820939111,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14432283252885554305,
                482712639194005250,
                7470676601254790177,
                455033472879236793,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3604499013639514197,
                342323805150670024,
                17022638049303803964,
                155443994082178536,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1978486236644889836,
                15605684180441002676,
                4198206343423572129,
                1727959820175294652,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3795277968253822496,
                17695733398306601144,
                3630256330993649688,
                721607267163900122,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4695531163806766087,
                10886771449417299503,
                11840039228046760786,
                512525046931850958,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10913895217832321972,
                3770030661950334370,
                6151927442266275524,
                1292944690970248437,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10599094103006592264,
                12542131616281443320,
                1430586969816149996,
                2710974291533852549,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7794543208916734858,
                17796298115276782566,
                1304338845377675689,
                2774341346828675114,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15413686586183710060,
                4282950946959865681,
                12468819903398334486,
                3452887238831860906,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8265431794405444089,
                6313720691241465735,
                5387215027464943235,
                2772669278500202075,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4092435130272977672,
                8791045127581272540,
                4854418211896585166,
                2726501271898560425,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8342891172119842065,
                1641792333151075246,
                4571807070950757902,
                2167571532713116647,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11303053901539226696,
                12228445827917389496,
                4660195488983424372,
                2022750706895894083,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16076516080189796181,
                15124482018933369217,
                5913482720334837561,
                2852950771344116814,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3463568493525523619,
                16153399299030728421,
                15779317044745383062,
                3418212841541707148,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2793265555998880163,
                1931316348749406945,
                11288579267425521398,
                42459247144223319,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4443542588103170464,
                14145344772469271304,
                4248200838437722853,
                705121969194421778,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2921214226137148697,
                1712313332591251028,
                6495168436606491359,
                1495389009471717127,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15624686375694486388,
                443582740113892155,
                1025100341537690547,
                2799253923529665598,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14637145809652314319,
                4488988844129063240,
                13428659107962089718,
                709862589335909109,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1968045249005411504,
                12223547856440500768,
                13129776061456894886,
                1658666102898463454,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3317008230114070215,
                1779868525391853127,
                909275357343458871,
                218146088950593227,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14253726626927028607,
                2723497237129920254,
                5983464738069683550,
                1864155461063685376,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5521702629940914837,
                1192342653473101716,
                5315233943409539613,
                255687440438973164,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3090741302049649642,
                4686872950633991720,
                445951849718796955,
                925250925484463560,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9170188212931918802,
                7705373954496185614,
                4871003565748059678,
                1052168518047901741,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11246922928163191173,
                16031129498489333813,
                4604294947161728372,
                2106278240448567583,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16833191995032459388,
                11100604248899544594,
                8545287446626289170,
                681273647369613699,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12872535865569536795,
                5395251129824555391,
                12690235627724623889,
                3302795397492673975,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16055196176080587927,
                10543877580999756861,
                3093604625069447790,
                809994217891949900,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1072418327572052802,
                6244681026771208267,
                13776436409204100397,
                331463940359368379,
            ])),
        ];
        let mds = vec![
            vec![
                F::from(ark_ff::BigInteger256::new([
                    6902477072542150086,
                    7221639482827751511,
                    1990569282217595855,
                    3220409607912989869,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    5903681048566164378,
                    16503228923043335145,
                    3722670196623930890,
                    1140896713125015717,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    4980904410419478982,
                    10826467472832450424,
                    900713949051834629,
                    360734860423210599,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    13215156709089814492,
                    6852009750764929101,
                    12875360309885606457,
                    2977655111042830121,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    8708544764889019386,
                    12467995693242677539,
                    16916304857210373407,
                    2545560780413952485,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    3555500913630574269,
                    11953666945571418925,
                    814912162069252867,
                    1069221994900704823,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    14043396070532138115,
                    1352786383366260828,
                    5772463268216227497,
                    2382468501258145045,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    15058715286514775735,
                    2919534455330376988,
                    6011478722315462935,
                    3436864815929611916,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    5747822252281566517,
                    16221434893540440739,
                    8114027232621015249,
                    3470595653539790746,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    17341255238496725178,
                    17003308038716631116,
                    4340580200081325446,
                    469471952589115127,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    11513179155460798841,
                    6773777953776601004,
                    8046974397429544201,
                    2236586949462193735,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    16537902547153213365,
                    1809693199847241091,
                    16469800678846571979,
                    403287177782130911,
                ])),
            ],
            vec![
                F::from(ark_ff::BigInteger256::new([
                    17094928785586417994,
                    2386656095295440226,
                    2735361612540868012,
                    2275168110699125122,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    3227426277963232763,
                    5701141666680075796,
                    5129023789831223808,
                    343406172286331327,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    17244151092475639290,
                    14007168312584407847,
                    9812255071685125020,
                    989704242875634250,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    14213026712522693750,
                    12337047994983381827,
                    10503537055464894754,
                    3439373912929507741,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    2810966539403465297,
                    4305105633724835391,
                    2343314614835874551,
                    832955730076273735,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    15569552930908133890,
                    6435735090518845527,
                    5062466906863067144,
                    183523514204422587,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    9187710459312047594,
                    15369137284799259900,
                    3897700945214145440,
                    2580156042750577871,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    16654167252118053903,
                    18313364753436458327,
                    17869312527477066836,
                    988587104298212182,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    3233271889494286655,
                    7122543309000988188,
                    14959068840002787540,
                    3346243421652719130,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    13587395726199272855,
                    15958778855278590566,
                    2548752810180062559,
                    1608115010857152439,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    4257979473811463986,
                    133289902994550613,
                    8498126331277310237,
                    187086702458984027,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    5352352670830865400,
                    15774617109451185088,
                    10049226577197526580,
                    2323806862580033784,
                ])),
            ],
            vec![
                F::from(ark_ff::BigInteger256::new([
                    11501814366956685725,
                    2934296439440146034,
                    8385398615025772079,
                    112170127408004517,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    3897898536828460850,
                    9297389743752109686,
                    9309096027092965359,
                    1186513693866170871,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    7535062552067618312,
                    6933662358414352665,
                    8458850695939596224,
                    677407033331658539,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    14978327406775093293,
                    16979051249984322473,
                    4334851460277718511,
                    585544952077414118,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    8972043816151650710,
                    17048194747698437559,
                    11980592122189385254,
                    3206224221943193673,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    17537458587665360109,
                    6552460849215122613,
                    11731382243046920317,
                    1459204658566790946,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    14861206247180000522,
                    10294103532347852435,
                    9347257539476397603,
                    1658454772184026950,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    13317294048738838042,
                    8218732099048582202,
                    9580904531247530219,
                    2997102853037475436,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    7551591306047180305,
                    2818535117390242238,
                    3345156963891813924,
                    2396613443339476766,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    9106649196560987005,
                    10695337432157678882,
                    12172100952367594550,
                    2646891177443820865,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    10427800360303069716,
                    2899600711927662304,
                    2140205576751010768,
                    143370194764622314,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    12448414416790807716,
                    17045734089810171598,
                    5835710031290712740,
                    1330325702473663789,
                ])),
            ],
            vec![
                F::from(ark_ff::BigInteger256::new([
                    7295610461349448455,
                    16396145670715962418,
                    463847759633801746,
                    1733483767965486052,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    16680095079942410241,
                    8388892995092911268,
                    17114004468671733466,
                    3084184969240298537,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    5211220068355749425,
                    8071129789367163086,
                    9645106182690674147,
                    1707293213730414064,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    6583520209347427950,
                    336598246028600744,
                    15224190682371178907,
                    190323000953655092,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    5946675841666569074,
                    16872735985270792237,
                    17263601346378515274,
                    1836900630365211050,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    7061769801748076414,
                    14009841441184331927,
                    3654622180316491181,
                    1140463726960152884,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    17321486515052374825,
                    8091135085966359986,
                    15932592976399888126,
                    3006715565356270893,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    5161041980507120791,
                    13385080690453891949,
                    199760733069306900,
                    732398362837937141,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    17748650576550995568,
                    16659346234129592261,
                    15841951448404545760,
                    2556529470064872718,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    7700571948264664107,
                    9186813375559840334,
                    15459903007505284208,
                    30738623011941424,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    8794580278383247557,
                    16013526996351388218,
                    346993009863965808,
                    768912918898078470,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    1881867472800955058,
                    8954728817961453226,
                    18021774068630538011,
                    1652425282753650212,
                ])),
            ],
            vec![
                F::from(ark_ff::BigInteger256::new([
                    13985741635216627132,
                    12407771987333138946,
                    16695206735928261022,
                    3421899082157330832,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    12845439684889658790,
                    4597100175228550818,
                    17271005301649702230,
                    2988291970009693865,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    8426266061661586402,
                    6226833420080241928,
                    13227131203317082702,
                    1354500770740590379,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    1000990456685393204,
                    763124022466411999,
                    3119605009821028052,
                    3383756670960065470,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    3012212536588859183,
                    17986764545341249944,
                    8240973039265073900,
                    430558483470666258,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    15364441646349948965,
                    13255166992398859193,
                    9610915713725212386,
                    2988650375942215576,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    9676139982800953280,
                    16167332820973923478,
                    18156317151746915660,
                    1529131524954919568,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    13994095476444732539,
                    16454656444137685697,
                    5653899344884076879,
                    1985100831438215935,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    13342781370477046359,
                    14289735283453433354,
                    8129313632367281859,
                    3278943630246554585,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    5897390281593905646,
                    17396209291675674079,
                    16539039921026772993,
                    1194096050252399849,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    13725558713176799851,
                    3951176265141491363,
                    12038768711153029312,
                    2524576079774492410,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    1939269374347900653,
                    17831738007882847769,
                    17969681281014388062,
                    2141781274720337787,
                ])),
            ],
            vec![
                F::from(ark_ff::BigInteger256::new([
                    13849847072448499313,
                    5354718224414138108,
                    810453635710387742,
                    155443288166824263,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    5719070643301396179,
                    2964196467928967994,
                    1185989159056251486,
                    480625612805244682,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    16831429063877779543,
                    16045516160534579729,
                    497418556463541693,
                    3205810368881175342,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    4174763539291661655,
                    13772387214189457094,
                    12268630322308827937,
                    2946587864735233724,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    6102063797165771622,
                    1970979040216477111,
                    7465542722701728570,
                    71509966124464704,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    11951003166516425973,
                    15893026937288872483,
                    16556797592513699090,
                    2498937270440531777,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    5812943746690259886,
                    11728555133495353803,
                    9975221622622977926,
                    1178829453428573373,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    14817351059071272343,
                    9560807481432919376,
                    6947693421686132853,
                    3051956461235576498,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    9433148624108861398,
                    1450882287560189542,
                    7016886555397825411,
                    2568092189185760438,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    16598751469031836098,
                    10086924647981398521,
                    15768993193540791619,
                    2766159384598225159,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    13480848767432323532,
                    16046435072110351751,
                    7669627237682079104,
                    3148736268903361963,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    14351973656199165922,
                    3339760524052420922,
                    3883647249572437802,
                    679432466576619150,
                ])),
            ],
            vec![
                F::from(ark_ff::BigInteger256::new([
                    16774155330277413846,
                    13847086973599401416,
                    11065548766046290345,
                    2447159919043633808,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    13431738069856618795,
                    4270924579821422935,
                    4942390607758415338,
                    2141895200565172830,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    6451499883225608540,
                    12693697503673730824,
                    390075300876671544,
                    3400358507942535425,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    3971580952058084411,
                    906828161474256815,
                    11606856000522090163,
                    2175997158064755315,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    14360351235744253538,
                    3360657259037310742,
                    2920870809073623744,
                    1951504798067303064,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    15274429193106645147,
                    16568971909446590073,
                    16744715055323613080,
                    370543010004020768,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    15190319687126418628,
                    12579656232013150270,
                    2649301750975996912,
                    490320636224842884,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    8053783960318952533,
                    17460163299269935623,
                    6108596158814703594,
                    3430156163900268084,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    9053035429916825593,
                    9966512860411242383,
                    11138443643734401738,
                    121986818457962145,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    12751052833030741237,
                    18342682449283384686,
                    4900159279849472647,
                    2936158427931866449,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    13723567241228161596,
                    10604177156852571577,
                    11461173017540359603,
                    2416493384915712984,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    6460745423630251649,
                    6660166962566632507,
                    5665639915738439778,
                    2486805489959404893,
                ])),
            ],
            vec![
                F::from(ark_ff::BigInteger256::new([
                    9973034688361616533,
                    16167110057518197024,
                    2659864786562795086,
                    2080243757137049668,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    6431940367449089025,
                    10037987027953163493,
                    12286711225226702147,
                    2039530331420839194,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    3259824057540573249,
                    2843413991021467981,
                    6222864817603235503,
                    1255455245317499620,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    7154978998772743390,
                    6266639300731930013,
                    10612652429673185433,
                    1765977734066807313,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    2591655914430026060,
                    3757304355784780187,
                    13458189432987549687,
                    902018481755121388,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    4373219330607527759,
                    5163498430844324653,
                    535372625161056620,
                    3125679372859789105,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    3531375930666877211,
                    17516029403471459293,
                    2287861388778179181,
                    222397648217419841,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    10509307930072331514,
                    16501969542560200751,
                    4154491500790988462,
                    451361115354449472,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    6371511889464986336,
                    4528538897593747386,
                    7090504553758566471,
                    1479867522487487676,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    1224204763950395348,
                    9495269310608116016,
                    7434594328374228741,
                    40778463965586753,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    9637800329899181560,
                    17047733512460316227,
                    5095235958438700906,
                    256155621215673551,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    11135135997900173556,
                    16553062736144287065,
                    12486062585837657023,
                    2896690927836497525,
                ])),
            ],
            vec![
                F::from(ark_ff::BigInteger256::new([
                    15654628036816988385,
                    6374851807014492473,
                    1797122716394711315,
                    1263862575911752696,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    7788182797446126574,
                    12401039346283122728,
                    17653536864347165039,
                    3241609254428858143,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    17412719078831418722,
                    15874421145417255320,
                    17076588307972608246,
                    3457862656590998447,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    1083248903760531723,
                    13634661533785323886,
                    14898173371637738179,
                    699957942222268272,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    9992278889899395183,
                    3396715107535199494,
                    3708557895719582735,
                    3188608328231378950,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    12419912503907924644,
                    4087802026517750681,
                    10643119766260798721,
                    295989193326007145,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    666672086949675636,
                    8507211221373316764,
                    11859485680580926785,
                    2720303591400331303,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    4915087249587366254,
                    13385097202066066625,
                    15116620926381254972,
                    1250471853176376437,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    901359597531054020,
                    2214918763123538256,
                    10085435686840006462,
                    3427155141390050677,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    3662850465180983895,
                    4989158777455818311,
                    3166886711512501765,
                    2999764334147577029,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    14726448358224047076,
                    822969261121198913,
                    3422078348895281234,
                    3209338606203772303,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    16139856908249571807,
                    645033808966096673,
                    568952406016809807,
                    1483702965819540347,
                ])),
            ],
            vec![
                F::from(ark_ff::BigInteger256::new([
                    9981673906981175908,
                    8616833260792358375,
                    6129339867864005808,
                    1604802851565385637,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    9273673316364956286,
                    3689436948438266677,
                    10275734679344895750,
                    1780218689184307592,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    1382879105951414990,
                    942203146203612659,
                    10720606080894597430,
                    2307886224252788257,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    12076568550948233461,
                    11742164542033270799,
                    15721492085791717390,
                    307901112272885511,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    15420826402181723291,
                    5081060504027218498,
                    2991323794128380773,
                    1686603472595082549,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    14192134887324678940,
                    10332623825352476735,
                    944065125768037247,
                    2717612718998667153,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    3809278703119524056,
                    6987166948326592502,
                    17285844270209108664,
                    3432498802129449463,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    16121498736926706419,
                    548300714231056568,
                    934154162032620612,
                    905176164056827161,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    12582598914591261286,
                    8795959041491561631,
                    2773771287040033770,
                    1051923958250483197,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    10963627990725909456,
                    6459772822612726086,
                    7552934864069536827,
                    171312679765767421,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    9616784996481499965,
                    2373643567723148397,
                    1574738703719904269,
                    1575230209634082144,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    17018284274703956118,
                    10313498431536534402,
                    6760494817214363952,
                    490955814547412887,
                ])),
            ],
            vec![
                F::from(ark_ff::BigInteger256::new([
                    1697249066067332098,
                    12724371661611917589,
                    235638315281331422,
                    1769900373214040743,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    10235924633928133298,
                    5370161150952486776,
                    3605447678694620400,
                    806159613612009427,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    882113019686136378,
                    10164688351988652668,
                    15702633345320538601,
                    3194132300439571591,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    8793248213005205621,
                    6404604842727856560,
                    8533805999352870076,
                    1870418911566375286,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    8824435667394842826,
                    9123691217427733320,
                    13535023643561340496,
                    3411570899766443899,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    14501740111274903910,
                    8078923727706327435,
                    14743605577187262287,
                    3174444313082724010,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    2192662395969884631,
                    14821489030564881466,
                    6643769071659590000,
                    929297359818329140,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    2695102807322142483,
                    3752655954207191323,
                    16111132497977210052,
                    3236567285848856870,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    9565538190246981229,
                    5441816193168062617,
                    8340898076953189644,
                    3205125279631296351,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    1663232585350733584,
                    2965978511339521828,
                    15005120147427202576,
                    2532025582315057197,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    8496069071044746883,
                    1314586518472352272,
                    213005073995133173,
                    2046692460403522784,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    11718017715168897767,
                    17991019177575236867,
                    11782134955832484873,
                    2749375455072947405,
                ])),
            ],
            vec![
                F::from(ark_ff::BigInteger256::new([
                    8655084581764550317,
                    17668294088865039121,
                    6874862199125251819,
                    593774946046162082,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    3925382554421119336,
                    10189069876047665953,
                    598154154688080682,
                    378776720294168162,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    18076197410638570885,
                    11550401446844228544,
                    5761753762452926077,
                    3238872714159207987,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    1387107222522254307,
                    7654495700332178707,
                    13011531586797915713,
                    488041943505226287,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    14729022787398362011,
                    14927495992335576630,
                    13199484292254356348,
                    1883415576403709038,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    48636178699461173,
                    9464383655303594068,
                    11419345738588576345,
                    429766313226899203,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    11325746896855201662,
                    13503703224335197192,
                    17870097717764637918,
                    3101254544699626636,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    1978389362448623780,
                    2459939631252642825,
                    8428370379301623382,
                    160419679122719313,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    15917441397793917909,
                    4387228210398769320,
                    11686776670529230729,
                    2974006904852132579,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    7519891633130323184,
                    9836361601639311447,
                    13578454172303307275,
                    59600584089713905,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    16969529632095493565,
                    6608439798319114777,
                    10860082183743688926,
                    1290206845248631553,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    12429798292996189463,
                    4288033146968506498,
                    5120463726514091723,
                    899767444846305209,
                ])),
            ],
        ];
        return Ok(crate::PoseidonParameters::new(
            ark,
            mds,
            FULL_ROUNDS,
            PARTIAL_ROUNDS[10],
            t.into(),
            ALPHA,
        ));
    } else if 13 == t {
        let ark = vec![
            F::from(ark_ff::BigInteger256::new([
                2946226504836579341,
                17266765802170466922,
                11683730724418709128,
                1401683200642732321,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12204292773100606266,
                6694472978730084262,
                107935625385032394,
                3024725539494139958,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7070821344719996200,
                17184088172072909762,
                1629028102418491940,
                2053277566527080284,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17483469670941741798,
                11764697873241107389,
                11484696434900188826,
                2424425249506267013,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1888308766769100040,
                15214578765228257191,
                11383282666311254240,
                816277393826413851,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10346428831813174199,
                15828950085091660657,
                2367526108030252255,
                149351906209437120,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5847033839554444329,
                3510146799203623263,
                10206138079709808850,
                131324777054539464,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11340572816718888851,
                10840177512018281875,
                11069793377573978677,
                2221121103584020784,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16584596943328947739,
                4448335938636138558,
                17212916347490528689,
                3376318951195555050,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12407881622117779417,
                18339216029745319817,
                14302244379866120483,
                1314394535801964127,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12786381243153688828,
                16849452251883848147,
                14945149175918309636,
                3155957045183652106,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8112662980361177557,
                2779078275128366513,
                16760399028067751446,
                2281399537334714887,
            ])),
            F::from(ark_ff::BigInteger256::new([
                431569872192260151,
                13947564649551820201,
                17670545070669946716,
                3009233375201988536,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10084404722836031070,
                8953580540078619908,
                12678801065174415617,
                1704386702928361121,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11894826960941351528,
                1392884174186120386,
                4294405198180441757,
                871665225280683700,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7350144181434867273,
                14612988840676789711,
                11464938676866124058,
                1825301186959942412,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10048804679398905344,
                1701748318570085863,
                11592467410794108655,
                2707133380784239832,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6253713814723964908,
                3404982471858467767,
                16607703701381428403,
                1701720724230325785,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1028561593358980469,
                359036472139384700,
                1296336883669300110,
                3484943069981446268,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16284722646184852415,
                13594586364219829136,
                16325939356360911822,
                2992399883846309775,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6796901171742746899,
                15024054836245980314,
                4695680129902215204,
                194559586274594892,
            ])),
            F::from(ark_ff::BigInteger256::new([
                761204015766033209,
                1065668948258648927,
                4754019444002471326,
                605873141923871507,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1888885367383771581,
                16700662768644761818,
                13230195644673831038,
                747590905981779113,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12526722733638174294,
                6066995581734507347,
                3834239309806601930,
                2370097675783837986,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14862842164269984109,
                15106418650653265129,
                16743432306852370972,
                168001192951010052,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13613213227809950704,
                13555511871419758244,
                15870216571412166780,
                1282792050794826839,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7911002428224077041,
                6931349483548448195,
                13475694588203618230,
                3429685740095865973,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2625433404952906234,
                13867505051289220432,
                3156160881647632976,
                1530521621697506680,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11053233373538131670,
                8156428275914279397,
                3373352555427071720,
                349892817524443885,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7151259269389722077,
                3513920363041844691,
                16028651691329125666,
                1052323259889170933,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8663104968235378519,
                10456929556229454574,
                11449856359595716729,
                1039643521566169999,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7528168623566582410,
                12400355379028771325,
                11659565452646605402,
                159875730842870839,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3855748083056224900,
                13664440551026092041,
                12216435069483054956,
                2178964997710509589,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2226714954510087232,
                13609711552192415503,
                15982072897310990025,
                680678257349406400,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1494702258032894249,
                2162346722845317536,
                15647716386288471820,
                1230495932078126183,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4496176278551275165,
                5399896298495811064,
                10944675633521525143,
                1523595181636922406,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14855383457758448793,
                5753106084598694034,
                13045589305221261829,
                454039588744927851,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15670125965233131084,
                8985948171320524861,
                2357480735231874646,
                1358393644069047990,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17510129458540256195,
                14704200946030036670,
                16979447239490887813,
                616186581637771891,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4559632508010915599,
                9957803550523869564,
                4673863238217346501,
                1518396839779450555,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1622685069286475165,
                5361158868661563125,
                16915918577474874862,
                1254747269804836136,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11790963495003962026,
                909630416772021807,
                9570468811030965551,
                3057951596656283503,
            ])),
            F::from(ark_ff::BigInteger256::new([
                925841957606634072,
                17041296536063774613,
                16042683885610359887,
                2740372395359403660,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5069137563894949432,
                710407508721116036,
                3815512241552998568,
                2617447939286769208,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5390334067677505106,
                14501523292543419522,
                12110540167619290853,
                447349512126914199,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17594627018391057787,
                3909240068383086044,
                14505918427377510548,
                2265234419359557521,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13956207746370440189,
                8446468679740983557,
                11036342898413848627,
                1677531786993217264,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9430596551262119475,
                6536108717531260783,
                16386493357649910384,
                283679195615467892,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6671717027366876342,
                1965642880756034920,
                3659634573038757275,
                3286732217979177755,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9317895081207035243,
                8488785282908067665,
                4944618440061969917,
                128801354686319898,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14327823682286648756,
                2455404650063603393,
                5196736942228920811,
                2612086317193084984,
            ])),
            F::from(ark_ff::BigInteger256::new([
                232366752323626472,
                10705522762032283440,
                13750100637081397872,
                455104011008458448,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7229527008488300025,
                12550701829884714241,
                8538761283598962874,
                1644390427219206795,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3394608219977225371,
                3015792479785655887,
                6622864284366204715,
                427434446931442161,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5903903472160189811,
                16447671566280687589,
                4624661322966683327,
                2588383396602244178,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2929964549748191464,
                13164498009580095172,
                7698824265433465784,
                2966664471816990737,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2977118994648687594,
                12149459422028234177,
                12987892406958259907,
                1386337764240065869,
            ])),
            F::from(ark_ff::BigInteger256::new([
                834512899700651102,
                14918078001121322513,
                5115161391608082096,
                2294230864299010190,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2472789034392999798,
                11208368528092774918,
                14417543339572709543,
                160732219178944546,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9049689074684735215,
                5553953588972573384,
                6406126007550230300,
                531362340683031233,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16425025843469370651,
                1033224391147435344,
                2868137364656539650,
                107589186863944237,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10874626907856246914,
                7929914449342168802,
                778728329739178558,
                1820186459701334440,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3209959940629443860,
                8640380696936548276,
                6741908246411506796,
                500151495284116722,
            ])),
            F::from(ark_ff::BigInteger256::new([
                276042366194143471,
                3951450244473395506,
                8780340323148520044,
                921298491500585397,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16490278598358258441,
                6214926832067374532,
                1373847687474425279,
                1926859779308949094,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3769507280322916142,
                10302954914487450128,
                3908912717299367186,
                2599751881914302463,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17546848198148154488,
                6922782994525420090,
                10086883849395923140,
                1815436446370935799,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16787630084505598046,
                17959393434854560168,
                8251560541027239572,
                3234636839732122277,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16463518822831596684,
                12263532213688861644,
                11421490830219195635,
                1558420995428326435,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11751849832393406053,
                9743609787526318477,
                10420868739256156534,
                801011329646664573,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2028107532564754934,
                16592322353538396349,
                5577992155796650334,
                1434997884942267753,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14402451636371511069,
                12205122236444527995,
                12672063230756103061,
                2305377628379491231,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2700889252816626759,
                4642688717355681190,
                137463137849883741,
                1277529095219814460,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9151535870572900588,
                16642430785823267649,
                10234771859026609434,
                1441291187238734715,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11669462893492956786,
                302171352751362052,
                17119910235589207967,
                1168666619518449282,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16271354972585130685,
                8620162644547464143,
                7010336678104099519,
                2355817905971035516,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12917283397028108845,
                11969319915241721139,
                1236077425036237856,
                2155031685303845479,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16767195007666462031,
                7712450391110089287,
                5171781934197301751,
                2385843496910570663,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13980856470594340142,
                2389802300768850549,
                1268935455355842519,
                248538198994714974,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17303021414429806782,
                9029257388075423766,
                13760380273812940583,
                91072472827331164,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9972602696329957905,
                14200965595905532566,
                8838983412773410569,
                572780555554441589,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4486469805365416207,
                14453821098988560599,
                2256969722416558747,
                889362644825759118,
            ])),
            F::from(ark_ff::BigInteger256::new([
                18100124509689709843,
                10527786883811955038,
                3832738299528094137,
                2057312277282886904,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3247840710841338766,
                15565027136201974476,
                17332146529851735384,
                1629635197285020571,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7900127312504256635,
                17880711033168179469,
                4235084603719247974,
                3400275628257120216,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17860167979087966323,
                8652896347503899279,
                17499522438630103671,
                1022797577877049573,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6892877882435610004,
                14667262615561004222,
                2101299964578261515,
                3066959121999192419,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16677605819306328279,
                14767277516446654515,
                16739796779485738398,
                579543791861893003,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17273712624897276754,
                15248760467465160998,
                13476365717299643645,
                1002229264835708446,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4753952034199123545,
                2073538938754232294,
                1587137506546933753,
                2498075351815438228,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11579604701806783334,
                9874570022874066020,
                17325427351856755917,
                2256122505159727179,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13067683976871322353,
                5723491623794072842,
                9325369888467015261,
                1339491465820843576,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15613394514849376702,
                10112217536433367029,
                13036590188473987369,
                286689369738158807,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4851173569026232120,
                2736728132893952310,
                8444369295801640093,
                577067707080387501,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9242911652264643082,
                10081287043528381906,
                1732992366592166446,
                2114637087454410938,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1271761850783389607,
                7801189928685592601,
                551169469891876172,
                2273383722366009587,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6510081394607225842,
                14104997380411955871,
                1663827513283455715,
                260265874954909708,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12801819893100479196,
                6061474720180943725,
                3560720275361188850,
                526778305375841147,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17029797305591356592,
                9771359322477735905,
                6257100243281652237,
                2261879111012118438,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17275431533479935523,
                7194401597238856215,
                13003443849902142551,
                2133570340893641628,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8731812747043179206,
                11233481591007404096,
                9884939567646574231,
                532711765629448225,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2862016162991067704,
                10380234765713610349,
                7491576637596144651,
                504723359985444578,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2653323980099092353,
                6527387393994800274,
                16342684204241864310,
                703283207541439756,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6535309660479465025,
                2917255722952594720,
                5520190392180004891,
                2246698307525126967,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10783446284854064165,
                9231055797411133183,
                15618590269604661051,
                887977377830115899,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16126025179047830501,
                11412845018801643570,
                18412438654002352623,
                3397453572596986052,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2385484129511900354,
                542303949627683562,
                4986669964129786855,
                3095848862057758996,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9274401932133378878,
                11848876139679368051,
                15487987337573102951,
                1786475864510181935,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17918887812334716302,
                15556980234345423441,
                15287079393562120384,
                1465472947243703771,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1563522191179520071,
                16523016349936322274,
                6314395335327205692,
                3367188256248876329,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13438842766574965968,
                2305090353258666821,
                14156544625085700867,
                2934195855293332598,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10846037056673759397,
                8355093695349240396,
                12822207588689098575,
                1714477583170661678,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16401723250878121904,
                8951416183205543063,
                14244912505404162014,
                2662875100080229015,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12780452838830005740,
                3250074631702120801,
                17865861621267451186,
                2831458223342059675,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9468332655426487415,
                4806242973787840769,
                5059639463380367312,
                3223039532324879768,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13838711868556220808,
                13522934526002367000,
                10412207817149203918,
                2877851020440308548,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17212519127951122769,
                1626839299509717467,
                4903123844706613700,
                3076548319281031674,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11288386477621633653,
                10574991579265183695,
                9474702363485474455,
                2959105273337449476,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10942015378328419066,
                3500732494244937955,
                1354666881975437816,
                1081935488641733086,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17137060911110560225,
                309094630092427601,
                4434615608632422212,
                2552137401590781729,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5502111035592226063,
                4250207830089963484,
                118061142540362168,
                3193275021663908328,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11416053427923265404,
                17959938911927261348,
                15829036776607900536,
                918607389661870033,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15037035590906629092,
                12744041159572559920,
                12007018255627507744,
                2096612522044048833,
            ])),
            F::from(ark_ff::BigInteger256::new([
                651970042048773843,
                2898957510051994969,
                10685138858714852963,
                1474571657755508607,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12204759756727233880,
                6733582184896809945,
                17684081084646378780,
                2654139685584884306,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3961545870248654494,
                4867845151118949860,
                16769522999123018819,
                894219515397532344,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7394537706675721745,
                1622436354752123034,
                891093833193836263,
                2082266503748337180,
            ])),
            F::from(ark_ff::BigInteger256::new([
                484872824113390088,
                17737105472613636607,
                4895116544137997913,
                3256684062249055939,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15255714065499738873,
                18079370924564275796,
                5018709614983057450,
                1264019743481261850,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16504349012166589277,
                14633499952312999391,
                12969645162134116991,
                812274167135520741,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5343829410877949619,
                17399724896760632638,
                14254425048168055614,
                3235768964988567891,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11981618973235895032,
                17645262373711119269,
                5191307907867696981,
                600312762970063590,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16642445120688253245,
                8575140257543449460,
                6409450178527876149,
                1569362040447791391,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6112217720989830872,
                3500555513717197214,
                7042393073180101897,
                3029302846435613191,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17707703319291120345,
                4033218193975340610,
                13252279624162236877,
                1048235891266006242,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14509289577197216228,
                5741766061717079435,
                13350470380500797074,
                1599471557393313386,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16144370379366386941,
                11975828993598380480,
                2001559934053556,
                2580941137865914188,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10442539540175582624,
                12550106833538765298,
                3588807516583649442,
                1542583791281882304,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3798855622032777381,
                1267521780232833112,
                14612802095682803251,
                1960179705745836893,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15467970755487210945,
                9024986879805272853,
                14283276712323140485,
                2898079922870712468,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5412857978914759238,
                12299766599988379791,
                4893952034611161222,
                3013198783081825561,
            ])),
            F::from(ark_ff::BigInteger256::new([
                18169100455109432816,
                13198030729589168442,
                14122891966856038885,
                2111111504212612379,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8899807769423973683,
                16115206697988765900,
                7278722629238672989,
                1632790819424833539,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7696883640876740163,
                1510234015258663930,
                15895657094605170223,
                2926865601734426078,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17829922088025957,
                5286798716704784330,
                806844342143197126,
                739001944182549694,
            ])),
            F::from(ark_ff::BigInteger256::new([
                497637983335638740,
                3830292978259126880,
                1423208052293495229,
                2648349829706598566,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13380546137453548667,
                1435927462634711631,
                16657768175118891233,
                2845340724685551020,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13936475398704690781,
                13305875298533768641,
                649706515727373004,
                734145459181406568,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5567440671326517577,
                12143081373864879867,
                15655934829231011688,
                2269489591715273321,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1584461024927136869,
                9971023658316084968,
                14666661924345047591,
                606943075824397162,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15431634280843719479,
                18374087336108312703,
                15852652603058877594,
                2861544690620647264,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12746781130539560304,
                12358893498727001576,
                10973494975321372087,
                52284682743673199,
            ])),
            F::from(ark_ff::BigInteger256::new([
                760301708888056725,
                17702363893708709164,
                8749746084176187987,
                1971348464295551683,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6108010347588688184,
                175099272361310495,
                4562633634400568540,
                3322202415468982880,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12659544989994085910,
                12788813246269848862,
                8583686549634834319,
                3424528049353693749,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15384443568875229505,
                12467407731070457101,
                4855357628094345180,
                27774650456476766,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5423976176162136887,
                143778029907442114,
                7603285553566159598,
                3295714753286016567,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8884734483496675399,
                9494866913161941620,
                16843319200467353860,
                2081746047685614097,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9898137213290751276,
                6443225265236147937,
                945387326828507615,
                605034472136984395,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14260831821573445484,
                17783176505245390802,
                17866509513451403422,
                3141345800067986208,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3850097550750252072,
                15745447422155883417,
                3491726991101773360,
                643428680672814503,
            ])),
            F::from(ark_ff::BigInteger256::new([
                919355169716159519,
                11465136604152491411,
                7624461051848172506,
                2945136231800268581,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10175206951044719839,
                17559518969911952173,
                14724540420880169371,
                1562584714222584554,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4215593906491779425,
                17042824776717263805,
                15375697360705207576,
                1859720332940155547,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12211357976442713432,
                8302273131058397419,
                13149123401030963495,
                351527839618809848,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3342997633984283806,
                6580897890187821933,
                14712081616905224935,
                1561353102612451729,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8890869579837822447,
                9299617058987272696,
                18295795121557682900,
                112172140408337495,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11147915032139281237,
                13718179940227826992,
                1835392489550418614,
                1664930233089617541,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7337095124781502497,
                1453227719106119970,
                6194281034073123639,
                3361772284759078899,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10970110874432660659,
                7843684230347773806,
                9279404161522682375,
                1459890180896078673,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3390720268845415452,
                2389116476538175172,
                10352749991845370461,
                553146654008667978,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4978002555881328004,
                3007091287218191521,
                16535988920595059442,
                384236280935958538,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10292947768018875557,
                7219107704971935435,
                10903355289553861165,
                794221623157161493,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16263838205282286214,
                300407438171680738,
                12653663188726497543,
                1840624929277085025,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14336204043347308292,
                12420584022306283454,
                8234400613008720996,
                2684007604402629254,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15837830338576151593,
                17110992356016639132,
                2258229560407482128,
                1077018823211213786,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15001368068810046264,
                15647093233691252641,
                1607340205796211411,
                3281195958805201504,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2684157678100506774,
                3438272692696258773,
                9183345074120700943,
                1859507494268833912,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13378081839523022539,
                16283212527465696087,
                15313391292609327362,
                404753353382187593,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13099899814697695235,
                1929516474011191478,
                4312453135035374285,
                3097394229458165949,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3367119442396095003,
                8937244909497359802,
                4800268247604975480,
                498035498577056791,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17031204291035488408,
                15497141025063346739,
                4937824245715715567,
                2200985938078761314,
            ])),
            F::from(ark_ff::BigInteger256::new([
                970853391910517481,
                1694816472703980336,
                14515006325814442586,
                3416217322059238179,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15382387552618960579,
                10146100358007872643,
                7537670274373684481,
                733667759882888005,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15871631397418709265,
                14006169465256654696,
                17875970919873564662,
                1192093763989285774,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10888740099141559809,
                14109990862445184887,
                15247908895534074208,
                878079793365198933,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11253260427680481372,
                11504529075793252012,
                3471283586848867843,
                277778167634977749,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2579220351969214772,
                9248541216893628411,
                1326091091110719308,
                76364690733139411,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12675876946414813824,
                11659804563637595038,
                8381585260203380393,
                463891753012189646,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9669234368098322137,
                7340877417084825328,
                8601910711592019397,
                3301196384619656493,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9169375481012376630,
                6544971661915988547,
                7922233383437512986,
                1951630379996115975,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4675156114069943698,
                14535618336068062556,
                7529784847174947619,
                191730773127357835,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1644362221228436822,
                13857025281839379826,
                2622120433097805165,
                958337731827793038,
            ])),
            F::from(ark_ff::BigInteger256::new([
                166542739071623532,
                2035873962871711462,
                11577171074782301284,
                2085441228637341683,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15991988998497110760,
                15329892519484901517,
                9970652657335662479,
                829155180678653082,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10184680432714161461,
                5566213524528646348,
                13183856593952870303,
                2142924867865300330,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8928892474714467728,
                4735929153104683673,
                16450571850048193527,
                1645269340419016880,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9349443414948459998,
                933860673783568658,
                6465336298832146526,
                3210776816582692435,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12593307191793889041,
                10572724150473310773,
                10700561362555274609,
                2233924990469257630,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13183826532224522306,
                15133029763306242482,
                6244243154742187825,
                924142288769654908,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16661613698371555289,
                14500992319531116175,
                6014517054613506090,
                788605984931298191,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7007154570871159616,
                11405879438854705521,
                13856812801407838840,
                2652687603473114436,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2666828308483942635,
                2867021109437830448,
                5700130361309932863,
                3395448363534896426,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17412370150596118966,
                9337527292448915650,
                4938531934496797474,
                2632988602950013875,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15335144003510907441,
                11066109664451968332,
                4434293913758339398,
                2441923740978994205,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16938290206927833701,
                7479787108845704280,
                10577270228027206326,
                2030989034622956304,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17092734250042084982,
                3959077207759269170,
                10390094891253219920,
                3020601836715144363,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11586448047523940067,
                17617730619044582731,
                3912103441983536712,
                563795485906771749,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10816324474234053487,
                16477846613165684378,
                11418849543266162024,
                3097049135515669532,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16500205739230255747,
                3020021011164015799,
                1235608978632026059,
                3291922877373447368,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4851289825451177101,
                16062148658506731565,
                14587432592213506459,
                898272967305535726,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5078597837679100683,
                7515356192024369531,
                10763974042184588739,
                218461046373272265,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12365948113486402197,
                7291949757982112269,
                170604537687643643,
                2376374789967073128,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3419237705627427397,
                16180611049156523720,
                12940331367110848963,
                3409053939370347972,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11979237740857760728,
                15732986353159209652,
                2064303486333859021,
                2713084679157309451,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13359520377455285708,
                17535986471778958214,
                6066098226447018921,
                388861307374288014,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12885054778698977213,
                9218305119388083249,
                5338204169253592884,
                2641071517541109938,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8885688255595728106,
                4501385603479053769,
                8560079362077549733,
                93182806724038309,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5828630259371621071,
                8322177095384756080,
                7737468148195642143,
                2674348778533929416,
            ])),
            F::from(ark_ff::BigInteger256::new([
                18247698033634670187,
                3632176598102555870,
                15309922457429650221,
                2561470582507623304,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10134155531697105370,
                13874993336960772164,
                120926211250141084,
                1384812324041198528,
            ])),
            F::from(ark_ff::BigInteger256::new([
                621538171316433992,
                7337435609984134468,
                1830071337958148943,
                2781785719141794989,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13469807406408669329,
                14288489689238505217,
                16377086202535271544,
                3098996034907117078,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10024602646690155730,
                17593038468992037174,
                5008379915894352674,
                1931023386004310531,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14392784254990637944,
                17133420486550649533,
                1463327889656965339,
                749522753374641383,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12455179005107014824,
                12342796897267427972,
                15329288531556553559,
                1699984037472157815,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1771542152598966351,
                7842930995567544795,
                9310075128329253577,
                1282325117240924503,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7521313590472591674,
                3423097365763472336,
                3477360082254647642,
                848792418653388222,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10085036827012134668,
                15665082088852198944,
                7052186614176868722,
                2582327279830794339,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12476074060974471996,
                3823026377923044563,
                1876545222636277180,
                325228693855900219,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4964376095008059238,
                14788433185231634209,
                3144548900359267813,
                2387043168192608965,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8434157369492391684,
                1399216338929346910,
                497189560375855316,
                1925410724921130134,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16907039221203035204,
                7114131419245583435,
                12533739114637930527,
                320235082551441071,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6548380147236267772,
                8457755526472578109,
                11304209780263924397,
                2262965618160724995,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13583819266510925817,
                17540778986908752845,
                8714241752513306071,
                2123687285676360701,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12920936915167066001,
                1303118682853335788,
                17096931057772895034,
                1491661622831777066,
            ])),
            F::from(ark_ff::BigInteger256::new([
                18442788759015761313,
                16094571045690859182,
                1349782127102508890,
                52078032712158071,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9826881466420996304,
                4925917554904846789,
                12185721450535193083,
                3441032606980150190,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2651163447008429899,
                17069367828318242220,
                13087292531199187322,
                764899163463942707,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14587722143179709340,
                1499017639447504806,
                5309740065035799718,
                2255138379148733969,
            ])),
            F::from(ark_ff::BigInteger256::new([
                18071689956022501331,
                4208593701108494904,
                920131484375740334,
                995606799428260625,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1215570147278764981,
                7484817518500110787,
                15297561481971201600,
                2378226490946577529,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6701504958943787262,
                12159685646913680148,
                14086604912660376886,
                1453794700666641414,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9334528486080198115,
                9026285089644657125,
                4585688782002471959,
                2724806293365756884,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12581709950149966147,
                7321088671534691329,
                4539903610141644553,
                1523176448338356669,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10291832074258343181,
                1435704338978152137,
                9047009950207140611,
                2469187173665855554,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15861169395772737563,
                3142406623288005500,
                3677368509227659428,
                643605647225731089,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16168860801951952341,
                14630734772533428017,
                14617135328905978010,
                469625567989565040,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16061657360525776276,
                8404017342743768445,
                10881568610495349040,
                1719511776348974498,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3380103864659439132,
                4646690760991297390,
                7899645907112276685,
                2434566900741525811,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9786128945568247350,
                11640217797751278830,
                9289548027592035265,
                786746464943278535,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4662492732364294379,
                2458970123503892620,
                8863290070584661334,
                251751744650574975,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11900596029172988672,
                14098203130941552385,
                11987822201766677738,
                2811885092023744147,
            ])),
            F::from(ark_ff::BigInteger256::new([
                747844002064217027,
                15042831961905322644,
                10242180382496214664,
                1006591563553826703,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15128257678568995582,
                12513557032206362762,
                11848082437784506057,
                2979693874020362086,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8350840692696812099,
                3909422949104074788,
                104851033891432138,
                1046726708734376926,
            ])),
            F::from(ark_ff::BigInteger256::new([
                751902212708065231,
                1485857540489805786,
                4107406411472979366,
                646633513373065661,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13877242558684923142,
                6858999653099557420,
                5770194604057055449,
                2983885949373114094,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10788810082500411790,
                14652358284940263884,
                391765152671350718,
                1537630067646952652,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8662048254351783137,
                3010353462755791008,
                16465364522620817210,
                930110183749534363,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6303830352500904782,
                1258080730625306603,
                6259925207784850898,
                2442839863006184817,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15835597237213088452,
                14232189044507140396,
                17543802875219054218,
                307588745611552214,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6211640479738348020,
                1705598603494130038,
                7066597468816611473,
                1715221113461633479,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14398628652909114356,
                6442901830458823505,
                8927289586777609538,
                887153270470725554,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15114924013106271606,
                15064605309542568454,
                7342783209613908451,
                250591488468742087,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14848972516612983642,
                16172087456927020068,
                1528940713302251435,
                3176368908629752320,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2229420996002375673,
                5041286048406560084,
                15382655832786709822,
                1206275287769482751,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4092602473137367496,
                5484625947646506197,
                18267290188188181039,
                1092698411201227605,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9249187538137589268,
                1477356741524338787,
                4820199200472039475,
                2989069314909592659,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5564276524123322292,
                7270699650656853796,
                6104114048427075864,
                2507727385709261065,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5849644130356935466,
                5169974196945025998,
                14642979402380253619,
                418634030079601164,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12225569617883140308,
                13126160199133894714,
                10423167437112904160,
                2164798415711728225,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3029040264657009691,
                11485096135958994360,
                10431279827997692731,
                3109788314838637626,
            ])),
            F::from(ark_ff::BigInteger256::new([
                18328521412225719372,
                10131165150923326119,
                1916670505659797312,
                1421703616959756452,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1310058254606747341,
                9810800161247612799,
                15805266735570472098,
                926670662982372336,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8091558463765964375,
                6510661231649337483,
                13607265264821562184,
                1987474508750681057,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14271016891392158823,
                12397700625321792918,
                4547586061163512332,
                3354467428275665235,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16041474051294533310,
                8109012166835622531,
                1949769753781022995,
                3416299243799878580,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5214504327762648767,
                11984882584619327731,
                282343165713648555,
                1515155601453525468,
            ])),
            F::from(ark_ff::BigInteger256::new([
                18208110575778081288,
                12870506257634284775,
                13921211038114627649,
                3333518293998393693,
            ])),
            F::from(ark_ff::BigInteger256::new([
                18032116934169910049,
                11555227301157740228,
                9399948666221821046,
                1676023714133963222,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6304380669092372029,
                15342045879669644166,
                13002363521824627565,
                2497431761268065084,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15057794087547398182,
                14676515517114377742,
                5848548789349882384,
                1211581201650249870,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11806440008422149263,
                11759545090429987820,
                7901644809630752544,
                1378887349592193410,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3381141106532562131,
                12039178583475780509,
                475929142752300204,
                1049394966704405121,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3604319260073639563,
                14234628327667801401,
                5260532086927355248,
                2844593387087425554,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6879385558668425438,
                17517687038567339562,
                16163732437830702325,
                3128397739388593430,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5704708633274898905,
                7047855095953015604,
                7694216488820103984,
                2205495571540831272,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17799562837409159357,
                11785597958026116837,
                1065192701050623568,
                620746292146370452,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6979369762160485950,
                8874601488418137202,
                3997125366077242007,
                1286090739415588666,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2026177202743523745,
                15082746377558398211,
                12622376268369459050,
                549083133067360205,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12597759897686568698,
                3829262835817141487,
                2455102261324341957,
                1075493238576505429,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7613289745863651072,
                17401945743228059395,
                9395479094790913823,
                1263746882567770597,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16485232110060711636,
                5455670915555037281,
                3702652939304784547,
                725165421194311098,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16715452110961610731,
                10783761738060228032,
                10701749474036719376,
                1346202202821757555,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12402524897201330528,
                7476278165144166863,
                17823983785848520628,
                3269516453412053235,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13965237519659213149,
                6331588578149754077,
                16224043343370731150,
                1277958150912448906,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5517383673323624863,
                14967626657344353930,
                2612934253466534045,
                3431150477149263224,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9610269065699565894,
                17825198935037724573,
                2243655835251447271,
                2639497359913851143,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14485880867964803751,
                16547679931963635456,
                3491441965934620822,
                2693955976997035218,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5399499969117585432,
                17473982459141377037,
                16958281904542730277,
                896775450139344865,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2198190335522167947,
                6712432015753592155,
                4845574532649291752,
                2033602723456898981,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1918945375003660554,
                16759350683957065365,
                65498658833553997,
                1585866543571799234,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7229799517844544004,
                4049046997595947554,
                5562323917195722003,
                693632160906084949,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15866866044574648159,
                11564495007705935791,
                15653255073686722197,
                732616942317269736,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7547975152777862435,
                723678079695596872,
                2031819832242004530,
                2758876007995638774,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2438393487627770430,
                14307394466278672430,
                17537161207323926820,
                1953023384069299066,
            ])),
            F::from(ark_ff::BigInteger256::new([
                18446641147111060790,
                1429452251769420165,
                6993879040421496899,
                1881638880636450659,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2304809369764810078,
                11606106372721084407,
                11343907343228182496,
                2660317198965882750,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4429736232355479533,
                4503755620860341876,
                1778523939408820461,
                409110269154244701,
            ])),
            F::from(ark_ff::BigInteger256::new([
                262176277782508963,
                1090805162536577057,
                17009124016302391276,
                2371793873858140986,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8362362995520144867,
                9010871522259618944,
                10310161726168646333,
                568622161300345069,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14540989720663925078,
                18183022587808112919,
                18354582884038458914,
                876022077144579378,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4097707454380028908,
                12636089861079533587,
                2844581869809757502,
                1841560128494409364,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14324370538350920006,
                482317090574552028,
                15408940108852759223,
                357797899754933440,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7191057041884265726,
                17701856172878850637,
                875044303596527622,
                2700506646109493237,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15464028802817920129,
                8941697518812385404,
                3600671733306187881,
                3427685617076719326,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13508212972715702239,
                8407692419153537742,
                644508859379538462,
                199042768671646849,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10474111452176677949,
                18244577776700099277,
                5395350584853825506,
                942947969465169176,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9926313487575132586,
                8697621741262711527,
                6864117049595615006,
                2826045830128090968,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10009605161344871474,
                8983542837973196619,
                4684103069082941807,
                2942400012472950303,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1259164384898203525,
                4122306053902645488,
                94767147324972225,
                917515283996229333,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8654954863884126364,
                16977157384067639611,
                3248530887611003664,
                3398449952997202215,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1104053337879893854,
                18253896926989378353,
                18296545647291762036,
                2388321713716964781,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6826054153259750812,
                7738412954026374912,
                821687559417168395,
                365197834280245645,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13900752896348732800,
                15760288817406024922,
                1529927915507051086,
                3229496822560043758,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17225929808602789033,
                9837335936141689410,
                15544119226245612632,
                2603622563957706587,
            ])),
            F::from(ark_ff::BigInteger256::new([
                929691620402416480,
                6299253830143839882,
                2477667751481409648,
                3133293409232510470,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7158071368804423627,
                5404175852536632449,
                948395796259106877,
                2368492840768567028,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9266345163892845891,
                3613878073693156234,
                243352783029560269,
                1581781232918529792,
            ])),
            F::from(ark_ff::BigInteger256::new([
                519855252571788422,
                3603376640813920457,
                12436555393980483500,
                385639159389340627,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2527890888390134533,
                5465107763917900530,
                11959942638002871390,
                1118854615764497343,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5570312546165519477,
                1373635975311231173,
                9834866753082653324,
                3267194463718493878,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7125978199592201557,
                5485588281190254809,
                3151657643005168889,
                131271180465649963,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11006512344269790727,
                11336804094955503449,
                11892335804858129262,
                1287440808085943760,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7607371969929921893,
                12185941786709003801,
                15241454664697425026,
                979221009202500433,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9316618512408496121,
                16160845557591707403,
                7590079586425152042,
                2217193986671479761,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4672365389975159461,
                3952348499922831733,
                17855048552828233495,
                1362239269376483717,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16337494513789842094,
                10582185345831052112,
                14900341745564765619,
                2845534714869139280,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14122029480226876041,
                2698526615297147401,
                11340009665631995956,
                2484452473010330149,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15113269931250065279,
                9023707287257894901,
                3417899291431264970,
                3155969226876683684,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8593666772062581128,
                12196029749184519587,
                14707451092704771258,
                2267673081506369399,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16618096437709188798,
                14544846139076135994,
                3048599913116416282,
                3088716428490355885,
            ])),
            F::from(ark_ff::BigInteger256::new([
                402237598390896141,
                7172683836892909142,
                11618742346649817589,
                1592459616510300980,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10022559581258134393,
                339003402040339710,
                4887098031375846203,
                722275946851953003,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10834733482316540730,
                6269146095770961816,
                5412616019060329672,
                3471848066716005620,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3513169312153274723,
                13876128580813305786,
                12193852086520416758,
                3137163223177982593,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12006637772554652888,
                3122723919997613028,
                2167286960169067740,
                2447280063578658701,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2374672740653711286,
                696814233636988721,
                2583314159849179193,
                913866803649000103,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11112035660076264838,
                8357352377090575303,
                17325301459352387035,
                961353996801840025,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9834457720909024595,
                14777777878440798612,
                9118324094720718617,
                494486465555144782,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3748451273261766325,
                13297681749300726586,
                17786593370506263215,
                939818615530836606,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3555139750542928730,
                14771171876011261678,
                10361556156564297776,
                153539260306052924,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9252142855891621069,
                17515867444443527466,
                1400610959372273313,
                1946537205026698180,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12248617681969291077,
                6891557884704823627,
                15318821296120235095,
                3163735111670247332,
            ])),
            F::from(ark_ff::BigInteger256::new([
                18211136560840954903,
                10873066192431218050,
                910653815224135250,
                3183700287971648689,
            ])),
            F::from(ark_ff::BigInteger256::new([
                825774577836686582,
                5752612807496225547,
                15467689086065842792,
                240571009147130919,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12496433615379034500,
                18201336237223048950,
                6210773188324262964,
                1055667283108115949,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15382594017582533864,
                12407231356063974572,
                13317691497289301479,
                1768712090442457506,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14589940145571971823,
                8819989985978186707,
                17277540871849214999,
                973373868477964471,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14559799587159180889,
                14283065764493906377,
                15062094059593602960,
                1854273860256057651,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7472057617019317581,
                2676124036386499624,
                13592784222915727551,
                810424849033648491,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9798209236688679244,
                7304878435729741093,
                7176874997747150511,
                2262346572503052976,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12685278357039828004,
                10394074193884086226,
                12805674262262168131,
                1673481643487578401,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11449195353550126228,
                10531996572959514719,
                4843973075590642494,
                2691709798681595865,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10015516581767445034,
                3237515246253832438,
                15438649956502542385,
                3087699439795330455,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15373780734023696632,
                15603443238714183238,
                11009258632955455798,
                2794507412758769136,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5231008379407568979,
                4698351808681278091,
                13988406081349285045,
                2401263238610746171,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14407816076198657002,
                17014246882255720760,
                4942480706106677547,
                367820587846388612,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12390306544479005234,
                12506492225706037578,
                11270944824450946943,
                1949207945744698092,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11461944657117530701,
                5939663288152137796,
                18248858169419620597,
                37424867763781739,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15332077626446159127,
                13750253719459873760,
                14149169784221500190,
                3447083569941219127,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17023226702318948972,
                2823050450184080985,
                9116900620518675457,
                799379256338102444,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5951797239262039908,
                17943988645414311868,
                4249556217169223589,
                122718180543739919,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12898694022710179605,
                18430107476094458127,
                1346698538205151850,
                2529322537760333066,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2927053488083255585,
                16486548475208651071,
                3966280806203906095,
                3408594389707518522,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10368997425238780598,
                9351897699576227871,
                10106830836032949728,
                1974717881168674415,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13080965308543504918,
                18024364005326979107,
                9273948974439365019,
                735304725553147696,
            ])),
            F::from(ark_ff::BigInteger256::new([
                311144282549711010,
                4817696423143432341,
                4714424671867112788,
                1281446920595220360,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9474475049247739105,
                18291527010786490248,
                2819595728516540271,
                810415492688798198,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7104193152161547188,
                5112960080336067136,
                14650556992402049609,
                2699282490734717893,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4546966072170911072,
                6681422139785431518,
                18137504336798303602,
                2156108513021905359,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6646778141585114690,
                997792260933243629,
                14274684372918659475,
                2680408595493847692,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11670058292685173275,
                14188235926309657087,
                12226886222015804545,
                66214820094919358,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5218429804098380076,
                3384772290600020972,
                4322507421340371052,
                2929282897324744521,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3620658913744354191,
                13855347220695198337,
                4274706276440513912,
                1780939279538764683,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1976101712163449955,
                6569847855299650986,
                15828148405279791310,
                3434432632770780492,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3263383220802053111,
                10485411623650054503,
                12768407380718265596,
                156901433151239171,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12943715874051193715,
                11343369998454844735,
                9946910857251337637,
                2507953190207628066,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17017980656306761007,
                1769239386808479037,
                16432792985490677023,
                744675539251237909,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7544403165671071830,
                12796438696634641497,
                14015661147175758864,
                1378140385244989986,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5484226930615808724,
                736845589529436991,
                13430793620986271659,
                2199369204272484373,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3494914063311750604,
                7891954027445360303,
                14795462819652197541,
                2616786652852774128,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14670176352958753646,
                15937918852949759402,
                9831216924746976126,
                2013187741522040115,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17877855217223885551,
                56489043448012876,
                6622949043854512525,
                1086052339662154131,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11227060241253448714,
                1707016613950210429,
                11044851937932437944,
                612160198886736421,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13093083585007319500,
                9912455925557912484,
                14574477462225185177,
                5937707205340556,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2492989092751793284,
                7926358806245546240,
                14567169528743204971,
                1964215491997761254,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11384291635634891566,
                11504746383450094647,
                11399653412022904752,
                2399189598235402926,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9466964979672162883,
                7916026738848823966,
                12788270391560692041,
                1760360842562134611,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8920519390785756431,
                14536347930268698012,
                13460652072157801056,
                1521030658808738668,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13947201340682383718,
                15738020065678959837,
                7831193209750025658,
                2889392736800551486,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4402470659290508056,
                4872352633345508571,
                10333824347008039048,
                587070091102269799,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7729561126952083399,
                14616299767501002449,
                2837426584130899028,
                693153364659319041,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4887658217037015994,
                11721767833516329051,
                4442668199127485001,
                996928543209496448,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15181872622489477962,
                17310390275110609598,
                3441333197611683246,
                2111327730234583032,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17111893525207072872,
                2032166220555613697,
                11703297052468014784,
                2569421527441144752,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11916337024101550158,
                9091190970463060412,
                13488912873489865333,
                3213374420701717100,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8761245826794167094,
                10956081175582603230,
                13632012328423228019,
                498084521292182573,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2123940284934457418,
                16518663485077948728,
                8440036303465742943,
                1094017991363950282,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9105556537788420129,
                10412771071128731058,
                17669485483660524743,
                821775380802165445,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12244723968599909560,
                10698383799936323461,
                16266008240284447693,
                2031934942757173137,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7279714887758525028,
                4240294001354492258,
                11255187053785132456,
                1303819440989509720,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14291538045130916106,
                442191052225302630,
                14513336126766781380,
                2896567295059646444,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3991344217095025716,
                6605772598248690027,
                7966801376591493955,
                655018726688168321,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1957591554928677287,
                9015146140403472266,
                6865070759369104348,
                2006458415275159153,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4816463169164815089,
                732352954996193133,
                12141785764848887555,
                941017606369659102,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8623279888915973170,
                14669931975725882178,
                6708426416797711737,
                315444976707246843,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10216170112135028786,
                16790087327723372283,
                5744971771394618053,
                847090404933028573,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16981597757635511766,
                4773715608297400533,
                15539257000294865368,
                1076281665643025801,
            ])),
            F::from(ark_ff::BigInteger256::new([
                622583206610997717,
                9187893173145156020,
                6123361998075658203,
                120541871751569152,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2266045442391795164,
                15230958060784080863,
                14710310785901429464,
                2115528437782963130,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8631275371987544949,
                4475385947193396448,
                4704711140559481360,
                547714776489776688,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12666249684701862254,
                9914151843234272225,
                13428764138766111800,
                1144187082748938878,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8633048092883276414,
                9998900410588779214,
                10629088102327339869,
                514925237495856135,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1560439680940405323,
                11062803614788125126,
                16845862156145827311,
                2923176090967997193,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9386813540665345611,
                3182340925192197484,
                18194127345950958791,
                2336980065116270124,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16162624750706286546,
                15010254155277963210,
                10666761242502680022,
                3349627624890566132,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7803502544391574820,
                13254720549124433242,
                13758313284534662631,
                837269640708937865,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14587743055084135115,
                17707195553561961789,
                5246888990754796879,
                2753309362576839220,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16196377966778289666,
                5984179833382778906,
                15854801639779622568,
                1215817692308281676,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16078954176767327610,
                7226881663080494949,
                907729229116237298,
                3289122490348828916,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6459612994104354057,
                4921720851506120188,
                8490597328648253920,
                1859482823371845177,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1265205396127177277,
                9951977679318576461,
                2008866551208704568,
                1893723776933494858,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1081460426577099167,
                8928662007637707717,
                11344645413083383939,
                3387690380446698735,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17471199051896168060,
                1163165396576669167,
                3318477493732714256,
                2858504220717987750,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14723802952993605224,
                8290083791903505795,
                12378387282879396823,
                3463679561045751397,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12196593319594952809,
                12551696649549875497,
                16494765709363571100,
                220262004585021502,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11847892106994045873,
                7396856269467739886,
                5305789989985772168,
                2418017742966133967,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8738936964078209590,
                1287386258306980436,
                9091038802204785002,
                3107930727184113329,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11733805186532827307,
                11409124434216130262,
                4913375933751141771,
                2458534702488919733,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5301847813304227012,
                7940697683677149123,
                9489838169688581458,
                580891482379734666,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8273357998248595747,
                13391668918900735235,
                955980096357715197,
                1585801807868430888,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6347350562894616401,
                15309774222245572639,
                17894178459573493759,
                455657239988591063,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4032115290197352487,
                5988331387412702677,
                13501948679276424904,
                1214714154873772888,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14757951932589699127,
                12293929435325650698,
                10543180945171524499,
                2128521148766875703,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6781464369963360831,
                3038820790700846367,
                44878341271628271,
                1858793582852816831,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12410584890112622222,
                13478383461452024483,
                3548703115399858135,
                1896712411231270207,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9629497041701743371,
                7228898575423455485,
                2900600117600233846,
                1941393203850602348,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11802148154953511474,
                616780514452656760,
                2151823284436465456,
                988733040873132219,
            ])),
            F::from(ark_ff::BigInteger256::new([
                624990167928495441,
                11675509168281719080,
                6775094908870524778,
                1590827624621381090,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14140818511476919545,
                14361184531797481404,
                3703454964992603461,
                1136172614705805213,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12702206122997727707,
                16766920626176196207,
                14848401470437537705,
                1093459094150606972,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1713428192690015772,
                3500609532974076872,
                9816066394396231792,
                3385958667639202407,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10549044256423915904,
                779332575221789555,
                7591666447453199211,
                3077334314645826970,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12655750970156826681,
                9198531513343458007,
                9411412823267984653,
                3331376769598236262,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14772056516644161750,
                13263816233226157778,
                12001290695741920083,
                1744002814647720044,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9725155088302534890,
                16781576755309902326,
                11157171517462419912,
                1056142650808030537,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17882676712054752702,
                8610677368100582613,
                6520253512936091040,
                686086958965232496,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2659272299107062857,
                2282424468385797239,
                10601342305109757398,
                2842523951197784697,
            ])),
            F::from(ark_ff::BigInteger256::new([
                319723586891183770,
                16312916312391968744,
                7828146508727796536,
                3342273024119615605,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15513760951264421416,
                3302695403135810656,
                2992116258601472022,
                172754127601588578,
            ])),
            F::from(ark_ff::BigInteger256::new([
                744981810667961392,
                8120090275487665971,
                9020666848470498109,
                2614423198801033617,
            ])),
            F::from(ark_ff::BigInteger256::new([
                18086864254432696527,
                13587871164154663820,
                17183435412817533101,
                3111218346783496887,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12345820006909992535,
                3260249875584186465,
                2524364510507551432,
                2550793417994858919,
            ])),
            F::from(ark_ff::BigInteger256::new([
                374574737032012817,
                13496049437817941472,
                6283191980660042781,
                2953427599523378234,
            ])),
            F::from(ark_ff::BigInteger256::new([
                345269932858968765,
                6014647373626797532,
                16989597429211812828,
                91698042494879145,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2792201409712117770,
                14977344600952596566,
                16270812249729033960,
                1617767121830911738,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5685789519963335577,
                4685930166882986599,
                6294152727060066739,
                916260277033445985,
            ])),
            F::from(ark_ff::BigInteger256::new([
                414180914674526340,
                14902535104711354153,
                5570927484212776718,
                3271789015508762694,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5583021251757941099,
                13254111985650813528,
                12411985517591277842,
                61874617301072883,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9430500007409638522,
                12686919273714920362,
                12553427357496830019,
                1272709825718933335,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3184488318871928869,
                3493188328318322445,
                6591096772445965239,
                2525732364274301809,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12040104852219455809,
                15154983196110948275,
                1506730592948746350,
                720276856599392166,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8105874590682791810,
                10894761394664159226,
                5321747207243066887,
                1299569507662183378,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17974822913009790377,
                14930734225210376690,
                7191446919662534340,
                2741746371912501049,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3779174332762933332,
                16026324576198937400,
                4920766770023721180,
                1325569210963539436,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15550044370880192249,
                5049765787563068326,
                7350075829197278748,
                2954528424418845661,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16925012853378600833,
                2499591646680681258,
                10890852673894001004,
                199898205437264352,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5873367451476062139,
                14918325927377794652,
                8076115733883983165,
                2539014391565383555,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5025110518545305168,
                6320454467030158375,
                7133380326072520773,
                3130891123135695738,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2563890619069573205,
                15626216059644008972,
                13117283873061133655,
                1673345186387931591,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13369321340873089464,
                535134348793876017,
                15280047833815481993,
                3355231304200217075,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4851709305427023965,
                8783820569978758645,
                15765730584947575073,
                1622421209639920881,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14686018652463310159,
                13063629512556597729,
                3153713818489459350,
                1380731815450493437,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3979004534698863360,
                222936917219997543,
                7715297330441986558,
                1002975296585169190,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13648474316668793734,
                7121753201611042454,
                16561724219729401143,
                3199865301265440330,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8521185920010735410,
                16723313756045107784,
                4759351981116443558,
                2124599190936967341,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10814394384982432751,
                1180833633418719653,
                3745829824438517608,
                454281366743242307,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9545235375407748704,
                17549131547737890787,
                18212986665516582223,
                2443570436762163679,
            ])),
            F::from(ark_ff::BigInteger256::new([
                18016373554066666716,
                14094723315705059691,
                14256449562763899475,
                1652269156811607433,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11972825349805212523,
                2892734387164577316,
                312434924881826858,
                752112099160160792,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15051416372608421736,
                12170016101027915329,
                11506976452587687140,
                636594453382822665,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8230272036075358226,
                13573671075935535569,
                10468787461524052542,
                1130852464162691004,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4827202203356451108,
                16630555109741326421,
                17347985590259408045,
                761659245814096627,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15676598369310722340,
                8175301844200399430,
                12625085207110825924,
                3293040850918549896,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11220219556398165327,
                6726554070506260326,
                6716994644134510805,
                1978311662710897703,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3765032046062568231,
                17771848839204468199,
                13142565988052786727,
                1244494208921333235,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17681056019671952348,
                1611255053516750230,
                1083552551948914871,
                1940935499782183749,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17838851752458567250,
                12925393349513442188,
                5153087232255363669,
                2647542551245690587,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15433271850315020175,
                1224035378117979101,
                8093761286073249530,
                3091655866682275244,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5916774397181995248,
                11592493856990838565,
                14905332891601099239,
                1611396041995087131,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10165282386778720126,
                15898550138570793488,
                18150650494673336253,
                1996172924959342757,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13486114102921346347,
                9140623467412563415,
                3867875908159120947,
                1759041589600324191,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10340803425477081613,
                5044883570549898337,
                12277467089620765436,
                210292436334428125,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9014558174477545347,
                2494951297753420077,
                3411148071011787381,
                2155100957516875405,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7512865427738921464,
                13483503737866169643,
                7540629705186495914,
                2187183625455205419,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12416674126725629826,
                17137092398421667652,
                15767154648015431951,
                2856452382903433021,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8473369703009871735,
                10157342458695746994,
                13522995189320375427,
                43304168092326033,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6606727282479134553,
                13436915131746903195,
                1509261245986701507,
                3477807963036574974,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12747891183075235823,
                10128760721544115518,
                6413773513302217959,
                1043337599562785608,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4727067924326450872,
                492808864679178395,
                8240715853383365198,
                1224427220110609903,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11857728979419745315,
                7299001193561165202,
                2192082590971843719,
                1624919219170613006,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12563169166537931205,
                1337543431748547259,
                6261739039645885863,
                946913929845721173,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13169531019530215231,
                13220013392219966670,
                13293882605017213236,
                2172354463811468393,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4688106273254505778,
                15600539005579537657,
                18174490451936142162,
                2499749785203535914,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10106170585044635574,
                8443299455890917395,
                6153941892451855738,
                1363635838345671014,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12118812823962722512,
                2878281548067159409,
                15778491591491626837,
                2998429629805595684,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8519519205337657909,
                17597038085557304354,
                5707704981571198443,
                1099595161086538008,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2630523648506194409,
                16668640928104981942,
                11461128267751420868,
                2008698455625615682,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4861861427105934534,
                16450706922731596502,
                2274806577708215672,
                1143003695018841778,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9248385425492322835,
                3503751931492264111,
                4097158941807537038,
                1399047836460257352,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7849056258016929387,
                124195536708300123,
                17087066946059787211,
                397517526278235044,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14740388605434383379,
                1394526963135886609,
                12245315269013237352,
                2015830946174324588,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15967138626420918200,
                3261963388853009787,
                13408302841994414708,
                3101844020467134499,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7935217780424522111,
                3150474585965342432,
                10430350468049573045,
                1860415566157927557,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8300550444598703098,
                8194384448828603254,
                5081926569180979258,
                1481859703349770141,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12611928131866712310,
                8001066238413932237,
                16630908982582467563,
                2564785771271213641,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5776022479717480514,
                315400836787967210,
                6411407345483501513,
                775454368582641405,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7983468241182786001,
                6530384842710373128,
                1227871076205175355,
                774116276917908697,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16561898808021670876,
                12053304375562840416,
                13383887841630556376,
                2619657341628821449,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3655855288398640352,
                889129083081445517,
                5828175327972833006,
                1490147266322305634,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2073662369262083180,
                15580333806325490315,
                16292077384379301493,
                2350975138631977179,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15628877033666216762,
                3415674624737187799,
                11924748127564121381,
                2567153654412993677,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7328958824777016519,
                10823345291133306081,
                14791659753864986,
                2887889944335901123,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13855630665324053843,
                10501178583160334141,
                9366008233932927361,
                3168262908198183552,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14787099511755576197,
                16900263725076821456,
                15670188581579193703,
                2720535893433640131,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2278587347688434621,
                8460683120051439577,
                600513776104921154,
                2107071366024431215,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3827773909774360400,
                9970501888602777822,
                4273538910831587007,
                790268952961569046,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4523419177337849648,
                9488089777094833333,
                10749813072890584058,
                247943984908287524,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6645785623322050808,
                2215645231525341116,
                18164576486386842813,
                1417171256747886151,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11359716714275928870,
                13158760755293219726,
                9170086164727341643,
                364709102753245053,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1864005610956645267,
                14627682532745980574,
                17996044607153954114,
                915063943320094432,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13147421729740173042,
                8113636862000265896,
                491770373771261301,
                726315600079128057,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7109981000894997117,
                14414140562548921622,
                9635658832810707695,
                2530736172093131828,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2424040150625817424,
                10020209584497473866,
                1741034247427635853,
                3367777765935730005,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1618513277061117854,
                2143893871381809685,
                4317438611773850034,
                2101213899656773700,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11918488225528054170,
                15691325071362486690,
                6271413291364998209,
                2842713491190332345,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5878315537668258235,
                7324049032174595943,
                14195298237662088955,
                2287662784296132619,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9203257613439864746,
                6429718855841093099,
                3424026684771929535,
                1097620603979641083,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11792602489290275562,
                2421782135842764220,
                7400444687385989248,
                725483761601831176,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16451554924739686974,
                9266340212661447088,
                14745116323630436025,
                3474608689918006062,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9123452450590675329,
                12026001016661503186,
                17873067947688586603,
                1442250471137533372,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1974851714309803300,
                9658711403384797761,
                7349644618341129265,
                1259114708004395716,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4282367373058915770,
                12295516465439671671,
                10004547008128439532,
                505340919029471667,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3155989602436002224,
                2319632500602061580,
                14915087691450313134,
                3251038173509805582,
            ])),
            F::from(ark_ff::BigInteger256::new([
                300511614966094396,
                13891506959569641781,
                4105396343131091747,
                1270038407521844767,
            ])),
            F::from(ark_ff::BigInteger256::new([
                110295059199807833,
                1011811265611881522,
                7855076837615895947,
                400340753466024892,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17373522419694919284,
                14199803395662496680,
                135270522131398559,
                1957609566477128066,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16526944156946352906,
                333656290518988710,
                16142910872403479332,
                408593060323899374,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7519877741871426935,
                2849381969913559621,
                14547709170808193826,
                289790440170711465,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1160550778782128665,
                1748033475305958502,
                15257425121457171514,
                2423437728469806312,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2387225583960457472,
                17546451781213090244,
                10091474758221477955,
                2317365589692013299,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15354468134369365152,
                2030318881570441793,
                5538454975329935932,
                2643746217223991606,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3755464344096463385,
                16502118382615027973,
                15662047153293532172,
                2412239807013991151,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1355383784202019133,
                15391229433564071966,
                14231599175186721062,
                3011691762276580461,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10943667707635023346,
                879627326250041355,
                8215838058947909090,
                3137090335077785853,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12639871776714705243,
                13774826408333175695,
                6507160315858318561,
                1421690321292364129,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14428531898249864209,
                7635960998996435694,
                1177529595922091245,
                3138989117986450186,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17965287558068597472,
                6231040386088967427,
                3327481243063978986,
                194734699531116706,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7351008047189177938,
                5935423743658805404,
                5607140413328677802,
                505383564198973561,
            ])),
            F::from(ark_ff::BigInteger256::new([
                18251897161005737005,
                5714139721146503401,
                8900964639726405581,
                3240878998762076038,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2604844377480997925,
                18281342742356976978,
                12499757439158287733,
                3476597010448164526,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13393399279858765606,
                3362838079496889861,
                3729730279014376428,
                656226523607782547,
            ])),
            F::from(ark_ff::BigInteger256::new([
                948415619160579634,
                1407245575887629307,
                14507663276069742498,
                140852898142369235,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14868609736894024096,
                10018095073075711999,
                16034318610357805693,
                2258875662206573366,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12350198255220729533,
                14141385432830960150,
                14089449728735099114,
                996940051245245030,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15745806857147847748,
                2768304420120410918,
                13865716858704005757,
                2804121637728820193,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6448070426060573494,
                10775508560337890380,
                4976988451822599460,
                464488168976896875,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11351832197668380072,
                5649365891177621111,
                5519533260507400938,
                514744397216772101,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16325591120843021225,
                1596722060692361190,
                1696831840635733104,
                2074430964445240679,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10242487633296831137,
                14247851818792534560,
                3512910125022837754,
                3123318107169772541,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3723956278790672267,
                2484793875181160161,
                7768568334565346859,
                2711656888802086946,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5691641446185494567,
                17565365746014864959,
                10928962674855765435,
                127036456129688136,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8539384502223274677,
                12582135471779978166,
                6772743035582669858,
                424543639836232266,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7739959898543139266,
                2747228099017180169,
                727925549718740165,
                539349171147966983,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13214760878704701769,
                17181691158484334244,
                2490469246157781041,
                217284729460894117,
            ])),
            F::from(ark_ff::BigInteger256::new([
                588949738200175934,
                4997007633763496593,
                15304254876486244026,
                2823776025767547232,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2961506456046643158,
                4552212251933143264,
                12115803219069578752,
                886582335709268366,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17771345806174170130,
                7579548968094318868,
                12289483028666717306,
                2069844761308359229,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7967685166357106540,
                1898750225871942661,
                7490906367958898576,
                1876379507295175636,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17985588331727240062,
                294480873144853017,
                14052303816128488049,
                2366665899754565214,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3183453106044059851,
                18397829102683737276,
                7329160925806946967,
                736825138867573758,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16674873647295512430,
                3883797769065111478,
                17011737372417885162,
                999894754144888543,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6816147502686636780,
                7953816962053992620,
                10900845229840212671,
                286025585516078649,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10633579346396064326,
                8185240829226082009,
                973027313939692556,
                591568458682902421,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4923243152619030901,
                4374494037608804694,
                4281895632105380007,
                1301256188920586479,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3445877326589719129,
                6828233235921872333,
                15340040897603731464,
                409760908970178843,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12338513213425194399,
                17023669412945173589,
                3012980628982608176,
                1119241441416062204,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15112096197487922972,
                892845961559636810,
                15400224076556276122,
                681499442664818646,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8431449892550667953,
                9644615587562671053,
                6356911590609496919,
                2119985056558321560,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13049278204719811412,
                15414198972201113451,
                4254120733524845735,
                564170194057756905,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13461817160879942571,
                5821366526751979915,
                11164740455937138951,
                1263407618705931742,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11569655922292360253,
                12321338903041474573,
                7366023110134625793,
                2620667506169706085,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5641660369300883930,
                10165076147908209864,
                7006265076657507757,
                3236787916751149222,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11903884186184323400,
                11442070584743625351,
                8898948483776946238,
                1967595028877651587,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6938204081740386520,
                1025169413160891683,
                1055517440518612732,
                576557203954380120,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3956771627072598187,
                593935232691868331,
                12104158306666971093,
                1239031101399631207,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7410926834696565156,
                11506680683829198896,
                16233059922558676075,
                2100804306064063511,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8893732907220585649,
                12098416934871809400,
                1733620106625871224,
                508980561143792495,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17975555756457702886,
                8570700788242134488,
                6987524030952316074,
                3467356855359768543,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11703969864212322797,
                6073180566401743495,
                4735863209258681696,
                2704133918423451264,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9721912925259292827,
                1325747538397449112,
                13172825394657839239,
                1258230036591516903,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13909721734453035301,
                12751073417028119359,
                10892115405345833007,
                2170449400839469852,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1455364272970876622,
                2991108459417576303,
                9657724253466767819,
                113379782447524982,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1209791565704265103,
                4081137859392489177,
                11835064164962227641,
                709673802431646787,
            ])),
            F::from(ark_ff::BigInteger256::new([
                976056605486006068,
                4911889327214500849,
                772523946272675970,
                1793584473193352995,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15651195765058081656,
                15496778527484115463,
                8601188336351368447,
                2724950178332843673,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6049463315547554124,
                16647145627599736363,
                17113298417041726647,
                390934470759867073,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5436161768342780737,
                8775637606233769868,
                5994906980810389075,
                2368182334319739598,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16926946406226894944,
                5277272807369716098,
                10272212494607204902,
                340440503872418962,
            ])),
            F::from(ark_ff::BigInteger256::new([
                19622923510031872,
                8734572644310779779,
                14617173792985728415,
                664080905499619506,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11933962792717306203,
                2828119254343441371,
                2636181480828374216,
                2383730146135588877,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10814656870077836689,
                13244340156927910936,
                2039384302550685725,
                2833283213493736402,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4688952747103076215,
                11341151318641627172,
                9976163589030258429,
                925782778211396139,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4924151136635672606,
                11047872697002914329,
                9482582441827311954,
                181196959699654626,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2905380337608545432,
                17464624987303289824,
                14891382136133913060,
                1653236493357883649,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11976449395395358733,
                11083329156360575802,
                2120161934317385778,
                2913707813846752872,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1398867917440056305,
                11016646321354203590,
                11342092192033423839,
                1570876305054011942,
            ])),
            F::from(ark_ff::BigInteger256::new([
                736623328321544459,
                18304755083903790813,
                17125452871046181549,
                776511448761911201,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2168870093725256066,
                3210305763170463992,
                13917793955521390087,
                2686815701181332535,
            ])),
            F::from(ark_ff::BigInteger256::new([
                840281714591956523,
                10261214619349848040,
                2076436207285117859,
                2884917094026879390,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9162562452280657725,
                9212799284149066261,
                15877770105284125094,
                926364264979422103,
            ])),
            F::from(ark_ff::BigInteger256::new([
                18243891585920428077,
                18358140202896244422,
                748646760415987985,
                883705372785050666,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14555943629589972491,
                2495266761742847460,
                16467297697872953557,
                2422590815083138680,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1712100555590631795,
                11886555135551717099,
                12152303205213917886,
                261369801220534766,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4132767694571387299,
                18271995256211408234,
                7141409406897920932,
                1219278969752508365,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15356041187268852784,
                4210486058257000053,
                1077739752026781327,
                2507951460716597035,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15319889299572250798,
                14850694718865064577,
                6250423249973030422,
                1771198161639682313,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5968205123193605617,
                5631781548641980189,
                38372606686582292,
                2745452968575978404,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10316556726073605817,
                16827401578984901077,
                17408562083546258737,
                163474622052768278,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12914615482577353918,
                16049445227246128266,
                11472581243135066503,
                1651675186140628619,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13668205519024143056,
                15175983906859388038,
                3622570529524901653,
                3153677107983246162,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13967003341963913763,
                11086850736967374530,
                8312176490190831723,
                44714892513795980,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8683072208475552137,
                1772770783570938263,
                3630305069928005665,
                1790968708116524018,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8585599462854184681,
                3182423117162865839,
                13868477832726312079,
                803330272646129722,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10661996710026219883,
                5486315193619198760,
                18321618839552498942,
                2879719977073407100,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3310144545466086954,
                10721778900877608586,
                11650509448940603383,
                1175644152716908439,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6918804102741088800,
                9221399707367078124,
                1947002635097759444,
                566840100086756657,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13750458747339835948,
                13275888881686898942,
                17023539933907675676,
                1525143253082312594,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9826358825548789516,
                12281858536068664811,
                1705120528613965019,
                2440463828023267900,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17856192524926451985,
                3936636789273701923,
                13165509904177884278,
                2009579895882228631,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6842679263176394585,
                16567980563805762423,
                731233608712209179,
                1794597669162853158,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2929464555482635221,
                13134826650062025195,
                8359422099164721891,
                10124264694613367,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10566493371346607589,
                17039876145733936281,
                1344630396314830,
                2625842826398269802,
            ])),
            F::from(ark_ff::BigInteger256::new([
                386133427619348460,
                14336415065231988676,
                283562196569463175,
                2098606555875311624,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13150294070713250663,
                15444722327323656977,
                4932495248089423567,
                179967934196538118,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4555947769420838994,
                790854958893302092,
                2944527508913585896,
                3266146574037657468,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6932983785791627224,
                17631512596987502646,
                9351884141124208193,
                1365040695553056580,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5354500673998955795,
                4926856151490399934,
                15152751886127826601,
                3386833807275785593,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7851077646535995066,
                979206998519115,
                11150635267140483957,
                2949089972508706242,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8147593468605349467,
                17565022384931432882,
                17510113614672274513,
                2963723544433812370,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12708101376763062269,
                5043754195621010616,
                6467007314910833226,
                1723376651181267568,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12131297241267027542,
                5866668033890504829,
                17440922801804459639,
                1879827480788876163,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3507892015553586890,
                13650386832458459662,
                11740036500936788740,
                598675179579720151,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9793589755926923821,
                16252554144852708227,
                16637283827327582853,
                3368960814035474185,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2897681432349460352,
                8607223135713950134,
                10072991909178198676,
                1610004346930522991,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10626459338146933429,
                16677252679578769815,
                6408021842861395865,
                2272109712090253417,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4033620625152888787,
                9167200212905329650,
                2288462746602642433,
                2412249167611019264,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16284012449768331851,
                5636541775177249081,
                8141551103513841309,
                402748733203117140,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17304940736575353083,
                12976903433373739407,
                2758505243718496471,
                852009250115736695,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5078080477981893802,
                5880641002963318580,
                15619505732050499652,
                1505593480586182557,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11942233229455486380,
                15902088832949143911,
                7484723644045154856,
                684741429830291617,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9723970540735433409,
                16106368058458332798,
                7158022036060255056,
                2713100956260081293,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8823050016473703654,
                15239792311489878062,
                7194580165888497349,
                3220709949611209742,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1732993032588589818,
                12921666435861389766,
                13858345322994170318,
                482779888798603138,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2397544514766978887,
                18405807160580674289,
                17173112326239514779,
                716646369405005419,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10044021294468282360,
                16606546876719754375,
                13272461606217163251,
                3476823014259275501,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13233398849344983437,
                425654511640168275,
                12234606506229563525,
                32349424707976251,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13008619030652785664,
                1553638542481439280,
                9226933594814013855,
                2093118758683654398,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8177947118074898907,
                14732407590875267627,
                7063362279239467435,
                3322371677445940100,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7470636356360415915,
                15175118725526769409,
                2082736439178045768,
                2961349839434248762,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2253618849330278154,
                15912420302430253012,
                4915776841367865605,
                335779753709585761,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14742940361168294528,
                2248589040381565504,
                13997441105976567245,
                2751548694934314116,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2789126194603259174,
                3202374671645298205,
                5369407959350106815,
                1051656286164493554,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5486206209587040617,
                17886798529185585227,
                7312762573898959486,
                1913055928060705354,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11223064745370906389,
                8330170168544559875,
                10049154318325183304,
                405727658834424223,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5107138435016760578,
                1768510278031054218,
                4877982846027024488,
                1196354551076974313,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3219845091577376222,
                5658840863686193654,
                1704595469222041108,
                1246342845277692115,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16869659093213413417,
                16125721633410973723,
                17368452599545403449,
                2300477042531479218,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16828213430157052365,
                18230178843775384886,
                12708537517220713639,
                2088350841972461895,
            ])),
            F::from(ark_ff::BigInteger256::new([
                969149901257731458,
                715427468769014905,
                13478506672997600544,
                3183536730906654096,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17204581207592231354,
                1881632369373261785,
                13511473817904153507,
                1797675879239010804,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10864722356003826434,
                436971717913516700,
                4921039075918512261,
                302223612231728590,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15880875350848446355,
                17206513181179070618,
                13696887803170346757,
                933159654511468913,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8843464527182664309,
                3420760891656548376,
                8473081154094942963,
                722620514382160550,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13500535930835148746,
                7364807010947006758,
                3491848919600637135,
                1827226870468894382,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17965220661853177711,
                5781485232442308554,
                8055344023759580377,
                3188175103049004468,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7745415310369082412,
                2016078782348072256,
                9035210900144859539,
                1153066188859647497,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17008122416953977979,
                6715205264788718162,
                11070569586372645231,
                708773840765835483,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3611917460783075719,
                6902397528260206714,
                7212556260839933231,
                3023621653466544065,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4790716765062250880,
                18303074171025221624,
                18028328160695428355,
                3060153651088436701,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11589793113579618579,
                7169331945083698025,
                917492353542120746,
                907001372217021543,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2514108024704302266,
                17654996302508987816,
                2267719205600451603,
                918630399251834498,
            ])),
            F::from(ark_ff::BigInteger256::new([
                451703699084791929,
                9525415006079986080,
                8401016022033511392,
                1620254187559985205,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2576263126309894481,
                3312774001207560959,
                8626030518129687378,
                1707026406046357754,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9773220882380925905,
                15285974886986698872,
                2453717134852764239,
                1532432461704654400,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3242437487850784342,
                14466134660478576621,
                2039244012954544382,
                274871477250856947,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17659415096591184308,
                2292825229208295855,
                12854357017156751302,
                2041147208106905105,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9173416224279725350,
                9904143998979049607,
                7240995503738027641,
                1853548584890659844,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7917814652429594785,
                1770801906241204888,
                10444297878741387035,
                94139246016421164,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14572900975455943942,
                16719148704077111272,
                6476773158921262423,
                1114265467667416101,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17961494124849796233,
                8406096611404312062,
                6349752722720672050,
                570125475691131808,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12414445830959575906,
                8918561668582866142,
                14504077508237028237,
                3446719018695856393,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3618566726313604899,
                18024148667658592259,
                8085009024567074690,
                1979738607576415569,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8112399972284654754,
                10657780735897795954,
                13695616841425802796,
                901574700379008903,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10794180891872981736,
                14632681765762274850,
                8451602409697783958,
                2295576814404156184,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17965090464741533268,
                1522812788372765232,
                16891871741913349528,
                3213982998707613133,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7177793303744187691,
                10686227783162472112,
                8733948564668562817,
                2572006994948797609,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8874807366352196739,
                8751146549638043222,
                4072164911806379028,
                925087504888276176,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13428989660638688934,
                1317067624645392701,
                9278039478738486255,
                1743476840165014544,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5482573796159797712,
                15496646964908427082,
                7867498401564406562,
                306660449495275725,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9140631895537796002,
                4845113708629431920,
                8051741032111580350,
                2227027295651757713,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17433790669533953296,
                5883452759732055798,
                2161272199428239096,
                1024689714813270020,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9109662991344731975,
                17631950833939861261,
                9219139324690152268,
                806606809295668156,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6086791392107883969,
                2848029274760076000,
                12025316411709143501,
                1711050782770175208,
            ])),
            F::from(ark_ff::BigInteger256::new([
                18311234655807905301,
                11886599954819401826,
                12553775736220362054,
                2029147832243311027,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6807315637245641420,
                10149619230693218652,
                14682740289959803218,
                1569170921167939943,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5749876827280743373,
                6886276242839878428,
                4381751636189264456,
                2549885203013072464,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13423566475011231712,
                1820289455684459834,
                14752513840207402867,
                2286668026718895043,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9265639810379515322,
                4842462614809047544,
                6386438816308312054,
                1473036594549510014,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16602202586107948035,
                10646424276112478305,
                17701864041809528461,
                2413369513309639958,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12985020496276507676,
                812812526622195377,
                1636649594584676664,
                2621116647545276730,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4359910437365171695,
                12085264673923412252,
                15399864391429824724,
                3121598488997949356,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4733862548372373538,
                2422503016217820153,
                15480775332612892887,
                762532578904880890,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10452985160532596590,
                10957487220976020213,
                13521400523479286002,
                1905454262537497513,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2742514999470979772,
                17135732048370903548,
                2969621941989905595,
                1568878989478438320,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14558298742820271151,
                8713882989662093489,
                14758339328635352489,
                2054934716325493654,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12998243183474908708,
                2911544296799288727,
                3423695590358795536,
                1186025027121611058,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6573344432486349930,
                691501357094612255,
                7486538751129106847,
                1924232325171279415,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16202183101920452815,
                12570171034495147346,
                1002441180490971289,
                2104011023133714264,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14706887129817451877,
                12559469648836827413,
                5008307429511392945,
                2397973759991210230,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2379636744856443569,
                7037723355177451080,
                6857438800578474540,
                353918331165955177,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14901072681474195118,
                12311325484281525922,
                16343266326279085464,
                3225001715927708063,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12810943456105274379,
                4775083719117918691,
                8702332993268566452,
                1591449440221964966,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6744419592302360019,
                2541950094621165437,
                9187323496102782904,
                381414143671135185,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13035857823115327251,
                5644338573397394676,
                781562203110684562,
                2379360623684191033,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17336101178738551230,
                11488604414068273383,
                15938620665564126535,
                1953873464514394456,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6128259995022457913,
                12454046939554023854,
                5889873181901216168,
                609778124141481083,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6372790488054960212,
                12201581673180872073,
                3837909810458864108,
                2148747813250814752,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16496231621568899608,
                12124696891600496487,
                4388543932041050160,
                361839037465251420,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2965368933465367251,
                3148443494113807326,
                18058953651222739844,
                1519246510120074722,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15479992265191884222,
                1048513586318522048,
                8070541973826920769,
                1788116209765948046,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6696072070116171741,
                8368339209834044601,
                4531969564739737244,
                2604388482614280328,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17911185479563048909,
                343843235453470709,
                1490992068472172550,
                104944417341399850,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4494917060810144362,
                8725076227062923491,
                15885477132354195768,
                1738676193573864751,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11301270482228421365,
                15896746255796514637,
                14669495197056756183,
                824371042011111981,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15242993577499356320,
                13374842422067984090,
                508752023688846656,
                2140418658747464417,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4385926662253030066,
                3262295704616811463,
                3690316070550611875,
                2555675793919291282,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10055747842573306498,
                10068188327008853819,
                8700293157466275929,
                950555327928730844,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5918153798238108476,
                12755420941938245298,
                5224908503823660156,
                3065813226431477484,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10808665731673829036,
                2393828603212235048,
                11258968314276089251,
                1426428266891634272,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5780003182597315392,
                5064247853739100979,
                3738431480346359680,
                3090615470060214055,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17804346911147246387,
                6569995923812294662,
                11648618227636637293,
                284895169888680573,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11074977834694122704,
                9264290168631344448,
                14686616734062275594,
                661978555837340668,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2326443045228063626,
                1238953533371304765,
                12906714983645311826,
                50076255818103208,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14669853212383476851,
                9212379078723722623,
                6901402445813243983,
                926232607672140268,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10639918538899902715,
                8689059026270354270,
                4081793615842623384,
                1296708737154045481,
            ])),
            F::from(ark_ff::BigInteger256::new([
                238611820762138162,
                6778436964845239924,
                11447324842695804589,
                938640997294605338,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8859816008572579631,
                7199865475326251204,
                14426703326736730361,
                2519653205482415471,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17542432957694180971,
                12847146455949466018,
                14402252818435618808,
                3148839498920740935,
            ])),
            F::from(ark_ff::BigInteger256::new([
                18081958748468984113,
                12877870639592911976,
                12118391278343816523,
                1206152578040589284,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14881070377541526980,
                512576322870016595,
                15112333787488647671,
                3404780682011094307,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11982849735492342285,
                12088039013584013251,
                15082283154877598665,
                1128108557261537674,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12683322874711033302,
                11317297875230580418,
                12553320977582665921,
                832226082844293057,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15639856961243781746,
                405394334564759502,
                4701253721016112978,
                1213303802532832261,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14117400411313687171,
                11661518363361345604,
                152400301892783644,
                472339390933420782,
            ])),
            F::from(ark_ff::BigInteger256::new([
                453327157578157142,
                4159389438149380994,
                5291564263461851106,
                3417504293339665046,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13759790782116664188,
                18323100985454458145,
                6603951217486199538,
                894601808676046955,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11675548869900021293,
                14102329021832814557,
                15642620071614719399,
                2191860261716202409,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7805340003198221707,
                12991009009075253196,
                16831118683192128012,
                1954868970479577473,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12337545487159299670,
                14821584430587008671,
                4161356446022149568,
                360638717761123294,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4015291653909916883,
                15965449498971912720,
                13935509282915747158,
                593704445919807108,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7936391874231239302,
                6535363810831910717,
                12698756765225938075,
                1949489487554783033,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5016862425462509637,
                12841985321387061300,
                12233084486354141847,
                383929457167666984,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13573003676141020641,
                9470681275194592365,
                13447686568444013020,
                1606702238118574343,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14958244504929792987,
                9377657771538894995,
                11113300060728482155,
                3381896870886077474,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5356121088733306883,
                1558405737804539884,
                11502726054942970015,
                1920706737044884324,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14114273942502929155,
                14089764410170283978,
                17485377308826340060,
                2901556693610942807,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1288726275089231669,
                8739474136807168387,
                14160071243951139766,
                2628991153143719294,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16817150083690245037,
                11907746759583563105,
                1945569465766537648,
                2405513992687880601,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16560087983094391545,
                12326273608811295334,
                7857073032265521764,
                3291756055471584857,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7005559843494862669,
                7623145056534556205,
                3862478901107055463,
                1803351895192171688,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10213628153658223409,
                376634756350146086,
                3442225394002919671,
                54797138751699881,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4282346611705924555,
                2253064561004887694,
                10145537115443020750,
                2788437451283457423,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3611082780443069613,
                6721013603066854496,
                10420704419036570828,
                2029854791842459376,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9245827372962078200,
                7985505560632955514,
                7877542779893899024,
                3304022132481928238,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13142874298027376374,
                15615641549411535466,
                9939547749566740531,
                1265964009951584494,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6887012740529552159,
                8017075952070013399,
                6350143539745064284,
                1331854660714441024,
            ])),
            F::from(ark_ff::BigInteger256::new([
                180569462582539803,
                14376451137058096984,
                11072550519691613416,
                3195639892058224868,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15137316622623550590,
                8393126165005807653,
                8353349529793354677,
                1419274313358041016,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7905162104385193923,
                6505376196396994891,
                10272824684668836318,
                2749727014705609823,
            ])),
            F::from(ark_ff::BigInteger256::new([
                18062332598535842491,
                6946451576415048289,
                17230993401487937037,
                2557148901401865776,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15719101845995101220,
                3182201285143927962,
                6506273456424223958,
                2169280936320392539,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14552571448292970328,
                772028968177580164,
                8928914334341821272,
                1134580764418077918,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1323601468548124792,
                7877698046157210752,
                8192571851807576113,
                811805233677312666,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10120954910615364085,
                18161100597590544284,
                11787820911433853473,
                2399951559872479101,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10829724305765064126,
                10339945600740930821,
                14939776015588299204,
                1976591384704856995,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4391678205158103461,
                12081792610057798912,
                11465957525200582888,
                3221777714779243433,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11029091259291030470,
                14907571244466840285,
                2008231690233070759,
                584354542866472532,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3741189644808754244,
                5336122838333180101,
                17707653844744812696,
                1810720036957165210,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4112502333249072887,
                16638853303753855344,
                5153179265171405354,
                2004467652391931195,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15478134057783341818,
                9690251914477806042,
                1076134495890944087,
                3215964862483084478,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8518256458019197797,
                7204595270709607586,
                14006637151170012246,
                2568385958785273121,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8523570243714790306,
                16661520918882904109,
                17373934166120688234,
                40567910629182006,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14359995256493350987,
                2341206076910190293,
                5866591652295599513,
                1939244215152718392,
            ])),
            F::from(ark_ff::BigInteger256::new([
                26098238516360271,
                8570337290510669457,
                16976952141088933849,
                3177487041118860123,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5924321898600686511,
                2441763613894872461,
                17448350536122428448,
                3299612729278537385,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8850806227231597658,
                18017436945811395989,
                10836082831464811737,
                3362232787167643205,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1190459019871640111,
                12848180788106283710,
                2772415988409162162,
                563690404398587704,
            ])),
            F::from(ark_ff::BigInteger256::new([
                973974613954364972,
                3668834197417650286,
                3754469655224382440,
                2550235168638295884,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10203605103019084721,
                7023537713551715709,
                13836353948458854074,
                3412752947584271692,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2397801727679395567,
                17523452427427451302,
                17238023994324438960,
                2994940682222582000,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14399530012481014609,
                5756368334336480924,
                16156117716969400218,
                1015858607712694316,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10991806977045180287,
                18419193668661749038,
                14663866100434944011,
                2524684867628129875,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7922211989439153491,
                13924775220571171242,
                122157713004900091,
                3465535406601897481,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9478562444490547358,
                7352788189399822052,
                10579132337366522718,
                1277439381186000350,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10658242688224736458,
                2529390229131799527,
                8218650571847041656,
                1871455562607741463,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7686803764741674267,
                10970664480139806967,
                3678792621636540908,
                2850560351815611920,
            ])),
            F::from(ark_ff::BigInteger256::new([
                600773557762311042,
                3865388151069802992,
                6162101260398146404,
                2542786380547416767,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6643233365240347297,
                12352834209477022566,
                25960993034182060,
                515551613325656353,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5965511656475033382,
                14557779400183954870,
                11238197892527176782,
                29917499869891589,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14183243739143090441,
                11903376006969994437,
                7225046312636445580,
                38137396638793753,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5925593263362973559,
                2654672361042766191,
                3779378018660278939,
                2974990708929711182,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9448676023314791072,
                9514385565752657600,
                8221799997709242804,
                441500800843202000,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8362114093354124572,
                11032147563569948439,
                12037741477319007257,
                3098709223185006854,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9707976719483464939,
                686626752421992607,
                1811942651260291103,
                2722294990572317496,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11422536429538987613,
                6424347383717181436,
                10446603498032657758,
                735104320958753536,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2441104272336393589,
                11510486992668692930,
                7761213732731370652,
                76523624884399820,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8112605991720156373,
                16499941151678612848,
                8519426460759440649,
                633556126072139018,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15281095756403395960,
                5352302258990168045,
                7219942978514440090,
                3145992187852065800,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2979608422584477871,
                2313480078318378707,
                13883226328602254080,
                2885781163420358755,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13143900244341445757,
                2817209490104626233,
                13427293993341520974,
                2611036368940860863,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8647759855901479050,
                1911989958817786652,
                13234732296099681931,
                2285761372460974197,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2187166284701849919,
                4455073722484773207,
                726340960934757774,
                1115449504756897693,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15832241187341096425,
                18417707004990930153,
                1849951626402553920,
                3070365053625781714,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16225192244377941093,
                3932345351296423997,
                7923358670358145498,
                3292441896028510506,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11092899358235763610,
                4189754034594032205,
                14247101393540051668,
                3012173605405428090,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10630211517692173937,
                11103218712917527578,
                15818302856826454342,
                1499788944096129974,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16475138402489855254,
                11497447163113733737,
                3923265420519424390,
                174020269048122183,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15527930647401881250,
                1469777097282419952,
                9464858132738218956,
                629802971682475530,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1581002092052836848,
                1497582235695352519,
                2653291601829913105,
                1870860496798886865,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17698592609223313431,
                9865970122438623729,
                12043665270046283518,
                2276929691845860107,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10155495496018117811,
                1683784949585782370,
                2158270290578635825,
                551830432694828854,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8567199496778518797,
                2572331807087853103,
                5857937308597469714,
                2760672906710756356,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5207654060660914588,
                7534172574022866794,
                14313563710947781718,
                2719253349266969791,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10531611660878601471,
                4782422831818866034,
                10255338481953683582,
                2829008423976054774,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5245365453067337280,
                4214995006664288748,
                7483680236294613281,
                741908813673254569,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7569742375889612842,
                15512552606529409181,
                6524698362399874317,
                1598140113109508783,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12309103044070916037,
                6934088745172660033,
                2163635988286770552,
                2130435348434279066,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3779056747078640988,
                16430252977105769398,
                8165163838191278324,
                3357391811868987916,
            ])),
            F::from(ark_ff::BigInteger256::new([
                18428345597452174617,
                8464053526827968700,
                9407205801447987363,
                3164479114237627257,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1356567156093248372,
                4524852900133227307,
                730110115764496541,
                3341201298839317898,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3202525046730235290,
                18265978340284062629,
                2038277458962409978,
                3311452271911354566,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2117921319503781969,
                9177057994695254687,
                7838875167606125806,
                2151203710355329639,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11705794160993273547,
                8323155959613528623,
                2439836961398007215,
                3427783438671434051,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10701697302229038339,
                3350456314792364054,
                7687505573486808644,
                667420770529635857,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15289296005719321675,
                7323917197262499624,
                5076025622719219050,
                1012715260416627560,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17409816131929594167,
                4007461908550970841,
                4446810491010970858,
                863595855594756070,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8163200734378067892,
                200939071725933303,
                4565077799751631429,
                3275283980278992777,
            ])),
            F::from(ark_ff::BigInteger256::new([
                594423551181313558,
                566361663114003271,
                2949508825847479997,
                1190396185896134836,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5418875495071260232,
                12194967121056225383,
                2314367884048850375,
                890165014064260787,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16737825098216013962,
                18362935470651336495,
                4840411567476195948,
                1669153352148312256,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11471878116308823707,
                5422397098533489310,
                12823152680886099888,
                1109521855653827454,
            ])),
            F::from(ark_ff::BigInteger256::new([
                921381686877148924,
                4178074860075525439,
                8829082328326460857,
                2360742534628864599,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3324496496491093667,
                14249958387333979894,
                18189169623785843150,
                248822495912812798,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11985941081564125147,
                6604498436846482458,
                11242726895356822327,
                2536857835112946550,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10954397342680496058,
                9417063334711916457,
                17058902994467399059,
                738063435087776762,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8872826878686818053,
                1012377036480115516,
                242639108684276645,
                65997886509451975,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9161080275406554328,
                14337754285802557792,
                3823864564237148639,
                943085182411492251,
            ])),
            F::from(ark_ff::BigInteger256::new([
                422987984630232271,
                3683521400913747303,
                7154597974233305052,
                2901629807476585179,
            ])),
            F::from(ark_ff::BigInteger256::new([
                2015489735994879423,
                11529250380653511180,
                14313740956790827860,
                3038336858417871334,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10706967294103848538,
                45697621713279227,
                7802538260480810451,
                1283353193281268626,
            ])),
            F::from(ark_ff::BigInteger256::new([
                627668112864843625,
                3993506187511489865,
                12817583849721667951,
                2955588958840897477,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11164448729072033569,
                2515565815215248007,
                14393894039586815821,
                395828822253163763,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4340417880744620541,
                4205688216642407052,
                13620334406163158987,
                2705614719722301845,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9029466522798836844,
                8898095517833975899,
                5805939591423794759,
                675230978476145235,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15048003120605842977,
                2702174462558530147,
                3576827411782104678,
                578313500411492406,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10129889288099128060,
                13831127533772461133,
                7891700311751397182,
                3354245350313888514,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7665301101195432266,
                1714066101333928558,
                16131937463611539533,
                1236992626833665664,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8791807146825819503,
                15137314783150313460,
                14016094627064734531,
                572446609744257524,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11233472154585347670,
                2379539473678250807,
                14111277158605599439,
                148533377758440735,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4493624059835398911,
                1260878784534028913,
                10677237412725530883,
                753843446903691789,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7994111676963272290,
                8338038420594185278,
                7063989557722863726,
                208072297204300110,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13777278387840829582,
                7774293390666356418,
                6994835883507097392,
                1236092971293323350,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14930039751769525893,
                15261894221242482780,
                9312608332644873647,
                1027224620066456130,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3893497371818141095,
                13961237366081189777,
                17657644853411394089,
                1316995071212575480,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15731912511007063476,
                3806683881636417028,
                17139907224662248274,
                209405746619613871,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13219316893297748002,
                16824125843933074413,
                17492669380002962334,
                808465703185235226,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6803638613927304274,
                12562094482684078799,
                11732167699762516680,
                93527433926564741,
            ])),
            F::from(ark_ff::BigInteger256::new([
                169389557281371701,
                6429627227858183745,
                17552064216974603829,
                838700173527397266,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16280565575249410315,
                16310542919129358412,
                2685889789395553124,
                196982650014776157,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5616356562578165923,
                3014582682805553041,
                5502707309397776636,
                2085276028655884074,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16314060910800643456,
                6401917100927008649,
                12773165280906125982,
                2754740571506974019,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1015366844509117316,
                15843804574101550849,
                1311830959241592112,
                2887347872678867257,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16612345464556565165,
                18094269540124612864,
                1569761246613460575,
                1578091200822805787,
            ])),
            F::from(ark_ff::BigInteger256::new([
                11044307633909875717,
                14955127434603925392,
                15974545852967749236,
                1881367576198343859,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7827361377604035139,
                1145670500998874126,
                7886169657449758092,
                654005490095303420,
            ])),
            F::from(ark_ff::BigInteger256::new([
                7989009091462871640,
                14439387444800280689,
                10271249728685444121,
                1154990795001153171,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4349680589564504527,
                6467328880527819412,
                11347093120625861744,
                1324978255743471581,
            ])),
            F::from(ark_ff::BigInteger256::new([
                615593403119455311,
                13844045839147961886,
                1191661104378970109,
                320505081833586824,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5795657098595121350,
                11049905165223947513,
                1490489215925400573,
                1032082603431559498,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13009316782803416384,
                1539290567144388907,
                7987073917820865855,
                1805433820059970681,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8101589771400844164,
                17176058981078963093,
                3576368787524572360,
                1801850145436736691,
            ])),
            F::from(ark_ff::BigInteger256::new([
                9895366802429364071,
                359069494169695459,
                12025335035924384325,
                1768189312267373332,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4158326998970466407,
                6063188074827602348,
                17955764330148620083,
                1491669542491329459,
            ])),
            F::from(ark_ff::BigInteger256::new([
                18165563099499136365,
                14935339838532922309,
                12033897133451002762,
                2413834891535238752,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8425112091950554040,
                5953697229170423588,
                3540317011512558109,
                3167268361884270166,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15922521750049620253,
                4672719281394501687,
                14719955264187997271,
                479698709102287156,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15750632834814221946,
                12928085278190687833,
                9070673386065363106,
                2791188819157489046,
            ])),
            F::from(ark_ff::BigInteger256::new([
                12330708334969699096,
                2358733681430042767,
                158964150326913570,
                980592176447744421,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16557102345709210428,
                12603350885004677604,
                13470048961818506305,
                1375056968693967227,
            ])),
            F::from(ark_ff::BigInteger256::new([
                6718085397905181991,
                6362961633345481934,
                8454065650295278893,
                1944449593484485804,
            ])),
            F::from(ark_ff::BigInteger256::new([
                328208354807739485,
                8501131608923885265,
                7950401892244721670,
                2903004285687151029,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3660635041246561989,
                1809043093008887779,
                14422823941542423686,
                1439105650692539028,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4595344614439974800,
                14528277641243140578,
                9028711803107273855,
                3099343984482279853,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10221792935050796461,
                4867980768152729510,
                15082191862784486905,
                3157062001905356891,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17735724140416712906,
                18253364930740505067,
                2680068401888238169,
                3391807460912016830,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8061777782083599919,
                15401713115324714427,
                18446025958822097263,
                797640806322592595,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1503432840977132444,
                16232854320290562396,
                2457063197224358714,
                2615920250232500390,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15152483176757394682,
                12169921183471948882,
                7841655099785227199,
                207157012224852118,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5954832704914781975,
                10989207168720247532,
                6767179598278113708,
                2465203551911990684,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4632452865507404368,
                18054681259392523243,
                8008166821611526744,
                819053288295391770,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15160727793112068366,
                16995299052960483904,
                6308187122630371741,
                266818240615042288,
            ])),
            F::from(ark_ff::BigInteger256::new([
                3931563633418363789,
                5110993879323860019,
                5870320209333059472,
                3305599957921016464,
            ])),
            F::from(ark_ff::BigInteger256::new([
                14501628718838290483,
                17916823194565106058,
                15501766732740048715,
                3355902063658987803,
            ])),
            F::from(ark_ff::BigInteger256::new([
                13052852401424699505,
                15895177474202202207,
                8721578681989872528,
                1123379062363821481,
            ])),
            F::from(ark_ff::BigInteger256::new([
                1221980192351945181,
                4178735466387675158,
                16466391257712721883,
                178596073230540207,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16974739331684539953,
                11853641750499284202,
                3001723982731125320,
                336780137540533888,
            ])),
            F::from(ark_ff::BigInteger256::new([
                978373485421047692,
                8372568917600189458,
                1483833188362380766,
                531197782841434581,
            ])),
            F::from(ark_ff::BigInteger256::new([
                4666933637117073501,
                5876665058356520436,
                11253135415903512619,
                852234289097613175,
            ])),
            F::from(ark_ff::BigInteger256::new([
                15884373983723211997,
                8111403652001794855,
                4276441888690784561,
                908546893264819837,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8866428312699848434,
                1899572255188295879,
                8227572145458643757,
                2178340200163206709,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5894041548351985979,
                8668440032302739694,
                12395030351867429136,
                873631513371565866,
            ])),
            F::from(ark_ff::BigInteger256::new([
                8937248957884706698,
                1279433951606206668,
                4994740404025231581,
                3077651958718141001,
            ])),
            F::from(ark_ff::BigInteger256::new([
                17029710245604451432,
                6677965431517268924,
                11096960646915688675,
                2047335043558627665,
            ])),
            F::from(ark_ff::BigInteger256::new([
                5732305911749430600,
                9073828671573934930,
                10150812331105606226,
                1599710035642046009,
            ])),
            F::from(ark_ff::BigInteger256::new([
                10011466920930008745,
                9091394326088025174,
                6701518547973820387,
                571831633973433080,
            ])),
            F::from(ark_ff::BigInteger256::new([
                16864740443784839800,
                1984399877798825926,
                14016485366012444759,
                994242657025515208,
            ])),
        ];
        let mds = vec![
            vec![
                F::from(ark_ff::BigInteger256::new([
                    4156222283127006610,
                    2576966725917287048,
                    11469133228845717554,
                    938523259136714963,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    6191534145221952873,
                    3390555444566475373,
                    15467715681119670074,
                    3111976289233385203,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    5296773495730251258,
                    6821256314901034337,
                    17289938784009476097,
                    821212357641505726,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    11632361144471566095,
                    15187290104846507774,
                    5865880534534933611,
                    1261345427510100985,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    3382566304987030720,
                    15025621710563151230,
                    6208871011437276237,
                    817760083313279411,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    9930994009022809178,
                    818472115569774776,
                    1358165537922908220,
                    3099838837379503168,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    5980193944486321220,
                    190226155688025313,
                    16371164848031446209,
                    3007888066289001261,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    2919396551198580407,
                    17943220441042295479,
                    12040068766413999505,
                    1821423053350454229,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    9314588206698328528,
                    1721389724085450511,
                    6187820859417924346,
                    1942118387666999994,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    5914884922121175747,
                    3833741754935206335,
                    1009767280434354432,
                    585283959895493904,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    2438637046736236637,
                    15000153815796678017,
                    12207160111414100311,
                    2497336907372432012,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    10093260217473075617,
                    8658337804537030111,
                    7536799500029032994,
                    803816128718551427,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    1886127464563810550,
                    7305776834940125286,
                    11588391329762920867,
                    339007086387977328,
                ])),
            ],
            vec![
                F::from(ark_ff::BigInteger256::new([
                    11138482299652290569,
                    13265416706758951282,
                    18134069499610077686,
                    3282121388096335262,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    18015005696228941918,
                    17183822795142032309,
                    863542246500247680,
                    715848670349221149,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    8289278386073354737,
                    3874093091987728659,
                    14219285048366807797,
                    1466166119966654864,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    10976628014959805620,
                    1774287869803588021,
                    1822145494642739222,
                    1400322965363883878,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    210406921669500227,
                    15575967250449134061,
                    8753360539663771676,
                    1786800819138576771,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    5779639489881535252,
                    8128600885980009251,
                    10933902111409290014,
                    2483028938262197330,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    4538479273044989007,
                    7028878048236473488,
                    13947143494983754385,
                    1594978304058585924,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    12204883956257380862,
                    5578957875009969594,
                    9251956120094177956,
                    2987678262835419557,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    14361426614748938699,
                    9432992343696561631,
                    508585988029579387,
                    3148217008974314769,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    8655385854177018492,
                    1736863330222208803,
                    11354225642559056752,
                    330203633318291129,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    5527178138238069150,
                    14285126163144528336,
                    899799923210844992,
                    1691114392137505959,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    18303394512985295122,
                    18291417508409196530,
                    13816872809926651557,
                    1807022198661173314,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    7278031143799317992,
                    4735022632294142894,
                    7872262683970443956,
                    3070949352362522182,
                ])),
            ],
            vec![
                F::from(ark_ff::BigInteger256::new([
                    6929850980466786879,
                    4487156175874547055,
                    11305521237070620559,
                    555667650203486692,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    3199263630111641681,
                    11488479728737885,
                    3072088774912015212,
                    2870689725155212457,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    11531397659136611543,
                    13988910817897367246,
                    12452197043886384781,
                    2984174957324553009,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    3899841979835044625,
                    3797149904811616090,
                    17653968344391138784,
                    3010229525716914208,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    15539464153154011771,
                    17878535810514555445,
                    1712784599476165843,
                    1316305292646537655,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    17738015494362259022,
                    2900285357608812025,
                    13488178297370121422,
                    3206045020149295267,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    17374511539925816455,
                    2966716896408797907,
                    12249547284711212321,
                    1611175512196757062,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    8281468328501390671,
                    4278671131463752600,
                    3863165073111449270,
                    145910917042380190,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    1560405811145759906,
                    582284356918286074,
                    16933002432834618556,
                    664050947586107252,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    8020931682669714264,
                    3920633416144555254,
                    16529310878387613639,
                    2178074322997868981,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    7000958169354441848,
                    11004833171811036376,
                    11030462426443541219,
                    2690130720129533346,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    12423755208293319763,
                    7510343311375223923,
                    17403569314935974130,
                    532910490974697541,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    3740793436950713163,
                    16733341149645434835,
                    9233997987435354610,
                    3429701157827557263,
                ])),
            ],
            vec![
                F::from(ark_ff::BigInteger256::new([
                    11989115087072075023,
                    1410348597508702422,
                    6232201024971843643,
                    2529218372333372789,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    4843808549364103127,
                    9953264037088735868,
                    5317005076425183337,
                    1819228454306576224,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    5444474958697745407,
                    13634134040389549312,
                    2827727845010459948,
                    746936017465684388,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    14514663534648028964,
                    4570679811435633358,
                    4762959067081597502,
                    191458848487782375,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    7033886115390171304,
                    7059024674366727467,
                    11850325547356354072,
                    687839926312717822,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    18373761097116648624,
                    9181507856259723060,
                    2024759399965568550,
                    1159476766577132825,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    16834712585818560177,
                    10964309710456384960,
                    4328255222960888756,
                    899415944214721990,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    5557733799796303899,
                    1782275687493463358,
                    1793030320648034688,
                    1254869584252475868,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    14865204843339318239,
                    11317903747536454681,
                    6414617612582827462,
                    289568456499656672,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    12525247270976379353,
                    11003775163592523106,
                    10947973858251528226,
                    872609171033314500,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    4597686119125751379,
                    17926495043561582303,
                    7532971639025860004,
                    1282154065396188026,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    11777431256109613082,
                    6283502660159747267,
                    5467627770761500238,
                    1760650215429693708,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    3361871064120637745,
                    12710724076600064792,
                    12885042294624305583,
                    2169988593426098646,
                ])),
            ],
            vec![
                F::from(ark_ff::BigInteger256::new([
                    596847239515178282,
                    7209299815435195333,
                    11814805954131632299,
                    352541304722005082,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    14607481924649840940,
                    16456538380732095149,
                    4439340973105106633,
                    2548208845395653106,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    11205664478639517905,
                    8555315413312395111,
                    13801678630061767232,
                    1304201163100865499,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    15529044287671509950,
                    13313038255055583607,
                    710616962820568739,
                    3121028695412865704,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    9376314248762368805,
                    12512261853202099457,
                    15127582823441108167,
                    1095742037699668695,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    7041475228950812819,
                    16610756263738246126,
                    13602552782211605623,
                    346824108189088336,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    13224410955020836978,
                    7514107529075039182,
                    12235452849363852879,
                    3156041368930991694,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    16904411389530147601,
                    613804859317424156,
                    13781696218336109202,
                    1144648074389703771,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    1465006810694931673,
                    1489968311346622436,
                    3927981492322350443,
                    1995667585452276416,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    9715011384548804027,
                    7310879001787220199,
                    958717391646913794,
                    1104729657139644238,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    5287867968542801962,
                    5602542034148742241,
                    1867126922501446566,
                    1589588977156366208,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    15518065325688997996,
                    12724800539987406976,
                    9894438692123593398,
                    3403995151585056352,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    16029803019830499077,
                    14500980991256622586,
                    9798964205114145991,
                    1970812336860436809,
                ])),
            ],
            vec![
                F::from(ark_ff::BigInteger256::new([
                    18253535533125793780,
                    112323020349665049,
                    11993618587708670387,
                    760181116924529757,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    3180166422755109930,
                    723053938530230066,
                    13557307025815828143,
                    2158853626494910049,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    15525341060735071736,
                    14832109342433138255,
                    13114823813245280541,
                    1388288540904782858,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    7846604268261002633,
                    10527936183732745651,
                    10915860314682459438,
                    282123877547519397,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    4592861281684654709,
                    16625518233219534741,
                    3149680647452626670,
                    495462621540451666,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    7817093270569869920,
                    10877427732227514083,
                    3592209965820258143,
                    93530639065228722,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    5002587333489930444,
                    1523426594405835546,
                    3100100173793070016,
                    437391025308803078,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    1282247388617829998,
                    3876274682011191982,
                    16859219536645820679,
                    2499277619823414762,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    11140541166192023007,
                    11804746728318236311,
                    5600277291742250709,
                    2999108570803107145,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    15866078025696868635,
                    17896746453023590641,
                    9780053982665465929,
                    1170030147326295568,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    2114886901745268786,
                    3715564468408928267,
                    5739684596875329954,
                    2336510763810260035,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    8965725716209040518,
                    9818700921318317926,
                    3400619666804230954,
                    997650461252991186,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    12279582700304590797,
                    10757880970990130950,
                    15270218870687729469,
                    2652069098941162791,
                ])),
            ],
            vec![
                F::from(ark_ff::BigInteger256::new([
                    6635707400654659016,
                    12170831545671629483,
                    17017715475528253996,
                    1782211603218771551,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    16711702989432593804,
                    3095374687583600688,
                    12093391716991935566,
                    2139775104219493924,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    13401409036450054805,
                    15014403513956736595,
                    17556716080286621254,
                    683528118055009864,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    8943459839651788319,
                    1326398006681121531,
                    17002900622950871963,
                    1517424904442956685,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    8024034772867631474,
                    11721828528119387766,
                    466848876695184303,
                    1312786436598738336,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    11636086479359710418,
                    7767716781044489991,
                    2768968153739398077,
                    3287251270559298344,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    16398973607353394002,
                    1974721765594618198,
                    9526429831351731158,
                    322000028453262107,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    11697298604031766710,
                    15380445503573865150,
                    3354893313663716693,
                    2128948800163990951,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    15574936233930735881,
                    14588399945742708331,
                    7884646857565262676,
                    2335858523154592432,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    7264565348698074794,
                    5984068310483710915,
                    14923204523792143181,
                    2943826976202952766,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    13213082684111708440,
                    17591227327938905889,
                    4117432038129245231,
                    124204511405722691,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    4998749247985560390,
                    13086299108898050979,
                    7537715174207616513,
                    2560352352709480086,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    9663126704522931909,
                    340366317835313046,
                    7472968469408810085,
                    811695745165152192,
                ])),
            ],
            vec![
                F::from(ark_ff::BigInteger256::new([
                    4137931148615820511,
                    10449412695405010632,
                    2654690054129597858,
                    198068339704001257,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    9948783535081686479,
                    7749276963025247309,
                    1196324374058850378,
                    1893379254767590438,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    7334538918964646404,
                    13129220068813661468,
                    9405094801683494098,
                    1188282315260506445,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    3314346741049847110,
                    18077721505748942032,
                    366670458229195716,
                    3118095946294295938,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    1798248109637686565,
                    6987026883788902145,
                    10539136784462721320,
                    2342691012134326853,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    12726745619471697031,
                    10742263333198523018,
                    9115693511467035979,
                    3475263933573338576,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    3340032884869948923,
                    12980701270731233638,
                    4970092723861255653,
                    2982316726788572920,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    8466678295268320307,
                    6560375335978322372,
                    9920850731444442337,
                    340013877010664438,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    14110375598079931164,
                    11078997750489431490,
                    2923991006942153254,
                    2708148913086706338,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    7696447433203408881,
                    3904874852608178890,
                    14689081875358347838,
                    2851233568562338975,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    11373243846313582383,
                    6340097761029835609,
                    17268216835539712221,
                    3139633349402695255,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    9902850743928176044,
                    11114491039002326287,
                    17058626493232876443,
                    2318853420406552089,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    16340116219861922732,
                    11664602595395438407,
                    11051819278066504112,
                    2749263933633685814,
                ])),
            ],
            vec![
                F::from(ark_ff::BigInteger256::new([
                    7333476479067550098,
                    12937440736602535961,
                    8905096784388345338,
                    491175696571814242,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    6503213390694568362,
                    10905988067689223650,
                    15485145390497121710,
                    913809388159818598,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    12962003948989813166,
                    11372220811563227686,
                    3974856551703170067,
                    1307569170548989322,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    11747421782194173074,
                    5991975923576399117,
                    3156549445396184659,
                    2560689834420501242,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    16327717470506070962,
                    994614544540829137,
                    10109282824219184731,
                    2568988316016742179,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    16300587234717829472,
                    2569501156219954360,
                    13886487578384967333,
                    2657469271349485071,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    16888705425522272248,
                    2527555801928970342,
                    16262773682701540481,
                    648814610626257066,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    14858334965229294295,
                    4655147256049353064,
                    17112218245802309978,
                    739124307512127445,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    8954711619719921781,
                    14426236325317483720,
                    16323768606801423086,
                    3442950784522320925,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    2765604462446427834,
                    18176528215067876556,
                    5508432289265632668,
                    2271753655433031008,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    4499318692041553372,
                    14252826299261217680,
                    2403574147162492814,
                    1742206487791049254,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    5981560270152784560,
                    11771108630765244231,
                    8372522680381861699,
                    467577632690926841,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    1926150444824776063,
                    10146031456580143686,
                    14209427668074318503,
                    1057007971088624467,
                ])),
            ],
            vec![
                F::from(ark_ff::BigInteger256::new([
                    11714301757080886248,
                    11948686290112646695,
                    1174361908730085061,
                    1778748087432378463,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    5448738923446496233,
                    17165801824043237,
                    17417959385600829783,
                    1824054258536407629,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    3179852807796944518,
                    16366272294217608831,
                    246665133967966863,
                    2131661400743796006,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    5652308915945600666,
                    4421976841984392641,
                    6993786808221055572,
                    247899566688230726,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    11696745542693585266,
                    18046560568070370942,
                    10861000649550592791,
                    2513139037383868393,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    1329596765006859449,
                    16173749336478716069,
                    2921275956860619329,
                    1326400198319800707,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    15952623350888276173,
                    16668386656275088414,
                    15149997470664743108,
                    243744170780763883,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    6539935666538380087,
                    1980719720669065442,
                    13383457934168992066,
                    1927131268706804563,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    726022817531162023,
                    15442588220780559008,
                    4489669512507920478,
                    1560469002923734423,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    1676688383117600557,
                    1190387942125955373,
                    7267650833640096098,
                    1014803866054245416,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    16430200669741562709,
                    9350015279417676233,
                    18045496101137082444,
                    2689367358901573500,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    3588501979450476963,
                    16781106304107290851,
                    973081265548589304,
                    2445673093771498530,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    15256513869776659556,
                    8248126140198345869,
                    3042694960605158439,
                    2354783039219961461,
                ])),
            ],
            vec![
                F::from(ark_ff::BigInteger256::new([
                    7859581207912083659,
                    3680156647267927892,
                    13740914325891555659,
                    1289058373613339505,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    11562414709102485939,
                    13971556397076503985,
                    7781481372104032915,
                    3264324321420675646,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    1954036446773880801,
                    3344901933345435035,
                    6525769392858920541,
                    2751870152224405226,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    12644161184473169944,
                    17902347025083726586,
                    5725354411966455247,
                    3073790441605843110,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    18355177152526973325,
                    16958542729460458384,
                    5934444251471043491,
                    3450631486176175978,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    9237303288216457697,
                    12825486437619490886,
                    15362716261962355078,
                    2864768779019742483,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    5742136674693172655,
                    14069893693136827288,
                    8858005568186206035,
                    931065555586856742,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    214729479870260177,
                    3168179993983408812,
                    15475713381113203350,
                    2962257666648315584,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    6559639150494048480,
                    12431370545634665556,
                    10392245343534190217,
                    1303493394460324580,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    13015081040160258201,
                    5382328087418747023,
                    10835429684202697429,
                    1850528184737316563,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    6551170348872469285,
                    4075506424154119026,
                    5644681017070499457,
                    3429890127844870717,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    13982609866703412918,
                    8051367758535321607,
                    2464332626477746560,
                    3207230223922015082,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    8335876261400883782,
                    1745789012627482603,
                    3001100076740392898,
                    528210283165109741,
                ])),
            ],
            vec![
                F::from(ark_ff::BigInteger256::new([
                    16127976257387652100,
                    4380607600339160975,
                    297990129792059828,
                    3307285516846842845,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    6143280624390519308,
                    1767312091574588777,
                    11629669997555725587,
                    170770569091162283,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    3575280166892054238,
                    3362683121302559967,
                    8213845199068909166,
                    725733058061349411,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    13055757658958295089,
                    3863168400517265147,
                    9536692932507239058,
                    1999391379089830098,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    3865812897777009382,
                    14149147265008245775,
                    3096941919164820665,
                    2587532804802642317,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    3008425168723161889,
                    9402854417546464417,
                    4143637911473200957,
                    1551631864299089361,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    2269388185283930768,
                    18188613687271001660,
                    8415121170575731279,
                    2567522409533490857,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    14398685065172802918,
                    2186885732558982984,
                    17844679687583442669,
                    237641914863093294,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    15836889352541468696,
                    18414710471775069823,
                    16275300359115496864,
                    2467323399442554520,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    12026135555344111292,
                    17500065895664225,
                    10771767757380454392,
                    43269615937971036,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    9099485175083159187,
                    10696522425043978463,
                    10133129876417510587,
                    3182671081487909621,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    13903967401998623694,
                    8054689978147000846,
                    3143623026566916680,
                    315919524994871334,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    7674572004774874249,
                    12176559662991647433,
                    9298429763068194238,
                    2566108520155440385,
                ])),
            ],
            vec![
                F::from(ark_ff::BigInteger256::new([
                    115304693174306338,
                    18031839111177089969,
                    3036983702799964125,
                    2485059262449488463,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    1366194499183935037,
                    850222378995938475,
                    14293214961614899880,
                    1641498043765152306,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    6782188735028508204,
                    8320515606410762563,
                    10578172417441393890,
                    2874277775996694768,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    7104678664402081025,
                    5788349103154457762,
                    12486675483628199938,
                    1863677076438614600,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    11076121366183614073,
                    16496297997550877763,
                    12167222899915531365,
                    2404734610908786166,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    8022943098596505253,
                    14172731390460966407,
                    11239888195485842840,
                    68518362452923291,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    3750450595168812269,
                    15438648161421020690,
                    18248400322178509274,
                    2224070671245396448,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    8777214732395262015,
                    3296357799410703559,
                    15480718323494612107,
                    639920279630787726,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    12573261807583812562,
                    10141162304384443726,
                    3599741122235033498,
                    3407721216023466832,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    3736658596595466184,
                    12697387120716929024,
                    18156466052144322229,
                    2423117179425531631,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    14113370957820065885,
                    15654503523071928372,
                    5372708722281097336,
                    223832654441554334,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    2242964646051611735,
                    10435420888007138977,
                    13139697409395917293,
                    147055317425391004,
                ])),
                F::from(ark_ff::BigInteger256::new([
                    4753341709649038825,
                    622827003846660644,
                    1964636521678053185,
                    1869301094511500264,
                ])),
            ],
        ];
        return Ok(crate::PoseidonParameters::new(
            ark,
            mds,
            FULL_ROUNDS,
            PARTIAL_ROUNDS[11],
            t.into(),
            ALPHA,
        ));
    } else {
        return Err(PoseidonError::InvalidWidthCircom {
            width: t as usize,
            max_limit: 13usize,
        });
    }
}
